// ============================================================
// File Name		:	Functions
// Description		:	Functions unique to CityRPG.
// ============================================================
// Table of Contents
// 
// 1. Namespaceless
// 2. fxDTSBrick[Data]
// 3. GameConnection
// 4. MinigameSO
// 5. Player
// 6. ServerCmd
// 7. ShapeBase
// 8. WeaponImage
// ============================================================

function test_raycasts()
{
	%time = getRealTime();
	
	for(%a = 0; %a < 100000; %a++)
		containerRayCast("0 0 0", "100 100 100", $TypeMasks::All);
	
	%time = getTimeString((getRealTime() - %time) / 1000);
	
	echo("\c3100,000 Raycasts:" SPC %time SPC "sec.");
}

function test_functions()
{
	%time = getRealTime();
	
	for(%a = 0; %a < 10000000; %a++)
		test();
	
	%time = getTimeString((getRealTime() - %time) / 1000);
	
	echo("\c310,000,000 Empty Functions:" SPC %time SPC "sec.");
}

function test() { }

// ============================================================
// Section 1 : Namespaceless
// ============================================================
// Technical Commands
// +- Commands that serve to aid in the ease of coding.

// Adds a field to the end of %str.
// @string
function addField(%str, %text)
{
	return setField(%str, getFieldCount(%str), %text);
}

// Adds a field at the end of %str if %text is not already a field.
// @string
function addFieldInto(%str, %text)
{
	%fields = getFieldCount(%str);
	
	for(%a = 0; %a < %fields; %a++)
		if(getField(%str, %a) $= %text)
			return %text;
	
	return addField(%str, %text);
}

// CODE FROM BOT_HOLE
// check if the target is in our field of view 
function doFOVCheck( %subject, %target, %returnDot, %forwardVec)
{
	if(%subject.getClassName $= "Player")
	{
		// get all required information
		if( %forwardVec )
			%subjectEye = %subject.getForwardVector();
		else
			%subjectEye = %subject.getEyeVector();
	}
	else
	{
		%subjectEye = %subject.getWorldBoxCenter();
		%subjectEye = vectorNormalize(%subjectEye);
	}
	%posSubject = %subject.getPosition();
	%posTarget = %target.getPosition();
	
	// draw a line between us and the subject
	%line = vectorNormalize( vectorSub( %posTarget, %posSubject ) );
	
	// compare our eye to the line
	%dot = vectorDot( %subjectEye, %line );

	// return the dot product if they want it, otherwise return true or false
	if( %returnDot )
		return %dot;
	else
	{
		// this will return 0 or 1
		%fovCheck = %dot > 0.1;
		
		// take into consideration if they want us to look backwards only?
		// if( %bot.hFOVRange < 0 )
			// %fovCheck = !%fovCheck;
			
		return %fovCheck;
	}
}

// Removes all fields matching %text.
// @string
function removeFields(%str, %text)
{
	%matches = 0;
	
	for(%a = 0; %a < getFieldCount(%str); %a++)
	{
		%match[%matches] = %a;
		%matches++;
	}
	
	for(%a = %matches - 1; %a >= 0; %a--)
		%str = removeField(%str, %match[%matches[%a]]);
	
	return %str;
}

// Returns the absolute value of all of %vector's components.
// @int
function vectorSum(%vector)
{
	%x = getWord(%vector, 0);
	%y = getWord(%vector, 1);
	%z = getWord(%vector, 2);
	
	return mAbs(%x) + mAbs(%y) + mAbs(%z);
}

// CRPG Commands
// +- Commands with use exclusive to CityRPG.
function CRPG_CreateMinigame()
{
	echo("Creating CityRPG Minigame.");
	if(isObject(CRPGMinigame))
		CRPGMinigame.endGame();
	
	%player = "CRPG_Civilian_Armor";
	
	new ScriptObject(CRPGMinigame)
	{
		class = "MiniGameSO";
		owner = -1;
		numMembers = 0;
		
		title = "CityRPG Minigame";
		colorIdx = "2";
		inviteOnly = false;
		UseAllPlayersBricks = true;
		PlayersUseOwnBricks = false;
		
		Points_BreakBrick = 0;
		Points_PlantBrick = 0;
		Points_KillPlayer = 0;
		Points_KillSelf = 0;
		Points_Die = 0;
		
		respawnTime = "10000";
		vehiclerespawntime = "10000";
		brickRespawnTime = "30000";
		playerDataBlock = %player;
		
		useSpawnBricks = true;
		fallingdamage = true;
		weapondamage = true;
		SelfDamage = true;
		VehicleDamage = true;
		brickDamage = true;
		
		enableWand = true;
		EnableBuilding = true;
		enablePainting = true;
		
		StartEquip0 = 0;
		StartEquip1 = 0;
		StartEquip2 = 0;
		StartEquip3 = 0;
		StartEquip4 = 0;
		
		ZAPT_Points_KillPlayer = "2";
		ZAPT_Outbreak = "0";
		ZAPT_FF = "0";
		ZAPT_Mobs = "0";
		zombieAssembledOnce = "1";
		zombieTimeLeft = "1";
		ZAPT_Points_GettingEaten = "0";
		ZAPT_GameMode = "0";
		zombieStressLevel = "0";
		ZAPT_PlayerZombies = "0";
		zombieGameLost = "0";
		ZAPT_Enabled = false;
		zombieTimeLeftCd = "10";
		ZAPT_Time_Outbreak = "30";
		zombieLastDeathTime = "0";
		zombieSelectedInfected = "0";
		zombieTimeSinceTank = "0";
		ZAPT_Points_EatingPlayer = "3";
		ZAPT_Tank = "0";
		zombiesDisabled = "0";
		ZAPT_Points_KillTank = "10";
		ZAPT_Time_Rescue = "180";
		zombieStressCalcTime = "0";
		ZAPT_Points_KillBot = "1";
		ZAPT_Timer = "0";
		ZAPT_Time_Reset = "10";
		ZAPT_Suicide = "0";
		zombieTimeAssembled = "1451.488069";
		zombieAssembledAs = "0";
		zombieStress = "100";
		ZAPT_Points_KillTeam = "-5";
		zombieLastHumanCount = "1";
		ZAPT_Rescue = "0";
		zombieSafehouseVictory = "1";
	};
	
	for(%a = 0; %a < ClientGroup.getCount(); %a++)
		CRPGMinigame.addMember(ClientGroup.getObject(%a));
}

function CRPG_MassProfileUpdate(%property, %action, %value)
{
	for(%a = 0; %a < ClientGroup.getCount(); %a++)
	{
		%client = ClientGroup.getObject(%a);
		
		if(isObject(%profile = %client.profile))
		{
			eval(%profile.getID() @ "." @ %property SPC %action SPC %value @ ";");
			%client.sendStatus();
		}
	}
}

function CRPG_UpdateFactoryProfit(%oldVal, %newVal)
{
	schedule(0, 0, "messageAll", '', '\c3+ CityRPG\c0 - Factory Profit changed from \c3$%1\c0 to \c3$%2\c6.', %oldVal, %newVal);
}

function CRPG_UpdateFactoryRest(%oldVal, %newVal)
{
	schedule(0, 0, "messageAll", '', '\c3+ CityRPG\c0 - Factory Brick Rest Time changed from \c3 \c3%1s\c0 to \c3%2s\c6.', %oldVal, %newVal);
}

function CRPG_UpdatePeace(%oldVal, %newVal)
{
	schedule(0, 0, "messageAll", '', '\c3+ CityRPG\c0 - City-wide peace has been %1\c0.', (%newVal ? "<color:ffaaaa>Enabled" : "Disabled"));
}

function CRPG_UpdateJailBreaks(%oldVal, %newVal)
{
	schedule(0, 0, "messageAll", '', '\c3+ CityRPG\c0 - Jail Breaks are now \c3%1\c6.', %newVal ? "Enabled" : "Disabled");
}

function CRPG_UpdateTriggers(%oldVal, %newVal)
{
	schedule(0, 0, "messageAll", '', '\c3+ CityRPG\c0 - Lot triggers are now \c3%1\c0 flat-bricks tall.', (%newVal * 5));
	
	for(%a = 0; %a < TriggerSS.getCount(); %a++)
	{
		%brick = TriggerSS.getObject(%a);
		%db = %brick.getDatablock();
		
		if(isObject(%db.triggerDatablock))
		{
			if(isObject(%brick.trigger))
				%brick.trigger.delete();
			
			%trigX = getWord(%db.triggerSize, 0);
			%trigY = getWord(%db.triggerSize, 1);
			%trigZ = getWord(%db.triggerSize, 2);
			
			if(mFloor(getWord(%brick.rotation, 3)) == 90)
				%scale = (%trigY / 2) SPC (%trigX / 2) SPC (%trigZ / 2);
			else
				%scale = (%trigX / 2) SPC (%trigY / 2) SPC (%trigZ / 2);
			
			%brick.trigger = new trigger()
			{
				datablock	= %db.triggerDatablock;
				position		= getWords(%brick.getWorldBoxCenter(), 0, 1) SPC getWord(%brick.getWorldBoxCenter(), 2) + ((getWord(%db.triggerSize, 2) / 4) + (%db.brickSizeZ * 0.1));
				rotation		= "1 0 0 0";
				scale			= %scale;
				polyhedron	= "-0.5 -0.5 -0.5 1 0 0 0 1 0 0 0 1";
				parent		= %brick;
			};
		}
	}
	
	for(%a = 0; %a < LotSS.getCount(); %a++)
	{
		%db = LotSS.getObject(%a);
		%db.triggerSize = setWord(%db.triggerSize, 2, %newVal);
	}
}

// Cleans up the GameMode's junk before ending the server.
// @null
function destroyCityRPG()
{
	$CRPG_Temp::ServerDestroying = true;
	
	echo("\n\c1CityRPG is being destroyed.");
	echo("\c1 +- Step 1. CityCleanup " @ (isObject(CityCleanup) ? "\c4exists" : "\c2does not exist") @ "\c1.");
	
	if(isObject(CityCleanup))
	{
		echo("\c1 +- Step 2. CityScripts " @ (isObject(CityScripts) ? "\c4exists" : "\c2does not exist") @ "\c1.");
		
		if(isObject(CityScripts))
		{
			for(%a = 0; CityScripts.getCount(); %a++)
				CityScripts.getObject(0).delete();
			
			echo("\c1 +- +- " @ %a @ " ScriptObject(s) and SimSet(s) destroyed.");
			CityScripts.delete();
		}
		
		echo("\c1 +- Step 3. CityGroups " @ (isObject(CityGroups) ? "\c4exists" : "\c2does not exist") @ "\c1.");
		
		if(isObject(CityGroups))
		{
			for(%a = 0; CityGroups.getCount(); %a++)
			{
				%obj = CityGroups.getObject(0);
				
				for(%b = mClamp(%b, 0, %b); %obj.getCount(); %b++)
					%obj.getObject(0).delete();
				
				%obj.delete();
			}
			
			echo("\c1 +- +- " @ %a @ " SimGroup(s) destroyed.");
			echo("\c1 +- +- " @ %b @ " SimGroup Children destroyed.");
			
			CityGroups.delete();
		}
		
		echo("\c1 +- Step 4. CRPGMinigame " @ (isObject(CRPGMinigame) ? "\c4exists" : "\c2does not exist") @ "\c1.");
		
		if(isObject(CRPGMinigame))
		{
			CRPGMinigame.delete();
		}
	}
	
	CityCleanup.delete();
	
	echo("\c1 +- Step 5. Destroy global variables.");
	deleteVariables("$CRPG_Temp::*");
	deleteVariables("$CRPG::*");
	
	echo("\n");
}

// Other Commands
// +- Fun Bullshit and Test Commands.
function MissionComplete()
{
	for(%a = 0; %a < ClientGroup.getCount(); %a++)
		ClientGroup.getObject(%a).sendMissionComplete();
}

function updateEventTables()
{
	$CRPG_Temp::Events = false;
	
	CRPG();
	
	for(%a = 0; %a < ClientGroup.getCount(); %a++)
		serverCmdRequestEventTables(ClientGroup.getObject(%a));
}

function FreeCrate(%value)
{
	%crates = 0;
	for(%a = 0; %a < ClientGroup.getCount(); %a++)
	{
		%client = ClientGroup.getObject(%a);
		%obj = %client.player;
		
		if(isObject(%obj))
		{
			%hasLot = false;
			
			for(%b = 0; %b < TriggerSS.getCount(); %b++)
			{
				if(TriggerSS.getObject(%b).getGroup().bl_id == %client.bl_id)
				{
					%hasLot = true;
					break;
				}
			}
			
			if(%hasLot)
			{
				%obj.crpgCrateOwner = %client.bl_id;
				%obj.crpgCrateRefPlastic = 0;
				%obj.crpgCratePlastic = 0;
				%obj.crpgCrateWorth = mClamp(%value, 0, %value);
				%obj.crpgCrateMission = -1;
				%obj.mountImage(CRPG_Crate_Image, 0);
				%crates++;
			}
		}
	}
	
	talk(%crates @ " people were given crates.");
}

function KDR()
{
	for(%a = 0; %a < ClientGroup.getCount(); %a++)
	{
		%client = ClientGroup.getObject(%a);
		%profile = %client.profile;
		
		if(isObject(%profile))
		{
			%kills = %client.profile.kills;
			%deaths = %client.profile.deaths;
			%ratio = mFloatLength(%kills / %deaths, 2);
			
			%arrests = %client.profile.expLaw;
			%prestige = mFloatLength((%kills + %arrests) / %deaths, 2) * 100;
			
			messageAll('', '\c4%1\c6: \c2%2 Kills\c6, \c0%3 Deaths\c6. \c3%4 K/D Ratio\c6. \c1%5 Arrests\c6. \c5%6 Prestige\c6.', %client.name, %kills, %deaths, %ratio, %arrests, %prestige);
		}
	}
}

function kickAll(%reason)
{
	while(ClientGroup.getCount())
		ClientGroup.getObject(0).delete(%reason);
}

// ============================================================
// Section 2 : fxDTSBrick[Data]
// ============================================================
function fxDTSBrick::getCodeError(%brick, %client)
{
	%admin	= %client.isCRPGAdmin();
	%db		= %brick.getDatablock();
	%profile	= %client.profile;
	%price	= %db.getPrice();
	
	// General Errors
	// We need a profile.
	if(!isObject(%profile))
		return 'CRPG_Error_General_NoProfile';
	
	// Brick needs to be usable by the planter.
	if(!%brick.isUseable(%client))
		return 'CRPG_Error_General_Restricted';
	
	// Lot Placement Errors
	if(%db.crpgLot)
	{
		// Lots cannot be stacked.
		if(%brick.isInLot(false))
			return 'CRPG_Error_Lot_Double';
		
		// Lots must be on the ground.
		if(!%brick.isGrounded())
			return 'CRPG_Error_Lot_Float';
		
		// Lots cannot be seperated.
		if(%brick.isIsolated())
			return 'CRPG_Error_Lot_Disconnect';
	}
	
	// Brick Placement
	else if(!%admin)
	{
		// Bricks must be on a lot.
		if(!%brick.isInLot(false))
			return 'CRPG_Error_Brick_Baseplate';
		
		// Bricks cannot be bisected by two lots.
		if(%brick.isInMultipleLots())
			return 'CRPG_Error_Brick_Overlap';
		
		// Bricks must be within a trigger.
		// This test is specific in that it
		// sees if the brick is in a lot, but
		// hovering over the Z axis.
		if(%brick.isOverLot(%brick))
			return 'CRPG_Error_Brick_Float';
		
		// Bricks must be COMPLETELY within a lot.
		if(!%brick.isInLot(true))
			return 'CRPG_Error_Brick_Hanging';
	}
	
	// Money Errors
	// If the brick is an investment (cost money), we need the money.
	if(!%admin && %profile.money < %price && %db.category $= "CityRPG")
	//if(!%admin && %db.price > 0 && %profile.money < %price && %db.category $= "CityRPG")
		return 'CRPG_Error_General_Funds';
	
	// If the brick is not an investment, we need the refined plastic.
	if(!%admin && %profile.refplastic < %price && %db.category !$= "CityRPG")
	//if(!%admin && %db.price >= 0 && %profile.refplastic < %price && %db.category !$= "CityRPG") 
		return 'CRPG_Error_General_Funds';
	
	return -1;
}

function fxDTSBrick::getLot(%brick)
{
	%db = %brick.getDatablock();
	
	if(%db.crpgLot)
		return %brick;
	
	// Brick Dimensions
	%brickMinX = mFloatLength(getWord(%brick.getWorldBox(), 0), 1);
	%brickMinY = mFloatLength(getWord(%brick.getWorldBox(), 1), 1);
	%brickMinZ = mFloatLength(getWord(%brick.getWorldBox(), 2), 1);
	
	%brickMaxX = mFloatLength(getWord(%brick.getWorldBox(), 3), 1);
	%brickMaxY = mFloatLength(getWord(%brick.getWorldBox(), 4), 1);
	%brickMaxZ = mFloatLength(getWord(%brick.getWorldBox(), 5), 1);
	
	// Trigger Search
	%pos = %brick.getWorldBoxCenter();
	%mask = $TypeMasks::TriggerObjectType;
	
	// Rectangle Compensation
	if(mFloor(getWord(%brick.rotation, 3)) == 90)
		%boxSize = getWord(%db.brickSizeY, 1) / 2.5 SPC getWord(%db.brickSizeX, 0) / 2.5 SPC getWord(%db.brickSizeZ, 2) / 2.5;
	else
		%boxSize = getWord(%db.brickSizeX, 1) / 2.5 SPC getWord(%db.brickSizeY, 0) / 2.5 SPC getWord(%db.brickSizeZ, 2) / 2.5;
	
	initContainerBoxSearch(%pos, %boxSize, %mask);
	
	while(isObject(%trigger = containerSearchNext()))
	{
		if(%trigger.getDatablock().lotTrigger)
		{
			// "Box in a Box" Check
			%triggerMinX = mFloatLength(getWord(%trigger.getWorldBox(), 0), 1);
			%triggerMinY = mFloatLength(getWord(%trigger.getWorldBox(), 1), 1);
			%triggerMinZ = mFloatLength(getWord(%trigger.getWorldBox(), 2), 1);
			
			%triggerMaxX = mFloatLength(getWord(%trigger.getWorldBox(), 3), 1);
			%triggerMaxY = mFloatLength(getWord(%trigger.getWorldBox(), 4), 1);
			%triggerMaxZ = mFloatLength(getWord(%trigger.getWorldBox(), 5), 1);
			
			if(%triggerMinX <= %brickMinX && %triggerMinY <= %brickMinY && %triggerMinZ <= %brickMinZ && 
				%triggerMaxX >= %brickMaxX && %triggerMaxY >= %brickMaxY && %triggerMaxZ >= %brickMaxZ)
				return %trigger.parent;
		}
	}
	
	return -1;
}

function fxDTSBrick::getPrice(%brick)
{
	return %brick.getDatablock().getPrice();
}

function fxDTSBrick::isCrimeHarbor(%brick, %client)
{
	%trigger = %brick.trigger;
	
	if(!isObject(%trigger))
	{
		%lot = %brick.getLot();
		
		if(!isObject(%lot))
			return false;
		
		%trigger = %lot.trigger;
		
		if(!isObject(%trigger))
			return false;
	}
	
	%triggerMinX = mFloatLength(getWord(%trigger.getWorldBox(), 0), 1);
	%triggerMinY = mFloatLength(getWord(%trigger.getWorldBox(), 1), 1);
	%triggerMinZ = mFloatLength(getWord(%trigger.getWorldBox(), 2), 1);
	
	%triggerMaxX = mFloatLength(getWord(%trigger.getWorldBox(), 3), 1);
	%triggerMaxY = mFloatLength(getWord(%trigger.getWorldBox(), 4), 1);
	%triggerMaxZ = mFloatLength(getWord(%trigger.getWorldBox(), 5), 1);
	
	for(%a = 0; %a < ClientGroup.getCount(); %a++)
	{
		%cc = ClientGroup.getObject(%a);
		
		if(isObject(%client) && %cc.bl_id == %client.bl_id)
			continue;
			
		%cp = %cc.profile;
		
		if(!isObject(%cp))
			continue;
		
		%player = %cc.player;
		
		if(!isObject(%player))
			continue;
		
		%playerX = mFloatLength(getWord(%player.getEyePoint(), 0), 1);
		%playerY = mFloatLength(getWord(%player.getEyePoint(), 1), 1);
		%playerZ = mFloatLength(getWord(%player.getEyePoint(), 2), 1);
		
		if(!%cp.getWantedLevel())
			continue;
		if(%triggerMinX <= %playerX && %triggerMinY <= %playerY && %triggerMinZ <= %playerZ && 
			%triggerMaxX >= %playerX && %triggerMaxY >= %playerY && %triggerMaxZ >= %playerZ)
			return true;
	}
	
	return false;
}

// Load a brick group's profile.
// @ProfileSO
function fxDTSBrick::getProfile(%brick)
{
	%blid		= %brick.getGroup().bl_id;
	%profile	= Profiler.fetchProfile(%blid);
	
	if(!isObject(%profile))
		error("fxDTSBrick::getProfile() - [Could not load profile for absentee client BL_ID " @ %blid @ "]");
	
	return %profile;
}

function fxDTSBrick::isGrounded(%brick)
{
	%db = %brick.getDatablock();
	%pos = vectorSub(%brick.getPosition(), "0 0" SPC (%db.brickSizeZ * 0.2));
	
	if(mFloor(getWord(%brick.rotation, 3)) == 90)
		%boxSize = getWord(%brick.getDatablock().brickSizeY, 1) / 2.5 SPC getWord(%brick.getDatablock().brickSizeX, 0) / 2.5 SPC getWord(%brick.getDatablock().brickSizeZ, 2) / 2.5;
	else
		%boxSize = getWord(%brick.getDatablock().brickSizeX, 1) / 2.5 SPC getWord(%brick.getDatablock().brickSizeY, 0) / 2.5 SPC getWord(%brick.getDatablock().brickSizeZ, 2) / 2.5;
	
	%mask = $TypeMasks::FxBrickAlwaysObjectType;
	
	%plantX = getWord(%brick.getPosition(), 0);
	%plantY = getWord(%brick.getPosition(), 1);
	
	initContainerBoxSearch(%pos, %box, %mask);
	while(%searchObj = containerSearchNext())
	{
		if(!%searchObj.isPlanted())
			continue;
		
		if(%searchObj.isDead())
			continue;
		
		if(%searchObj == %brick)
			continue;
		
		return false;
	}
	
	return true;
}

function fxDTSBrick::isInLot(%brick, %strict)
{
	// Brick Dimensions
	%brickMinX = mFloatLength(getWord(%brick.getWorldBox(), 0), 1);
	%brickMinY = mFloatLength(getWord(%brick.getWorldBox(), 1), 1);
	%brickMinZ = mFloatLength(getWord(%brick.getWorldBox(), 2), 1);
	
	%brickMaxX = mFloatLength(getWord(%brick.getWorldBox(), 3), 1);
	%brickMaxY = mFloatLength(getWord(%brick.getWorldBox(), 4), 1);
	%brickMaxZ = mFloatLength(getWord(%brick.getWorldBox(), 5), 1);
	
	// Trigger Search
	%pos = %brick.getWorldBoxCenter();
	%mask = $TypeMasks::TriggerObjectType;
	
	// Rectangle Compensation
	if(mFloor(getWord(%brick.rotation, 3)) == 90)
		%boxSize = getWord(%brick.getDatablock().brickSizeY, 1) / 2.5 SPC getWord(%brick.getDatablock().brickSizeX, 0) / 2.5 SPC getWord(%brick.getDatablock().brickSizeZ, 2) / 2.5;
	else
		%boxSize = getWord(%brick.getDatablock().brickSizeX, 1) / 2.5 SPC getWord(%brick.getDatablock().brickSizeY, 0) / 2.5 SPC getWord(%brick.getDatablock().brickSizeZ, 2) / 2.5;
	
	initContainerBoxSearch(%pos, %boxSize, %mask);
	
	while(isObject(%trigger = containerSearchNext()))
	{
		if(%trigger.getDatablock().lotTrigger)
		{
			// %strict means it must fit ENTIRELY inside, not just overlapping.
			if(%strict)
			{
				// "Box in a Box" Check
				%triggerMinX = mFloatLength(getWord(%trigger.getWorldBox(), 0), 1);
				%triggerMinY = mFloatLength(getWord(%trigger.getWorldBox(), 1), 1);
				%triggerMinZ = mFloatLength(getWord(%trigger.getWorldBox(), 2), 1);
				
				%triggerMaxX = mFloatLength(getWord(%trigger.getWorldBox(), 3), 1);
				%triggerMaxY = mFloatLength(getWord(%trigger.getWorldBox(), 4), 1);
				%triggerMaxZ = mFloatLength(getWord(%trigger.getWorldBox(), 5), 1);
				
				if(%triggerMinX <= %brickMinX && %triggerMinY <= %brickMinY && %triggerMinZ <= %brickMinZ && 
					%triggerMaxX >= %brickMaxX && %triggerMaxY >= %brickMaxY && %triggerMaxZ >= %brickMaxZ)
					return true;
			}
			// We already know it's overlapping because we're doing a box search to find it in the first place!
			else
				return true;
		}
	}
	
	return false;
}

function fxDTSBrick::isInMultipleLots(%brick)
{
	%inLot = false;
	
	// Trigger Search
	%pos = %brick.getWorldBoxCenter();
	%mask = $TypeMasks::TriggerObjectType;
	
	// Rectangle Compensation
	if(mFloor(getWord(%brick.rotation, 3)) == 90)
		%boxSize = getWord(%brick.getDatablock().brickSizeY, 1) / 2.5 SPC getWord(%brick.getDatablock().brickSizeX, 0) / 2.5 SPC getWord(%brick.getDatablock().brickSizeZ, 2) / 2.5;
	else
		%boxSize = getWord(%brick.getDatablock().brickSizeX, 1) / 2.5 SPC getWord(%brick.getDatablock().brickSizeY, 0) / 2.5 SPC getWord(%brick.getDatablock().brickSizeZ, 2) / 2.5;
	
	initContainerBoxSearch(%pos, %boxSize, %mask);
	
	while(isObject(%trigger = containerSearchNext()))
	{
		if(%trigger.getDatablock().lotTrigger)
		{
			if(!%inLot)
				%inLot = true;
			else
				return true;
		}
	}
	
	return false;
}

function fxDTSBrick::isIsolated(%brick)
{
	%db = %brick.getDatablock();
	%pos = %brick.getPosition();
	%box = ((%db.brickSizeX / 2) + 0.1) SPC ((%db.brickSizeY / 2) + 0.1) SPC (%db.brickSizeZ * 0.2);
	%mask = $TypeMasks::FxBrickAlwaysObjectType;
	
	%plantX = getWord(%brick.getPosition(), 0);
	%plantY = getWord(%brick.getPosition(), 1);
	
	initContainerBoxSearch(%pos, %box, %mask);
	while(%searchObj = containerSearchNext())
	{
		if(!%searchObj.isPlanted())
			continue;
		
		if(%searchObj.isDead())
			continue;
		
		if(%searchObj == %brick)
			continue;
		
		if(!%searchObj.getDataBlock().crpgLot && %searchObj.getDataBlock().category !$= "Baseplates")
			continue;
		
		return false;
	}
	
	return true;
}

function fxDTSBrick::isOverLot(%brick)
{
	// Brick Dimensions
	%brickMinX = mFloatLength(getWord(%brick.getWorldBox(), 0), 1);
	%brickMinY = mFloatLength(getWord(%brick.getWorldBox(), 1), 1);
	%brickMinZ = mFloatLength(getWord(%brick.getWorldBox(), 2), 1);
	
	%brickMaxX = mFloatLength(getWord(%brick.getWorldBox(), 3), 1);
	%brickMaxY = mFloatLength(getWord(%brick.getWorldBox(), 4), 1);
	%brickMaxZ = mFloatLength(getWord(%brick.getWorldBox(), 5), 1);
	
	// Trigger Search
	%pos = %brick.getWorldBoxCenter();
	%mask = $TypeMasks::TriggerObjectType;
	
	// Rectangle Compensation
	if(mFloor(getWord(%brick.rotation, 3)) == 90)
		%boxSize = getWord(%brick.getDatablock().brickSizeY, 1) / 2.5 SPC getWord(%brick.getDatablock().brickSizeX, 0) / 2.5 SPC getWord(%brick.getDatablock().brickSizeZ, 2) / 2.5;
	else
		%boxSize = getWord(%brick.getDatablock().brickSizeX, 1) / 2.5 SPC getWord(%brick.getDatablock().brickSizeY, 0) / 2.5 SPC getWord(%brick.getDatablock().brickSizeZ, 2) / 2.5;
	
	initContainerBoxSearch(%pos, %boxSize, %mask);
	
	while(isObject(%trigger = containerSearchNext()))
	{
		if(%trigger.getDatablock().lotTrigger)
		{
			// "Box in a Box but also sticking out above it" Check
			%triggerMinX = mFloatLength(getWord(%trigger.getWorldBox(), 0), 1);
			%triggerMinY = mFloatLength(getWord(%trigger.getWorldBox(), 1), 1);
			%triggerMinZ = mFloatLength(getWord(%trigger.getWorldBox(), 2), 1);
			
			%triggerMaxX = mFloatLength(getWord(%trigger.getWorldBox(), 3), 1);
			%triggerMaxY = mFloatLength(getWord(%trigger.getWorldBox(), 4), 1);
			%triggerMaxZ = mFloatLength(getWord(%trigger.getWorldBox(), 5), 1);
			
			if(%triggerMinX <= %brickMinX && %triggerMinY <= %brickMinY && %triggerMinZ <= %brickMinZ && 
				%triggerMaxX >= %brickMaxX && %triggerMaxY >= %brickMaxY && %triggerMaxZ < %brickMaxZ)
				return true;
		}
	}
	
	return false;
}

function fxDTSBrick::isUseable(%brick, %client)
{
	%db = %brick.getDatablock();
	
	if(%db.adminOnly && !%client.isCRPGAdmin())
		return false;
	
	return true;
}

function fxDTSBrick::onClicked(%brick, %client, %profile, %obj, %pos)
{
	if(!isObject(%brick) || !isObject(%client) || !isObject(%profile) || !%profile.getJob().ninja || !isObject(%obj) || %brick.getID() == %obj.getID())
		return false;
	
	if(!isObject(%col.Shape))
	{
		%start = %obj.getPosition();
		%end = vectorSub(%start, "0 0 1");
		
		%masks = ($TypeMasks::FxBrickObjectType | $TypeMasks::PlayerObjectType | $TypeMasks::StaticObjectType | $TypeMasks::TerrainObjectType | $TypeMasks::VehicleObjectType);
		
		%floor = getWord(containerRayCast(%start, %end, %masks, %obj), 0);
		
		if(!isObject(%floor))
		{
			if(%obj.getEnergyLevel() > 25)
			{
				%zBrickPlane = getWord(%brick.getWorldBox(), 5);
				%zObjPlane = getWord(%obj.getWorldBox(), 2);
				
				if(%zBrickPlane > %zObjPlane)
				{
					%vel = %obj.getVelocity();
					%velZ = getWord(%vel, 2);
					
					if(mFloor(%velZ) < 15)
					{
						%velZ = mClampF(%velZ + 7.5, -mAbs(%velZ), 15.0);
						%energy = mAbs(%velZ) * 1.5;
							
						if(%obj.getEnergyLevel() > %energy)
						{
							%obj.setEnergyLevel(%obj.getEnergyLevel() - %energy);
							%obj.setVelocity(getWords(%vel, 0, 1) SPC %velZ);
						}
					}
				}
			}
		}
	}
}

// Data
function fxDTSBrickData::getPrice(%this)
{
	if(%this.price)
		return %this.price;
	
	%volume = %this.getVolume();
	%price = mAbs(mCeil(%volume / 10));
	
	return %price;
}

function fxDTSBrickData::onChop(%this, %brick, %client)
{
	%brick.onChop(%client);
}

function fxDTSBrickData::onMine(%this, %brick, %client)
{
	%volume = %this.getVolume();
	
	if(!%this.isRock && %volume < 90)
	{
		%obj = %client.player;
		%profile =  %client.profile;
		
		if(%client.crpgEvil && isObject(%obj) && isObject(%profile) && !%profile.isInmate())
		{
			%aimVec = vectorScale(%obj.getAimVector(), 6);
			%time = mClamp(%client.minigame.brickRespawnTime, 0, %client.minigame.brickRespawnTime);
			
			if(%client.bl_id != %brick.getGroup().bl_id)
			{
				%client.commitCrime("Destruction", 5, false, true, false, false, %time * 2);
				
				if(isObject(%client.mission))
				{
					%client.mission.onSmashBrick(%brick);
				}
			}
			
			
			%brick.fakeKillBrick(%aimVec, mClamp(%volume / 10, 1, mCeil(%volume)), %time, %client);
		}
	}
	
	%brick.onMine(%client);
}

// Input Events
function fxDTSBrick::onChop(%brick, %client)
{
	$InputTarget_["Self"] = %brick;
	
	if(isObject(%obj = %client.player))
	{
		$InputTarget_["Player"] = %obj;
	}
	else
	{
		$InputTarget_["Player"] = -1;
	}
	
	if(isObject(%client))
	{
		$InputTarget_["Client"] = %client;
		
		if(isObject(%profile = %client.profile))
		{
			$InputTarget_["ProfileSO"] = %profile;
		}
	}
	else
	{
		$InputTarget_["Client"] = -1;
	}
	
	$InputTarget_["MiniGame"] = getMiniGameFromObject(%brick);
	
	if(isObject(%brick.getGroup().client))
	{
		if(isObject(%brick.getGroup().client.player))
		{
			$InputTarget_["OwnerPlayer"] = %brick.getGroup().client.player;
		}
		else
		{
			$InputTarget_["OwnerPlayer"] = -1;
		}
		
		$InputTarget_["OwnerClient"] = %brick.getGroup().client;
	}
	else
	{
		$InputTarget_["OwnerPlayer"] = -1;
		$InputTarget_["OwnerClient"] = -1;
	}
	
	%brick.processInputEvent("onChop", %client);
}

function fxDTSBrick::onMine(%brick, %client)
{
	$InputTarget_["Self"] = %brick;
	
	if(isObject(%obj = %client.player))
	{
		$InputTarget_["Player"] = %obj;
	}
	else
	{
		$InputTarget_["Player"] = -1;
	}
	
	if(isObject(%client))
	{
		$InputTarget_["Client"] = %client;
		
		if(isObject(%profile = %client.profile))
		{
			$InputTarget_["ProfileSO"] = %profile;
		}
	}
	else
	{
		$InputTarget_["Client"] = -1;
	}
	
	$InputTarget_["MiniGame"] = getMiniGameFromObject(%brick);
	
	if(isObject(%brick.getGroup().client))
	{
		if(isObject(%brick.getGroup().client.player))
		{
			$InputTarget_["OwnerPlayer"] = %brick.getGroup().client.player;
		}
		else
		{
			$InputTarget_["OwnerPlayer"] = -1;
		}
		
		$InputTarget_["OwnerClient"] = %brick.getGroup().client;
	}
	else
	{
		$InputTarget_["OwnerPlayer"] = -1;
		$InputTarget_["OwnerClient"] = -1;
	}
	
	%brick.processInputEvent("onMine", %client);
}

function fxDTSBrick::onLotEnter(%brick, %obj)
{
	$InputTarget_["Self"] = %brick;
	
	if(isObject(%obj))
	{
		$InputTarget_["Player"] = %obj;
	}
	else
	{
		$InputTarget_["Player"] = -1;
	}
	
	if(isObject(%obj.client))
	{
		$InputTarget_["Client"] = %obj.client;
		
		if(isObject(%profile = %obj.client.profile))
		{
			$InputTarget_["ProfileSO"] = %profile;
		}
	}
	else
	{
		$InputTarget_["Client"] = -1;
	}
	
	$InputTarget_["MiniGame"] = getMiniGameFromObject(%brick);
	
	if(isObject(%brick.getGroup().client))
	{
		if(isObject(%brick.getGroup().client.player))
		{
			$InputTarget_["OwnerPlayer"] = %brick.getGroup().client.player;
		}
		else
		{
			$InputTarget_["OwnerPlayer"] = -1;
		}
		
		$InputTarget_["OwnerClient"] = %brick.getGroup().client;
	}
	else
	{
		$InputTarget_["OwnerPlayer"] = -1;
		$InputTarget_["OwnerClient"] = -1;
	}
	
	%brick.processInputEvent("onLotEnter", %obj);
}

function fxDTSBrick::onLotLeave(%brick, %obj)
{
	$InputTarget_["Self"] = %brick;
	
	if(isObject(%obj))
	{
		$InputTarget_["Player"] = %obj;
	}
	else
	{
		$InputTarget_["Player"] = -1;
	}
	
	if(isObject(%obj.client))
	{
		$InputTarget_["Client"] = %obj.client;
		
		if(isObject(%profile = %obj.client.profile))
		{
			$InputTarget_["ProfileSO"] = %profile;
		}
	}
	else
	{
		$InputTarget_["Client"] = -1;
	}
	
	$InputTarget_["MiniGame"] = getMiniGameFromObject(%brick);
	
	if(isObject(%brick.getGroup().client))
	{
		if(isObject(%brick.getGroup().client.player))
		{
			$InputTarget_["OwnerPlayer"] = %brick.getGroup().client.player;
		}
		else
		{
			$InputTarget_["OwnerPlayer"] = -1;
		}
		
		$InputTarget_["OwnerClient"] = %brick.getGroup().client;
	}
	else
	{
		$InputTarget_["OwnerPlayer"] = -1;
		$InputTarget_["OwnerClient"] = -1;
	}
	
	%brick.processInputEvent("onLotLeave", %obj);
}

function fxDTSBrick::onResourceDepleat(%brick, %client)
{
	$InputTarget_["Self"] = %brick;
	
	if(isObject(%obj = %client.player))
	{
		$InputTarget_["Player"] = %obj;
	}
	else
	{
		$InputTarget_["Player"] = -1;
	}
	
	if(isObject(%client))
	{
		$InputTarget_["Client"] = %client;
		
		if(isObject(%profile = %client.profile))
		{
			$InputTarget_["ProfileSO"] = %profile;
		}
	}
	else
	{
		$InputTarget_["Client"] = -1;
	}
	
	$InputTarget_["MiniGame"] = getMiniGameFromObject(%brick);
	
	if(isObject(%brick.getGroup().client))
	{
		if(isObject(%brick.getGroup().client.player))
		{
			$InputTarget_["OwnerPlayer"] = %brick.getGroup().client.player;
		}
		else
		{
			$InputTarget_["OwnerPlayer"] = -1;
		}
		
		$InputTarget_["OwnerClient"] = %brick.getGroup().client;
	}
	else
	{
		$InputTarget_["OwnerPlayer"] = -1;
		$InputTarget_["OwnerClient"] = -1;
	}
	
	%brick.processInputEvent("onResourceDepleat", %client);
}

function fxDTSBrick::onServiceAccepted(%brick, %client)
{
	$InputTarget_["Self"] = %brick;
	
	if(isObject(%obj = %client.player))
	{
		$InputTarget_["Player"] = %obj;
	}
	else
	{
		$InputTarget_["Player"] = -1;
	}
	
	if(isObject(%client))
	{
		$InputTarget_["Client"] = %client;
		
		if(isObject(%profile = %client.profile))
		{
			$InputTarget_["ProfileSO"] = %profile;
		}
	}
	else
	{
		$InputTarget_["Client"] = -1;
	}
	
	$InputTarget_["MiniGame"] = getMiniGameFromObject(%brick);
	
	if(isObject(%brick.getGroup().client))
	{
		if(isObject(%brick.getGroup().client.player))
		{
			$InputTarget_["OwnerPlayer"] = %brick.getGroup().client.player;
		}
		else
		{
			$InputTarget_["OwnerPlayer"] = -1;
		}
		
		$InputTarget_["OwnerClient"] = %brick.getGroup().client;
	}
	else
	{
		$InputTarget_["OwnerPlayer"] = -1;
		$InputTarget_["OwnerClient"] = -1;
	}
	
	%brick.processInputEvent("onServiceAccepted", %client);
}

function fxDTSBrick::onServiceRejected(%brick, %client)
{
	$InputTarget_["Self"] = %brick;
	
	if(isObject(%obj = %client.player))
	{
		$InputTarget_["Player"] = %obj;
	}
	else
	{
		$InputTarget_["Player"] = -1;
	}
	
	if(isObject(%client))
	{
		$InputTarget_["Client"] = %client;
		
		if(isObject(%profile = %client.profile))
		{
			$InputTarget_["ProfileSO"] = %profile;
		}
	}
	else
	{
		$InputTarget_["Client"] = -1;
	}
	
	$InputTarget_["MiniGame"] = getMiniGameFromObject(%brick);
	
	if(isObject(%brick.getGroup().client))
	{
		if(isObject(%brick.getGroup().client.player))
		{
			$InputTarget_["OwnerPlayer"] = %brick.getGroup().client.player;
		}
		else
		{
			$InputTarget_["OwnerPlayer"] = -1;
		}
		
		$InputTarget_["OwnerClient"] = %brick.getGroup().client;
	}
	else
	{
		$InputTarget_["OwnerPlayer"] = -1;
		$InputTarget_["OwnerClient"] = -1;
	}
	
	%brick.processInputEvent("onServiceRejected", %client);
}

function fxDTSBrick::onTimeDay(%brick, %round)
{
	if(ClientGroup.getCount())
	{
		%client = %round % ClientGroup.getCount();
		%client = ClientGroup.getObject(%client);
		
		$InputTarget_["Self"] = %brick;
		
		$InputTarget_["MiniGame"] = getMiniGameFromObject(%brick);
		
		if(isObject(%brick.getGroup().client))
		{
			if(isObject(%brick.getGroup().client.player))
			{
				$InputTarget_["OwnerPlayer"] = %brick.getGroup().client.player;
			}
			else
			{
				$InputTarget_["OwnerPlayer"] = -1;
			}
			
			$InputTarget_["OwnerClient"] = %brick.getGroup().client;
		}
		else
		{
			$InputTarget_["OwnerPlayer"] = -1;
			$InputTarget_["OwnerClient"] = -1;
		}
		
		%brick.processInputEvent("onTimeDay", %client);
	}
}

function fxDTSBrick::onTimeNight(%brick, %round)
{
	if(ClientGroup.getCount())
	{
		%client = %round % ClientGroup.getCount();
		%client = ClientGroup.getObject(%client);
		
		$InputTarget_["Self"] = %brick;
		
		$InputTarget_["MiniGame"] = getMiniGameFromObject(%brick);
		
		if(isObject(%brick.getGroup().client))
		{
			if(isObject(%brick.getGroup().client.player))
			{
				$InputTarget_["OwnerPlayer"] = %brick.getGroup().client.player;
			}
			else
			{
				$InputTarget_["OwnerPlayer"] = -1;
			}
			
			$InputTarget_["OwnerClient"] = %brick.getGroup().client;
		}
		else
		{
			$InputTarget_["OwnerPlayer"] = -1;
			$InputTarget_["OwnerClient"] = -1;
		}
		
		%brick.processInputEvent("onTimeNight", %client);
	}
}

function fxDTSBrick::onWorkDone(%brick, %client)
{
	$InputTarget_["Self"] = %brick;
	
	if(isObject(%obj = %client.player))
	{
		$InputTarget_["Player"] = %obj;
	}
	else
	{
		$InputTarget_["Player"] = -1;
	}
	
	if(isObject(%client))
	{
		$InputTarget_["Client"] = %client;
		
		if(isObject(%profile = %client.profile))
		{
			$InputTarget_["ProfileSO"] = %profile;
		}
	}
	else
	{
		$InputTarget_["Client"] = -1;
	}
	
	$InputTarget_["MiniGame"] = getMiniGameFromObject(%brick);
	
	if(isObject(%brick.getGroup().client))
	{
		if(isObject(%brick.getGroup().client.player))
		{
			$InputTarget_["OwnerPlayer"] = %brick.getGroup().client.player;
		}
		else
		{
			$InputTarget_["OwnerPlayer"] = -1;
		}
		
		$InputTarget_["OwnerClient"] = %brick.getGroup().client;
	}
	else
	{
		$InputTarget_["OwnerPlayer"] = -1;
		$InputTarget_["OwnerClient"] = -1;
	}
	
	%brick.processInputEvent("onWorkDone", %client);
}

function fxDTSBrick::onWorked(%brick, %client)
{
	$InputTarget_["Self"] = %brick;
	
	if(isObject(%obj = %client.player))
	{
		$InputTarget_["Player"] = %obj;
	}
	else
	{
		$InputTarget_["Player"] = -1;
	}
	
	if(isObject(%client))
	{
		$InputTarget_["Client"] = %client;
		
		if(isObject(%profile = %client.profile))
		{
			$InputTarget_["ProfileSO"] = %profile;
		}
	}
	else
	{
		$InputTarget_["Client"] = -1;
	}
	
	$InputTarget_["MiniGame"] = getMiniGameFromObject(%brick);
	
	if(isObject(%brick.getGroup().client))
	{
		if(isObject(%brick.getGroup().client.player))
		{
			$InputTarget_["OwnerPlayer"] = %brick.getGroup().client.player;
		}
		else
		{
			$InputTarget_["OwnerPlayer"] = -1;
		}
		
		$InputTarget_["OwnerClient"] = %brick.getGroup().client;
	}
	else
	{
		$InputTarget_["OwnerPlayer"] = -1;
		$InputTarget_["OwnerClient"] = -1;
	}
	
	%brick.processInputEvent("onWorked", %client);
}

function fxDTSBrick::onWorkReady(%brick, %client)
{
	$InputTarget_["Self"] = %brick;
	
	if(isObject(%obj = %client.player))
	{
		$InputTarget_["Player"] = %obj;
	}
	else
	{
		$InputTarget_["Player"] = -1;
	}
	
	if(isObject(%client))
	{
		$InputTarget_["Client"] = %client;
		
		if(isObject(%profile = %client.profile))
		{
			$InputTarget_["ProfileSO"] = %profile;
		}
	}
	else
	{
		$InputTarget_["Client"] = -1;
	}
	
	$InputTarget_["MiniGame"] = getMiniGameFromObject(%brick);
	
	if(isObject(%brick.getGroup().client))
	{
		if(isObject(%brick.getGroup().client.player))
		{
			$InputTarget_["OwnerPlayer"] = %brick.getGroup().client.player;
		}
		else
		{
			$InputTarget_["OwnerPlayer"] = -1;
		}
		
		$InputTarget_["OwnerClient"] = %brick.getGroup().client;
	}
	else
	{
		$InputTarget_["OwnerPlayer"] = -1;
		$InputTarget_["OwnerClient"] = -1;
	}
	
	%brick.processInputEvent("onWorkReady", %client);
}

// Output Events
function fxDTSBrick::buyPlastic(%brick, %price, %max, %client)
{
	if(isObject(%client))
	{
		%profile = %client.profile;
		
		if(isObject(%profile))
		{
			%brickgroup = %brick.getGroup();
			%owner = %brickgroup.client;
			
			if(isObject(%brickGroup))
			{
				// Load a brick group's profile.
				%brickProfile = %brick.getProfile();
				
				if(isObject(%brickProfile))
				{
					if(%brickProfile.plastic < %max)
					{
						%plastic = %profile.plastic;
						
						if(%plastic || %brickProfile.getID() == %profile.getID())
						{
							%remainder = %max - %brickProfile.plastic;
							%buying = mClamp(%plastic, 0, %remainder);
							%sellPrice = mFloor(%buying * %price);
							
							if(%sellPrice > %brickProfile.bank)
							{
								%buying = mFloor(%brickProfile.bank / %price);
								%sellPrice = mFloor(%buying * %price);
							}
							
							if(%buying > 0)
							{
								if(%brickProfile.getID() != %profile.getID())
								{
									%client.salesBrick			= %brick;
									%client.salesBrickProduct	= "plastic";
									%client.salesBrickItem		= %buying;
									%client.salesBrickMarkup	= 0;
									%client.salesBrickName		= "Raw Plastic";
									%client.salesBrickPrice		= %sellPrice;
									
									commandToClient(%client, 'messageBoxYesNo', "Do you want to sell" SPC %buying SPC " Raw Plastic?",
										"<font:courier new:20>SALES RECEIPT<just:left>\n" NL
										"<font:courier new:16>RAW PLASTIC	 : " @ %buying NL
										"PRICE OFFERED  : $" @ %sellPrice NL
										"MERCHANT STOCK : " @ mFloor(%brickProfile.plastic) NL
										"MERCHANT QUOTA : " @ %max NL
										"YOUR REMAINDER : " @ (%profile.plastic - %buying) NL
										"\n\n<just:center><font:courier new:20>ACCEPT TRANSACTION?",
										'Event_Buy', 'Event_Cancel');
								}
								else
								{
									commandToClient(%client, 'messageBoxOK', "Quota Information",
										"<font:courier new:20>SALES QUOTA<just:left><font:courier new:16>\n" NL
										"YOUR STOCK	 : " @ mFloor(%brickProfile.plastic) NL
										"REF. PLASTIC  : " @ mFloor(%brickProfile.refplastic) NL
										"PRICE OFFERED : $" @ mFloatLength(mClampF(%price, 0.00, 2.00), 2) NL
										"BRICK QUOTA	: " @ mFloor(%max) NL
										"\n<font:courier new:20><just:center>Relax.");
								}
							}
							else if(%brickProfile.getID() == %profile.getID())
							{
								commandToClient(%client, 'messageBoxOK', "Quota Information",
									"<font:courier new:20>SALES QUOTA<just:left><font:courier new:16>\n" NL
									"YOUR STOCK	 : " @ mFloor(%brickProfile.plastic) NL
									"REF. PLASTIC  : " @ mFloor(%brickProfile.refplastic) NL
									"PRICE OFFERED : $" @ mFloatLength(mClampF(%price, 0.00, 2.00), 2) NL
									"BRICK QUOTA	: " @ mFloor(%max) NL
									"\n<font:courier new:20><color:ff0000><just:center>NO BANK FUNDS!!");
							}
							else
								messageClient(%client, '', "\c6Sorry, but this vendor does not have enough money to buy any plastic.");
						}
						else
							messageClient(%client, '', "\c6Sorry, but you do not have plastic to sell.");
					}
					else if(%brickProfile.getID() == %profile.getID())
					{
						commandToClient(%client, 'messageBoxOK', "Quota Information",
							"<font:courier new:20>SALES QUOTA<just:left><font:courier new:16>\n" NL
							"YOUR STOCK	 : " @ mFloor(%brickProfile.plastic) NL
							"REF. PLASTIC  : " @ mFloor(%brickProfile.refplastic) NL
							"PRICE OFFERED : $" @ mFloatLength(mClampF(%price, 0.00, 2.00), 2) NL
							"BRICK QUOTA	: " @ mFloor(%max) NL
							"\n<font:courier new:20><color:ff0000><just:center>QUOTA INTAKE MET!!");
					}
					else
						messageClient(%client, '', "\c6Sorry, but this vendor has reached their plastic intake limit.");
				}
			}
		}
	}
}

function fxDTSBrick::doTransaction(%brick, %client)
{
	if(isObject(%client))
	{
		%ownerID = %brick.getBL_IDFromObject();
		
		if(%client.getBLID() == %owner)
		{
			%gui = "TransactionOwner";
			commandToClient(%client, 'DisplayCRPGGUI', %gui);
			%client.lastGUI = $sim::time;
		}
		else
		{
			%gui = "TransactionRegular";
			// WIP
		}
	}
}

function fxDTSBrick::doWork(%brick, %profit, %client)
{
	if(!isObject(%client))
		return;
	
	if(!isObject(%obj = %client.player))
		return %client.sendMessageBox("Does Not Compute", "Sorry!\n\nYou have to spawn first.");
	
	if(!isObject(%profile = %client.profile))
		return %client.sendMessageBox("Does Not Compute", "Sorry!\n\nThe server has encountered an error, you do not have a profile.");
	
	if(!%brick.crpgBrickWorked || %brick.crpgBrickWorked != %obj.getID())
		return %client.sendMessageBox("Does Not Compute", "The events on this brick are broken.\n" NL
			"To make a Factory Brick, use this event series:" NL
			"onWorked -> Self -> DoWork");
	
	if($sim::time - %brick.crpgWorkedTime < $CRPG::FactoryRest)
		return %client.bottomPrint("<just:center>BRICK OVERWORKED!", 2);
	
	if($sim::time - %obj.crpgWorkTime < 0.50)
		return;
	
	%brickGroup	= %brick.getGroup();
	%owner		= %brickgroup.client;
	
	if(!isObject(%brickGroup))
		return %client.sendMessageBox("Does Not Compute", "Sorry!\n\nThe server has had an error and this brick has no brickgroup. Please pick another factory!");
	
	%brickProfile = %brick.getProfile();
	
	if(!isObject(%brickProfile))
		return %client.sendMessageBox("Does Not Compute", "Sorry!\n\nThe server has had an error and this brick's group does not have a profile tied to it. Please pick another factory!");
	
	if(%brickProfile.getJob().getID() != JobFactoryOwnerSO.getID())
		return %client.sendMessageBox("Does Not Compute", "Sorry!\n\nThe brick owner is not a Factory Owner.");
	
	%drone	= mClamp(mFloor(%profile.exp["Drone"] / 250), 0, 5);
	%hunger	= mFloor(5 * (mClamp(%profile.hunger, 0, 100) / 100));

	%rollMin	= 2 + %drone;
	%rollMax	= 4 + %drone + %hunger;
	
	%chance = getRandom(%rollMin, %rollMax);
	
	%brick.crpgWorked[%obj.getID()] += %chance;
	%obj.crpgWorkTime = $sim::time;
	
	if(%brick.crpgWorked[%obj.getID()] < 100)
	{
		%worked = mFloor(100 * (%brick.crpgWorked[%obj.getID()] / 100));
		%client.sendProgress(%worked, "BRICK WORKED");
		
		if(%obj.crpgWorkLastOwner !$= "" && %obj.crpgWorkLastOwner != %brickGroup.bl_id && isObject(%owner.mission))
			%owner.mission.onPatronConvert(%client);
		
		%obj.crpgWorkLastOwner = %brickGroup.bl_id;
	}
	else
	{
		%brick.schedule($CRPG::FactoryRest * 1000, "onWorkReady", %client);
		%brick.crpgWorkedTime = $sim::time;
		%brick.crpgWorked[%obj.getID()] = "";
		
		%profile.hunger = mClamp(%profile.hunger - 1, 0, %profile.hunger);
		
		// EXP
		%profile.addEXP(1, "Drone");
		
		// Money
		%wages = mFloor($CRPG::FactoryProfit * %profit);

		//CHANGE: 12/26/15:
		//We made it so that the employer pays the workers as long as the employer has money.

		//%netProfit = $CRPG::FactoryProfit - %wages;
	
		if(%brickProfile.bank < %wages)
		{
			%paidWages = 0;

			//commandToClient(%client,'MessageBoxOK',"Oops!","The employer has run out of money to pay you!");
			%client.sendMessageBox("Does Not Compute", "Your employer ran out of money!");
		}
		else
		{
			%paidWages = %wages;
		}

		//%profile.money += %wages;
		//%brickProfile.bank += %netProfit;

		%profile.money += %paidWages;
		%brickProfile.bank -= %paidWages;
		
		if(%paidWages > 0)
		{
			commandToClient(%client, 'CRPGCashChange', %paidWages, %profile.money, %profile.bank, %profile.debt);
		}
		commandToClient(%client, 'BottomPrint', "<just:center>\c6WORK FINISHED!", 2);
		%brick.onWorkDone(%client);
		
		// Plastic
		%modified = mClamp(getRandom(0, %brickProfile.plastic), 0, 10);
		
		if(%modified)
		{
			%brickProfile.plastic -= %modified;
			%brickProfile.refplastic += %modified;
			%brickProfile.addEXP(1, "Store");
		}
		
		if(isObject(%owner))
			%owner.sendStatus();
	}
}

function fxDTSBrick::explodePlastic(%brick, %client)
{
	if(isObject(%client))
	{
		%db = %brick.getDatablock();
		%profile = %client.profile;
		
		if(isObject(%db) && isObject(%profile))
		{
			%payout = mFloor(getRandom(%db.payoutMin, %db.payoutMax));
			%remainder = %payout;
			
			while(%remainder > 0)
			{
				%bonus = %profile.getJob().getLaborBonus(%client);
				
				if(getRandom(0, 100) - %bonus > 0)
				{
					%fraction = getRandom( mClamp(mCeil(%payout / 4), 0, %payout), %remainder);
					%remainder -= %fraction;
					
					%item = new Item()
					{
						datablock = "CRPG_Plastic_" @ getRandom(1, 3) @ "_Item";
						plasticValue = %fraction;
						worker = %client;
						spawnTime = $sim::time;
					};
					
					%scaleFactor = mClampF( (%fraction / %payout) + 0.5, 0.5, 1.5);
				}
				else
				{
					%remainder -= 1;
					
					%gemMin = 50;
					%gemMax = 250;
					
					%item = new Item()
					{
						datablock = CRPG_Gem_Item;
						cashValue = getRandom(%gemMin, %gemMax);
						worker = %client;
						spawnTime = $sim::time;
					};
					
					%scaleFactor = 1;
				}
				
				if(isObject(%item))
				{
					%trans = %brick.getWorldBoxCenter();
					%trans = %trans SPC 0 SPC 0 SPC getWord("-1 1", getRandom(0, 1)) SPC (getRandom(0, 3000) / 1000);
					%vel = vectorScale(getRandom(-300, 300) SPC getRandom(-300, 300) SPC "500", 1/100);
					
					%item.canPickup = false;
					%item.playThread(0, "bounce");
					%item.setScale(%scaleFactor SPC %scaleFactor SPC %scaleFactor);
					%item.setTransform(%trans);
					%item.setVelocity(%vel);
					%item.schedule(60000, "schedulePop");
					
					MissionCleanup.add(%item);
				}
				
				%brick.playSound(CRPG_Resource_Depleat_Sound);
			}
		}
	}
}

function fxDTSBrick::sellFood(%brick, %fill, %markup, %name, %client)
{
	if(isObject(%client))
	{
		%profile = %client.profile;
		
		if(isObject(%profile))
		{
			%obj = %client.player;
			
			if(isObject(%obj))
			{
				if(%profile.hunger < 50)
				{
					
					%name = StripMLControlChars(stripTrailingSpaces(%name));
					%name = %name $= "" ? "Delicious Cake" : %name;
					%price = mCeil(%fill * (1 + (%markup / 100)));
					%profit = mCeil(%fill * (%markup / 100));
					
					%client.salesBrick			= %brick;
					%client.salesBrickProduct	= "food";
					%client.salesBrickItem		= %fill;
					%client.salesBrickMarkup	= %profit;
					%client.salesBrickName		= %name;
					%client.salesBrickPrice		= %price;
					
					commandToClient(%client, 'messageBoxYesNo', "Do you want to buy" SPC %name @ "?", "A service brick is offering to sell you \c3" @ %name @ "\c6 for \c3$" @ %price @ "\c6.\n\nDo you wish to accept?", 'Event_Buy');
				}
				else
				{
					messageClient(%client, '', '\c6A service is offering to sell you \c3%1\c6, but you\'re too full.', %name);
				}
			}
		}
	}
}

function fxDTSBrick::sellItem(%brick, %id, %markup, %client)
{
	if(isObject(%client))
	{
		%profile = %client.profile;
		
		if(isObject(%profile))
		{
			%obj = %client.player;
			
			if(isObject(%obj))
			{
				%item = SalesSO.getIDItem(%id);
				
				if(isObject(%item))
				{
					%name = %item.uiName;
					%profit = SalesSO.getItemPrice(%id) * (%markup / 100);
					%price = SalesSO.getItemPrice(%id) * (1 + (%markup / 100));
					
					if(%obj.getToolCount() < %obj.dataBlock.maxTools)
					{
						%client.salesBrick			= %brick;
						%client.salesBrickProduct	= "item";
						%client.salesBrickItem		= %item;
						%client.salesBrickMarkup	= %profit;
						%client.salesBrickName		= %name;
						%client.salesBrickPrice		= %price;
						
						commandToClient(%client, 'messageBoxYesNo', "Do you want to buy" SPC %name @ "?", "A service brick is offering to sell you a(n) \c3" @ %name @ "\c6 for \c3$" @ %price @ "\c6.\n\nDo you wish to accept?", 'Event_Buy');
					}
					else
					{
						messageClient(%client, '', '\c6A service is offering to sell you \c3%1\c6, but you\'re carrying too much already.', %name);
					}
				}
			}
		}
	}
}

function fxDTSBrick::sellPlastic(%brick, %price, %client)
{
	if(isObject(%client))
	{
		if(!%client.isLandOwner())
			return %client.sendMessageBox("No Land", "This vendor is trying to sell you plastic, but seeing as you do not own any land there's not really much of a point to that.");
		
		%profile = %client.profile;
		
		if(isObject(%profile))
		{
			%brickgroup = %brick.getGroup();
			%owner = %brickgroup.client;
			
			if(isObject(%brickGroup))
			{
				// Load a brick group's profile.
				%brickProfile = %brick.getProfile();
				
				if(isObject(%brickProfile))
				{
					if(%brickProfile.getID() != %profile.getID())
					{
						%plastic = %brickProfile.refplastic;
						
						if(%plastic >= 100)
						{
							%plastic = 100;
							%total = mCeil(%plastic * %price);
							
							if(%total <= %profile.money)
							{
								%client.salesBrick			= %brick;
								%client.salesBrickProduct	= "refplastic";
								%client.salesBrickItem		= %plastic;
								%client.salesBrickMarkup	= 0;
								%client.salesBrickName		= "Refined Plastic";
								%client.salesBrickPrice		= %total;
								
								commandToClient(%client, 'messageBoxYesNo', "Do you want to buy" SPC %plastic SPC " Refined Plastic?",
									"<font:courier new:20>SALES RECEIPT<just:left>\n" NL
									"<font:courier new:16>REF. PLASTIC	: " @ %plastic NL
									"PRICE GIVEN	 : $" @ %total NL
									"MERCHANT STOCK : " @ mFloor(%brickProfile.refPlastic) NL
									"YOUR STOCK	  : " @ mFloor(%profile.refPlastic) NL
									"\n\n<just:center><font:courier new:20>ACCEPT TRANSACTION?",
									'Event_Buy');
							}
							else
								commandToClient(%client, 'messageBoxYesNo', "Does Not Compute",
								"Vendors sell Refined Plastic in orders of 100.\n" NL
								"This particular vendor is selling 100 Plastic Crates for $" @ %total @ " each.\n" NL
								"Please have this money on hand if you want to complete this order.");
						}
						else
							messageClient(%client, '', "\c6Sorry, but this vendor has run out of plastic.");
					}
					else
						messageClient(%client, '', "\c6Sorry, but you cannot buy plastic from yourself.");
				}
			}
		}
	}
}

function fxDTSBrick::sellVehicle(%brick, %client)
{
	if(isObject(%client))
	{
		%profile = %client.profile;
		
		if(isObject(%profile))
		{
			%obj = %client.player;
			
			if(isObject(%obj))
			{
				%brickgroup = %brick.getGroup();
				%owner = %brickgroup.client;
				
				if(isObject(%brickGroup))
				{
					// Load a brick group's profile.
					%brickProfile = %brick.getProfile();
					
					// Handle sales.
					if(isObject(%brickProfile))
					{
						%ownerJob = %brickProfile.getJob();
						
						if(isObject(%ownerJob) && %ownerJob.sellVehicle)
						{
							%client.salesBrick			= %brick;
							%client.salesBrickProduct	= "vehicle";
							%client.salesBrickItem		= "";
							%client.salesBrickMarkup	= "";
							%client.salesBrickName		= "";
							%client.salesBrickPrice		= "";
							
							commandToClient(%client, 'DisplayCRPGGUI', "Vehicles");
						}
						else
							commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "Sorry!\n\nThis brick owner is not liscensed to sell vehicles.");
					}
					else
						commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "Sorry!\n\nThe server has had an error and this brick's group does not have a profile tied to it. Please pick another store!");
				}
				else
					commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "Sorry!\n\nThe server has had an error and this brick's does not have an owner. Please pick another store!");
			}
		}
	}
}

// ============================================================
// Section 3 : GameConnection
// ============================================================
// Sets a player in the arrest position and starts the count down.
// @null
function GameConnection::arrest(%client, %police)
{
	%profile = %client.profile;
	
	if(isObject(%profile))
	{
		%obj = %client.player;
		
		if(isObject(%obj))
		{
			%obj.setArrested(true);
			%arrested = true;
		}
		else
		{
			%stars = %profile.getWantedLevel();
			
			if(%stars)
			{
				%profile.setInmate();
				%arrested = true;
			}
			
			%client.spawnPlayer();
			
			if(%stars)
				commandToClient(%client, 'BottomPrint', "<just:center>\c6ARRESTED!", 2);
			else
				messageClient(%client, '', "\c6The Police had to release you, due to lack of evidence.");
		}
		
		if(%arrested)
		{
			%profile.bank += %profile.money;
			%profile.money = 0;
			
			// Returns stolen money.
			%fieldCount = getFieldCount(%profile.stolenMoney);
			if(%fieldCount > 0)
			{
				for(%index = 0; %index < %fieldCount; %index++)
				{
					%victimBL_ID = strReplace(getField(%profile.stolenMoney, %index), "|", "");
					%amountStolen = getField(%profile.stolenMoney, %index + 1);
					
					%victimProfile = Profiler.getProfile(%victimBL_ID);	
					%victim = findClientByBL_ID(%victimBL_ID);
					
					if(%profile.bank >= %amountStolen)
					{
						%victimProfile.bank += %amountStolen;
						%profile.removeFunds(%amountStolen, false);
						if(isObject(%victim))
							messageClient(%victim, '', "\c6 - Your stolen fund of \c3$" @ %amountStolen @ "\c6 was returned to your bank.");
					}
					else if(%profile.bank > 0)
					{
						%profile.bank = %remainder;
						%profile.removeFunds(%remainder, false);
						if(isObject(%victim))
							messageClient(%victim, '', "\c6 - Your partial stolen fund of \c3$" @ %amountStolen @ "\c6 was returned to your bank.");
					}
					%index++;
				}
				%profile.stolenMoney = "";
			}
		}
	}
}

// Applies a crime to %client.
// @null
function GameConnection::commitCrime(%client, %message, %demerits, %blatant, %witness, %victim, %visible, %cluelife, %cluetrans)
{
	// We need a real victim.
	%profile = %client.profile;
	
	if(!isObject(%profile))
		return -1;
	
	// There must be a reason for the crime.
	if(%message $= "")
		return -1;
	
	// Without demerits, a crime is not a crime.
	%demerits = mClamp(%demerits, 0, %demerits);
	
	if(%demerits <= 0)
		return -1;
	
	// Begin Clue Generation!
	
	if(!%blatant)
	{
		%obj = %client.player;
		
		if(isObject(%obj))
		{
			%detected = false;
			
			if(%witness)
			{
				initContainerRadiusSearch(%obj.getPosition(), 64, $TypeMasks::PlayerObjectType | $TypeMasks::FxBrickObjectType);
				
				while(isObject(%col = containerSearchNext()))
				{
					// If %col is a player.
					if(%col.getType() & $TypeMasks::PlayerObjectType)
					{
						if(!isObject(%col.client.profile) || %col.client.getID() == %client.getID())
							continue;
						
						if(%col.crpgArrested || %col.crpgStunned)
							continue;
						
						if(isObject(%victim))
						{
							if(%col == %victim && !%victim.client.profile.getJob().police)
								continue;
						}
						
						%police = %col.client;
						%job = %col.client.profile.getJob();
						
						// if(!%job.police)
							// continue;

						%fovCheck = doFOVCheck(%col, %obj, 0, 1);
						
						%start	= %col.getEyePoint();
						%masks	= ($TypeMasks::FxBrickObjectType | $TypeMasks::StaticObjectType | $TypeMasks::TerrainObjectType | $TypeMasks::VehicleObjectType);
						
						%end		= %obj.getEyePoint();
						%hit		= getWord(containerRayCast(%start, %end, %masks), 0);
						
						if(!isObject(%hit) && %fovCheck)
						{
							%detected = true;
							break;
						}
						
						%end		= %obj.getWorldBoxCenter();
						%hit		= getWord(containerRayCast(%start, %end, %masks), 0);
						
						if(!isObject(%hit) && %fovCheck)
						{
							%detected = true;
							break;
						}
					}
					// camera
					else if(%col.getDatablock().crpgCamera)
					{
						%start	= %col.getWorldBoxCenter();
						%masks	= ($TypeMasks::FxBrickObjectType | $TypeMasks::StaticObjectType | $TypeMasks::TerrainObjectType | $TypeMasks::VehicleObjectType);
						
						%end		= %obj.getEyePoint();
						%hit		= getWord(containerRayCast(%start, %end, %masks), 0);	
						
						// FOV is the same no matter what side of the brick youre on...
						// need to calculate a vector depending on the angle and base the fov on that vector. (brick forward vector basically)
						// talk(doFOVCheck(%col, %obj, 1, 1));
						// talk(%col.angleID);
						
						%angleID = %col.getAngleID;
						switch(%angleID)
							{
								case 0:
									
							}
						if(!isObject(%hit))
						{
							%detected = true;
							break;
						}
					}
					else
						continue;
				}
				
				if(%detected)
				{
					%profile.setDemerits(%profile.demerits + %demerits);
					commandToClient(%client, 'CenterPrint', "\c6You have committed a crime!\n\c6[\c3" @ %message @ "\c6]", 3);
					
					if(isObject(%police))
						commandToClient(%police, 'CenterPrint', "\c6You have whitnessed a crime!\n\c6[\c3" @ %message @ "\c6]", 3);
					
					SystemSO.crimes++;
					SystemSO.solved++;
				}
				else if(mFloor(%cluelife) > 0)
				{
					%clue = new Item()
					{
						datablock	= (%visible ? CRPG_Clue_Item : CRPG_Clue_Hidden_Item);
						
						isClue		= true;
						clueHidden	= !%visible;
						cluePerp		= %client;
						clueBLID		= %client.bl_id;
						clueMessage	= %message;
						clueDems		= %demerits;
					};
					
					if(%cluetrans !$= "")
						%clue.setTransform(%cluetrans);
					else
						%clue.setTransform(setWord(%obj.getTransform(), 2, getWord(%obj.getTransform(), 2) + 2));
					
					%clue.canPickup = false;
					%clue.schedule(%cluelife, "schedulePop");
					MissionCleanup.add(%clue);
					CluesSS.add(%clue);
					
					SystemSO.crimes++;
					SystemSO.unsolved++;
				}
				else
				{
					SystemSO.crimes++;
					SystemSO.unsolved++;
				}
			}
			else if(mFloor(%cluelife) > 0)
			{
				%clue = new Item()
				{
					datablock	= (%visible ? CRPG_Clue_Item : CRPG_Clue_Hidden_Item);
					
					isClue		= true;
					clueHidden	= !%visible;
					cluePerp		= %client;
					clueBLID		= %client.bl_id;
					clueMessage	= %message;
					clueDems		= %demerits;
				};
				
				if(%cluetrans !$= "")
					%clue.setTransform(%cluetrans);
				else
					%clue.setTransform(setWord(%obj.getTransform(), 2, getWord(%obj.getTransform(), 2) + 2));
				
				%clue.canPickup = false;
				%clue.schedule(%cluelife, "schedulePop");
				MissionCleanup.add(%clue);
				CluesSS.add(%clue);
				
				SystemSO.crimes++;
				SystemSO.unsolved++;
			}
			else
			{
				SystemSO.crimes++;
				SystemSO.unsolved++;
			}
		}
		else if(!%witness)
		{
			%profile.setDemerits(%profile.demerits + %demerits);
			commandToClient(%client, 'CenterPrint', "\c6You have committed a crime!\n\c6[\c3" @ %message @ "\c6]", 3);
			
			SystemSO.crimes++;
			SystemSO.solved++;
		}
		else
		{
			SystemSO.crimes++;
			SystemSO.unsolved++;
		}
	}
	else
	{
		%profile.setDemerits(%profile.demerits + %demerits);
		commandToClient(%client, 'CenterPrint', "\c6You have committed a crime!\n\c6[\c3" @ %message @ "\c6]", 3);
		
		SystemSO.crimes++;
		SystemSO.solved++;
	}
	
	return (isObject(%clue) ? %clue.getID() : -1);
}

// Removes a player from jail, grants them their items, and prepares their clothing to change.
// @null
function GameConnection::escapePrison(%client)
{
	%profile = %client.profile;
	
	if(isObject(%profile) && %profile.isInmate())
	{
		if($CRPG::JailBreaks)
		{
			JobsSO.messageBranch("Law", "\c5Police HQ\c6: Attention all units, \c3" @ %client.name @ "\c6 has escaped from the Prison Facility.");
			
			if(!%profile.isInmateLife())
				%profile.setDemerits(1500);
			else
				%profile.setDemeritsMax();
			
			%profile.sentence	= 0;
			%profile.felon		= true;
			
			%client.schedule(60000, "applyOutfitParts");
			%client.schedule(60000, "applyOutfitColors");
			%client.sendStatus();
			
			if(isObject(%client.player))
			{
				%client.player.clearTools();
				%client.player.giveDefaultEquipment();
			}
		}
		else if(isObject(%obj = %client.player))
		{
			%obj.teleportHome(%client, true);
		}
	}
}

// Gets the name of a bitmap to display in lieu of the player's clan prefix.
// @string
function GameConnection::getAward(%client)
{
	%prefix = "<bitmap:[%CRPGTAG:AWARD_PATH]/";
	%suffix = ">";
	
	if(%client.profile.awardBeta)
		return %prefix @ "beta" @ %suffix;
}

// Fetches a player's profile (creating it if nessicary).
// @ProfileSO
function GameConnection::getProfile(%client)
{
	return Profiler.fetchProfile(%client.bl_id);
}

// Gives the client a mission.
// @Mission
function GameConnection::giveMission(%client, %id)
{
	if(!isObject(%client.profile))
		return;
	
	MissionsSO.giveAssignment(%id, %client);
}

// Tests if the Client can run CityRPG Admin Operations.
// @bool
function GameConnection::isCRPGAdmin(%client)
{
	return (%client.isSuperAdmin ? !%client.disableAdminHacks : false);
}

// Determines if the Client owns a lot.
// @bool
function GameConnection::isLandOwner(%client)
{
	for(%a = 0; %a < TriggerSS.getCount(); %a++)
	{
		%brick = TriggerSS.getObject(%a);
		
		if(%brick.getGroup().bl_id == %client.bl_id)
		{
			return true;
		}
	}
	
	return false;
}

// Loads the %client's profile at the end of mission loading.
// Also prepares some client varaibles for the live game.
// @null
function GameConnection::loadProfile(%client)
{
	echo("\c4Loading" SPC %client.name SPC "(" @ %client.bl_id @ ")");
	
	if(!Profiler.isProfileActive(%client.bl_id))
	{
		%profile = %client.getProfile();
		
		if(isObject(%profile))
		{
			%client.profile = %profile;
			%message = '\c2Your profile was loaded successfully.';
		}
		else
		{
			switch(%profile)
			{
				case -1:
					%message = '\c0Your profile failed to load. Please inform the Admin.';
					
				case -2:
					%message = '\c0Your profile failed to be created. Please inform the Admin.';
					
				default:
					%message = '\c0Your profile did not register for unknown reasons. Please inform the Admin.';
			}
		}
	}
	else
	{
		%profile = %client.getProfile();
		%client.profile = %profile;
		
		for(%a = 0; %a < ClientGroup.getCount(); %a++)
		{
			%subClient = ClientGroup.getObject(%a);
			
			if(%subClient.bl_id == %client.bl_id && %subClient.getID() != %client.getID())
			{
				%message = '\c3Your profile is already in use. Actions will affect all instances of %1 in this server.';
				break;
			}
		}
		
		if(%message $= "")
			%message = '\c2Your profile is ready.';
	}
	
	if(isObject(%profile)) %client.clanSuffix = %profile.getTitle();
	else %client.clanSuffix = "\c3 the Unknown";
	
	schedule(0, 0, "messageClient", %client, '', %message, %client.bl_id);
	%client.schedule(0, "sendWelcome");
}

// Logs stolen money and its owner.
function GameConnection::logStolenMoney(%client, %blid, %amount)
{
	%profile = %client.profile;
	
	%ownerBLID = "|" @ %blid @ "|";
	%fieldCount = getFieldCount(%profile.stolenMoney);
	if(%fieldCount == 0)
		%profile.stolenMoney = %ownerBLID TAB %amount;
	else
	{
		for(%index = 0; %index < %fieldCount; %index++)
		{
			%field = getField(%profile.stolenMoney, %index);
			if(%field $= %ownerBLID)
			{
				%found = true;
				break;
			}
		}
		if(%found)
		{
			%oldAmount = getField(%profile.stolenMoney, %index + 1);
			%newAmount = %oldAmount + %amount;
			
			%profile.stolenMoney = setField(%profile.stolenMoney, %index + 1, %newAmount);
		}
		else
			%profile.stolenMoney = %profile.stolenMoney TAB %ownerBLID TAB %amount;
	}
}

// Changes the Client's camera to zombie goast mode.
// @bool (success)
function GameConnection::setGhostCamera(%client, %transform)
{
	%camera = %client.camera;
	
	if(isObject(%camera))
	{
		%camera.setMode("Observer");
		
		if(%transform $= "")
			if(isObject(%client.player))
				%transform = %client.player.getTransform();
			else
			{
				%transform = pickSpawnPoint();
				%transform = vectorAdd(getWords(%transform, 0, 2), "0 0 2.5") SPC getWords(%transform, 3, 6);
			}
		
		%camera.setTransform(%transform);
		%client.setControlObject(%camera);
		
		%image = %camera.getMountedImage(0);
		
		if(isObject(%image))
			%camera.unMountImage(0);
		
		%camera.mountImage(CRPG_Death_Image, 0);
		%camera.isGhostCam = true;
		
		return true;
	}
	
	return false;
}

// Client Updates

// Political candidates.
// @null
function GameConnection::sendCandidates(%client)
{
	%candList = SystemSO.getCandidates();
	%candNum = getFieldCount(%candList);
	
	if(%candNum > 0)
	{
		%cand[1] = -1;
		%cand[2] = -1;
		%cand[3] = -1;
		%cand[4] = -1;
		%cand[5] = -1;
		
		switch(%candNum)
		{
			case 1:
				%cand[3] = getField(%candList, 0);
			
			case 2:
				%cand[2] = getField(%candList, 1);
				%cand[4] = getField(%candList, 0);
			
			case 3:
				%cand[2] = getField(%candList, 1);
				%cand[3] = getField(%candList, 0);
				%cand[4] = getField(%candList, 2);
			
			case 4:
				%cand[1] = getField(%candList, 3);
				%cand[2] = getField(%candList, 1);
				%cand[4] = getField(%candList, 0);
				%cand[5] = getField(%candList, 2);
			
			default:
				%cand[1] = getField(%candList, 3);
				%cand[2] = getField(%candList, 2);
				%cand[3] = getField(%candList, 0);
				%cand[4] = getField(%candList, 1);
				%cand[5] = getField(%candList, 4);
		}
		
		for(%a = 1; %a <= 5; %a++)
		{
			%bl_id = -1;
			%name = "";
			%incumbent = false;
			
			%parts = 0;
			%nodes = "";
			%colors = "";
			%decals = "";
			
			if(isObject(%cand[%a]))
			{
				%bl_id = %cand[%a].bl_id;
				%name = %cand[%a].name;
				%incumbent = (%cand[%a].bl_id $= SystemSO.getMayor().bl_id ? true : false);
				
				%outfit = %cand[%a].getOutfit().getOutfitString(%cand[%a]);
				%parts = getLine(%outfit, 0);
				%nodes = getLine(%outfit, 2);
				%colors = getLine(%outfit, 3);
				%decals = getLine(%outfit, 1) TAB %cand[%a].faceName;
			}
			
			echo("Sending #" @ %a @ ". - " @ %bl_id SPC "(" @ %cand[%a].faceName @ ")");
			commandToClient(%client, 'Vote_SetCandidate', %a, %bl_id, %name, %incumbent, %parts, %nodes, %colors, %decals);
		}
	}
}

// Campaign updates
// @null
// FIXME: should this be in another place?
function GameConnection::sendCampaign(%client)
{
	%profile = %client.profile;
	
	if(isObject(%profile))
		commandToClient(%client, 'CRPGCampaignChange', %profile.campaign, %profile.getCampaignMoneyRequired());
}

// Lot, Weapon, and Vehicle prices.
// @null
function GameConnection::sendInformation(%client)
{
	%profile = %client.profile;
	
	if(isObject(%profile))
	{
		%data = %profile.getData();
		
		// Lots
		for(%a = 0; %a < LotSS.getCount(); %a++)
		{
			%lot = LotSS.getObject(%a);
			
			if(%lot.crpgLot)
			{
				%name = %lot.uiName;
				%price = mClamp(%lot.price, 0 , %lot.price);
				
				%lnames = addField(%lnames, %name);
				%lprices = addField(%lprices, %price);
			}
		}
		
		commandToClient(%client, 'Stats_SetLots', %lnames, %lprices);
		
		// Vehicles
		commandToClient(%client, 'Vehicles_Clear');
		
		for(%a = 0; %a < SalesSO.vehicles; %a++)
		{
			%vehicle = SalesSO.vehicle[%a];
			
			if(isObject(%vehicle))
			{
				%db = %vehicle.getID();
				%name = trim(stripMLControlChars(trim(%vehicle.uiName)));
				%corp = trim(stripMLControlChars(trim(SalesSO.vcorp[%a])));
				%price = mFloor(SalesSO.vprice[%a]);
				%rating = mClampF(mFloor(SalesSO.vrating[%a]), 0, 5);
				%desc = trim(stripMLControlChars(trim(SalesSO.vdesc[%a])));
				
				if(%name $= "")
					%name = "Vehicle";
				
				if(%corp $= "")
					%corp = "Unknown";
				
				if(%price < 0)
					%price = 0;
				
				commandToClient(%client, 'Vehicle_AddVehicle', %db, %name, %corp, %price, %rating, %desc);
				
				%vnames = addField(%vnames, %name);
				%vprices = addField(%vprices, %price);
			}
		}
		
		commandToClient(%client, 'Stats_SetVehicles', %vnames, %vprices);
		
		// Weapons
		for(%a = 0; %a < SalesSO.items; %a++)
		{
			%item = SalesSO.item[%a];
			
			if(isObject(%item))
			{
				%name = %item.uiName;
				%name = stripTrailingSpaces(%name);
				%price = mClamp(SalesSO.price[%a], 0 , SalesSO.price[%a]);
				
				%inames = addField(%inames, %name);
				%iprices = addField(%iprices, %price);
			}
		}
		
		commandToClient(%client, 'Stats_SetWeapons', %inames, %iprices);
		%client.sendCampaign(); // FIXME where does this go
	}
}

// Job Operations
function GameConnection::sendJobOps(%client)
{
	if(%client.hasJobOps && %client.hasJobOps == %client.profile.job)
		return;
	
	%client.hasJobOps = %client.profile.job;
	%job = %client.profile.getJob();
	
	commandToClient(%client, 'Stats_ClearJobOps');
	
	for(%a = 0; %a < %job.numOpCategories; %a++)
	{
		commandToClient(%client, 'Stats_AddJobOpCat', %job.opCategory[%a]);
		
		for(%b = 0; %b < %job.numCategoryOps[%a]; %b++)
		{
			commandToClient(%client, 'Stats_AddJobOpOp', %a, %job.opName[%a, %b], %job.opRange[%a, %b]);
		}
	}
}

// Sends a client message box so I can return and send a message
// box in one line with no console errors. Also exists for the
// custom message box in a future update.
// @null
function GameConnection::sendMessageBox(%client, %title, %message)
{
	commandToClient(%client, 'MessageBoxOK', %title, %message);
}

// Dun a dun a dun, duna dun.. dun doop doop da do doo...
// @null
function GameConnection::sendMissionComplete(%client)
{
	if($sim::time - %client.crpgLastMissionComplete > 10)
	{
		%message = "<bitmap:" @ $CRPG::ClientPath @ "images/graphics/mission_passed>";
		commandToClient(%client, 'CenterPrint', %message, 7);
		
		%client.play2D(CRPG_Mission_Complete_Sound);
		%client.crpgLastMissionComplete = $sim::time;
	}
}

// Allows script to accumilate a sum before sending them a cash change.
// and then call this function.
// @null
function GameConnection::sendMassUpdate(%client)
{
	if(isObject(%profile = %client.profile))
	{
		if(%client.massCashChange > 0)
		{
			commandToClient(%client, 'CRPGCashChange', %client.massCashChange, %profile.money, %profile.bank, %profile.debt);
			%client.massCashChange = "";
		}
		
		if(%client.massPlasticChange > 0)
		{
			commandToClient(%client, 'CRPGPlasticChange', %client.massPlasticChange);
			%client.massPlasticChange = "";
		}
	}
}

// Updates the client's progress bar.
// @null
function GameConnection::sendProgress(%client, %complete, %action)
{
	%action = (%action $= "" ? "ERROR NAME" : %action);
	%complete = mClamp(%complete, 0, 100);
	%message = "<just:center>\c6" @ %action NL "<color:00ff00>";
	
	if(%complete < 100)
	{
		%worked = mFloor(40 * (%complete / 100));
		
		for(%a = 0; %a < 40; %a++)
			if(%a == %worked) %message = %message @ "<color:ff0000>|";
			else %message = %message @ "|";
	}
	else
	{
		%message = %message @ "\c2Completed!";
	}
	
	commandToClient(%client, 'BottomPrint', %message, 2);
}

// Sends the client information about their datum.
// Will ignore network safety measures if %full = true.
// @null
function GameConnection::sendStatus(%client, %full)
{
	%profile = %client.profile;
	
	if(isObject(%profile))
	{
		%data = %profile.getData(%client);
		
		if(%client.crpgStatusString !$= "" && !%full)
		{
			%newNames = getLine(%data, 0);
			%newValues = getLine(%data, 1);
			%oldValues = getLine(%client.crpgStatusString, 1);
			
			for(%a = 0; %a < getFieldCount(%newNames); %a++)
			{
				%newName = getField(%newNames, %a); 
				%newValue = getField(%newValues, %a);
				%oldValue = getField(%oldValues, %a);
				
				if(%newValue !$= %oldValue)
				{
					%fields++;
					if(%finalNames !$= "") %finalNames = %finalNames TAB %newName;
					else %finalNames = %newName;
					
					if(%finalValues !$= "") %finalValues = %finalValues TAB %newValue;
					else %finalValues = %newValue;
				}
			}
			
			if(%fields)
			{
				%names = 0;
				while(strlen(%finalNames) > 0 && %names <= 5)
				{
					%names++;
					%names[%names] = getSubStr(%finalNames, 0, 255);
					
					if(strLen(%finalNames) >= 255)
						%finalNames = getSubStr(%finalNames, 255, strLen(%finalNames));
					else
						%finalNames = "";
				}
				
				%values = 0;
				while(strlen(%finalValues) > 0 && %values <= 5)
				{
					%values++;
					%values[%values] = getSubStr(%finalValues, 0, 255);
					
					if(strLen(%finalValues) >= 255)
						%finalValues = getSubStr(%finalValues, 255, strLen(%finalValues));
					else
						%finalValues = "";
				}
				
				%client.crpgStatusString = %data;
				commandToClient(%client, 'SetCRPGStatus', %names1, %names2, %names3, %names4, %names5, %values1, %values2, %values3, %values4, %values5);
			}
		}
		else
		{
			%finalNames = getLine(%data, 0);
			%finalValues = getLine(%data, 1);
			
			%names = 0;
			while(strlen(%finalNames) > 0 && %names <= 5)
			{
				%names++;
				%names[%names] = getSubStr(%finalNames, 0, 255);
				
				if(strLen(%finalNames) >= 255)
					%finalNames = getSubStr(%finalNames, 255, strLen(%finalNames));
				else
					%finalNames = "";
			}
			
			%values = 0;
			while(strlen(%finalValues) > 0 && %values <= 5)
			{
				%values++;
				%values[%values] = getSubStr(%finalValues, 0, 255);
				
				if(strLen(%finalValues) >= 255)
					%finalValues = getSubStr(%finalValues, 255, strLen(%finalValues));
				else
					%finalValues = "";
			}
			
			%client.crpgStatusString = %data;
			commandToClient(%client, 'SetCRPGStatus', %names1, %names2, %names3, %names4, %names5, %values1, %values2, %values3, %values4, %values5);
		}
	}
}

// Opens up the GUI, sends welcome message with date string, and problems.
// @null
function GameConnection::sendWelcome(%client)
{
	commandToClient(%client, 'DisplayCRPGComponents');
	messageClient(%client, '', "\c6Welcome to CityRPG, \c3" @ %client.name @ "\c6. Today is " @ CalendarSO.getTimeString(true));
	
	if(isObject(%profile = %client.profile))
	{
		// Checks to make sure your job is real.
		if(!isObject(%profile.getJob()))
		{
			%profile.job = Profiler.getDefaultValue("job");
			%profile.problem = addField(%profile.problem, "\c6Somehow, your job has become invalid. This problem has been fixed by setting you to the default job.");
		}
		
		// Reads off problems.
		if(%count = getFieldCount(%profile.problem))
		{
			getFieldCount(%profile.problem);
			
			messageClient(%client, '', "\c6============================================================");
			messageClient(%client, '', '\c6You have %1 offline message%2.', %count, (mAbs(%count) != 1 ? "s" : ""));
			
			for(%a = 0; %a < %count; %a++)
				if(getField(%profile.problem, %a) !$= "")
					messageClient(%client, '', '\c6 - %1', getField(%profile.problem, %a));
			
			%profile.problem = "";
			messageClient(%client, '', "\c6============================================================");
		}
	}
}

// Sends the client information about their weapon.
// Also clears it if they lack an ammo weapon.
// @bool (visibility)
function GameConnection::updateAmmo(%client)
{
	%obj = %client.player;
	
	if(isObject(%obj))
	{
		%image = %obj.getMountedImage(0);
		
		if(isObject(%image))
		{
			%ammoType = stripTrailingSpaces(%image.ammo);
			
			if(%ammoType !$= "")
			{
				if(%image.ammoUI !$= "") %ammoName = %image.ammoUI;
				else %ammoName = %ammoType;
				
				%ammoClip = mFloor(%obj.ammo[%ammoType]);
				%ammoMax = %image.ammoMagSize;
				%ammoCarrying = (%client.profile.ammo[%image.ammoMagName] $= "" ? "Inf" : %client.profile.ammo[%image.ammoMagName]);
				
				commandToClient(%client, 'CRPGAmmo', true, true, %ammoName, %ammoClip, %ammoMax, %ammoCarrying);
				return true;
			}
		}
	}
	
	commandToClient(%client, 'CRPGAmmo', false);
	return false;
}


// Events
function GameConnection::AddEXP(%client, %amount, %branch)
{
	%profile = %client.profile;
	
	if(!isObject(%profile))
		return;
	
	%profile.addEXP(%amount, %branch);
	%client.sendStatus();
}

function GameConnection::ShowGUI(%client, %gui)
{
	if($sim::time - %client.lastGUI > 3)
	{
		switch(%gui)
		{
			case 0: %gui = "Bank";
			case 1: %gui = "Police";
			case 2: %gui = "Stats";
			case 3: %gui = "Vote";
			default: return;
		}
		
		commandToClient(%client, 'DisplayCRPGGUI', %gui);
		%client.lastGUI = $sim::time;
	}
}

// ============================================================
// Section 4 : MinigameSO
// ============================================================
// Algorithm to determine a spawning location for the %client.
// Precedence in this order:
// +- (if inmate) Jail Brick
// +- (if killed last life) Medical Brick
// +- Personal Spawn Brick
// +- Job Spawn Brick
// +- Minigame Spawn
// +- Loads default algorithm.
// @vector7f
function CRPGMinigame::pickSpawnPoint(%so, %client)
{
	%profile = %client.profile;
	
	if(isObject(%profile))
	{
		%spawn = -1;
		
		if(%profile.isInmate())
			%spawn = SpawnsSO.getSpawnPointForBrick(CRPG_Jail_Spawn_Brick);
		
		if(%spawn == -1)
		{
			if(%client.bloodyDeath)
				%spawn = SpawnsSO.getSpawnPointForBrick(CRPG_Medical_Spawn_Brick);
			
			if(%spawn == -1)
			{
				// Sometimes the game calls MinigameSO::pickSpawnPoint with no %client.
				if(isObject(%client))
					%spawn = SpawnsSO.getSpawnPointForBrick(CRPG_Personal_Spawn_Brick, %client);
				
				if(%spawn == -1)
				{
					%jobSpawn = %profile.getJob().getSpawn();
					
					if(isObject(%jobSpawn))
						%spawn = SpawnsSO.getSpawnPointForBrick(%jobSpawn);
					
					if(%spawn == -1)
					{
						%spawn = SpawnsSO.getSpawnPointForBrick(brickSpawnPointData);
						
						if(%spawn == -1)
							return parent::pickSpawnPoint(%so, %client);
					}
				}
			}
		}
		
		return %spawn;
	}
	else
		return parent::pickSpawnPoint(%so, %client);
}

// ============================================================
// Section 5 : Player
// ============================================================
// Beam check by Hata
function Player::BeamCheck(%player,%start,%len,%masks,%vec,%name)
{
	%beam = vectorScale(%vec,%len); //ray final vector
	%end = vectorAdd(%start,%beam); //ray endpoint
	
	%ray = containerRayCast(%start,%end,%masks,%player);

	%col = firstWord(%ray); //col object
	%found = isObject(%col) && %col.type !$= "ClimbHelper"; //found object, and object not a climb tool
	if(%found) //if %found is true
		%hit = posFromRaycast(%ray); //use hit position
	else //otherwise
		%hit = %end; //use raycast endpoint

	if(%name $= "Aim Check") //if called by Aim Check fcn
		%found = %hit; //use hit position instead

	return %found; //return %found (true/false bool)
}

function Player::dropMoney(%player, %amount, %stolenOwner)
{	
	// Drop Cash
	%client = %player.client; //define client
	%profile = %client.profile;
	
	if(%client.profile.money > 0)
	{
		%dropMoney = mFloor(%amount); //round drop money down
		
		if(%dropMoney > %client.profile.money) //if drop money is greater than client's current money
				%dropMoney = %client.profile.money; //drop all the money
		%client.profile.money -= %dropMoney; //subtract drop money from client's total money
		
		if(!%stolenOwner)
			%stolenOwner = %client;
		
		//Spawn money item
		%cash = new Item()
		{
				datablock = CRPG_Cash_Item;
				cashValue = %dropMoney;
				owner = %client.getBLID();
				//stolen = %stolen;
				stolenOwner = %stolenOwner.getBLID();
				timeDropped = getSimTime();
				canPickup = false;
		};MissionCleanup.add(%cash);

		%cash.setTransform(setWord(%player.getTransform(), 2, getWord(%player.getTransform(), 2) + 2));
		
		%velocity = getRandom(5, 20);
		%cash.setVelocity(VectorScale(%player.getEyeVector(), %velocity));
		%cash.setShapeName("$" @ %cash.cashValue);
		
		commandToClient(%client, 'CRPGCashChange', -%dropMoney, %profile.money, %profile.bank, %profile.debt);
		%client.sendStatus(); //update GUI
	}
}
function Player::getAimVector(%obj)
{
	// Space Guy's code (iban no good at math)
	%fvec = %obj.getForwardVector();
	%fX = getWord(%fvec, 0);
	%fY = getWord(%fvec, 1);
	
	%evec = %obj.getEyeVector();
	%eX = getWord(%evec, 0);
	%eY = getWord(%evec, 1);
	%eZ = getWord(%evec, 2);
	
	%eXY = mSqrt((%eX * %eX) + (%eY * %eY));
	%aimVec = (%fX * %eXY) SPC (%fY * %eXY) SPC %eZ;
	
	return %aimVec;
}

function Player::getProfile(%obj)
{
	if(isObject(%client = %obj.client))
	{
		if(isObject(%profile = %client.getProfile()))
		{
			return %profle;
		}
	}
	
	return -1;
}

function Player::getToolCount(%obj)
{
	%count = 0;
	%maxTools = %obj.getDatablock().maxTools;
	
	for(%a = 0; %a < %maxTools; %a++)
		if(isObject(%obj.tool[%a]))
			%count++;
	
	return %count;
}

function Player::giveDefaultEquipment(%obj)
{
	%obj.unMountImage(0);
	%client = %obj.client;
	
	if(isObject(%client) && isObject(%profile = %client.profile))
	{
		%toolMax	= %obj.dataBlock.maxTools;
		%toolDef	= $CRPG::DefaultTools;
		%toolOwn	= %profile.tools;
		%toolJob	= %profile.getJob().getTools();
		%toolInm	= $CRPG::PrisonTools;
		
		// Regular Tools
		if(!%profile.isInmate())
		{
			// Dispense default tools.
			%givenDefault = 0;
			
			for(%a = 0; %a < %toolMax; %a++)
			{
				if(!isObject(%obj.tool[%a]))
				{
					%tool = getField(%toolDef, %givenDefault);
					
					if(isObject(%tool) && !%givenTool[%tool.getID()])
					{
						%obj.tool[%a] = %tool.getID();
						
						%givenDefault++;
						%givenTool[%tool.getID()] = true;
					}
					else
						%obj.tool[%a] = 0;
				}
			}
			
			// Dispense our job's tools.
			%givenJob = 0;
			
			for(%a = 0; %a < %toolMax; %a++)
			{
				if(!isObject(%obj.tool[%a]))
				{
					%tool = getField(%toolJob, %givenJob);
					
					if(isObject(%tool) && !%givenTool[%tool.getID()])
					{
						%obj.tool[%a] = %tool.getID();
						
						%givenJob++;
						%givenTool[%tool.getID()] = true;
					}
					else
						%obj.tool[%a] = 0;
				}
			}
			
			// Dispense our own tools
			%givenSelf = 0;
			
			for(%a = 0; %a < %toolMax; %a++)
			{
				if(!isObject(%obj.tool[%a]))
				{
					%tool = getField(%toolOwn, %givenSelf);
					
					if(isObject(%tool) && !%givenTool[%tool.getID()])
					{
						%obj.tool[%a] = %tool.getID();
						
						%givenSelf++;
						%givenTool[%tool.getID()] = true;
					}
					else
						%obj.tool[%a] = 0;
				}
			}
		}
		
		// Jail Tools
		else
		{
			for(%a = 0; %a < %toolMax; %a++)
			{
				%tool = getField(%toolInm, %a);
				
				if(isObject(%tool))
				{
					%obj.tool[%a] = %tool.getID();
					%givenTool[%tool.getID()] = true;
				}
				else
					%obj.tool[%a] = 0;
			}
		}
		
		for(%a = 0; %a < %toolMax; %a++)
		{
			%tool = %obj.tool[%a];
			
			if(isObject(%tool))
			{
				%tool.onPickup("", %obj);
				messageClient(%client, 'MsgItemPickup', '', %a, %tool.getID());
			}
			else
				messageClient(%client, 'MsgItemPickup', '', %a, 0);
		}
	}
}

function Player::isSunlit(%obj)
{
	if(EnvironmentSO.night)
		return false;
	
	%start = %obj.getEyePoint();
	%masks = ($TypeMasks::FxBrickAlwaysObjectType | $TypeMasks::StaticObjectType);
	
	%alpha = 0;
	%length = 1000;
	%runs = 0;
	
	while(%length > 0)
	{
		%stint = mClamp(%length, 1, 100);
		%length -= %stint;
		%end = vectorAdd(%start, "0 0" SPC %stint);
		
		%ray = containerRayCast(%start, %end, %masks);
		%hit = getWord(%ray, 0);
		%pos = posFromRaycast(%ray);
		
		if(isObject(%hit))
		{
			if(!(%hit.getType() & $TypeMasks::FxBrickAlwaysObjectType))
				return false;
			
			if(%hit.isRendering())
			{
				%alpha += getWord(getColorIDTable(%hit.getColorID()), 3);
				
				if(%alpha >= 1)
				{
					return false;
				}
			}
			
			%length += 100 - (getWord(%pos, 2) - getWord(%start, 2));
		}
		
		%start = %end;
		%runs++;
		
		// Failsafe.
		if(%length <= 0 || %runs > 100)
			break;
	}
	
	return true;
}

function Player::onAutoHeal(%obj, %amt)
{
	if(%obj.getState() !$= "Dead")
	{
		%damage = %obj.getDamageLevel();
		
		if(%damage > 0)
		{
			%factor = 0;
			%factor += mClampF(2 * (%obj.getEnergyLevel() / %obj.getDatablock().maxEnergy), 0.00, 1.50);
			
			if(isObject(%profile = %obj.client.profile))
			{
				%factor += mClampF(4 * (%profile.hunger / 100), 0.50, 4.00);
				
				if(EnvironmentSO.day)
				{
					if(!%obj.isSunLit())
					{
						%factor += mClampF(2 * %profile.getBeliefDay(), -2.00, 2.00);
					}
					else
					{
						%factor += %profile.getBeliefDay();
					}
				}
				else if(EnvironmentSO.night)
				{
					%factor += %profile.getBeliefNight();
				}
			}
			
			%damage -= %amt * %factor;
			%damage = mClamp(%damage, 0, %obj.getDatablock().maxDamage);
			
			%obj.setDamageLevel(%damage);
		
			if(!isEventPending(%obj.healSched))
				%obj.healSched = %obj.schedule(250, "onAutoHeal", %amt);
		}
	}
}

function Player::onArrestCycle(%obj)
{
	%obj.crpgArrestAmount--;
	
	%client	= %obj.client;
	%profile	= %client.profile;
	
	if(%obj.crpgArrestAmount > 0)
	{
		%worked = 100 - (100 * (%obj.crpgArrestAmount / 30));
		%client.sendProgress(%worked, "ARREST TIMER");
		
		if(!isEventPending(%obj.crpgArrestSchedule))
			%obj.crpgArrestSchedule = %obj.schedule(1000, "onArrestCycle");
		
		return false;
	}
	else
	{
		%stars = %profile.getWantedLevel();
		
		if(%stars)
		{
			if(isObject(%obj.crpgArrestClient))
			{
				%pigProfile = %obj.crpgArrestClient.profile;
				
				if(isObject(%pigProfile))
				{
					%pigProfile.money += %profile.demerits;
					%pigProfile.addEXP(%profile.getWantedLevel(), "Law");
					messageClient(%obj.crpgArrestClient, '', "\c6You have recieved \c3$" @ %profile.demerits @ "\c6 for the arrest.");
					commandToClient(%obj.crpgArrestClient, 'CRPGCashChange', %profile.demerits, %pigProfile.money, %pigProfile.bank, %pigProfile.debt);
					
					if(%stars >= 6)
						messageAll('', '\c6Elite criminal \c3%1\c6 was \c1legally apprehended\c6 by \c3%3 %2\c6.', %client.name, %obj.crpgArrestClient.name, %pigProfile.getJob().police ? "Officer" : "Vigilante");
				}
			}
			
			%profile.setInmate();
			
			if(isObject(%client.mission))
				%client.mission.onPlayerBusted();
		}
		
		if(%obj.crpgArrestClient.mission)
			%obj.crpgArrestClient.mission.onArrest(%client, %cp.getWantedLevel() ? true : false);
		
		%client.spawnPlayer();
		
		if(%stars)
		{
			commandToClient(%client, 'BottomPrint', "<just:center>\c6ARRESTED!", 2);
		}
		else
			messageClient(%client, '', "\c6The Police had to release you, due to lack of evidence.");
		
		return true;
	}
}

function Player::onClicked(%obj, %client, %profile, %player, %pos)
{
	if(isObject(%client) && isObject(%player) && %player.getID() != %obj.getID())
	{
		%oc = %obj.client;
		%op = isObject(%oc) ? %oc.profile : -1;
		
		// Evil Actions
		if(%client.crpgEvil)
		{
			// Handcuff Busting
			if(%obj.crpgArrested)
			{
				%obj.crpgArrestAmount	= mClamp(%obj.crpgArrestAmount, 5, %obj.crpgArrestAmount);
				%obj.crpgArrestHealth	-= getRandom(1 + mFloor(%profile.education / 2), 2 + %profile.education);
				
				if(%obj.crpgArrestHealth <= 0)
				{
					if(isEventPending(%obj.crpgArrestSchedule))
						cancel(%obj.crpgArrestSchedule);
					
					if(isObject(%op) && %op.getWantedLevel())
						%client.commitCrime("Harboring a Criminal", 50, false, true, %player, false, 30000);
					
					%obj.setArrested(false);
					
					commandToClient(%client, 'BottomPrint', "<just:center>\c6HANDCUFFS BROKEN!" NL %message, 2);
					
					if(isObject(%obj.client))
					{
						commandToClient(%obj.client, 'BottomPrint', "<just:center>\c6HANDCUFFS BROKEN BY\c3" SPC %client.name @ "\c6!" NL %message, 2);
					}
					
					if(isObject(%obj.crpgFalseArrestClue))
						%obj.crpgFalseArrestClue.delete();
				}
				else
				{
					%client.sendProgress(%obj.crpgArrestHealth, "HANDCUFF HEALTH");
					%obj.client.sendProgress(%obj.crpgArrestHealth, "HANDCUFF HEALTH");
				}
			}
			
			// Pickpocketing
			else if(%x = getWord(%obj.getWorldBoxCenter(), 2) - getWord(%pos, 2) > 4.25)
			{
				if(isObject(%op))
				{
					if($sim::time - %oc.lastPickpocketFrom[%player.getID()] <= 60)
					{
						commandToClient(%client, 'CenterPrint', "<just:right>\c3" @ %oc.name @ "\c6 is wary of you.", 1);
						return;
					}
					
					if($sim::time - %player.lastPickpocket <= 10)
					{
						commandToClient(%client, 'CenterPrint', "<just:right>\c5Don't be hasty,\n\c5or you'll make mistakes.", 1);
						return;
					}
					
					%player.lastPickpocket = $sim::time;
					%oc.lastPickpocketFrom[%player.getID()] = $sim::time;
					
					%prot = %player.rotation;
					if(getWord(%prot, 2) == -1) %prot = 360 - getWord(%prot, 3);
					
					%orot = %obj.rotation;
					if(getWord(%orot, 2) == -1) %orot = 360 - getWord(%orot, 3);
					
					%rotDif = mAbs(%orot - %prot);
					
					if(%rotDif <= 180)
					{
						%bill = %op.getRandomBill();
						
						if(%bill > 0)
						{
							commandToClient(%client, 'CenterPrint', "<just:right>\c6You stole \c2$" @ %bill @ "\c6 from \c3" @ %oc.name @ "\c6.", 1);
							
							%op.money -= %bill;
							%oc.sendStatus();
							
							%profile.money += %bill;
							//%profile.stolen += %bill; -- pointless
							%client.sendStatus();
							
							// Log the stolen money and its owner.
							%client.logStolenMoney(%oc.getBLID(), %bill);
						}
						else
						{
							commandToClient(%client, 'CenterPrint', "<just:right>\c3" @ %oc.name @ "\c6 has no money on them.", 1);
						}
						%client.commitCrime("Pickpocketing", 50, false, true, %player);
						
						// If pickpocket is successful without getting caught, keep pickpocketing.
						%oc.lastPickpocketFrom[%player.getID()] = 61;
					}
					else
					{
						%obj.emote(AlarmProjectile, true);
						
						if(!%op.getJob().police)
						{
							%lines =
								"Why are you digging around in my pockets for, %1?" NL
								"Police! Police! %1 is a thief!" NL
								"Get the hell out of my stuff you scrub!" NL
								"What are you doing with my wallet, %1?" NL
								"Achtung! %1 is a thief!" NL
								"%1 thief pig disgusting" NL
								"%1 is a thief! Help!";
							
							if(%op.gender == 2)
								%lines = %lines NL
									"Police! %1 tried to cop a feel!";
						}
						else
						{
							%lines =
								"You're going to steal from an Officer, %1? Seriously?" NL
								"Big mistake, %1." NL
								"I'm going to beat you into shit, %1." NL
								"Wow, %1. I think you want to be arrested." NL
								"Getting desperate, %1?" NL
								"Are you reaching for my handcuffs, %1? Because I know I am.";
						}
						
						%line = getLine(%lines, getRandom(0, getLineCount(%lines) - 1));
						%line = strReplace(%line, "%1", "\c3" @ %client.name @ "\c6");
						
						messageAll('', "<color:ffffaa><AUTO> \c3" @ %oc.name @ "\c6:" SPC %line);
						
						%client.commitCrime("Pickpocketing", 50, true);
					}
				}
			}
		}
	}
	
	return false;
}

function Player::onStunCycle(%obj)
{
	if(!%obj.crpgStunned)
		return;
	
	if(!%obj.crpgArrested)
	{
		%obj.crpgStunLeft--;
		
		%client	= %obj.client;
		
		if(isObject(%client))
		{
			%profile	= %client.profile;
			
			if(%obj.crpgStunLeft > 0)
			{
				%worked = 100 - (100 * (%obj.crpgStunLeft / %obj.crpgStunAmount));
				%client.sendProgress(%worked, "STUN TIMER");
				
				if(!isEventPending(%obj.crpgStunSchedule))
					%obj.crpgStunSchedule = %obj.schedule(1000, "onStunCycle");
				
				return false;
			}
			else
			{
				commandToClient(%client, 'BottomPrint', "<just:center>\c6NO LONGER STUNNED!", 2);
				%client.setControlObject(%obj);
				%obj.unMountImage($BackSlot);
				
				%obj.crpgStunned = "";
				%obj.crpgStunAmount = "";
				%obj.crpgStunLeft = "";
			}
		}
	}
	else
	{
		%obj.crpgStunned = "";
		%obj.crpgStunAmount = "";
		%obj.crpgStunLeft = "";
	}
}

function Player::onSunBurn(%obj)
{
	if(EnvironmentSO.day)
	{
		if(!isEventPending(%obj.sunBurnSched))
		{
			if(%obj.isSunLit())
			{
				%obj.sunBurnDamage += 1;
				%obj.sunBurnDamage = mClamp(%obj.sunBurnDamage, 1, 5);
				%obj.setWhiteOut(%obj.sunBurnDamage / 10);
				
				%pos = %obj.getPosition();
				%damageType = $DamageType::Direct;
				
				%obj.damage(%obj, %pos, %obj.sunBurnDamage, %damageType);
			}
			else
			{
				%obj.sunBurnDamage = 0;
			}
			
			cancel(%obj.sunBurnSched);
			%obj.sunBurnSched = %obj.schedule(250, "onSunBurn");
		}
	}
	else
	{
		if(isEventPending(%obj.sunBurnSched))
			cancel(%obj.sunBurnSched);
		
		%obj.sunBurnSched = "";
	}
}

function Player::setArrested(%obj, %arrested)
{
	if(%arrested && !%obj.crpgArrested)
	{
		%obj.crpgArrested			= true;
		%obj.crpgArrestAmount	= 30;
		%obj.crpgArrestClient	= %police;
		%obj.crpgArrestHealth	= 100;
		
		%obj.playThread(1, "sit");
		%obj.playThread(2, "armReadyBoth");
		%obj.mountImage(CRPG_Handcuffs_Image, 0);
		%obj.unMountImage($BackSlot);
		
		%arrested = %obj.onArrestCycle();
		
		if(isObject(%obj.client.camera))
		{
			%obj.client.camera.setMode("Corpse", %obj);
			%obj.client.setControlObject(%client.camera);
		}
	}
	if(!%arrested && %obj.crpgArrested)
	{
		%obj.crpgArrested			= "";
		%obj.crpgArrestAmount	= "";
		%obj.crpgArrestHealth	= "";
		%obj.crpgArrestSchedule	= "";
		
		%obj.unMountImage(0);
		%obj.playThread(2, "root");
		%obj.playThread(1, "standjump");
		%obj.addVelocity("0 0 7");
		%obj.spawnExplosion(CRPG_Handcuff_Projectile, 1);
		
		if(isEventPending(%obj.crpgArrestSchedule))
			cancel(%obj.crpgArrestSchedule);
		
		if(isObject(%obj.client))
			%obj.client.setControlObject(%obj);
	}
}

function Player::setBackImage(%obj)
{
	%client = %obj.client;
	%backItem = -1;
	
	if(!isObject(%obj.getObjectMount()))
	{
		if(isObject(%client))
		{
			%maxTier	= 0;
			%maxGun	= -1;
			%usedGun	= %obj.getMountedImage(0);
			
			for(%a = 0; %a < %obj.getDatablock().maxTools; %a++)
			{
				%item = %obj.tool[%a];
				%image = %item.image;
				
				if(isObject(%image) && %image.gunTier > %maxTier)
				{
					if(!isObject(%usedGun) || %usedGun.getID() != %image.getID())
					{
						%maxTier = %image.gunTier;
						%maxGun = %image.backpack.getID();
					}
				}
			}
		}
	}
	
	if(isObject(%maxGun))
	{
		%obj.mountImage(%maxGun, 1);
	}
	else
	{
		%obj.unMountImage(1);
	}
	
	return %backItem;
}

function Player::setCriticalCondition(%obj, %critical)
{
	if(%critical && !%obj.crpgCriticalCondition)
	{
		%obj.crpgCriticalCondition = true;
	}
	else if(!%critical && %obj.crpgCriticalCondition)
	{
		%obj.crpgCriticalCondition = "";
	}
	
	return %critical;
}

function Player::setStunned(%obj, %time)
{
	if(!%obj.crpgArrested)
	{
		%client = %obj.client;
		
		if(isObject(%client))
		{
			%profile = %client.profile;
			
			if(isObject(%profile))
			{
				%obj.crpgStunned		= true;
				%obj.crpgStunAmount	= %time;
				%obj.crpgStunLeft		= %time;
				
				%stunned = %obj.onStunCycle();
				
				if(isObject(%client.camera))
				{
					%client.camera.setMode("Corpse", %client.player);
					%client.setControlObject(%client.camera);
					%obj.mountImage(CRPG_Stun_Image, $BackSlot);
				}
			}
		}
	}
}

// Events
function Player::moveToBrick(%obj, %brick, %dir, %vel)
{
	if(%brick $= "")
		return;
	
	%ss = new SimSet();
	%name = "_" @ %brick;
	
	while(isObject(%name))
	{
		%brick = %name;
		%ss.add(%brick);
		%brick.setName("");
	}
	
	if(%ss.getCount())
	{
		%brick = %ss.getObject(getRandom(0, %ss.getCount() - 1));
		%db = %brick.getDatablock();
		
		%x = (%db.brickSizeX - (3 * getWord(%obj.getScale(), 0))) / 4;
		%y = (%db.brickSizeY - (3 * getWord(%obj.getScale(), 1))) / 4;
		%z = %db.brickSizeZ;
		
		%pos = getRandom(-%x*100, %x*100)/100 SPC getRandom(-%y*100, %y*100)/100 SPC %z;
		%pos = vectorAdd(%brick.getPosition(), %pos);
		
		switch(%dir)
		{	
			case 0: %dir = getwords(%obj.getTransform(), 3, 6);
			case 1: %dir = "1 0 0 0";
			case 2: %dir = "0 0 1" SPC ($pi/2);
			case 3: %dir = "0 0 1" SPC $pi;
			case 4: %dir = "0 0 1" SPC -($pi/2);
		}
		
		%obj.setTransform(%pos SPC %dir);
		
		if(!%vel)
			%obj.setvelocity("0 0 0");
		
		while(%ss.getCount())
		{
			%brick = %ss.getObject(0);
			%brick.setName(%name);
			%ss.remove(%brick);
		}
	}
	
	%ss.delete();
}

function Player::teleportHome(%obj, %source, %manual)
{
	%client = %obj.client;
	
	if(isObject(%client))
	{
		if(!%manual)
		{
			%lot = %obj.crpgLot;
			
			if(!isObject(%lot))
				%lot = %obj.lastTouchBrick;
			
			if(isObject(%lot) && %lot.isCrimeHarbor(%client))
			{
				%profile = %client.profile;
				
				if(isObject(%profile) && %profile.getJob().police)
				{
					return false;
				}
			}
		}
		
		%obj.dismount();
		%obj.setVelocity("0 0 0");
		%obj.setTransform(CRPGMinigame.pickSpawnPoint(%client));
		%obj.teleportEffect();
		return true;
	}
	
	return false;
}

// ============================================================
// Section 6 : ServerCmd
// ============================================================
// Non-GUI
function serverCmdGiveMoney(%client, %amount)
{
	%profile = %client.profile;
	
	if(isObject(%profile))
	{
		if(mFloatLength(%amount, 0) $= %amount && %amount > 0)
		{
			%amount = mClamp(%amount, 0, %profile.money);
			
			if(%amount)
			{
				%obj = %client.player;
				
				if(isObject(%obj))
				{
					// ShapeBase::onClick Raycast
					%start	= %obj.getEyePoint();
					%aimVec	= %obj.getAimVector();
					%range	= 6 * getWord(%obj.getScale(), 2);
					%end		= vectorAdd(%start, vectorScale(%aimVec, %range));
					%masks	= ($TypeMasks::FxBrickObjectType | $TypeMasks::PlayerObjectType | $TypeMasks::StaticObjectType | $TypeMasks::TerrainObjectType | $TypeMasks::VehicleObjectType);
					%ray		= containerRayCast(%start, %end, %masks, %obj);
					%col		= getWord(%ray, 0);
					
					if(isObject(%col) && %col.getType() & $TypeMasks::PlayerObjectType)
					{
						%cc = %col.client;
						%cp = %cc.profile;
						
						if(isObject(%cp))
						{
							%cp.money += %amount;
							commandToClient(%cc, 'CRPGCashChange', %amount, %cp.money, %cp.bank, %cp.debt);
							messageClient(%cc, '', '\c3%1\c6 has given you \c3$%2\c6.', %client.name, %amount);
							%cc.sendStatus();
							
							%profile.money -= %amount;
							commandToClient(%client, 'CRPGCashChange', -%amount, %profile.money, %profile.bank, %profile.debt);
							messageClient(%client, '', '\c3%1\c6 was given \c3$%2\c6.', %cc.name, %amount);
							%client.sendStatus();
							
							return;
						}
					}
					
					messageClient(%client, '', "\c6Target not Found.");
				}
			}
			else
			{
				messageClient(%client, '', "\c6You have no money to give.");
			}
		}
		else
		{
			messageClient(%client, '', "\c6You did not enter a valid, positive integer.");
		}
	}
}

//Grab First Person State
function serverCmdCRPG_RelayFirstPerson(%client,%val)
{
	%client.ThirdPersonActive = %val; //save "is Third Person?" to client (server-side)
	if(isObject(%client.player) && !%val)
	{
		if(%client.player.isAiming)
			commandToClient(%client,'HataCrosshair',false);
	}
	else if(isObject(%client.player) && %val)
	{
		if(%client.player.isAiming)
			commandToClient(%client,'HataCrosshair',true);
	}
}

function serverCmdReset(%client)
{
	%profile = %client.profile;
	
	if(isObject(%profile))
	{
		commandToClient(%client, 'MessageBoxYesNo', "DESTROY YOUR PROFILE?",
		"You are about to DESTROY YOUR OWN PROFILE.\n" NL
		"If you do this, everything about your profile will be erased and your lots will be destroyed.\n" NL
		"ARE YOU ABSOLUTELY SURE?",
		'Reset_Confirm');
	}
}

function serverCmdReset_Confirm(%client)
{
	%profile = %client.profile;
	
	if(isObject(%profile))
	{
		// Lots
		for(%a = 0; %a < TriggerSS.getCount(); %a++)
		{
			%brick = TriggerSS.getObject(%a);
			
			if(%brick.getGroup().bl_id == %client.bl_id && %brick.getDatablock().crpgLot)
			{
				%brick.killBrick();
			}
		}
		
		// Profile
		Profiler.deleteProfile(%client.bl_id);
		Profiler.createProfile(%client.bl_id);
		%client.profile = %client.getProfile();
		
		// Respawn
		for(%a = 0; %a < ClientGroup.getCount(); %a++)
			if(ClientGroup.getObject(%a).bl_id == %client.bl_id)
				ClientGroup.getObject(%a).spawnPlayer();
	}
}

//Server-side Movement Input Detection
$Dir[0] = "Forward";
$Dir[1] = "Left";
$Dir[2] = "Backward";
$Dir[3] = "Right";
function serverCmdSetMoveTrigger(%client,%dir,%val) //dir = W A S or D, val = true or false
{
	%client.KeyMove[$Dir[%dir]] = %val;
}

// Bullshit
function serverCmdAdminHacks(%client)
{
	if(%client.isSuperAdmin)
	{
		%client.disableAdminHacks = !%client.disableAdminHacks;
		
		if(%client.disableAdminHacks)
			messageClient(%client, '', "\c0You have disabled your admin hacks. The game will treat you like a normal player.");
		else
			messageClient(%client, '', "\c2You have enabled your admin hacks. The game will exempt you from many restrictions.");
	}
	else
		messageClient(%client, '', "\c5Sorry! You must be at least this cool to enable Admin Hacks.");
}

//function serverCmdCRPG_Test()
//{
//	messageAll('', "Triggered!");
//}

function serverCmdGiveBricks(%client, %targetBL_ID, %victimBL_ID)
{
	if(%client.isSuperAdmin)
	{
		%tbg = "BrickGroup_" @ %targetBL_ID;
		%vbg = "BrickGroup_" @ %victimBL_ID;
		
		if(!isObject(%tbg))
		{
			messageClient(%client, '', "Could not find target BG (" @ %tbg @ ").");
			return;
		}
		
		if(!isObject(%vbg))
		{
			messageClient(%client, '', "Could not find victim BG (" @ %vbg @ ").");
			return;
		}
		
		if(!%vbg.getCount())
		{
			messageClient(%client, '', "Victim BG has no bricks.");
			return;
		}
		
		messageAll('', '\c3%1\c0 is giving all of %2\'s bricks to %3.', %client.name, %victimBL_ID, %targetBL_ID);
		
		%FAILCOUNT = 50000;
		
		while(isObject(%brick = %vbg.getObject(0)) || (%FAILCOUNT-- < 0))
		{
			%tbg.add(%brick);
			
			if(%vbg.isMember(%brick))
			{
				mesasgeClient(%client, '', "Woah, why is it still a member?");
				return;
			}
		}
		
		if(%FAILCOUNT < 0)
			messageAll('', "SHIT HIT THE WALL! That was aclose infinite loop. TRY HARDER FAILBAN.");
		else
			messageAll('', "If you're seeing this, shit probably worked out just fine.");
	}
}

function serverCmdRainMoney(%client)
{
	if(%client.isCRPGAdmin())
	{
		%obj = %client.player;
		
		if(isObject(%obj))
		{
			for(%a = 0; %a < 250; %a++)
			{
				%pos		= vectorAdd(%obj.getPosition(), getRandom(-64, 64) SPC getRandom(-64, 64) SPC 100);
				%value	= getRandom(1, 5);
				
				%cash = new Item()
				{
					datablock = CRPG_Cash_Item;
					cashValue = %value;
				};
				
				if(isObject(%cash))
				{
					%cash.canPickup = false;
					%cash.setShapeName("$" @ %value);
					%cash.setTransform(%pos SPC getRandom(0, 1) SPC getRandom(0, 1) SPC getRandom(0, 1) SPC getRandom(0, 1));
					%cash.schedule(60000, "schedulePop");
					MissionCleanup.add(%cash);
				}
			}
		}
		else
		{
			messageClient(%client, '', "\c5Please spawn first.");
		}
	}
}

function serverCmdRespawnAllPlayers(%client)
{
	if(%client.isSuperAdmin)
	{
		for(%a = 0; %a < ClientGroup.getCount(); %a++)
		{
			%subClient = ClientGroup.getObject(%a);
			
			if(%subClient.hasSpawnedOnce)
				%subClient.spawnPlayer();
		}
		
		messageAll('', '\c5%1 has respawned all players.', %client.name);
	}
}

function serverCmdTimeUp(%client)
{
	%clean = mFloor($sim::time);
	
	%days = mFloor(%clean / 60 / 60 / 24);
	%daysN = %days != 1 ? "Days" : "Day";
	
	%hours = mFloor((%clean / 60 / 60) - (%days * 24));
	%hoursN = %hours != 1 ? "Hours" : "Hour";
	
	%minutes = mFloor((%clean / 60) - (%hours * 60) - (%days * 24 * 60));
	%minutesN = %minutes != 1 ? "Minutes" : "Minute";
	
	%seconds = %clean % 60;
	%secondsN = %seconds != 1 ? "Seconds" : "Second";
	
	messageClient(%client, '', "\c6The server has been up for \c3" @ %days SPC %daysN @ "\c6, \c3" @ %hours SPC %hoursN @ "\c6, \c3" @ %minutes SPC %minutesN @ "\c6, and \c3" @ %seconds SPC %secondsN @ "\c6 straight.");
}

function serverCmdWrench(%client)
{
	if(%client.isCRPGAdmin() && isObject(%obj = %client.player))
		%obj.mountImage(wrenchImage, 0);
}

// Admin
function serverCmdAdmin_DeductCash(%client, %victim, %input)
{
	if(%client.isCRPGAdmin())
	{
		if(mFloatLength(%input, 0) $= %input && %input > 0)
		{
			if(isObject(%victim))
			{
				%profile = %victim.profile;
				
				if(isObject(%profile))
				{
					%profile.removeFunds(%input, false);
					%victim.sendStatus();
					
					if(%victim != %client.getID())
					{
						messageClient(%victim, '', '\c6An admin has deducted \c3$%1\c6 from your account.', %input);
						commandToClient(%victim, 'CRPGCashChange', -%input, %profile.money, %profile.bank, %profile.debt);
						%victim.sendStatus();
					}
					
					serverCmdAdmin_GetProfileList(%client);
					commandToClient(%client, 'MessageBoxOK', "Success!", %victim.name @ " had money deducted.");
				}
			else
			{
				commandToClient(%client, 'MessageBoxOK', "Does Not Compute", %victim.name SPC "has no profile.");
			}
			}
			else
			{
				commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "Your specific victim does not exist, or the ID given to you by the server (" @ %victim @ ") is invalid.\n\nReopen the Admin Panel and try again.");
			}
		}
		else
		{
			commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "You did not enter a valid, positive Integer.");
		}
	}
	else
	{
		commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "You are not an Admin.\n\nFuck off.");
	}
}

function serverCmdAdmin_DeleteProfile(%client, %victim, %input)
{
	if(%client.isCRPGAdmin())
	{
		if(isObject(%victim))
		{
			Profiler.deleteProfile(%victim.bl_id);
			Profiler.createProfile(%victim.bl_id);
			%victim.profile = %victim.getProfile();
			
			if(%victim.hasSpawnedOnce)
				%victim.spawnPlayer();
			
			if(%victim != %client.getID())
				messageClient(%victim, '', '\c6Your CityRPG profile has been reset by an admin.');
			
			serverCmdAdmin_GetProfileList(%client);
			commandToClient(%client, 'MessageBoxOK', "Success!", %victim.name @ " had their profile reset.");
		}
		else
		{
			commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "Your specific victim does not exist, or the ID given to you by the server (" @ %victim @ ") is invalid.\n\nReopen the Admin Panel and try again.");
		}
	}
	else
	{
		commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "You are not an Admin.\n\nFuck off.");
	}
}

function serverCmdAdmin_EraseRecord(%client, %victim, %input)
{
	if(%client.isCRPGAdmin())
	{
		if(isObject(%victim))
		{
			%profile = %victim.profile;
			
			if(isObject(%profile))
			{
				%profile.setDemerits(0);
				%profile.felon = false;
				%profile.sentence = 0;
				
				if(%victim.hasSpawnedOnce)
					%victim.spawnPlayer();
				
				if(%victim != %client.getID())
					messageClient(%victim, '', '\c6Your criminal record has been erased by an admin.');
				
				commandToClient(%client, 'MessageBoxOK', "Success!", %victim.name @ " had their record erased.");
			}
			else
			{
				commandToClient(%client, 'MessageBoxOK', "Does Not Compute", %victim.name SPC "has no profile.");
			}
		}
		else
		{
			commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "Your specific victim does not exist, or the ID given to you by the server (" @ %victim @ ") is invalid.\n\nReopen the Admin Panel and try again.");
		}
	}
	else
	{
		commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "You are not an Admin.\n\nFuck off.");
	}
}

function serverCmdAdmin_GetProfileList(%client)
{
	commandToClient(%client, 'Admin_ClearProfileSO');
	
	if(%client.isCRPGAdmin())
	{
		for(%a = 0; %a < ClientGroup.getCount(); %a++)
		{
			%subClient = ClientGroup.getObject(%a);
			
			if(isObject(%profile = %subClient.profile))
			{
				commandToClient(%client, 'Admin_AddProfile', %subClient.name, %profile.money + %profile.bank, %profile.demerits, %profile.getJob().name, %subClient.getID());
			}
		}
	}
	else
	{
		commandToClient(%client, 'Admin_AddProfile', "Go", "Fuck", "Yourself", "Kalphiter.");
	}
}

function serverCmdAdmin_GrantCash(%client, %victim, %input)
{
	if(%client.isCRPGAdmin())
	{
		if(mFloatLength(%input, 0) $= %input && %input > 0)
		{
			if(isObject(%victim))
			{
				%profile = %victim.profile;
				
				if(isObject(%profile))
				{
					%profile.bank += %input;
					%victim.sendStatus();
					
					if(%victim != %client.getID())
					{
						messageClient(%victim, '', '\c6An admin has added \c3$%1\c6 to your account.', %input);
						commandToClient(%victim, 'CRPGCashChange', %input, %profile.money, %profile.bank, %profile.debt);
						%victim.sendStatus();
					}
					
					serverCmdAdmin_GetProfileList(%client);
					commandToClient(%client, 'MessageBoxOK', "Success!", %victim.name @ " had money granted.");
				}
				else
				{
					commandToClient(%client, 'MessageBoxOK', "Does Not Compute", %victim.name SPC "has no profile.");
				}
			}
			else
			{
				commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "Your specific victim does not exist, or the ID given to you by the server (" @ %victim @ ") is invalid.\n\nReopen the Admin Panel and try again.");
			}
		}
		else
		{
			commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "You did not enter a valid, positive Integer.");
		}
	}
	else
	{
		commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "You are not an Admin.\n\nFuck off.");
	}
}

function serverCmdAdmin_Pardon(%client, %victim, %input)
{
	if(%client.isCRPGAdmin())
	{
		if(isObject(%victim))
		{
			%profile = %victim.profile;
			
			if(isObject(%profile))
			{
				if(%profile.isInmate())
				{
					%profile.sentence = 0;
					
					if(%victim.hasSpawnedOnce)
						%victim.spawnPlayer();
					
					if(%victim != %client.getID())
						messageClient(%victim, '', '\c6You\'ve been pardoned by an admin.');
					
					commandToClient(%client, 'MessageBoxOK', "Success!", %victim.name @ " was pardoned.");
				}
				else
				{
					commandToClient(%client, 'MessageBoxOK', "Does Not Compute", %victim.name SPC "is not in prison.");
				}
			}
			else
			{
				commandToClient(%client, 'MessageBoxOK', "Does Not Compute", %victim.name SPC "has no profile.");
			}
		}
		else
		{
			commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "Your specific victim does not exist, or the ID given to you by the server (" @ %victim @ ") is invalid.\n\nReopen the Admin Panel and try again.");
		}
	}
	else
	{
		commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "You are not an Admin.\n\nFuck off.");
	}
}

function serverCmdAdmin_SetDemerits(%client, %victim, %input)
{
	if(%client.isCRPGAdmin())
	{
		if(mFloatLength(%input, 0) $= %input && %input >= 0)
		{
			if(isObject(%victim))
			{
				%profile = %victim.profile;
				
				if(isObject(%profile))
				{
					if(%profile.demerits != %input)
					{
						%profile.setDemerits(%input);
						%victim.sendStatus();
						
						if(%victim != %client.getID())
							messageClient(%victim, '', '\c6Your demerits were set to \c3%1\c6 by an admin.', %input);
						
						serverCmdAdmin_GetProfileList(%client);
						commandToClient(%client, 'MessageBoxOK', "Success!", %victim.name @ " had their demerits set to" SPC %input @ ".");
					}
					else
					{
						commandToClient(%client, 'MessageBoxOK', "Does Not Compute", %victim.name SPC "already has" SPC %input SPC " demerit(s).");
					}
				}
				else
				{
					commandToClient(%client, 'MessageBoxOK', "Does Not Compute", %victim.name SPC "has no profile.");
				}
			}
			else
			{
				commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "Your specific victim does not exist, or the ID given to you by the server (" @ %victim @ ") is invalid.\n\nReopen the Admin Panel and try again.");
			}
		}
		else
		{
			commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "You did not enter a valid, positive Integer.");
		}
	}
	else
	{
		commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "You are not an Admin.\n\nFuck off.");
	}
}

function serverCmdAdmin_SetHunger(%client, %victim, %input)
{
	if(%client.isCRPGAdmin())
	{
		if(mFloatLength(%input, 0) $= %input && %input >= 0)
		{
			if(isObject(%victim))
			{
				%profile = %victim.profile;
				
				if(isObject(%profile))
				{
					if(%profile.hunger != %input)
					{
						%profile.hunger = %input;
						%victim.sendStatus();
						
						if(%victim != %client.getID())
							messageClient(%victim, '', '\c6Your hunger was set to \c3%1%\c6 by an admin.', %input);
						
						serverCmdAdmin_GetProfileList(%client);
						commandToClient(%client, 'MessageBoxOK', "Success!", %victim.name @ " had their hunger set to" SPC %input @ "%.");
					}
					else
					{
						commandToClient(%client, 'MessageBoxOK', "Does Not Compute", %victim.name SPC "already has" SPC %input @ "% hunger.");
					}
				}
				else
				{
					commandToClient(%client, 'MessageBoxOK', "Does Not Compute", %victim.name SPC "has no profile.");
				}
			}
			else
			{
				commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "Your specific victim does not exist, or the ID given to you by the server (" @ %victim @ ") is invalid.\n\nReopen the Admin Panel and try again.");
			}
		}
		else
		{
			commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "You did not enter a valid, positive Integer.");
		}
	}
	else
	{
		commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "You are not an Admin.\n\nFuck off.");
	}
}

// Bank
function serverCmdBank_Deposit(%client, %amount)
{
	if(!isObject(%client.player))
		return %client.sendMessageBox("Does Not Compute", "You must spawn first.");
	
	%bank = false;
	
	%pos = %client.player.getEyePoint();
	%mask = $TypeMasks::FxBrickAlwaysObjectType;
	%radius = 16;
	initContainerRadiusSearch(%pos, %radius, %mask);
	
	while(isObject(%searchObj = containerSearchNext()))
	{
		for(%a = 0; %a < %searchObj.numEvents; %a++)
		{
			if(%searchObj.eventOutput[%a] $= "ShowGUI" && %searchObj.eventOutputParameter[%a, 1] == 0)
			{
				%bank = true;
				break;
			}
		}
		
		if(%bank) break;
	}
	
	if(!%bank)
		return %client.sendMessageBox("Does Not Compute", "There is no bank brick near you.");
	
	if(isObject(%client.profile))
	{
		if(mFloatLength(%amount, 0) $= %amount && %amount > 0)
		{
			if(%amount > %client.profile.money)
				%amount = %client.profile.money;
			
			if(%amount > 0)
			{
				%client.profile.bank += %amount;
				%client.profile.money -= %amount;
				%client.sendStatus();
				
				commandToClient(%client, 'MessageBoxOK', "Transaction Complete", "You have put $" @ %amount @ " into your bank account.");
			}
			else
				commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "You have no cash on hand to deposit.");
		}
		else
			commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "You must enter a valid, positive integer to deposit.");
	}
	else
		commandToClient(%client, 'MessageBoxOK', "No Profile", "You have no profile.");
}

function serverCmdBank_Enroll(%client)
{
	if(!isObject(%client.player))
		return %client.sendMessageBox("Does Not Compute", "You must spawn first.");
	
	%bank = false;
	
	%pos = %client.player.getEyePoint();
	%mask = $TypeMasks::FxBrickAlwaysObjectType;
	%radius = 16;
	initContainerRadiusSearch(%pos, %radius, %mask);
	
	while(isObject(%searchObj = containerSearchNext()))
	{
		for(%a = 0; %a < %searchObj.numEvents; %a++)
		{
			if(%searchObj.eventOutput[%a] $= "ShowGUI" && %searchObj.eventOutputParameter[%a, 1] == 0)
			{
				%bank = true;
				break;
			}
		}
		
		if(%bank) break;
	}
	
	if(!%bank)
		return %client.sendMessageBox("Does Not Compute", "There is no bank brick near you.");
	
	if(isObject(%client.profile))
	{
		if(!%client.profile.student)
		{
			if(!%client.profile.debt)
			{
				%amount = (%client.profile.education + 1) * 1000;
				
				if(%amount <= %client.profile.money + %client.profile.bank)
				{
					%client.profile.student = (%client.profile.education * 7) + 7;
					%client.profile.removeFunds(%amount, false);
					%client.sendStatus();
					
					commandToClient(%client, 'MessageBoxOK', "Transaction Complete", "You have paid $" @ %amount @ " out of pocket for your next semester of schooling.");
				}
				else
					commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "Your next diploma will cost $" @ %amount @ ", which you cannot afford.\n\nEither take a loan, or make some money.");
			}
			else
				commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "You cannot enroll with debt.\n\nPlease pay off the $" @ %client.profile.debt @ " you owe the school, first.");
		}
		else
			commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "You are already a student.");
	}
	else
		commandToClient(%client, 'MessageBoxOK', "No Profile", "You have no profile.");
}

function serverCmdBank_PayOff(%client, %amount)
{
	if(!isObject(%client.player))
		return %client.sendMessageBox("Does Not Compute", "You must spawn first.");
	
	%bank = false;
	
	%pos = %client.player.getEyePoint();
	%mask = $TypeMasks::FxBrickAlwaysObjectType;
	%radius = 16;
	initContainerRadiusSearch(%pos, %radius, %mask);
	
	while(isObject(%searchObj = containerSearchNext()))
	{
		for(%a = 0; %a < %searchObj.numEvents; %a++)
		{
			if(%searchObj.eventOutput[%a] $= "ShowGUI" && %searchObj.eventOutputParameter[%a, 1] == 0)
			{
				%bank = true;
				break;
			}
		}
		
		if(%bank) break;
	}
	
	if(!%bank)
		return %client.sendMessageBox("Does Not Compute", "There is no bank brick near you.");
	
	if(isObject(%client.profile))
	{
		if(mFloatLength(%amount, 0) $= %amount && %amount > 0)
		{
			if(%amount > %client.profile.money + %client.profile.bank)
				%amount = %client.profile.money + %client.profile.bank;
			
			if(%amount > 0)
			{
				if(%amount > %client.profile.debt)
					%amount = %client.profile.debt;
				
				if(%amount > 0)
				{
					%client.profile.debt -= %amount;
					%client.profile.removeFunds(%amount, false);
					%client.sendStatus();
					
					if(%client.profile.debt > 0)
						commandToClient(%client, 'MessageBoxOK', "Transaction Complete", "You have put $" @ %amount @ " towards your debt.\n\n$" @ %client.profile.debt @ " remains.");
					else
						commandToClient(%client, 'MessageBoxOK', "Transaction Complete", "You have put $" @ %amount @ " towards your debt.\n\nYou are debt free.");
				}
				else
					commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "You no debt.");
			}
			else
				commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "You have no cash to pay off the loan with.");
		}
		else
			commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "You must enter a valid, positive integer to put towards your debt.");
	}
	else
		commandToClient(%client, 'MessageBoxOK', "No Profile", "You have no profile.");
}

function serverCmdBank_TakeLoan(%client)
{
	if(!isObject(%client.player))
		return %client.sendMessageBox("Does Not Compute", "You must spawn first.");
	
	%bank = false;
	
	%pos = %client.player.getEyePoint();
	%mask = $TypeMasks::FxBrickAlwaysObjectType;
	%radius = 16;
	initContainerRadiusSearch(%pos, %radius, %mask);
	
	while(isObject(%searchObj = containerSearchNext()))
	{
		for(%a = 0; %a < %searchObj.numEvents; %a++)
		{
			if(%searchObj.eventOutput[%a] $= "ShowGUI" && %searchObj.eventOutputParameter[%a, 1] == 0)
			{
				%bank = true;
				break;
			}
		}
		
		if(%bank) break;
	}
	
	if(!%bank)
		return %client.sendMessageBox("Does Not Compute", "There is no bank brick near you.");
	
	if(isObject(%client.profile))
	{
		if(!%client.profile.student)
		{
			if(!%client.profile.debt)
			{
				%amount = (%client.profile.education + 1) * 1000;
				
				%client.profile.student = (%client.profile.education * 7) + 7;
				%client.profile.debt = %amount;
				%client.sendStatus();
				
				commandToClient(%client, 'MessageBoxOK', "Transaction Complete", "You have taken a $" @ %amount @ " student loan for your next semester of schooling.");
			}
			else
				commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "You cannot take a new loan with debt.\n\nPlease pay off the $" @ %client.profile.debt @ " you owe the school, first.");
		}
		else
			commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "You are already a student..");
	}
	else
		commandToClient(%client, 'MessageBoxOK', "No Profile", "You have no profile.");
}

function serverCmdBank_Withdraw(%client, %amount)
{
	if(!isObject(%client.player))
		return %client.sendMessageBox("Does Not Compute", "You must spawn first.");
	
	%bank = false;
	
	%pos = %client.player.getEyePoint();
	%mask = $TypeMasks::FxBrickAlwaysObjectType;
	%radius = 16;
	initContainerRadiusSearch(%pos, %radius, %mask);
	
	while(isObject(%searchObj = containerSearchNext()))
	{
		for(%a = 0; %a < %searchObj.numEvents; %a++)
		{
			if(%searchObj.eventOutput[%a] $= "ShowGUI" && %searchObj.eventOutputParameter[%a, 1] == 0)
			{
				%bank = true;
				break;
			}
		}
		
		if(%bank) break;
	}
	
	if(!%bank)
		return %client.sendMessageBox("Does Not Compute", "There is no bank brick near you.");
	
	if(isObject(%client.profile))
	{
		if(mFloatLength(%amount, 0) $= %amount && %amount > 0)
		{
			if(%amount > %client.profile.bank)
				%amount = %client.profile.bank;
			
			if(%amount > 0)
			{
				%client.profile.bank -= %amount;
				%client.profile.money += %amount;
				%client.sendStatus();
				
				commandToClient(%client, 'MessageBoxOK', "Transaction Complete", "You taken out $" @ %amount @ " from your bank account.");
			}
			else
				commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "You have no cash in the bank to withdraw.");
		}
		else
			commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "You must enter a valid, positive integer to withdraw.");
	}
	else
		commandToClient(%client, 'MessageBoxOK', "No Profile", "You have no profile.");
}

// Commands
function serverCmdCommand_Reload(%client)
{
	%obj = %client.player;
	
	if(isObject(%obj) && %obj.getImageState(0) $= "Ready")
	{
		%image = %obj.getMountedImage(0);
		
		if(isObject(%image) && stripTrailingSpaces(%image.ammo) !$= "" && %obj.ammo[%image.ammo] < %image.ammoMagSize)
		{
			%obj.setImageAmmo(0, 0);
			%obj.ammo[%image.ammo] = 0;
		}
	}
}

function serverCmdCommand_ToggleVillainy(%client)
{
	if(isObject(%profile = %client.profile))
		%profile.setVillainy(!%client.crpgEvil);
}

// Events
function serverCmdEvent_Buy(%client)
{
	// Source brick
	%brick	= %client.salesBrick;
	// The class of product. Handles sales logic.
	%product	= %client.salesBrickProduct;
	// Item being sold.
	%item		= %client.salesBrickItem;
	// Profit made by the owner.
	%profit	= %client.salesBrickMarkup;
	// The name or label applied to the sales object.
	%name		= %client.salesBrickName;
	// Actual price the client will pay.
	%price	= %client.salesBrickPrice;
	
	// Clear metadata
	%client.salesBrick			= "";
	%client.salesBrickProduct	= "";
	%client.salesBrickItem		= "";
	%client.salesBrickMarkup	= "";
	%client.salesBrickName		= "";
	%client.salesBrickPrice		= "";
	
	if(!isObject(%brick))
		return %client.sendMessageBox("Does Not Compute", "Sorry!\n\nYou cannot buy something without a sales brick.");
	
	%obj = %client.player;
		
	if(!isObject(%obj))
		return %client.sendMessageBox("Does Not Compute", "Sorry!\n\nYou have to spawn before you can buy stuff.");
	
	if(vectorDist(%brick.getPosition(), %obj.getPosition()) > 8)
		return %client.sendMessageBox("Does Not Compute", "Sorry!\n\nYou are too far from the sales brick to buy from it.");
	
	%profile = %client.profile;
	
	if(!isObject(%profile))
		return %client.sendMessageBox("Does Not Compute", "Sorry!\n\nYou have no profile.");
	
	%brickgroup = %brick.getGroup();
	%owner = %brickgroup.client;
	
	if(!isObject(%brickGroup))
		return %client.sendMessageBox("Does Not Compute", "Sorry!\n\nThis brick has no brickgroup.\nPlease pick another store!");
	
	// Load a brick group's profile.
	%brickProfile = %brick.getProfile();
	
	if(!isObject(%brickProfile))
		return %client.sendMessageBox("Does Not Compute", "Sorry!\n\nThis brick has no profile.\nPlease pick another store!");
	
	%completed = false;
	%brickJob = %brickProfile.getJob();
	
	if(%product $= "food")
	{
		if(!%brickJob.sellFood)
			return %client.sendMessageBox("Does Not Compute", "Sorry!\n\nThis store owner cannot sell food.");
		
		if(%profile.money < %price)
			return %client.sendMessageBox("Does Not Compute", "Sorry!\n\nThis meal costs $" @ %price @ ", but you only have $" @ %profile.money @ " on hand.\n\nYou cannot afford it.");
		
		%plastic = mClamp(mFloor(%item / 10), 1, 10);
		
		if(!isObject(%owner) && %brickProfile.refPlastic < %plastic)
			return %client.sendMessageBox("Does Not Compute",
			"Sorry!\n\nYou cannot buy food from an offline client who has ran out of Refined Plastic.");
		
		// Different profile?
		if(%brickProfile.getID() != %profile.getID())
		{
			%completed = true;
			
			// Customer Update
			%profile.hunger += %item;
			
			if(%profile.hunger > 110)
				%profile.hunger = 110;
			
			%profile.removeFunds(%price, true);
			commandToClient(%client, 'CRPGCashChange', -%price, %profile.money, %profile.bank, %profile.debt);
			messageClient(%client, '', '\c6You paid \c3$%1\c6 for \"\c3%2\c6\". Delicious.', %price, %name);
			
			// Owner Update
			if(%brickProfile.refPlastic >= %plastic)
			{
				%brickProfile.refPlastic -= %plastic;
				%brickProfile.bank += %profit;
				%brickProfile.addEXP(mClampF(mFloor(%profit / 10), 1, 100), "Store");
				
				if(isObject(%owner))
				{
					commandToClient(%owner, 'CRPGCashChange', %profit, %brickProfile.money, %brickProfile.bank, %brickProfile.debt);
					messageClient(%owner, '', '\c6A customer purchased "\c3%2\c6" for \c3$%1\c6 profit.', %profit, %name);
				}
			}
			else if(isObject(%brickgroup.client))
			{
				commandToClient(%brickgroup.client, 'CRPGCashChange', %profit, %brickProfile.money, %brickProfile.bank, %brickProfile.debt);
				messageClient(%brickgroup.client, '', '\c6A customer purchased "\c3%2\c6", but you did not get profit because \c0you do not have enough Refined Plastic\c6.', %profit, %name);
			}
		}
		else if(%brickProfile.refPlastic >= %plastic)
		{
			%completed = true;
			
			%profile.hunger += %item;
			
			if(%profile.hunger > 110)
				%profile.hunger = 110;
			
			%brickProfile.refPlastic -= %plastic;
			%profile.removeFunds(%price - %profit, true);
			commandToClient(%client, 'CRPGCashChange', -(%price - %profit), %profile.money, %profile.bank, %profile.debt);
			messageClient(%client, '', '\c6You you paid \c3$%1\c6 directly for \"\c3%2\c6\".', %price - %profit, %name);
		}
		else
			return %client.sendMessageBox("Does Not Compute",
			"Sorry!\n\nYou cannot buy a meal directly from yourself if you do not have the Refined Plastic to cover it.");
	}
	else if(%product $= "item")
	{
		if(%item.isFirearm && SystemSO.moralFelonGun && %profile.felon)
			return %client.sendMessageBox("Does Not Compute", "Sorry!\n\nCity Law prohibits felons from purchasing firearms.");
		
		if(%profile.money < %price)
			return %client.sendMessageBox("Does Not Compute",
			"Sorry!\n\nThis item costs $" @ %price @ ", but you only have $" @ %profile.money @ " on hand.\n\nYou cannot afford it.");
		
		%alreadyHas = false;
		
		for(%a = 0; %a < %obj.dataBlock.maxTools; %a++)
		{
			if(isObject(%obj.tool[%a]) && %obj.tool[%a].getID() == %item.getID())
			{
				%alreadyHas = true;
				break;
			}
		}
		
		if(%alreadyHas)
			return %client.sendMessageBox("Does Not Compute", "Sorry!\n\nYou already have this item.");
		
		%given = false;
		
		for(%a = 0; %a < %obj.dataBlock.maxTools; %a++)
		{
			if(!isObject(%obj.tool[%a]))
			{
				%item.onPickup("", %obj);
				%obj.tool[%a] = %item.getID();
				messageClient(%client, 'MsgItemPickup', '', %a, %item.getID());
				%given = true;
				break;
			}
		}
				
		if(!%given)
			return %client.sendMessageBox("Does Not Compute", "Sorry!\n\nYou have no room in your inventory.");
		
		%completed = true;
		
		if(!isObject(%owner) || %owner.getID() != %client.getID())
		{
			// Customer Update
			%profile.removeFunds(%price, true);
			commandToClient(%client, 'CRPGCashChange', -%price, %profile.money, %profile.bank, %profile.debt);
			messageClient(%client, '', '\c6You paid \c3$%1\c6 for \c3%2\c6.', %price, %name);
			
			// Owner Update
			%brickProfile.bank += %profit;
			%brickProfile.addEXP(mClampF(mFloor(%profit / 10), 1, 100), "Store");
			
			if(isObject(%owner))
			{
				commandToClient(%owner, 'CRPGCashChange', %profit, %brickProfile.money, %brickProfile.bank, %brickProfile.debt);
				messageClient(%owner, '', '\c6A customer purchased the \c3%2\c6 for \c3$%1\c6 profit.', %profit, %name);
			}
		}
		else
		{
			%profile.removeFunds(%price - %profit, true);
			commandToClient(%client, 'CRPGCashChange', -(%price - %profit), %profile.money, %profile.bank, %profile.debt);
			messageClient(%client, '', '\c6You you paid \c3$%1\c6 directly for the \c3%2\c6.', %price - %profit, %name);
		}
	}
	else if(%product $= "vehicle")
	{
		commandToClient(%client, 'Vehicle_Close');
		
		if(!%brickJob.sellVehicle)
			return %client.sendMessageBox("Does Not Compute", "Sorry!\n\nThis store owner cannot sell vehicles.");
		
		if(%profile.money < %price)
			return %client.sendMessageBox("Does Not Compute",
			"Sorry!\n\nThis vehicle costs $" @ %price @ ", but you only have $" @ %profile.money @ " on hand.\n\nYou cannot afford it.");
		
		if(!isObject(%item))
			return %client.sendMessageBox("Does Not Compute", "Sorry!\n\nVehicle requested does not exist.");
		
		%alreadyHas = false;
		
		for(%a = 0; %a < getFieldCount(%profile.garage); %a++)
		{
			%vehicle = getField(%profile.garage, %a);
			
			if(isObject(%vehicle) && %vehicle.getID() == %item.getID())
			{
				%alreadyHas = true;
				break;
			}
		}
		
		if(!%alreadyHas)
		{
			%profile.garage = setField(%profile.garage, getFieldCount(%profile.garage), %item.getName());
			%completed = true;
			
			if(!isObject(%owner) || %owner.getID() != %client.getID())
			{
				// Customer Update
				%profile.removeFunds(%price, true);
				commandToClient(%client, 'CRPGCashChange', %price, %profile.money, %profile.bank, %profile.debt);
				messageClient(%client, '', '\c6You paid \c3$%1\c6 for \"\c3%2\c6\".', %price, %name);
				
				// Owner Update
				%brickProfile.bank += %profit;
				%brickProfile.addEXP(mClampF(mFloor(%profit / 10), 1, 100), "Store");
				
				if(isObject(%owner))
				{
					commandToClient(%owner, 'CRPGCashChange', %profit, %profile.money, %profile.bank, %profile.debt);
					messageClient(%owner, '', '\c6A customer purchased "\c3%2\c6" for \c3$%1\c6 profit.', %profit, %name);
				}
			}
			else
			{
				%profile.removeFunds(%price - %profit, true);
				commandToClient(%client, 'CRPGCashChange', -(%price - %profit), %profile.money, %profile.bank, %profile.debt);
				messageClient(%client, '', '\c6You you paid \c3$%1\c6 directly for \"\c3%2\c6\".', %price - %profit, %name);
			}
		}
		else
			commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "Sorry!\n\nYou already have this vehicle.");
				
	}
	else if(%product $= "plastic")
	{
		if(%profile.plastic < %item)
			return %client.sendMessageBox("Does Not Compute", "Sorry!\n\nYou don't have enough plastic to sell.");
		
		if(%brickProfile.bank < %price)
			return %client.sendMessageBox("Does Not Compute", "Sorry!\n\nThe brick owner cannot afford your plastic!\nWhat a chump.");
		
		%completed = true;
		
		%brickProfile.bank -= %price;
		%profile.money += %price;
		
		%profile.plastic -= %item;
		%brickProfile.plastic += %item;
		
		messageClient(%client, '', '\c6You have sold \c3%1\c6 Raw Plastic for \c3$%2\c6.', %item, %price);
		%client.sendStatus();
		
		if(isObject(%owner))
			messageClient(%owner, '', '\c3%1\c6 has sold you \c3%2\c6 Raw Plastic.', %client.name, %item);
		else
			%brickProfile.problem = addField(%brickProfile.problem, "\c3" @ %client.name @ "\c6 has sold you \c3" @ %item @ "\c6 Raw Plastic.");
	}
	else if(%product $= "refplastic")
	{
		if(%profile.money < %price)
			return %client.sendMessageBox("Does Not Compute",
			"Sorry!\n\n" @ %item @ " Refined Plastic costs $" @ %price @ " from this vendor, but you only have $" @ %profile.money @ " on hand.\n\nYou cannot afford it.");
		
		%completed = true;
		
		%brickProfile.bank += %price;
		%brickProfile.refplastic -= %item;
		%profile.money -= %price;
		
		%obj.crpgCrateOwner = %client.bl_id;
		%obj.crpgCrateRefPlastic = %item;
		%obj.crpgCratePlastic = 0;
		%obj.crpgCrateWorth = 0;
		%obj.crpgCrateMission = -1;
		
		%obj.mountImage(CRPG_Crate_Image, 0);
		
		messageClient(%client, '', '\c6Purchased \c3%1\c6 Refined Plastic for \c3$%2\c6.', %item, %price);
		%client.sendStatus();
		
		if(isObject(%owner))
			messageClient(%owner, '', '\c3%1\c6 has bought \c3%2\c6 Refined Plastic.', %client.name, %item);
		else
			%brickProfile.problem = addField(%brickProfile.problem, "\c3" @ %client.name @ "\c6 has bought \c3" @ %item @ "\c6 Refined Plastic.");
	}
	else
	{
		error("serverCmdEvent_Buy() - [\"" @ %product @ "\" is not a valid product.]");
		messageClient(%client, '', "This event brick references a sales class which does not exist. (" @ %product @ ")");
		messageClient(%client, '', "Please inform the admins that this copy of CityRPG is broke as shit.");
	}
	
	%client.sendStatus();
	
	if(isObject(%owner))
		%owner.sendStatus();
	
	if(%completed)
		%brick.onServiceAccepted(%client);
	
	// Hey, is that a closing bracket I see?
}

function serverCmdEvent_Cancel(%client)
{
	%client.salesBrick			= "";
	%client.salesBrickProduct	= "";
	%client.salesBrickItem		= "";
	%client.salesBrickMarkup	= "";
	%client.salesBrickName		= "";
	%client.salesBrickPrice		= "";
}

function serverCmdEvent_DumpCrate(%client)
{
	%obj = %client.player;
	
	if(isObject(%obj))
	{
		%profile = %client.profile;
		
		if(isObject(%profile))
		{
			%valid = false;
			%illegal = false;
			
			if(isObject(%obj.crpgLot) && %obj.crpgLot.getGroup().bl_id == %client.bl_id)
			{
				%valid = true;
				%illegal = false;
				%plastic = %obj.crpgCratePlastic;
				%refplastic = %obj.crpgCrateRefPlastic;
				%worth = %obj.crpgCrateWorth;
			}
			else if(isObject(%obj.crpgCarTrigger))
			{
				%valid = true;
				%illegal = true;
				%plastic = mFloor(%obj.crpgCratePlastic / 2);
				%refplastic = mFloor(%obj.crpgCrateRefPlastic / 2);
				%worth = mFloor(%obj.crpgCrateWorth / 2);
			}
			
			if(%valid)
			{
				if(%plastic)
					%contents = %contents NL "RAW PLASTIC  : " @ %plastic;
				
				if(%refplastic)
					%contents = %contents NL "REF. PLASTIC : " @ %refplastic;
				
				if(%worth)
					%contents = %contents NL "MONEY		  : $" @ %worth;
				
				%profile.plastic += %plastic;
				%profile.refplastic += %refplastic;
				%profile.money += %worth;
				%client.sendStatus();
				
				if(%illegal)
					%client.commitCrime("Crate Theft", 50, false, true, false, false, 90000);
				
				commandToClient(%client, 'MessageBoxOK', "Crate Opened!", "<font:courier new:20>CRATE CONTENTS<just:left><font:courier new:16>" NL %contents);
				
				if(isObject(%client.mission) && %obj.crpgCrateMission.profile.bl_id == %client.bl_id)
				{
					%client.mission.onSellCrate();
				}
				
				%obj.crpgCrateDumped = true;
				%obj.unMountImage(0);
			}
		}
	}
}

function serverCmdEvent_SellItem(%client)
{
	%obj = %client.player;
	%profile = %client.profile;
	
	if(!isObject(%obj) || !isObject(%profile))
		return;
	
	if(isObject(%obj.crpgCarTrigger) && %obj.crpgSellItem > 0)
	{
		%item = %obj.tool[%obj.crpgSellItem];
		
		if(SalesSO.isItemValuable(%item))
		{
			%price = mClamp(SalesSO.getItemPrice(SalesSO.getItemID(%item)) / 10, 10, 10000);
			
			%profile.money += %price;
			%obj.tool[%obj.crpgSellItem] = 0;
			
			serverCmdUnUseTool(%client);
			%client.sendStatus();
			commandToClient(%client, 'MessageBoxOK', "Excellent!", "You have been paid $" @ %price @ " in cash.\n\nPlease doing business.");
			messageClient(%client, 'MsgItemPickup', '', %obj.crpgSellItem, 0);
		}
	}
}

// Police
function serverCmdPolice_GetWantedList(%client)
{
	%profile = %client.profile;
	
	if(isObject(%profile))
	{
		// Wanted List
		commandToClient(%client, 'Police_ClearWanted');
		
		for(%a = 0; %a < ClientGroup.getCount(); %a++)
		{
			%subClient = ClientGroup.getObject(%a);
			
			if(isObject(%subProfile = %subClient.profile) && %subProfile.getWantedLevel())
				commandToClient(%client, 'Police_AddWanted', %subClient.name, %subProfile.demerits, %subClient.getID());
		}
		
		%comitted	= SystemSO.crimes;
		%unsolved	= SystemSO.unsolved;
		%solved		= SystemSO.solved;
		%clues		= CluesSS.getCount();
		
		commandToClient(%client, 'Police_SetStats', %comitted, %unsolved, %solved, %clues);
		
		// Vehicle Recovery
		for(%a = 0; %a < getFieldCount(%profile.lostv); %a++)
		{
			%vehicle = getField(%profile.lostv, %a);
			
			if(isObject(%vehicle))
			{
				%name = %vehicle.uiName;
				%price = SalesSO.getVehiclePrice(%vehicle) / 10;
				%price = mClamp(%price, 0, %price);
				
				%vnames = addField(%vnames, %name);
				%vprices = addField(%vprices, %price);
				%vids = addField(%vids, %vehicle.getID());
			}
		}
		
		commandToClient(%client, 'Police_SetRecovery', %vnames, %vprices, %vids);
	}
}

function serverCmdPolice_FindVehicle(%client, %id)
{
	%profile = %client.profile;
	
	if(isObject(%profile))
	{
		if(isObject(%id))
		{
			%id = %id.getID();
			%vlist = %profile.lostv;
			
			for(%a = 0; %a < getFieldCount(%vlist); %a++)
			{
				%vehicle = getField(%vlist, %a);
				
				if(isObject(%vehicle) && %vehicle.getID() == %id)
				{
					%price = SalesSO.getVehiclePrice(%vehicle) / 10;
					%price = mClamp(%price, 0, %price);
					
					if(%price <= %profile.money + %profile.bank)
					{
						commandToClient(%client, 'MessageBoxOK', "Success!", "The Police recovered your " @ %vehicle.uiName @ ".");
						commandToClient(%client, 'CRPGCashChange', -%price, %profile.money, %profile.bank, %profile.debt);
						serverCmdPolice_GetWantedList(%client);
						
						%profile.removeFunds(%price, true);
						%profile.garage = addField(%profile.garage, %vehicle.getName());
						%profile.lostv = removeField(%profile.lostv, %a);
						
						%client.sendStatus();
						
						return;
					}
					else
					{
						commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "It costs $" @ %price @ " to recover your " @ %vehicle.uiName @ ".\n\nGet more money.");
						return;
					}
				}
			}
			
			commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "Requested vehicle (" @ %id @ ") exists, but is either already in your property or not stolen.");
		}
		else
			commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "Requested vehicle (" @ %id @ ") does not exist.");
	}
}

//Campaign
function serverCmdCampaign_Upgrade(%client) {
	%profile = %client.profile;
	
	if(isObject(%profile)) {
		%required = %profile.getCampaignMoneyRequired();
		
		if(%required > %profile.money)
			commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "You don't have enough money to upgrade your campaign ($" @ %required @ ").");
		else {
			%profile.money -= %required;
			%profile.campaign += 1;
			
			%client.sendStatus();
			%client.sendCampaign();
		}
	}
}

// Stats
function serverCmdStats_ApplyForJob(%client, %id)
{
	if(isObject(%client.profile))
	{
		if(mFloatLength(%id, 0) $= %id && isObject(%job = JobsSO.getJobSO(%id)))
		{
			if(%job.isApplicable(%client))
			{
				%job.setJobFor(%client, true);
				%client.sendStatus();
				
				if(isObject(%client.player))
					%client.player.setShapeNameColor("1 1 1 1");
			}
			else
			{
				%reason = %job.getError(%client);
				commandToClient(%client, 'MessageBoxOK', getField(%reason, 0), getField(%reason, 1));
			}
		}
		else
			commandToClient(%client, 'MessageBoxOK', "Job does not Exist", "The requested Job (" @ %id @ ") does not exist.");
	}
	else
		commandToClient(%client, 'MessageBoxOK', "No Profile", "You have no profile.");
}

function serverCmdStats_ChangeBelief(%client, %id, %change)
{
	%profile = %client.profile;
	
	if(isObject(%profile))
	{
		%change = mClamp(%change, -1, 1);
		
		if(%change != 0)
		{
			if(!%profile.beliefChangeTime > 0)
			{
				switch(%id)
				{
					case 1: %belief = "Order";
					case 2: %belief = "Peace";
					case 3: %belief = "Commercial";
					case 4: %belief = "Night";
					case 5: %belief = "Tradition";
					default: return %client.sendMessageBox("Does Not Compute", "I don't know any belief by ID" SPC %id @ ".");
				}
				
				%newVal = %profile.belief[%belief] + %change;
				
				if(mClamp(%newVal, 1, 11) == %newVal)
				{
					%resetTimer = 168;
					%resetTimer -= (12 * %profile.getBeliefChange());
					
					%profile.belief[%belief] += %change;
					%profile.beliefChangeTime = %resetTimer;
					%client.sendStatus();
				}
				else
				{
					%client.sendMessageBox("Does Not Compute", "You cannot change this belief in this direction further.");
				}
			}
			else
			{
				%client.sendMessageBox("Does Not Compute", "You need to wait" SPC %profile.beliefChangeTime @ " more in-game hours.");
			}
		}
		else
		{
			%client.sendMessageBox("Does Not Compute", "Tried to change belief by factor 0.");
		}
	}
}

function serverCmdStats_ChangeGender(%client)
{
	%profile = %client.profile;
	
	if(isObject(%profile))
	{
		if(%profile.gender != 2)
		{
			%client.sendMessageBox("Gender Change", "You are now seen as Female.");
			%profile.gender = 2;
		}
		else
		{
			%client.sendMessageBox("Gender Change", "You are now seen as Male.");
			%profile.gender = 1;
		}
		
		%profile.getOutfit().applyOutfit(%client);
		
		serverCmdStats_GetOutfit(%client);
		%client.sendStatus();
	}
	else
		commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "You cant change your gender because you don't have a profile yet.");
}

function serverCmdStats_FavoriteColor(%client, %color)
{
	%profile = %client.profile;
	
	if(isObject(%profile))
	{
		%r = mClampF(getWord(%color, 0), 0, 255) / 255;
		%g = mClampF(getWord(%color, 1), 0, 255) / 255;
		%b = mClampF(getWord(%color, 2), 0, 255) / 255;
		%a = 1.0;
		
		%color = %r SPC %g SPC %b SPC %a;
		%profile.favoriteColor = %color;
		
		messageClient(%client, '', "\c6Favorite color set!");
	}
}

function serverCmdStats_GetOutfit(%client)
{
	%profile = %client.profile;
	
	if(isObject(%profile))
	{
		%outfit = %profile.getOutfit().getOutfitString(%profile);
		
		%parts = getLine(%outfit, 0);
		%decal = getLine(%outfit, 1);
		%nodes = getLine(%outfit, 2);
		%colors = getLine(%outfit, 3);
		
		commandToClient(%client, 'Stats_SetOutfit', %parts, %decal, %nodes, %colors);
	}
}

function serverCmdStats_GetJobs(%client)
{
	%profile = %client.profile;
	
	if(isObject(%profile))
	{
		%cJob = %profile.getJob();
		
		for(%a = 0; %a < JobsSG.getCount(); %a++)
		{
			%job = JobsSG.getObject(%a);
			%tier[%job.expTree] = mFloor(%tier[%job.expTree]);
			
			if((%job.isAvailable(%client) || (isObject(%cJob) && %job.getID() == %cJob.getID())) && %job.expTier > %tier[%job.expTree])
			{
				%tier[%job.expTree] = %job.expTier;
			}
		}
		
		// Jarbs
		for(%a = 0; %a < JobsSG.getCount(); %a++)
		{
			%job = JobsSG.getObject(%a);
			
			if(%job.id != %cJob.id && %job.isListed())
			{
				// Can we ever get the job?
				if(%profile.felon && %job.cleanRecord)
					%color = 4;
				// Is it a job we have surpassed?
				else if(%tier[%job.expTree] > %job.expTier)
					%color = 1;
				// Is it a job we can apply for right now?
				else if(%job.isApplicable(%client))
					%color = 6;
				// Is it a job we can theoretically apply for?
				else if(%job.isAvailable(%client))
					%color = 7;
				// Okay, color it black then.
				else
					%color = 0;
				
				%jcols = addField(%jcols, %color);
				%jname = addField(%jname, %job.name);
				%jcost = addField(%jcost, %job.cost);
				%jedu = addField(%jedu, %job.education);
				%jids = addField(%jids, %job.id);
			}
		}
		
		commandToClient(%client, 'Stats_SetJobs', %jcols, %jname, %jcost, %jedu, %jids);
	}
}

function serverCmdStats_UpdateJobOps(%client, %packet0, %packet1, %packet2, %packet3, %packet4, %packet5)
{
	%profile = %client.profile;
	if(!isObject(%profile))
		return;
	
	%job = %profile.getJob();
	if(!isObject(%job))
		return;
	
	if($sim::time - %client.lastJobOps < 10)
		return;
	
	%client.lastJobOps = $sim::time;
	
	for(%a = 0; %a <= 5; %a++)
		%data = %data @ %packet[%a];
	
	%updates = "";
	
	for(%a = 0; %a < getLineCount(%data); %a++)
	{
		%line			= getLine(%data, %a);
		%category	= getField(%line, 0);
		%opID			= getField(%line, 1);
		%parameters	= getField(%line, 2);
		
		if(%job.opName[%category, %opID] !$= "")
		{
			%success = %job.onJobOp(%client, %profile, %category, %opID, %parameters);
			
			echo(%category SPC %opID SPC %success);
			
			if(%success !$= "-1")
				%updates = %updates NL %success;
		}
	}
	
	%job.onJobOpEnd(%client, %profile, %updates);
	%client.sendStatus();
}

// Vehicles
function serverCmdVehicles_BuyVehicle(%client, %vehicle)
{
	if(SalesSO.isVehicleProduct(%vehicle))
	{
		%price = SalesSO.getVehiclePrice(%vehicle);
		
		%client.salesBrickProduct	= "vehicle";
		%client.salesBrickItem		= %vehicle.getID();
		%client.salesBrickMarkup	= mFloor(%price / 10);
		%client.salesBrickName		= trim(stripMLControlChars(trim(%vehicle.uiName)));
		%client.salesBrickPrice		= %price;
		
		serverCmdEvent_Buy(%client);
	}
	else
		commandToClient(%client, 'MessageBoxOK', "Does Not Compute", "Sorry!\n\nThis vehicle does not exist or is not for sale.");
}

// ============================================================
// Section 7 : ShapeBase
// ============================================================
// Adds all three vectors of %shape's velocity to determine absolute velocity.
// @int
function ShapeBase::getAbsVelocity(%shape)
{
	%vel = %shape.getVelocity();
	%velX = getWord(%vel, 0);
	%velY = getWord(%vel, 1);
	%velZ = getWord(%vel, 2);
	
	%absVel = mAbs(%velX) + mAbs(%velY) + mAbs(%velZ);
	
	return %absVel;
}

// ============================================================
// Section 8 : WeaponImage
// ============================================================
// A cone of uncertainty that is based on the player's conditioning.
// @vector3f
function WeaponImage::getAimVector(%this, %obj, %slot, %aimVec)
{
	if(%aimVec $= "")
		%aimVec	= %obj.getAimVector();
	
	%mount	= %obj.getObjectMount();
	%spread	= 0.00000;
	
	if(isObject(%mount))
		%spread += %mount.getAbsVelocity() * 0.00050;
	else
	{
		%velocity = %obj.getAbsVelocity();
		%spread += %velocity * 0.00050;
		
		if(!%velocity)
			%spread /= %obj.isCrouched() ? 3 : 1;
		
		if(!%obj.isCrouched())
			%spread += 0.00050;
			
	}
	
	%spread += 0.00250 - (0.00250 * (%obj.getEnergyLevel() / %obj.getDatablock().maxEnergy));
	
	// Clamp the spread so it doesn't look ridiculous.
	%spread = mClampF(%spread, 0, 0.0125);
	%spread = mFloatLength(%spread, 5);

	%x = (getRandom() - 0.5) * 10 * $pi * %spread;
	%y = (getRandom() - 0.5) * 10 * $pi * %spread;
	%z = (getRandom() - 0.5) * 10 * $pi * %spread;
	%mat = MatrixCreateFromEuler(%x SPC %y SPC %z);
	%vector = VectorScale(%aimVec, 100);
	%aimVec = vectorNormalize(MatrixMulVector(%mat, %vector));
	
	return %aimVec;
}

// The shotvec is a cone of uncertainty that circles the aim vector.
// In other words, the shotvec is based on the weapon, the aimVec is
// based on the player.
// @vector3f
function WeaponImage::getShotVector(%this, %obj, %slot, %aimVec)
{
	if(%aimVec $= "")
		%aimVec = %obj.getAimVector();
	
	if(%this.raycastSpreadAmt > 0)
	{
		%spread = %this.raycastSpreadAmt;
		
		%x = (getRandom() - 0.5) * 10 * $pi * %spread;
		%y = (getRandom() - 0.5) * 10 * $pi * %spread;
		%z = (getRandom() - 0.5) * 10 * $pi * %spread;
		%mat = MatrixCreateFromEuler(%x SPC %y SPC %z);
		%vector = VectorScale(%aimVec, 100);
		%shotVec = vectorNormalize(MatrixMulVector(%mat, %vector));
	}
	else
	{
		%shotVec = %aimVec;
	}
	
	return %shotVec;
}

// Function called in WeaponImage::onFire to determine if a particular shot
// is critical.
// @bool (false unless overwritten)
function WeaponImage::isRaycastCritical(%this, %obj, %slot, %col, %pos, %normal)
{
	return false;
}

// Determines if a raycast hit is a headshot.
// No use on its own, but most weapons use it in isRaycastCritical.
// @bool
function WeaponImage::isRaycastHeadshot(%this, %obj, %slot, %col, %pos, %normal)
{
	if(fileName(%col.dataBlock.shapeFile) $= "m.dts")
	{
		%scaleZ = getWord(%col.getScale(), 2);
		
		if(getWord(%pos, 2) > getWord(%col.getWorldBoxCenter(), 2) - (3.3 * %scaleZ))
		{
			return true;
		}
	}
	
	return false;
}

// Checks if %obj has ammo for %this.ammo.
// @bool
function WeaponImage::onAmmoCheck(%this, %obj, %slot)
{
	%obj.setImageAmmo(%slot, (%obj.ammo[%this.ammo] > 0 ? true : false));
}

// Spawns explosions and noises.
// @null
function WeaponImage::onHitObject(%this, %obj, %slot, %col, %pos, %normal, %shotVec, %crit)
{
	%colType = %col.getType();
	%uniqueSound = false;
	
	if(%colType & ($TypeMasks::PlayerObjectType | $TypeMasks::VehicleObjectType))
	{
		if(isObject(%this.raycastExplosionPlayerSound) && %colType & $TypeMasks::PlayerObjectType)
		{
			serverPlay3d(%this.raycastExplosionPlayerSound, %pos);
			%uniqueSound = true;
		}
		
		if(%this.raycastDirectDamage > 0 && miniGameCanDamage(%obj, %col) == 1)
			%this.onRaycastDamage(%obj, %slot, %col, %pos, %normal, %shotVec, %crit);
	}
	else if(isObject(%this.raycastExplosionBrickSound) && %colType & $TypeMasks::FxBrickAlwaysObjectType)
	{
		serverPlay3d(%this.raycastExplosionBrickSound, %pos);
		%uniqueSound = true;
		
		%this.onRaycastDamageBrick(%obj, %slot, %col, %pos, %normal, %shotVec, %crit);
	}
	
	if(!%uniqueSound && isObject(%this.raycastExplosionSound))
		serverPlay3d(%this.raycastExplosionSound, %pos);
	
	// Spawn Explosion
	if(isObject(%this.raycastExplosionProjectile))
	{
		%p = new Projectile()
		{
			dataBlock = %this.raycastExplosionProjectile;
			initialPosition = %pos;
			initialVelocity = %normal;
			sourceObject = %obj;
			client = %obj.client;
			sourceSlot = 0;
			originPoint = %pos;
		};
		
		%p.setScale(%obj.getScale());
		MissionCleanup.add(%p);
		%p.explode();
	}
}

// Changes the mode of %this.
// @int (new mode ID)
function WeaponImage::onModeSwitch(%this, %obj, %slot)
{
	%original = mFloor(%obj.weaponMode[%this.getID()]);
	%obj.weaponMode[%this.getID()] = mFloor(%obj.weaponMode[%this.getID()]) + 1;
	
	if(%obj.weaponMode[%this.getID()] > mFloor(%this.numModes) - 1)
		%obj.weaponMode[%this.getID()] = 0;
	
	if(%original !$= %obj.weaponMode[%this.getID()])
	{
		%obj.playThread(2, "shiftRight");
		
		if(isObject(%client = %obj.client))
		{
			%mode = %this.mode[%obj.weaponMode[%this.getID()]];
			
			if(%mode $= "")
				%mode = "Unknown Mode";
			
			if(%obj.weaponMode[%this.getID()] == 0)
				%obj.weaponMode[%this.getID()] = "";
			
			if(%original !$= %obj.weaponMode[%this.getID()])
				commandToClient(%client, 'CenterPrint', "<just:right>\c6" @ %mode, 1);
		}
	}
	
	return %obj.weaponMode[%this.getID()];
}

// Applies damage to %col. Applies impulse. Removes head on %crit.
// @int (damage dealt)
function WeaponImage::onRaycastDamage(%this, %obj, %slot, %col, %pos, %normal, %shotVec, %crit)
{
	// Damage Type
	if(!%crit || %this.raycastCritDirectDamageType $= "")
		if(%this.raycastDirectDamageType !$= "")
			%damageType = %this.raycastDirectDamageType;
		else
			%damageType = $DamageType::Direct;
	else
		%damageType = %this.raycastCritDirectDamageType;
	
	// Damage Amount
	%directDamage = %this.raycastDirectDamage * getWord(%obj.getScale(), 2);
	
	// Damage Range Falloff
	if(%this.raycastWeaponOptimalRange > 0)
	{
		%dist = vectorDist(%obj.getMuzzlePoint(0), %pos) - %this.raycastWeaponOptimalRange;
		
		if(%dist > 0)
		{
			%directDamage = %directDamage * (%dist / (%this.raycastWeaponRange - %this.raycastWeaponOptimalRange));
		}
	}
	
	if(%directDamage <= 0)
		return 0;
	
	// Player Hook
	if(%col.getType() & $TypeMasks::PlayerObjectType)
	{
		%noCrits	= false;
		%damMod	= 1;
		
		if(isObject(%profile	= %col.getProfile()))
		{
			%noCrits	= %profile.isHeadProtected();
			%damMod	-= %profile.getArmor();
		}
		
		if(isObject(%obj) && isObject(%oprofile = %obj.getProfile()))
		{
			%damMod	+= (%oprofile.getBeliefWar() * 0.02);
		}
		
		// Headshot Head Removal
		if(%crit && !%noCrits)
		{
			%directDamage *= 4;
			
			%hideStr = "headskin";
			
			for(%i = 0; $hat[%i] !$= ""; %i++)
				%hideStr = %hideStr TAB $hat[%i];
			
			for(%i = 0; $accent[%i] !$= ""; %i++)
				%hideStr = %hideStr TAB $accent[%i];
			
			for(%i = 0; %i < getFieldCount(%hideStr); %i++)
			{
				%node = getField(%hideStr, %i);
				%col.hideNode(%node);
			}
		}
		
		// Impulses
		if(%this.raycastImpactImpulse > 0)
			%col.applyImpulse(%pos, vectorScale(vectorScale(%shotVec, %this.raycastImpactImpulse), %damMod));
		
		if(%this.raycastVerticalImpulse > 0)
			%col.applyImpulse(%pos, vectorScale(vectorScale("0 0 1", %this.raycastVerticalImpulse)), %damMod);
		
		// Damage Application
		%col.damage(%obj, %pos, %directDamage * %damMod, %damageType);
	}
	// Vehicle Hook
	else if(%col.getType() & $TypeMasks::VehicleObjectType)
	{
		if(%crit)
		{
			%directDamage *= 4;
		}
		
		// Impulses
		if(%this.raycastImpactImpulse > 0)
			%col.applyImpulse(%pos, vectorScale(%shotVec, %this.raycastImpactImpulse));
		
		if(%this.raycastVerticalImpulse > 0)
			%col.applyImpulse(%pos, vectorScale("0 0 1", %this.raycastVerticalImpulse));
		
		// Damage Application
		%col.damage(%obj, %pos, %directDamage, %damageType);
	}
	
	return %directDamage;
}

// A hack for applying damage to an fxDTSBrick object over time.
// @null
function WeaponImage::onRaycastDamageBrick(%this, %obj, %slot, %col, %pos, %normal, %shotVec, %crit)
{
	if(%col.getType() & $TypeMasks::FxBrickAlwaysObjectType)
	{
		// Damage
		%directDamage = %this.raycastDirectDamage * getWord(%obj.getScale(), 2);
		
		if(%directDamage > 0)
		{
			%volume = %col.getDatablock().getVolume();
			%health = %voume * getWord(getColorIDTable(%col.getColorID()), 3);
			
			if(%health > 250)
			{
				if($sim::time - %col.crpgRaycastDamageTime > 3)
					%col.crpgRaycastDamage = 0;
				
				%col.crpgRaycastDamageTime = $sim::time;
				%col.crpgRaycastDamage += (%directDamage - mFloor(%volume / 10));
				%col.crpgRaycastDamage = mClamp(%col.crpgRaycastDamage, 0, 270);
				
				if(%health < %col.crpgRaycastDamage)
				{	
					%time = mClamp(%client.minigame.brickRespawnTime, 0, %client.minigame.brickRespawnTime);
					%col.fakeKillBrick(%obj.getAimVector(), mClamp(%volume / 10, 1, %volume), %time, %obj.client);
					%col.crpgRaycastDamage = 0;
				}
			}
		}
	}
}

// Refuses %obj's ammo for %this.ammo based on %this.ammoMagSize
// @bool (success of reload)
function WeaponImage::onReload(%this, %obj, %slot)
{
	%obj.setImageAmmo(%slot, 1);
	%obj.ammo[%this.ammo] = %this.ammoMagSize;
	
	if(isObject(%obj.client))
		%obj.client.updateAmmo();
	
	return true;
}

// Empties %obj's magazine for %this.ammo because we are changing it.
// @int (amount of ammo left; always 0 unless overwritten)
function WeaponImage::onUnload(%this, %obj, %slot)
{
	%obj.ammo[%this.ammo] = 0;
	return %obj.ammo[%this.ammo];
}
