// ============================================================
// File Name		:	Package
// Description		:	Default Blockland functions that have had been modified.
// ============================================================
// Table of Contents
// 
// 1. Namespaceless
// 2. Armor
// 3. fxDTSBrick
// 4. GameConnection
// 5. Observer
// 6. Player
// 7. serverCmd
// 8. Misc
// ============================================================
package CRPG_Package_Main
{
	// ============================================================
	// Section 1 : Namespaceless
	// ============================================================
	// Deletes script objects.
	// @null
	function destroyServer()
	{
		destroyCityRPG();
		
		parent::destroyServer();
	}
	
	// Hacks all items into minigame.
	// @MinigameSO (CRPGMinigame)
	function getMiniGameFromObject(%obj)
	{
		return CRPGMinigame;
	}
	
	// Forges trust between server owner and mayor.
	// @int
	function getTrustLevel(%obj1, %obj2)
	{
		%trust = parent::getTrustLevel(%obj1, %obj2);
		
		if(%trust != 0 || !isObject(%obj1) || !isObject(%obj2))
			return %trust;
		
		%bg1 = getBrickGroupFromObject(%obj1);
		%bg2 = getBrickGroupFromObject(%obj2);
		
		if(!isObject(%bg1) || !isObject(%bg2))
			return %trust;
		
		%profile1 = Profiler.fetchProfile(%bg1.bl_id);
		%profile2 = Profiler.fetchProfile(%bg2.bl_id);
		
		if(!isObject(%profile1) || isObject(%profile2))
			return %trust;
		
		%host = 0;
		%lan = 0;
		%mayor = 0;
		
		if(%profile1.isMayor())
		{
			if(%profile2.isHost())
			{
				%mayor = 1;
				%host = 2;
			}
			else if(%profile2.isLocal())
			{
				%mayor = 1;
				%lan = 2;
			}
		}
		else if(%profile2.isMayor())
		{
			if(%profile1.bl_id == 999999)
			{
				%mayor = 2;
				%host = 1;
			}
			else if(%profile1.bl_id == 999999)
			{
				%mayor = 2;
				%lan = 1;
			}
		}
		
		if(%mayor)
		{
			%cname1 = %obj1.getClassName();
			%cname2 = %obj2.getClassName();
			
			if(%host)
			{
				if(%cname[%host] $= "fxDTSBrick")
				{
					%brick = %obj[%host];
					%lot = %brick.getLot();
					
					if(isObject(%lot))
					{
						%trust = (%lot.isInfastructure ? 1 : 0);
					}
					else
					{
						%trust = 1;
					}
				}
			}
			
			else if(%lan)
			{
				if(%cname[%lan] $= "fxDTSBrick")
				{
					%brick = %obj[%lan];
					%lot = %brick.getLot();
					
					if(isObject(%lot))
					{
						%trust = (%lot.isInfastructure ? 1 : 0);
					}
					else
					{
						%trust = 1;
					}
				}
			}
		}
		
		return %trust;
	}
	
	// Crime Enabled Code
	// @bool
	function miniGameCanDamage(%attackingObject, %victimObject)
	{
		%obj[1] = %attackingObject;
		%obj[2] = %victimObject;
		
		%result = parent::miniGameCanDamage(%obj[1], %obj[2]);
		
		if(%result)
		{
			// Are we even trying?
			if(isObject(%obj[1]) && isObject(%obj[2]))
			{
				// Loop through both objects to get the source objects.
				for(%i = 1; isObject(%obj[%i]); %i++)
				{
					// Don't assign this an [%i] to save memory.
					%className = %obj[%i].getClassName();
					
					// Get out client based on object type.
					switch$(%className)
					{
						case "AIPlayer":			%player[%i] = %obj[%i];
						case "FlyingVehicle":	%client[%i] = %obj[%i].getControllingClient();
						case "GameConnection":	%client[%i] = %obj[%i];
						case "Player":				%player[%i] = %obj[%i]; %client[%i] = %obj[%i].client;
						case "Projectile":		%client[%i] = %obj[%i].client;
						case "WheeledVehicle":	%client[%i] = %obj[%i].getControllingClient();
					}
					
					// No matter where we got the client, the player is always just a property away.
					if(isObject(%client[%i].player))
						%player[%i] = %client[%i].player;
					
					if(isObject(%client[%i].profile))
						%profile[%i] = %client[%i].profile;
				}
				
				if(isObject(%player[1]) && isObject(%player[2]) && (%player[1].isZombie || %player[2].isZombie) && %player[1].isZombie != %player[2].isZombie)
					return true;
				
				if(isObject(%client[1]) && isObject(%client[2]) && isObject(%profile[1]) && isObject(%profile[2]) && %profile[1].getID() != %profile[2].getID())
				{
					if(%profile[1].isInmate() || %profile[2].isInmate())
						return false;
					
					if(%player[1].crpgArrested || %player[2].crpgArrested)
						return false;
					
					if($CRPG::Peace)
						return false;
					
					if(%profile[1].getWantedLevel() || %profile[2].getWantedLevel()) // If wanted, allow shooting.
						return true;
					
					if(%client[1].crpgEvil)
						return true;

					if(%player[2].AssaultedAnyone || %player[2].Assaulted[%player[1]]) // Self defense OR vigilante. See Armor::Damage
						return true;
					
					return false;
				}
			}
		}
		
		return (%result ? !$CRPG::Peace : false);
	}
	
	// I'm so sorry.
	// @null
	function messageClient(%client, %type, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10, %a11)
	{
		if(%type !$= 'MsgClientInYourMiniGame')
			return parent::messageClient(%client, %type, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10, %a11);
	}
	
	// Summon the people's attention.
	// @null
	function talk(%message)
	{
		%name = "CONSOLE";
		messageAll('MsgAdminForce', '\c5%1\c6: %2', %name, %message);
	}
	
	// ============================================================
	// Section 2 : Armor
	// ============================================================
	// Picks up CashItem and grants money.
	// @obj
	function Armor::onCollision(%this, %obj, %col, %vec, %speed)
	{
		if(isObject(%col))
		{
			%client = %obj.client;
			%profile = %client.profile;
			
			if(isObject(%client) && isObject(%profile))
			{
				if((%col.getType() & $TypeMasks::ItemObjectType))
				{
					// Prevents n00bs from stealing your plastic within 30 seconds of it being dropped.
					if(!isObject(%col.worker) || !isObject(%col.worker.player) || %col.worker.getID() == %client.getID() || $sim::time - %col.spawnTime > 30)
					{
						// Money Pick-Ups
						if(%col.cashValue !$= "")
						{				
							// Money cannot get picked up by its owner for 10 seconds if stolen.
							if(getSimTime() - %col.timeDropped < 10000 && %col.owner == %client.getBLID())
									return;
							if(%col.owner !$= "" && ((%col.owner == %col.stolenOwner && %col.stolenOwner != %client.getBLID()) || (%col.owner != %client.getBLID() && %col.stolenOwner != %client.getBLID())))
								%stolen = true;

							if(%stolen && %client.crpgEvil)
							{
								if(mFloor(%col.cashValue) > 0)
								{
									%profile.money += mFloor(%col.cashValue);
									commandToClient(%client, 'CRPGCashChange', %col.cashValue, %profile.money, %profile.bank, %profile.debt);
									%client.play3D("errorSound", %col.getWorldBoxCenter());
									
									// Log the stolen money and its owner.
									%client.logStolenMoney(%col.stolenOwner, %col.cashValue);
									%client.commitCrime("Stealing Money", mFloor(%col.cashValue / 3) , false, true, findClientByBL_ID(%col.stolenOwner).player);
								}
							}
							else if(!%stolen)
							{
								if(mFloor(%col.cashValue) > 0)
								{
									%profile.money += mFloor(%col.cashValue);
									commandToClient(%client, 'CRPGCashChange', %col.cashValue, %profile.money, %profile.bank, %profile.debt);
									%client.play3D("errorSound", %col.getWorldBoxCenter());
								}
							}
							else
								return;
							%col.delete();
							return %obj;
						}
						
						// Plastic Pick-Ups
						if(%col.plasticValue !$= "")
						{
							if(mFloor(%col.plasticValue) > 0)
							{
								%profile.plastic += mFloor(%col.plasticValue);
								%client.massPlasticChange += %col.plasticValue;
								
								cancel(%client.sendMassUpdateSched);
								%client.sendMassUpdateSched = %client.schedule(500, "sendMassUpdate");
								%client.play3D("errorSound", %col.getWorldBoxCenter());
							}
							
							%col.delete();
							
							return %obj;
						}
						
						// Clue Pick-Ups
						if(%col.isClue && !%col.clueHidden)
						{
							if(%profile.getJob().police && %col.clueBLID !$= %client.bl_id)
							{
								%message = %col.clueMessage;
								
								if((isObject(%perp = %col.cluePerp) || isObject(%perp = findClientByBL_ID(%col.clueBLID))) &&
									(isObject(%perpProfile = Profiler.getProfile(%perp.bl_id))))
								{
									%perpProfile.setDemerits(%perpProfile.demerits + %col.clueDems);
									messageClient(%perp, '', "\c6A cop discovered evidence pinning you with a crime! [\c3" @ %message @ "\c6]");
									messageClient(%client, '', "\c6You have pinned \c3" @ %perp.name @ "\c6 with a crime! [\c3" @ %message @ "\c6]");
									
									SystemSO.unsolved--;
									SystemSO.solved++;
								}
								else if(isObject(%perpProfile = Profiler.getProfile(%col.clueBLID)))
								{
									%perpProfile.setDemerits(%perpProfile.demerits + %col.clueDems);
									messageClient(%client, '', "\c6You have pinned \c3" @ %perpProfile.name @ "\c6 with a crime! [\c3" @ %message @ "\c6]");
									
									SystemSO.unsolved--;
									SystemSO.solved++;
								}
								else
								{
									messageClient(%client, '', "\c6Sadly, this evidence leads to a dead end. It's a cold case!");
								}
								
								%col.delete();
							}
							
							return %obj;
						}
						
						if(%profile.isInmate())
							return;
						
						// Crate Pick-Ups
						if(%col.isBig)
						{
							%image = %col.getDatablock().image;
							%mounted = %obj.getMountedImage(0);
							
							if(isObject(%image) && !isObject(%mounted) && $sim::time - %col.dropTime >= 0.5)
							{
								if((%col.crpgOwner == %client.bl_id || %col.crpgMission.profile.bl_id == %client.bl_id) || (%client.crpgEvil && !isObject(%col.crpgMission)))
								{
									if(%col.isCrate)
									{
										%obj.crpgCrateOwner			= %col.crpgOwner;
										%obj.crpgCrateRefPlastic	= %col.crpgRefPlastic;
										%obj.crpgCratePlastic		= %col.crpgPlastic;
										%obj.crpgCrateWorth			= %col.crpgWorth;
										%obj.crpgCrateMission		= %col.crpgMission;
									}
									
									%obj.mountImage(%image, 0);
									%col.delete();
								}
								else if(%col.crpgMission != -1 && !%col.crpgMission.isMission)
								{
									%col.delete();
									echo("Mission crate of non-instantiated mission destroyed.");
								}
							}
							
							return %obj;
						}
					}
				}
				else if(%col.getType() & $TypeMasks::VehicleObjectType)
				{
					return %obj;
				}
			}
			
		}
		
		if(%col.doReturn)
			return %obj;
		
		return parent::onCollision(%this, %obj, %col, %vec, %speed);
	}
	
	// @obj
	// function Armor::Damage(%this, %obj, %source, %pos, %damage, %damageType)
	// {
		// if(isObject(%obj) && %obj.getType() & $TypeMasks::PlayerObjectType)
		// {
			// %oc = %obj.client;
			// %sc = isObject(%source.client) ? %source.client : %source;
			
			// if(isObject(%oc) && isObject(%sc))
			// {
				// %atk = %sc.player;
				
				// if(isObject(%atk) && %atk.getType() & $TypeMasks::PlayerObjectType)
				// {
					// %op = %oc.profile;
					// %sp = %sc.profile;
					

					// if(isObject(%op) && isObject(%sp) && %op.getID() != %sp.getID())
					// {
						// if($sim::time - %obj.assaultedAnyone < 60 || $sim::time - %obj.assaulted[%atk.getID()] < 120)
							// %selfDefense = true;
						// Used for assault.					
						// %obj has demerits and %source is a police OR %op has lost right to live...
						// if(!%op.getWantedLevel())// || !%sp.getJob().police) && %op.getWantedLevel() != 6)
						// {
							// ***** OLD AND BROKEN WITNESS SYSTEM *******
							// Self defense, defense of another, or this is just one instance of assault?
							// if($sim::time - %obj.assaultedAnyone > 60 && $sim::time - %obj.assaulted[%atk.getID()] > 120 && $sim::time - %atk.assaulted[%obj.getID()] > 120)
							// if($sim::time - %obj.assaultedAnyone > 60 && $sim::time - %obj.assaulted[%atk.getID()] > 120)
							// {
								// This is an attack, regardless of lethality. Keep track of it.
								// %atk.assaultedAnyone				= $sim::time;
								// %atk.assaulted[%obj.getID()]	= $sim::time;
								
								// If non-lethal and last guilty crime of assault was more than 60 seconds ago...
								// if(($sim::time - %atk.guiltyAssault[%obj.getID()] > 60) && %obj.getDamageLevel() + %damage < %obj.getDatablock().maxDamage)
								// {
									// %atk.guiltyAssault[%obj.getID()] = %sim::time;
									// if(!%op.getJob().police)
										// %sc.commitCrime("Assault with a Deadly Weapon", 25, false, true, %obj);
									// else
										// %sc.commitCrime("Assaulting an Officer", 50, true);
								// }
							// }
							
							// Self defense, defense of another, or this is just one instance of assault?
							// if($sim::time - %obj.assaultedAnyone > 60 && $sim::time - %obj.assaulted[%atk.getID()] > 120 && $sim::time - %atk.assaulted[%obj.getID()] > 120)

							
							// if(!%selfDefense)
							// {
								// This is an attack, regardless of lethality. Keep track of it.
								// %atk.assaultedAnyone				= $sim::time;
								// %atk.assaulted[%obj.getID()]	= $sim::time;
								// If non-lethal . . .
								// if(%obj.getDamageLevel() + %damage < %obj.getDatablock().maxDamage)
								// {
									// if(!%op.getJob().police)
										// %sc.commitCrime("Assault with a Deadly Weapon", 25, false, true, %obj);
									// else
										// %sc.commitCrime("Assaulting an Officer", 50, false, true);
									
									// Drop money.
									// %min = 50; //percentage multiplier for drop (minimum)
									// %max = 100; //percentage multiplier for drop (maximum)
									// %scale = %damage/100; //ratio
									// %amount = getRandom(%scale*%min,%scale*%max); //set a random amount between the scaled min and max
									// %obj.dropMoney(%amount);
								// }							
							// }
						// }
						// %sourceBL_ID = %sc.getBLID();
						// %needle = "|" @ %sourceBL_ID @ "|";
						// If victim stole money from shooter, drop all the stolen money possible.
						// When stolen money is at 0, remove the field from stolenMoney.
						// if(%selfDefense && strstr(%op.stolenMoney, %needle) >= 0 && %op.money > 0)
						// {						
							// %fieldCount = getFieldCount(%op.stolenMoney);
							// for(%index = 0; %index < %fieldCount; %index++)
							// {
								// %victimBL_ID = strReplace(getField(%op.stolenMoney, %index), "|", "");
								// %amountStolen = getField(%op.stolenMoney, %index + 1);
								
								// if(%victimBL_ID == %sourceBL_ID)
								// {
									// talk("removing some stolen money logs");
									// %obj.dropMoney(%amountStolen, %sc); 
									// if(%amountStolen > %op.money)
									// {
										// %amountDropped = %op.money;
										// %amountLeft = %amountStolen - %amountDropped;
										// %op.stolenMoney = setField(%op.stolenMoney, %index + 1, %amountLeft);
									// }
									// else 
									// {										
										// Removes the record as all the stolen money has been dropped.
										// %op.stolenMoney = removeField(%op.stolenMoney, %index + 1);
										// %op.stolenMoney = removeField(%op.stolenMoney, %index);
									// }								
									// break;
								// }
								// %index++;
							// }
														
						// }
					// }
				// }
			// }
		// }

		// parent::Damage(%this, %obj, %source, %pos, %damage, %damageType);
		
		// return %obj;
	// }

	function Armor::Damage(%data, %victim, %hitter, %pos, %damage, %damageType)
	{
			//Check for Player and Source
			%victimCl = %victim.client; //victim's client
			%isHitter = isObject(%hitter) && %hitter.getType() & $TypeMasks::PlayerObjectType; //hitter still exists, and is a player object
			if(!isObject(%victimCl) || !%isHitter) //if player's client DNE or source DNE
					return %parent; //Parent, and stop here

			%hitterCl = isObject(%hitter.client) ? %hitter.client : %hitter; //if source's client DNE, use source itself (e.g. bot)
			%vProfile = %victimCl.profile; //player's profile
			%hProfile = %hitterCl.profile; //attacker's profile

			if(!isObject(%vProfile) || !isObject(%hProfile) || %vProfile.getID() == %hProfile.getID()) //if victim profile DNE, or hitter profile DNE, or both profiles are the same player
					return %parent; //Parent, and stop here

			%selfDef = %victim.Assaulted[%hitter]; //if Victim recently assaulted the Hitter
			%vigilant = %victim.AssaultedAnyone; //if Victim recently assaulted someone else

			//Log the attack
			if(!%vigilant || !%selfDef) //if Hitter is not a vigilante or defending itself
			{
					%genTime = 60000; //time window for vigilantism
					%hitter.AssaultedAnyone = true; //note that Hitter assaulted someone
					schedule(%genTime,0,eval,%hitter @ ".AssaultedAnyone = false;"); //reset note

					%selfTime = 120000; //time window for self-defense
					%hitter.Assaulted[%victim] = true; //note that Hitter assaulted the Victim
					schedule(%selfTime,0,eval,%hitter @ ".Assaulted\[" @ %victim @ "\] = false;"); //reset note
					
					%record = true; //add to Hitter's wanted record
			}

			// Used for assault.                                        
			// %victim has no demerits, presumed assault
			if(!%vProfile.getWantedLevel() && %record)// || !%hProfile.getJob().police) && %vProfile.getWantedLevel() != 6)
			{
					// This is an attack, regardless of lethality. Keep track of it.
					// If non-lethal . . .
					if(%victim.getDamageLevel() + %damage < %victim.getDatablock().maxDamage)
					{
							if(!%vProfile.getJob().police)
									%hitterCl.commitCrime("Assault with a Deadly Weapon", 25, false, true, %victim);
							else
									%hitterCl.commitCrime("Assaulting an Officer", 50, false, true);

							// Drop money.
							%min = 50; //percentage multiplier for drop (minimum)
							%max = 100; //percentage multiplier for drop (maximum)
							%scale = %damage/100; //ratio
							%amount = getRandom(%scale*%min,%scale*%max); //set a random amount between the scaled min and max
							%victim.dropMoney(%amount); //drop the money
					}
			}

			//If victim stole money from someone, drop all the stolen money possible.
			%hitterBL_ID = %hitterCl.getBLID();
			%needle = "|" @ %hitterBL_ID @ "|";

			//When stolen money is at 0, remove the field from stolenMoney.
			if((%vigilant || %selfDef) && strStr(%vProfile.stolenMoney, %needle) >= 0 && %vProfile.money > 0) //if Hitter is a vigilante, and Victim has stolen money
			{                                                
					%fieldCount = getFieldCount(%vProfile.stolenMoney);
					for(%index = 0; %index < %fieldCount; %index++)
					{
							%victimBL_ID = strReplace(getField(%vProfile.stolenMoney, %index), "|", "");
							%amountStolen = getField(%vProfile.stolenMoney, %index + 1);

							if(%victimBL_ID == %hitterBL_ID)
							{
							//        talk("removing some stolen money logs");
									%victim.dropMoney(%amountStolen, %hitterCl); 
									if(%amountStolen > %vProfile.money)
									{
											%amountDropped = %vProfile.money;
											%amountLeft = %amountStolen - %amountDropped;
											%vProfile.stolenMoney = setField(%vProfile.stolenMoney, %index + 1, %amountLeft);
									}
									else 
									{                                                                                
											//Removes the record as all the stolen money has been dropped.
											%vProfile.stolenMoney = removeField(%vProfile.stolenMoney, %index + 1);
											%vProfile.stolenMoney = removeField(%vProfile.stolenMoney, %index);
									}                                                              
									break;
							}
							%index++;
					}
			}
			if((%vigilant || %selfDef) && !%hitterCL.crpgEvil && %victim.getDamageLevel() + %damage > %victim.getDatablock().maxDamage)
			{
				if(%victim.getDatablock().maxDamage - %victim.getDamageLevel() > 1)
					%damage = %victim.getDatablock().maxDamage - %victim.getDamageLevel() - 1;
				else
					%damage = 0;
			}
			return Parent::Damage(%data, %victim, %hitter, %pos, %damage, %damageType); //Return the Parent
	}
	
	// Change Datablocks
	// @null
	function Armor::onDamage(%this, %obj, %damage)
	{
		%result = parent::onDamage(%this, %obj, %damage);
		
		if(%obj.getState() !$= "Dead" && !%obj.isZombie && isObject(%obj.client))
		{
			%health = 100 - (100 * %obj.getDamagePercent());
			
			if(%health >= 50)
				%db = CRPG_Civilian_Armor;
			else if(%health >= 25)
				%db = CRPG_Civilian_Mid_Armor;
			else
				%db = CRPG_Civilian_Low_Armor;
			
			if(%db.getID() != %this.getID())
				%obj.setDatablocK(%db);
			
			if(%health != 100)
			{
				cancel(%obj.healSched);
				%obj.healSched = %obj.schedule(5000, "onAutoHeal", 1);
			}
		}		
		return %result;	
	}
	
	// Removes backpack.
	// @obj
   function Armor::onMount(%this, %obj, %col, %slot)
   {
      %result = parent::onMount(%this, %obj, %col, %slot);
		%obj.setBackImage();
		return %result;
   }
	
	// Reduces ninja fall damage.
	// @null
	function Armor::onImpact(%this, %obj, %col, %vel, %z)
	{
		%client = %obj.client;
		%profile = %client.profile;
		%job = isObject(%profile) ? %profile.getJob() : "";
		
		if(%job.ninja)
			%vel /= 1.5;
		
		return parent::onImpact(%this, %obj, %col, %vel, %z);
	}

	// Used for switching modes.
	// @obj
	function Armor::onTrigger(%this, %obj, %trigger, %state)
	{
		%client = %obj.client;
		%profile = %client.profile;
		%job = isObject(%profile) ? %profile.getJob() : "";
		
		// We only want to do stuff when they press the button.
		if(%state)
		{
			// Do different stuff based on what is pressed.
			switch(%trigger)
			{
				// Fire
				case 0:
					return parent::onTrigger(%this, %obj, %trigger, %state);
				
				// Jump
				case 2:
					return parent::onTrigger(%this, %obj, %trigger, %state);
				
				// Crouch
				case 3:
					return parent::onTrigger(%this, %obj, %trigger, %state);
				
				// Jet
				case 4:
					if(!isObject(%obj.getObjectMount()) && isObject(%image = %obj.getMountedImage(0)))
						%image.onModeSwitch(%obj, 0);
					
					return parent::onTrigger(%this, %obj, %trigger, %state);
				
				// Anything else.
				default:
					return parent::onTrigger(%this, %obj, %trigger, %state);
			}
		}
		
		return parent::onTrigger(%this, %obj, %trigger, %state);
	}
	
	// Re-Adds backpack.
	// @obj
   function Armor::onUnMount(%this, %obj, %col, %slot)
   {
      %result = parent::onUnMount(%this, %obj, %col, %slot);
		%obj.setBackImage();
		return %result;
   }
	
	
	// ============================================================
	// Section 3 : fxDTSBrick
	// ============================================================
	// Deletes the brick's trigger.
	// @null
	function fxDTSBrick::onDeath(%this, %brick)
	{
		if(isObject(%brick.trigger))
			%brick.trigger.delete();
		
		%group = %brick.getGroup();
		%db = %brick.getDataBlock();

		if(isObject(%group))
		{
			%profile = Profiler.getProfile(%group.bl_id);
			%client = %group.client;

			if(isObject(%profile) && (!isObject(%client) || !%client.isCRPGAdmin()))
			{
				if(!%brick.plasticMade)
				{
                                        if (strstr(%db, "Monopoly") >= 1)
                                        {
                                        	%taxes = mFloor(%this.getPrice() * (SystemSO.taxMonopoly / 100));
                                        }
                                        else
                                        {
                                        	%taxes = mFloor(%this.getPrice() * (SystemSO.taxLots / 100));
                                        }

					%refund = (%this.getPrice() - %taxes);
					echo(%refund);
					%profile.money += %refund;
					%client.massCashChange += %refund;
				}
				else
				{
					%profile.refplastic += %this.getPrice();
					//%client.massPlasticChange += %this.getPrice();
				}
				
				if(isObject(%client))
				{
					cancel(%client.sendMassUpdateSched);
					%client.sendMassUpdateSched = %client.schedule(32, "sendMassUpdate");
				}
			}
		}
		
		return parent::onDeath(%brick);
	}
	
	// Creates the brick's trigger.
	// @null
	function fxDTSBrick::onPlant(%this, %brick)
	{
		parent::onPlant(%this, %brick);
		
		if(isObject(%brick) && %brick.isPlanted())
		{
			// Trigger Field Rendering
			%db = %brick.getDatablock();
			
			if(isObject(%db.triggerDatablock))
			{
				if(isObject(%brick.trigger))
					%brick.trigger.delete();
				
				%trigX = getWord(%db.triggerSize, 0);
				%trigY = getWord(%db.triggerSize, 1);
				%trigZ = getWord(%db.triggerSize, 2);
				
				if(mFloor(getWord(%brick.rotation, 3)) == 90)
					%scale = (%trigY / 2) SPC (%trigX / 2) SPC (%trigZ / 2);
				else
					%scale = (%trigX / 2) SPC (%trigY / 2) SPC (%trigZ / 2);
				
				%brick.trigger = new trigger()
				{
					datablock		= %db.triggerDatablock;
					position			= getWords(%brick.getWorldBoxCenter(), 0, 1) SPC getWord(%brick.getWorldBoxCenter(), 2) + ((getWord(%db.triggerSize, 2) / 4) + (%db.brickSizeZ * 0.1));
					rotation			= "1 0 0 0";
					scale				= %scale;
					polyhedron		= "-0.5 -0.5 -0.5 1 0 0 0 1 0 0 0 1";
					parent			= %brick;
				};
				
				if(isObject(%brick.trigger))
					TriggerSS.add(%brick);
			}
			
			if(%db.crpgSpawn)
			{
				SpawnsSS.add(%brick);
			}
			
			%client = %brick.getGroup().client;
			
			if(!isObject(%client))
				%client = %brick.client;
			
			if(isObject(%client))
			{
				%profile = %client.profile;
				
				if(isObject(%profile) && !%client.isCRPGAdmin() && (!isObject($LastLoadedBrick) || $LastLoadedBrick.getID() != %brick.getID()))
				{
					%error = %brick.getCodeError(%client);
					
					if(%error != -1)
					{
						commandToClient(%client, 'DisplayCRPGError', %error);
						%brick.schedule(0, "delete");
						return;
					}
					
					%price = %this.getPrice();
					
                                        if(%db.category $= "CityRPG")
                                        {
                                            //if(%profile.plastic >= %price)
                                            //{
                                            //        %profile.plastic -= %price;
                                            //        commandToClient(%client, 'CRPGPlasticChange', -%price);
                                            //        %brick.plasticMade = true;
                                            //}
                                            //else
                                            //{
                                                    %profile.removeFunds(%price, true);
                                                    commandToClient(%client, 'CRPGCashChange', -%price, %profile.money, %profile.bank, %profile.debt);

                                                    if (strstr(%db, "Monopoly") >= 1)
                                                    {
                                                            %taxes = mFloor(%price * (SystemSO.taxMonopoly / 100));
                                                    }
                                                    else
                                                    {
                                                            %taxes = mFloor(%price * (SystemSO.taxLots / 100));
                                                    }
                                                    //%taxes = mFloor(%price * (SystemSO.taxTarriff / 100));
                                                    SystemSO.addToTreasury(%taxes);
                                            //}
                                        }
                                        else
                                        {
                                            	if(%profile.refplastic >= %price)
                                            	{
                                            	        %profile.refplastic -= %price;
                                            	        //commandToClient(%client, 'CRPGPlasticChange', -%price);
                                           	        %brick.plasticMade = true;
                                           	 }
					   	 //else
					   	 //{
						//	%brick.schedule(0, "delete");
						//	return;
					   	 //}                                            
                                        }
				}
			}

			
			for(%a = 0; %a < %brick.numEvents; %a++)
			{
				if($CRPG::Events::Time[%brick.eventInput[%a]])
				{
					TimeBrickSS.add(%brick);
					break;
				}
			}
		}
	}
	
	// A hack.
	// @null
	function fxDTSBrick::onPlayerTouch(%this, %obj)
	{
		%obj.lastTouchBrick = %this.getID();
		return parent::onPlayerTouch(%this, %obj);
	}
	
	// Deletes the brick's trigger.
	// @null
	function fxDTSBrick::onRemove(%this, %brick)
	{
		if(isObject(%brick.trigger))
			%brick.trigger.delete();
		
		return parent::onRemove(%brick);
	}
	
	// Spawns the vehicle object only if there is a client.
	// @null
	function fxDTSBrick::spawnVehicle(%brick)
	{
		if(isObject(%brick.getGroup().client))
			return parent::spawnVehicle(%brick);
	}
	
	// ============================================================
	// Section 4 : GameConnection
	// ============================================================
	// Gives an error message to clients in LAN games 
	// @integer
	function GameConnection::autoAdminCheck(%client)
	{
		%client.clanPrefix = "";
		%client.clanSuffix = "";
		
		for(%a = 0; %a < ClientGroup.getCount(); %a++)
		{
			%subClient = ClientGroup.getObject(%a);
			
			if(%subClient.bl_id == %client.bl_id && %subClient.getID() != %client.getID())
			{
				%subClient.schedule(0, "delete", "Another client using your BL_ID has entered the game.");
			}
		}
		
		if(%client.hasCRPG && %client.crpgVersion !$= $CRPG::Version)
		{
			commandToClient(%client, 'MessageBoxOK', "CityRPG GUI Error", "You are running an outdated version of CityRPG's GUI.\n\n" @ $CRPG::URL);
		}
		else if(!%client.hasCRPG)
		{
			if($Server::Lan)
			{
				commandToClient(%client, 'MessageBoxOK', "CityRPG GUI Error", "You are not running CityRPG's GUI and therefore cannot properly play the game.\n\n" @ $CRPG::URL);
			}
			else
			{
				commandToClient(%client, 'MessageBoxOK', "CityRPG GUI Error", "You have bypassed a security measure without the client.\n\nYou are not running CityRPG's GUI and therefore cannot properly play the game.\n\n" @ $CRPG::URL);
			}
		}
		
		echo(" +- CityRPG GUI" SPC (%client.crpgVersion ? "v" @ %client.crpgVersion : "is not installed!"));
		
		if(!BanManagerSO.isBanned(%so.bl_id))
			%client.schedule(0, "loadProfile");
		
		return parent::autoAdminCheck(%client);
	}
	
	// Applys outfit parts instead of the player's preference.
	// @integer
	function GameConnection::applyBodyParts(%client)
	{
		%result = parent::applyBodyParts(%client);
		
		%profile = %client.profile;
		
		if(isObject(%profile))
		{
			%profile.getOutfit().applyOutfitParts(%client);
			%profile.faceName = %client.faceName;
			%profile.headColor = %client.headSkin;
		}
		
		return %result;
	}
	
	// Applys outfit colors on top of the player's preference.
	// @integer
	function GameConnection::applyBodyColors(%client)
	{
		%result = parent::applyBodyColors(%client);
		
		%profile = %client.profile;
		
		if(isObject(%profile))
		{
			%profile.getOutfit().applyOutfitColors(%client);
			%profile.faceName = %client.faceName;
			%profile.headColor = %client.headSkin;
		}
		
		return %result;
	}
	
	// Overrides client ban message with user-friendly message.
	// @null
	function GameConnection::delete(%client, %reason)
	{
		if(%reason !$= "")
		{
			if(strpos(%reason, "%CRPGBAN%") > -1)
			{
				%reason = "<bitmap:" @ $CRPG::Path::Images @ "graphics/b&>\n\n\n\n\n\n";
				
				if(BanManagerSO.isBanned(%client.bl_id))
				{
					%bannum = -1;
					
					for(%a = BanManagerSO.numBans - 1; %a > -1; %a--)
					{
						if(BanManagerSO.victimBL_ID[%a] == %client.bl_id)
						{
							%bannum = %a;
							break;
						}
					}
					
					if(%bannum > -1)
					{
						%admin = BanManagerSO.adminName[%bannum];
					}
				}
				
				%adminReason = getField(BanManagerSO.isBanned(%client.bl_id), 1);
				%adminReason = strReplace(%adminReason, "%CRPGBAN%", "");
				
				if(%adminReason !$= "")
				{
					if(%admin !$= "")
						%reason = %reason NL "Banned by" SPC %admin @ "." NL "Reason: \"" @ %adminReason @ "\"";
					else
						%reason = %reason NL "Reason: \"" @ %adminReason @ "\"";
				}
				else
				{
					if(%admin !$= "")
						%reason = %reason NL "Banned by" SPC %admin SPC "for no reason.";
					else
						%reason = %reason NL "Banned for no reason.";
				}
			}
			else
			{
				// if(%client.hasRTB)
				// {
					// No GUI at all
					// if(strpos(%reason, "CITYRPG_PSEUDOBAN_NOGUI_" @ $CRPG_Temp::LastDenyTime) > -1)
						// %reason = "You will not able to join this server until you have the official CityRPG GUI!\n\n" @ $CRPG::RTBURL NL $CRPG::BETAURL;
					
					// GUI is old
					// else if(strpos(%reason, "CITYRPG_PSEUDOBAN_BADGUI_" @ $CRPG_Temp::LastDenyTime) > -1)
						// %reason = "Your client GUI is out of date. Please update it!\n" @ $CRPG::URL NL $CRPG::RTBURL;
				// }
				// else
				// {
					// No GUI at all
					if(strpos(%reason, "CITYRPG_PSEUDOBAN_NOGUI_" @ $CRPG_Temp::LastDenyTime) > -1)
						%reason = "You will not able to join this server until you have the official CityRPG GUI!\n\n" @ $CRPG::URL NL $CRPG::BETAURL;
					
					// GUI is old
					else if(strpos(%reason, "CITYRPG_PSEUDOBAN_BADGUI_" @ $CRPG_Temp::LastDenyTime) > -1)
						%reason = "Your client GUI is out of date. Please update it!\n\n" @ $CRPG::URL;
				// }
				
			}
		}
		
		parent::delete(%client, %reason);
	}
	
	// Sets player in minigame, creates it if it does not exist.
	// @null
	function GameConnection::onClientEnterGame(%client)
	{
		parent::onClientEnterGame(%client);
		
		// Make sure the minigame stays up.
		if(isObject(CRPGMinigame)) CRPGMinigame.addMember(%client);
		else CRPG_CreateMinigame();
		
		if(isObject(%profile = %client.profile))
		{
			%client.crpgLastWantedLevel = %profile.getWantedLevel();
			
			if(%profile.getWantedLevel() >= 6)
			{
				messageAll('', '\c0Attention\c6: Elite Criminal \c3%1\c6 has entered the city with \c0Wanted Level %2\c6. Lethal Force from all civilians is authorized.',
					%client.name, %profile.getWantedLevel());
			}
			else if(%client.bl_id == SystemSO.mayorBLID)
			{
				messageAll('', '\c2Mayor \c3%1\c2 has entered the city. Hail to the Chief!', %client.name);
				EnvironmentSO.schedule(100, 0, "playMayorTheme");
			}
		}
		
		%client.sendInformation();
		%client.sendJobOps();
	}
	
	// Saves tools.
	// Saves profile.
	// Completes an arrest if they are arrested.
	// @null
	function GameConnection::onClientLeaveGame(%client)
	{
		if(!$CRPG_Temp::ServerDestroying && Profiler.isProfileActive(%client.bl_id))
		{
			%obj		= %client.player;
			%profile	= %client.profile;
			
			if(isObject(%obj))
			{
				// Tool Saving
				%maxTools	= %obj.dataBlock.maxTools;
				%jobTools	= %profile.getJob().getTools();
				%tools		= "";
				
				for(%a = 0; %a < %maxTools; %a++)
				{
					%tool = %obj.tool[%a];
					%valid = false;
					
					if(isObject(%tool))
					{
						%valid = true;
						
						for(%b = 0; %b < %maxTools; %b++)
						{
							%jobTool = getField(%jobTools, %b);
							
							// %tool is a job tool. Forget about it, move on.
							if(isObject(%jobTool) && %jobTool.getID() == %tool.getID())
							{
								%valid = false;
								break;
							}
						}
						
						if(%valid)
						{
							if(!%profile.isInmate())
							{
								for(%b = 0; %b < %maxTools; %b++)
								{
									%jobTool = getField($CRPG::DefaultTools, %b);
									
									if(isObject(%jobTool) && %jobTool.getID() == %tool.getID())
									{
										%valid = false;
										break;
									}
								}
							}
							else
							{
								for(%b = 0; %b < %maxTools; %b++)
								{
									%jobTool = getField($CRPG::PrisonTools, %b);
									
									if(isObject(%jobTool) && %jobTool.getID() == %tool.getID())
									{
										%valid = false;
										break;
									}
								}
							}
						}
					}
					
					if(%valid)
						%tools = %tools @ %tool.getName() @ "\t";
				}
				
				%profile.tools = %tools;
				
				// Arrest Failsafe
				if(%obj.crpgArrested && %profile.getWantedLevel())
				{
					echo(%client.name SPC "tried to log out to escape jail.");
					%profile.setInmate(2);
					%profile.problem	= addField(%profile.problem, "\c6You tried to escape the police by logging out, but the jokes on you: Your sentence was \c0doubled\c6.");
				}
				
				// Remove Weapon
				%obj.unMountImage(0);
			}
			
			// Profile Saving
			echo("\c4Saving" SPC %client.name SPC "(" @ %client.bl_id @ ")");
			
			if(!isObject(%client.brickGroup) || !%client.brickGroup.getCount())
				Profiler.closeProfile(%client.bl_id);
			else
				Profiler.saveProfile(%client.bl_id);
		}
		
		if(isObject(%brickgroup = %client.brickGroup) && isObject(%quota = %brickGroup.QuotaObject))
		{
			if(%brickgroup.numPhysVehicles)
			{
				%quota.killobjects($TypeMasks::VehicleObjectType);
				echo("\c1Clearing" SPC %client SPC %brickgroup.numPhysVehicles SPC "Vehicle(s).");
			}
		}
		
		return parent::onClientLeaveGame(%client);
	}
	
	// Marks the client version.
	// @null
	function GameConnection::onConnectRequest(%client, %ip, %lan, %net, %prefix, %suffix, %arg5, %rtb, %arg7, %crpg, %arg9, %arg10, %arg11, %arg12, %arg13, %arg14, %arg15)
	{
		echo("7" SPC %arg7);
		echo("8" SPC %crpg);
		echo("9" SPC %arg9);
		
		if(!%client.isLocalConnection())
		{
			%client.hasCRPG = false;
			%client.crpgVersion = 0;
			
			for(%a = 0; %a < getLineCount(%crpg); %a++)
			{
				%line = getLine(%crpg, %a);
				
				if(getField(%line, 0) $= "CRPG")
				{
					%password	= getField(%line, 1);
					%version		= mFloatLength(getField(%line, 2), 1);
					
					echo(" - " @ %password);
					echo(" - " @ %version);
					
					if(%version !$= "")
					{
						%client.hasCRPG = true;
						
						if($CRPG::Password $= "" || %password $= $CRPG::Password)
							%client.crpgVersion = %version;
					}
					
					break;
				}
			}
		}
		else
		{
			%client.hasCRPG = ($CRPG_CL::Version !$= "" ? true : false);
			%client.crpgVersion = $CRPG_CL::Version;
		}
		
		return parent::onConnectRequest(%client, %ip, %lan, %net, %prefix, %suffix, %arg5, %rtb, %arg7, %crpg, %arg9, %arg10, %arg11, %arg12, %arg13, %arg14, %arg15);
	}
	
	// Puts player into ghost mode.
	// Spawns cash.
	// Demerits.
	// Mission fail.
	// @null
	function GameConnection::onDeath(%client, %sourceObject, %sourceClient, %damageType, %damageArea)
	{
		%damageType = "Invisible";
		
		%cp	= %client.profile;
		%obj	= %client.player;
		%sp	= isObject(%sourceClient) ? %sourceClient.profile : -1;
		
		if(isObject(%cp))
		{
			%cp.tools = "";
			%cp.deaths++;
			%cpJob = %cp.getJob();
		}
		
		if(isObject(%sp))
		{
			%sp.kills++;
			%spJob = %sp.getJob();
		}
		
		// Medical Respawn
		%client.bloodyDeath = true;
		
		if(isObject(%obj))
		{
			%db = %obj.getDatablock();
			
			// Ghost cam
			%transform = %obj.getTransform();
			%transform = vectorAdd(getWords(%transform, 0, 2), "0 0 2.5") SPC getWords(%transform, 3, 6);
			
			// Crosshair fix
			if(%obj.isAiming)
				commandToClient(%client,'HataCrosshair',false,false);
		
			// Remove Weapon
			%obj.unMountImage(0);
			
			// Drop Cash
			%obj.dropMoney(%client.profile.money);
			// if(%client.profile.money > 0)
			// {
				// %cash = new Item()
				// {
					// datablock = CRPG_Cash_Item;
					// cashValue = mFloor(%client.profile.money);
					// owner = %client.getBLID();
					// stolen = true;
				// };
				
				// %cash.canPickup = false;
				// %cash.setTransform(setWord(%obj.getTransform(), 2, getWord(%obj.getTransform(), 2) + 2));
				// %cash.setVelocity(VectorScale(%obj.getEyeVector(), 10));
				
				// MissionCleanup.add(%cash);
				
				// %cash.setShapeName("$" @ %cash.cashValue);
				
				// %client.profile.money = 0;
			// }
			
			// Drop Items
			for(%a = 0; %a < %db.maxTools; %a++)
			{
				%tool = %obj.tool[%a];
				
				if(SalesSO.isItemValuable(%tool) && !%client.profile.getJob().isJobItem(%tool) && %tool.canDrop)
				{
					%item = new Item()
					{
						dataBlock = %tool;
					};
					
					%item.setTransform(%obj.getPosition() SPC %obj.rotation);
					%item.setVelocity(getRandom(-8, 8) SPC getRandom(-8, 8) SPC 10);
					%item.schedulePop();
				}
				
				%obj.tool[%a] = "";
			}
		}
		
		%result = parent::onDeath(%client, %sourceObject, %sourceClient, %damageType, %damageArea);
		
		%client.setGhostCamera(%transform);
		%client.updateAmmo();
		
		// Crime
		if(isObject(%cp) && isObject(%sp) && %cp.getID() != %sp.getID())
		{
			%cpJob.onKilled(%client, %cp, %sourceClient, %sp, %sourceObject, %transform, %damageType);
			
			if(%cp.getWantedLevel() >= 6)
			{
				messageAll('', '\c6Elite criminal \c3%1\c6 was taken down by \c3%3 %2\c6 with deadly force.', %client.name, %sourceClient.name, %spJob.police ? "Officer" : "Vigilante");
				%client.arrest();
			}
			
			else if(%cp.getWantedLevel() && %spJob.police)
				messageClient(%sourceClient, '', '\c6You have to arrest \c3%1\c6 with your \c3Handcuffs\c6. Killing them does no good.', %client.name);
		}
		
		if(isObject(%client.mission))
			%client.mission.onPlayerKilled();
		
		// Death Handling
		commandToClient(%client, 'DisplayCRPGBlackout', true, 2500);
		%client.crpgRespawnSched = %client.schedule(CRPGMinigame.respawnTime, "spawnPlayer");
		%client.isDead = true;
		

		
		return %result;
	}
	
	// Does the following:
	//	- Deletes temp bricks.
	//	- Exits default spawn if no Profile and sets player in ghost mode.
	//	- Redistributes job and saved tools.
	// @null
	function GameConnection::spawnPlayer(%client)
	{
		if(%client.isDead)
		{
			cancel(%client.crpgRespawnSched);
			commandToClient(%client, 'DisplayCRPGBlackout', false, 1000);
		}
		
		%client.isDead = false;
		
		
		if(isObject(%client.player.tempBrick))
			%client.player.tempBrick.delete();
		
		
		%client.profile = %client.getProfile();
		
		if(isObject(%client.profile))
		{
			%client.camera.isGhostCam = false;
			
			%result = parent::spawnPlayer(%client);
			
			%obj = %client.player;
			
			if(isObject(%obj))
			{
				%obj.setShapeNameColor("1 1 1 1");
				%obj.setShapeNameDistance(1);
				%obj.giveDefaultEquipment();
				%obj.setBackImage();
				
				%client.profile.setVillainy(%client.crpgEvil);
			}
			
			if(!%client.profile.gender)
			{
				schedule(32, 0, "commandToClient", %client, 'MessageBoxYesNo', "So, about your gender...", "The game has automatically marked you as male.\n\nIf you want to be known as female, we can change it. Although there is no substantial difference between genders, you will be dressed differently by the system.\n\nClick \"Yes\" to be female.", 'ChangeGender', '');
				%client.profile.gender = 1;
			}
			
			if(!%client.crpgWelcomed)
			{
				schedule(250, 0, "commandToClient", %client, 'DisplayCRPGWelcome');
				%client.crpgWelcomed = true;
			}
			
			%client.sendStatus(true);
			%client.bloodyDeath = false;
			
			return %result;
		}
		else
			%client.setGhostCamera();
		
		return 0;
	}
	
	// ============================================================
	// Section 5: Observer
	// ============================================================
	// Respawns player while in ghost mode if spawn time is up.
	// @null
	function Observer::onTrigger(%this, %obj, %trigger, %state)
	{
		if(%obj.isGhostCam)
		{
			%client = %obj.getControllingClient();
			%minigame = %client.minigame;
			
			switch(%trigger)
			{
				case 0:
					if(%state)
					{
						%ms = getSimTime() - %client.deathTime;
						
						if(isObject(%minigame))
						{
							if(%ms >= %minigame.respawnTime)
								%client.spawnPlayer();
						}
						else if(%ms >= 1000)
							%client.spawnPlayer();
					}
				case 4:
					if(isObject(%client.oldFov))
						%client.setcontrolcamerafov(%client.oldFov);
			}
		}
		
		return parent::onTrigger(%this, %obj, %trigger, %state);
	}
	
	// ============================================================
	// Section 6 : Player
	// ============================================================
	// Calls JobSO.onActivate(%client, %obj).
	// @col
	function Player::activateStuff(%obj)
	{
		%client = %obj.client;
		
		if(isObject(%client))
		{
			%profile = %client.profile;
			
			if(isObject(%profile))
			{
				// JobSO::onActivate Call
				if(isObject(%job = %profile.getJob()))
					%job.onActivate(%client, %obj);
				
				// ShapeBase::onClick Raycast
				%start	= %obj.getEyePoint();
				%aimVec	= %obj.getAimVector();
				%range	= 3 * getWord(%obj.getScale(), 2);
				%end		= vectorAdd(%start, vectorScale(%aimVec, %range));
				%masks	= ($TypeMasks::FxBrickObjectType | $TypeMasks::PlayerObjectType | $TypeMasks::StaticObjectType | $TypeMasks::TerrainObjectType | $TypeMasks::VehicleObjectType);
				
				%ray		= containerRayCast(%start, %end, %masks, %obj);
				%col		= getWord(%ray, 0);
				%pos		= posFromRaycast(%ray);
				
				if(isObject(%col) && %col.getType() & $CRPG::ClickMasks && isFunction(%col.getClassName(), "onClicked"))
					%col.onClicked(%client, %profile, %obj, %pos);
			}
		}
		
		return parent::activateStuff(%obj);
	}
	
	// Hijacks the deathcry noise.
	// @null
	function Player::playDeathCry(%obj)
	{
		%db = %obj.getDatablock();
		
		if(%db.deathCry !$= "")
		{
			%cry = %db.deathCry;
			
			if(isObject(%cry))
				ServerPlay3D(%cry, %obj.getTransform());
		}
		else
			return parent::playDeathCry(%obj);
	}
	
	// Hijacks body removal.
	function Player::removeBody(%obj, %repeat)
	{
		if(!%repeat)
			%obj.crpgBodyRemovalSched = %obj.schedule(60000, "removeBody", true);
		
		else
			parent::removeBody(%obj, %repeat);
	}
	
	// Forces player name color to be whatever is indicitive of their status.
	// @null
	function Player::setShapeNameColor(%obj, %color)
	{
		%client = %obj.client;
		
		if(isObject(%client))
		{
			%profile = %client.profile;
			
			if(isObject(%profile))
			{
				if(%wl = %profile.getWantedLevel())
				{
					%pink = 1 - mFloatLength((255 * (%wl / 6)) / 255, 2);
					%color = "1" SPC %pink SPC %pink SPC "1";
				}
				else if(%profile.reincarnate)
					%color = "1 1 0 1";
				else if(%profile.getJob().expTree $= "Law")
					%color = "0.4 0.4 1 1";
				else
					%color = "1 1 1 1";
			}
		}
		
		parent::setShapeNameColor(%obj, %color);
	}
	
	// Changes player name distance.
	// @null
	function Player::setShapeNameDistance(%obj, %dist)
	{
		%client = %obj.client;
		%dist = 16;
		%distFactor = 2;
		
		if(isObject(%client))
		{
			%profile = %client.profile;
			
			if(isObject(%profile))
			{
				// Day n' Night
				%distFactor += mClampF(%profile.getBeliefDay(), 0, 1);
				
				if(EnvironmentSO.night)
				{
					%distFactor -= 1 * %profile.getBeliefNight();
				}
				
				// Wanted Level
				%distFactor *= 1 * %profile.getWantedLevel();
			}
		}
		
		if(EnvironmentSO.night)
		{
			%distFactor -= 1;
		}
		
		%dist *= %distFactor;
		
		parent::setShapeNameDistance(%obj, %dist);
	}
	
	// ============================================================
	// Section 7 : serverCmd
	// ============================================================
	// Will be used in filtering out Admin Events added by non-admins.
	function serverCmdAddEvent(%client, %enabled, %input, %delay, %classid, %target, %output, %param1, %param2, %param3, %param4, %param5, %param6)
	{
		%brick = %client.wrenchBrick;
		
		if(isObject(%brick))
		{
			%inputName = $InputEvent_Name["fxDTSBrick", %input];
			%className = getWord(getField($InputEvent_TargetList["fxDTSBrick", %input], %classid), 1);
			%outputName = $OutputEvent_Name[%className, %output];
			
			if(!%client.isCRPGAdmin() && (!isObject($LastLoadedBrick) || $LastLoadedBrick.getID() != %brick.getID()))
			{
				if($CRPG::Events::Admin[%className, %outputName])
				{
					messageClient(%client, '', '\c0Sorry. \c5%1::%2\c0 is Admin-Only.', %className, %outputName);
					return;
				}
			}
			
			if($CRPG::Events::Time[%inputName])
			{
				TimeBrickSS.add(%brick);
			}
		}
		
		return parent::serverCmdAddEvent(%client, %enabled, %input, %delay, %classid, %target, %output, %param1, %param2, %param3, %param4, %param5, %param6);
	}
	
	// Removes time bricks from the SS.
	function serverCmdClearEvents(%client)
	{
		%brick = %client.wrenchBrick;
		
		if(TimeBrickSS.isMember(%brick))
			TimeBrickSS.remove(%brick);
		
		return parent::serverCmdClearEvents(%client);
	}
	
	// Prevents admins from cheating.
	function serverCmdDropPlayerAtCamera(%client)
	{
		%result = parent::serverCmdDropPlayerAtCamera(%client);
		
		if(isObject(%result) && (%result.crpgArrested || %result.crpgStunned))
		{
			%client.camera.setMode("Corpse", %client.player);
			%client.setControlObject(%client.camera);
		}
		
		return %result;
	}
	
	// Sells item.
	function serverCmdDropTool(%client, %tool)
	{
		%obj = %client.player;
		
		if(isObject(%obj))
		{
			%item = %obj.tool[%tool];
			
			if(isObject(%item))
			{
				if(isObject(%profile = %client.profile))
				{
					if(%profile.getJob().isJobItem(%item))
						return;
					
					if(isObject(%obj.crpgCarTrigger))
					{
						if(SalesSO.isItemValuable(%item))
						{
							%price = mClamp(SalesSO.getItemPrice(SalesSO.getItemID(%item)) / 10, 10, 10000);
							%obj.crpgSellItem = %tool;
							commandToClient(%client, 'MessageBoxYesNo', "I think we can reach an agreement...", "The peddlers are willing to pay you $" @ %price @ " for your " @ %item.uiName @ ".\n\nDo you accept their offer?\nThere is no chance to get it back.", 'Event_SellItem');
							return;
						}
					}
				}
			}
		}
		
		if(%item.handsFree)
			%item.image.mountPoint = $BackSlot;
		
		parent::serverCmdDropTool(%client, %tool);
		
		if(%item.handsFree)
			%item.image.mountPoint = $HeadSlot;
	}
	
	// Reroutes Inmate chat to Team Chat.
	function serverCmdMessageSent(%client, %message)
	{
		if(%client.isDead)
		{
			messageClient(%client, '', '<color:BCD4E6>You feel so cold.');
			return;
		}
		
		// Filter tags.
		while((%kalphLoc = strPos(%message, "[%CRPGTAG:")) >= 0)
		{
			%kalphEnd = strPos(%message, "]", %kalphLoc);
			
			if(%kalphEnd >= 0)
			{
				%kalphInject = getSubStr(%message, %kalphLoc, %kalphEnd - %kalphLoc + 1);
				%message = strReplace(%message, %kalphInject, "");
			}
		}
		
		// Add Award
		%client.clanPrefix = %client.getAward();
		
		if(%client.profile.isInmate())
			return serverCmdTeamMessageSent(%client, %message);
		
		return parent::serverCmdMessageSent(%client, %message);
		
		if(getSimTime() - %client.spamProtectStart >= 10000 || %client.isAdmin)
		{
			%message = trim(stripMLControlChars(trim(%message)));
			%message = strlwr(%message);
			
			if(%message $= "?")
				%message = "wot";
			else
			{
				%message = strreplace(%message, "?", " :o");
				%message = strreplace(%message, ".", "");
				%message = strreplace(%message, "!", "");
				%message = strreplace(%message, ",", "");
				%message = strreplace(%message, "'", "");
				%message = strreplace(%message, "youre", "ur");
				%message = strreplace(%message, "you", "u");
				%message = strreplace(%message, "what", "wot");
				%message = strreplace(%message, "cool", "yay!");
				%message = strreplace(%message, "awesome", "yay!");
				%message = strreplace(%message, "hello", "hai");
			}
			
			if(strlen(%message) > 0)
			{
				if(%message $= %client.lastChatText)
				{
					messageClient(%client, '', "\c5Do not repeat yourself.");
				}
				
				if(%message !$= %client.lastChatText || %client.isAdmin)
				{
					if(getSimTime() - %client.lastChatTime < 2000)
						%client.spamMessageCount++;
					else
						%client.spamMessageCount = 0;
						
					if(%client.spamMessageCount < 4 || %client.isAdmin)
					{
						%client.lastChatTime = getSimTime();
						%client.lastChatText = %message;
						
						%name = strreplace(strlwr(%client.name), " ", "");
						
						if(getSubStr(%name, strLen(%name) - 1, 1) !$= "s" && getSubStr(%name, strLen(%name) - 1, 1) !$= "z")
							%name = %name @ "s";
						
						echo(%name @ ":" SPC %message);
						messageAll('', '\c3%1\c6: %2', %name, %message);
					}
					else
					{
						messageClient(%client, '', "\c5You are flooding the server.");
					}
				}
			}
		}
	}
	
	// Plant error filtering.
	// @fxDTSBrick
	function serverCmdPlantBrick(%client)
	{
		return parent::serverCmdPlantBrick(%client);
	}
	
	// Chat amongst jobs.
	// @null
	function serverCmdTeamMessageSent(%client, %message)
	{
		if(getSimTime() - %client.spamProtectStart >= 10000 || %client.isAdmin)
		{
			%message = trim(stripMLControlChars(trim(%message)));
			
			if(strlen(%message) > 0)
			{
				if(%message $= %client.lastChatText)
				{
					messageClient(%client, '', "\c5Do not repeat yourself.");
				}
				
				if(%message !$= %client.lastChatText || %client.isAdmin)
				{
					if(getSimTime() - %client.lastChatTime < 2000)
						%client.spamMessageCount++;
					else
						%client.spamMessageCount = 0;
						
					if(%client.spamMessageCount < 4 || %client.isAdmin)
					{
						%client.lastChatTime = getSimTime();
						%client.lastChatText = %message;
						
						%profile = %client.profile;
						
						if(isObject(%profile))
						{
							if(%profile.isInmate())
								%tree = "Jail";
							else
								%tree = %profile.getJob().expTree;
							
							if(%tree !$= "")
							{
								echo("(T-" @ %tree @ ")" @ %client.name @ ":" SPC %message);
								%message = "\c4[" @ %tree @ "]\c7" SPC %client.clanPrefix @ "\c3" @ %client.name @ "\c7" @ %client.clanSuffix @ "\c6:" SPC %message;
								
								for(%a = 0; %a < %client.minigame.numMembers; %a++)
								{
									%member = %client.minigame.member[%a];
									
									if(isObject(%member.profile))
									{
										if(%profile.isInmate() && %member.profile.isInmate())
											messageClient(%member, '', %message);
										
										else if(%profile.isInmate() && %member.profile.getJob().police)
											messageClient(%member, '', "\c1[P]" SPC %message);
										
										else if(!%profile.isInmate() && %member.profile.getJob().expTree $= %tree)
											messageClient(%member, '', %message);
										
										else if(%member.isAdmin)
											messageClient(%member, '', "\c5[A]" SPC %message);
									}
								}
							}
						}
						else
						{
							messageClient(%client, '', "\c5You have no profile.");
						}
					}
					else
					{
						messageClient(%client, '', "\c5You are flooding the server.");
					}
				}
			}
		}
	}
	
	// Teleports you to home base.
	// @null
	function serverCmdSuicide(%client)
	{
		if(!isObject(%client.player))
		{
			messageClient(%client, '', "\c6What body do you have to teleport?");
			return;
		}
		
		if(%client.player.getDamageLevel())
		{
			messageClient(%client, '', "\c6Your injuries prevent you from teleporting.");
			return;
		}
		
		if(%client.player.getMountedImage(0).isHeavy)
		{
			messageClient(%client, '', "\c6Iban's trans-space scripting powers cannot bend the fabric of distance if you are carrying a crate.");
			return;
		}
		
		if(%client.player.crpgArrested)
		{
			messageClient(%client, '', "\c6Stop right there, criminal scum! You may not teleport while under arrest.");
			return;
		}
		
		%time = $sim::time - %client.lastSuicideTeleport;
		
		if(%time < 90 && !%client.isAdmin)
		{
			%time = 89 - mFloor(%time);
			messageClient(%client, '', '\c6You must wait \c3%1\c6 more second%2 before teleporting home.', %time + 1, (%time + 1) > 1 ? "s" : "");
			return;
		}
		
		if(%client.profile.getWantedLevel())
		{
			initContainerRadiusSearch(%client.player.getPosition(), 64, $TypeMasks::PlayerObjectType);
			
			while(isObject(%col = containerSearchNext()))
			{
				if(isObject(%col.client.profile) && %col.client.bl_id != %client.bl_id)
				{
					%job = %col.client.profile.getJob();
					
					if(%job.police)
					{
						messageClient(%client, '', '\c6You cannot teleport in the presence of authority with demerits.');
						return;
					}
				}
			}
		}
		
		%client.player.teleportHome(%client, true);
		%client.lastSuicideTeleport = $sim::time;
	}
	
	// Filters, modifies, and erases inappropriate wrench data.
	function serverCmdSetWrenchData(%client, %data)
	{
		%brick = %client.wrenchBrick;
		
		if(isObject(%brick))
		{
			if(%brick.getDatablock().crpgLot)
				return;
			
			// Brick Properties
			%group	= %brick.getGroup();
			%profile	= Profiler.getProfile(%group.bl_id);
			
			if(isObject(%profile))
			{
				%count	= getFieldCount(%data);
				
				// Parse
				for(%a = 0; %a < %count; %a++)
				{
					%field	= getField(%data, %a);
					%name		= getWord(%field, 0);
					%value	= getSubStr(%field, strlen(%name) + 1, strlen(%field) - strlen(%name));
					
					switch$(%name)
					{
						case "N":		// Name
							
						case "LDB":		// Light DataBlock
						
						// Emitter Properties
						case "EDB":		// Emitter DataBlock
						case "EDIR":	// Emitter Direction
						
						// Item Properties
						case "IDB":		// Item DataBlock
							if(isObject(%value))
							{
								if(!%client.isCRPGAdmin())
								{
									commandToClient(%client, 'CenterPrint', "\c6Only \c3Admins\c6 can Spawn Bricks.\n\c6Use the \c3SellItem\c6 Event.", 5);
									%value = 0;
								}
								else
									commandToClient(%client, 'CenterPrint', "\c6This \c3" @ %value.uiName @ "\c6 is only spawning\n\c0BECAUSE YOU ARE AN ADMIN.\n\c6It \c0WILL NOT\c6 stop spawning until unset.\n\n\c6You've been warned.", 5);
							}
						
						case "IPOS":	// Item Position
						case "IDIR": 	// Item Direction
						case "IROT": 	// Item Rotation
						
						// Vehicle Properties
						case "VDB":		// Vehicle DataBlock
							
							if(isObject(%value))
							{
								if(%brick.getGroup().bl_id == %client.bl_id)
								{
									%vehicles = getFieldCount(%profile.garage);
									
									for(%b = 0; %b < %vehicles; %b++)
									{
										%vehicle = getField(%profile.garage, %b);
										
										if(isObject(%vehicle) && %vehicle.getID() == %value.getID())
										{
											%canSpawn = true;
											break;
										}
									}
									
									if(!%canSpawn)
									{
										if(!%client.isCRPGAdmin())
										{
											messageClient(%client, '', "\c3Can't Spawn!\c6 The owner of this brick does not have the \c3" @ %value.uiName @ "\c6.");
											%value = 0;
										}
										else
											commandToClient(%client, 'CenterPrint', "\c6This \c3" @ %value.uiName @ "\c6 is only spawning\n\c0BECAUSE YOU ARE AN ADMIN.\n\n\c6You've been warned.", 5);
									}
									
									%canSpawn = "";
								}
								else
								{
									commandToClient(%client, 'CenterPrint', "\c6You cannot spawn vehicles on any brick but your own.", 5);
								}
							}
						
						case "RCV":		// Re-Color Vehicle
						
						// Physical Properties
						case "RC":		// Raycasting
						case "C":		// Collision
						case "R":		// Rendering
					}
					
					%datumName[%a]		= %name;
					%datumValue[%a]	= %value;
				}
				
				// Re-assemble
				%data = "";
				
				for(%a = 0; %a < %count; %a++)
				{
					%data = %data @ %datumName[%a] SPC %datumValue[%a] @ (%a + 1 < %count ? "\t" : "");
				}
			}
		}
		
		return parent::serverCmdSetWrenchData(%client, %data);
	}
	
	// Prevents user from unmounting handcuffs.
	function serverCmdUnUseTool(%client)
	{
		if(isObject(%client.player) && !%client.player.crpgArrested && !%client.player.crpgStunned)
			%result = parent::serverCmdUnUseTool(%client);
			
		%client.updateAmmo();
		return %result;
	}
	
	// Prevents user from mounting tools while under arrest.
	function serverCmdUseTool(%client, %tool)
	{
		if(isObject(%client.player) && !%client.player.crpgArrested && !%client.player.crpgStunned)
		{
			%result = parent::serverCmdUseTool(%client, %tool);
			%client.player.setBackImage();
		}
		
		%client.updateAmmo();
		return %result;
	}
	
	// Prevents user from mounting bricks while under arrest.
	function serverCmdUseInventory(%client, %tool)
	{
		if(isObject(%client.player) && !%client.player.crpgArrested && !%client.player.crpgStunned)
			return parent::serverCmdUseInventory(%client, %tool);
	}
	
	// Banned minigame-related functions.
	function serverCmdCreateMinigame(%client)	{}
	function serverCmdEndMinigame(%client)		{}
	function serverCmdJoinMinigame(%client)	{}
	function serverCmdLeaveMinigame(%client)	{}
	function serverCmdUpdateMinigame(%client)	{}
	
	// ============================================================
	// Section 8 : Misc
	// ============================================================
	// Prevents players without the GUI from joining.
	// @bool TAB reason - True if banned or client lacks GUI, TAB reason for ban.
    function BanManagerSO::isBanned(%so, %blid)
    {
        %banned = parent::isBanned(%so, %blid);
       
        if(%banned $= "0")
        {
            %client = FindClientByBL_ID(%blid);
            if(!%client.hasCRPG)
            {
                $CRPG_Temp::LastDenyTime = getSimTime();
                %client.crpgBan = true;
                echo(" +- " @ %blid @ " has no GUI.");
                return true TAB "CITYRPG_PSEUDOBAN_NOGUI_" @ $CRPG_Temp::LastDenyTime;
            }
            else if(%client.crpgVersion !$= $CRPG::Version)
            {
                $CRPG_Temp::LastDenyTime = getSimTime();
                %client.crpgBan = true;
                echo(" +- " @ %blid @ " has outdated GUI.");
                return true TAB "CITYRPG_PSEUDOBAN_BADGUI_" @ $CRPG_Temp::LastDenyTime;
            }
            else
            {
                for(%a = 0; %a < ClientGroup.getCount(); %a++)
                {
                    %subClient = ClientGroup.getObject(%a);
                    if(%subClient.bl_id == %blid && %subClient.getID() != %client.getID())
                        return true TAB "There is already someone else using your Blockland ID in this server. Multiple clients are not allowed.";
                }
            }
           
            %beta = RestrictionSO.isAllowed(%blid);
           
            if(!%beta)
            {
                echo(" +- " @ %blid @ " is not on Beta List.");
                return true TAB "You are not on the Beta List.";
            }
            else
            {
                echo(" +- " @ %blid @ " is on Beta List.");
            }
        }
        else if(%client.hasCRPG)
        {
            %banned = getField(%banned, 0) TAB "%CRPGBAN%" @ getField(%banned, 1);
        }
       
        return %banned;
    }
	
	// Adds generic spawn poinst to the client thing.
	function brickSpawnPointData::onPlant(%this, %brick)
	{
		SpawnsSS.add(%brick);
		return parent::onPlant(%this, %brick);
	}
	
	// Prevents player from doing any damage with the hammer.
	// @col
	function hammerImage::onHitObject(%this, %obj, %slot, %col, %pos, %normal)
	{
		if(isObject(%col) && %col.getType() & ($TypeMasks::PlayerObjectType | $TypeMasks::VehicleObjectType | $TypeMasks::VehicleBlockerObjectType))
		{
			serverPlay3d(hammerHitSound, %pos);
			return %col;
		}
		
		return parent::onHitObject(%this, %obj, %slot, %col, %pos, %normal);
	}
	
	function WandImage::onHitObject(%this, %obj, %slot, %col, %pos, %normal)
	{
		if(isObject(%col) && %col.getType() & $TypeMasks::PlayerObjectType)
		{
			serverPlay3d(wandHitSound, %pos);
			return %col;
		}
		
		return parent::onHitObject(%this, %obj, %slot, %col, %pos, %normal);
	}
	
	// Prevents players from wrenching their baseplate.
	// @col
	function wrenchImage::onHitObject(%this, %obj, %slot, %col, %pos, %normal)
	{
		if(isObject(%col) && %col.getType() & $TypeMasks::FxBrickAlwaysObjectType)
		{
			if(%col.getDatablock().crpgLot)
			{
				if(isObject(%obj.client))
				{
					%owner	= %col.getGroup().name;
					%ownerID	= %col.getGroup().bl_id;
					
					%title = "Wrench -" SPC %owner SPC "- (BL_ID:" SPC %ownerID @ ")";
					%message = "<bitmap:base/client/ui/brickIcons/16x16 Base>\n\n\n\n\n\n\nYou cannot edit or manipulate the properties of your CityRPG Baseplate.\n\nYou can still paint them, however.";
					
					%obj.client.sendMessageBox(%title, %message);
				}
				
				serverPlay3d(wrenchMissSound, %pos);
				return %col;
			}
			else if(%col.getDatablock().specialBrickType $= "VehicleSpawn" && getTrustLevel(%obj, %col) < 3)
			{
				serverPlay3d(wrenchMissSound, %pos);
				return %col;
			}
		}
		
		return parent::onHitObject(%this, %obj, %slot, %col, %pos, %normal);
	}
	
	// Prevents important items from being destroyed.
	function Item::schedulePop(%obj)
	{
		if(!SalesSO.isItemValuable(%obj.getDatablock()))
			return parent::schedulePop(%obj);
	}
	
	// Prevents console errors on shutdown.
	function MiniGameSO::onRemove(%so)
	{
		if(!$CRPG_Temp::ServerDestroying)
			if(isFunction(parent, "onRemove"))
				return parent::onRemove(%so);
	}
	
	// Prevents arrested client from regaining control.
	function Observer::onTrigger(%this, %obj, %trigger, %state)
	{
		%client = %obj.getControllingClient();
		
		switch(%trigger)
		{
			case 0:
				if(%state)
				{
					if(isObject(%client) && %obj.isGhostCam && getSimTime() - %client.deathTime > %client.minigame.respawnTime)
						return %client.spawnPlayer();
				}
		}
		
		if(!isObject(%client) || (!%client.player.crpgArrested && !%client.player.crpgStunned))
			return parent::onTrigger(%this, %obj, %trigger, %state);
	}
	
	// Raycasting
	function WeaponImage::onFire(%this, %obj, %slot)
	{
		if(%this.raycastWeaponRange <= 0)
			return parent::onFire(%this, %obj, %slot);
		
		// Aiming
		%targets = %this.raycastWeaponTargets;
		%ptargets = %this.raycastWeaponPierceTargets;
		
		if(%this.raycastFromMuzzle)
		{
			%start = %obj.getMuzzlePoint(%slot);
			%aimVec = %obj.getMuzzleVector(%slot);
		}
		else
		{
			%start = %obj.getEyePoint();
			%aimVec = %obj.getAimVector();
		}
		
		// Bastardize the aimvec.
		%aimVec = %this.getAimVector(%obj, %slot, %aimVec);
		
		// Energy
		if(%this.kickBack > 0)
			%obj.setEnergyLevel(mClamp(%obj.getEnergyLevel() - %this.kickBack, 0, %obj.getDatablock().maxEnergy));
		
		// Raycasting
		if(%this.raycastSpreadCount <= 0)
			%shellcount = 1;
		else
			%shellcount = %this.raycastSpreadCount;
		
		for(%i = 0; %i < %shellcount; %i++)
		{
			%count = 0;
			%shotvec = %this.getShotVector(%obj, %slot, %aimVec);
			%rangeRem = %this.raycastWeaponRange * getWord(%obj.getScale(), 2);
			
			// Hit Detection
			while(%rangeRem > 0)
			{
				%range = mClamp(%rangeRem, 0, 100);
				%rangeRem -= %range;
				
				if(%count)
					%point = %end;
				else
					%point = %start;
				
				%end = vectorAdd(%point, vectorScale(%shotVec, %range));
				%ray = ContainerRayCast(%point, %end, %targets, %obj);
				%col = getWord(%ray, 0);
				
				if(isObject(%col))
				{
					%colType	= %col.getType();
					%pos		= posFromRaycast(%ray);
					%normal	= normalFromRaycast(%ray);
					%end		= %pos;
					%crit		= %this.isRaycastCritical(%obj, %slot, %col, %pos, %normal);
					
					%this.onHitObject(%obj, %slot, %col, %pos, %normal, %shotVec, %crit);
					
					// Piercing
					if(%colType & %ptargets)
					{
						%rangeRem = %rangeRem + vectorLen(vectorSub(%pos, %point)) - 1;
						%end = vectorAdd(%end, %shotVec);
					}
					else
					{
						%rangeRem = 0;
					}
				}
				
				%count++;
			}
			
			// Tracer Projectile
			if(isObject(%this.raycastTracerProjectile))
			{
				%projectile = %this.raycastTracerProjectile;
				%muzzle = %obj.getMuzzlePoint(%slot);
				
				%p = new (%this.projectileType)()
				{
					dataBlock = %projectile;
					initialVelocity = vectorScale(vectorNormalize(vectorSub(%end, %muzzle)), %projectile.muzzleVelocity);
					initialPosition = %muzzle;
					sourceObject = %obj;
					sourceSlot = %slot;
					client = %obj.client;
				};
				
				%p.setScale(%obj.getScale());
				MissionCleanup.add(%p);
			}
		}
		
		// Ammo Script
		if(isObject(%obj.client))
			%obj.client.updateAmmo();
	}
	
	// Resets highjacking.
	function Vehicle::mountObject(%obj, %player, %slot)
	{
		if(getTrustLevel(%obj, %player) == 3)
			%obj.crpgHijacked = false;
		
		return parent::mountObject(%obj, %player, %slot);
	}
	
	// Tumble Prevention && Click-Mounting
	function Vehicle::onActivate(%obj, %player, %client, %pos, %dir)
	{
		if(!%obj.getAbsVelocity() && getWord(%player.getWorldBox(), 2) < getWord(%obj.getWorldBox(), 5))
		{
			%db = %obj.getDatablock();
			%vSpeed = %obj.getAbsVelocity();
			
			echo(%vSpeed);
			if(%db.numMountPoints && %vSpeed <= %db.minImpactSpeed)
			{
				if(!%obj.crpgLocked)
				{
					%mounted = false;
					
					for(%a = 0; %db.numMountPoints; %a++)
					{
						if(!isObject(%obj.getMountedObject(%a)))
						{
							%obj.mountObject(%player, %a);
							%mounted = true;
							break;
						}
					}
					
					if(!%mounted)
					{
						commandToClient(%client, 'CenterPrint', "\c6Vehicle is Full", 2);
					}
				}
				else
				{
					commandToClient(%client, 'CenterPrint', "\c6Vehicle is Locked.", 2);
				}
			}
		}
	}
	
	// Rewrites entire onCollision script.
	function Vehicle::onCollision(%this, %obj, %col, %vec, %speed)
	{
		// Can we damage
		if(miniGameCanDamage(%obj, %col))
		{
			%oVel = %obj.getVelocity();
			%cVel = %col.getVelocity();
			
			%difVel = vectorSub(%oVel, %cVel);
			%colSpeed = mAbs(getWord(%difVel, 0)) + mAbs(getWord(%difVel, 1)) + mAbs(getWord(%difVel, 2));
			
			// Is the impact speed above minimum
			if(%speed > %this.minImpactSpeed)
			{
				%db			= %col.getDatablock();
				%pos			= %col.getPosition();
				%damage		= mFloor(%this.runOverDamageScale * ((%speed - %this.minImpactSpeed) / %this.minImpactSpeed));
				%damageType = $DamageType::Vehicle;
				
				if(%damage > 1)
				{
					%db.damage(%col, %obj, %pos, %damage, %damageType);
				}
			}
			
			%velInc = (%speed - %this.minImpactSpeed) / %this.minImpactSpeed;
			
			if(mFloor(%velInc) > 1)
			{
				%vel = %col.getVelocity();
				%vel = vectorAdd(%vel, vectorScale(%vec, %velInc));
				
				%col.setVelocity(%vel);
			}
		}
	}
};