// ============================================================
// Section 1 : ProfileManagerSO
// ============================================================
// Constructor - Initiates backup schedules and adds properties.
// @null
function ProfileManagerSO::onAdd(%so)
{
	// Back-Up
	%so.backupEvery = mFloor(%so.backupEvery);
	
	if(%so.backupEvery > 0)
		%so.backupSched = %so.schedule(%so.backupEvery * 60000, "backupProfiles");
	
	// SimGroup
	%so.group = new SimGroup();
	
	// Properties
	%so.properties = 0;
	
	// name, default, is object ids?, hide from client?)
	// Record Keeping
	%so.addProperty("name", "", false, true);
	%so.addProperty("ip", "", false, true);
}

// Destructor - Initiates backup before dying.
// @null
function ProfileManagerSO::onRemove(%so)
{
	%so.backupProfiles();
	%so.group.delete();
}

// Adds a property.
// @bool
function ProfileManagerSO::addProperty(%so, %name, %initial, %id, %hidden)
{
	if(%name !$= "")
	{
		%so.property[%so.properties]	= %name;
		%so.defaults[%so.properties]	= %initial;
		%so.objectID[%so.properties]	= (%id ? true : false);
		%so.hideData[%so.properties]	= %hidden;
		%so.defaultsByName[%name]		= %initial;
		%so.properties++;
		
		return true;
	}
	else
		error("ID" SPC %so.properties SPC "has no name.");
	
	return false;
}

// Saves all active profiles.
// Note: Function can be called out of sync with backup schedules for on-demand saving.
// @int (# of profiles saved successfully)
function ProfileManagerSO::backupProfiles(%so)
{
	if(ClientGroup.getCount())
	{
		//echo("\c5Backing up Profiles.");
		
		%count = 0;
		
		for(%a = 0; %a < ClientGroup.getCount(); %a++)
		{
			%client = ClientGroup.getObject(%a);
			
			if(%so.isProfileActive(%client.bl_id))
				%count += %so.saveProfile(%client.bl_id);
		}
	}
	
	if(!isEventPending(%so.backupSched))
	{
		%so.backupEvery = mFloor(%so.backupEvery);
		
		if(%so.backupEvery > 0)
			%so.backupSched = %so.schedule(%so.backupEvery * 60000, "backupProfiles");
	}
	
	return %count;
}

// Deletes a scriptobject after saving information
// @bool
function ProfileManagerSO::closeProfile(%so, %id)
{
	if(%so.isBL_ID(%id))
		if(%so.isProfileActive(%id))
		{
			%so.saveProfile(%id);
			%so.getProfile(%id).delete();
			
			if(!%so.isProfileActive(%id))
				return true;
			else
				error("ERROR: Failed to close profile for BL_ID (" @ %id @ ").");
		}
		else
			return false;
	else
		error("ERROR: Supplied invalid BL_ID (" @ %id @ ").");
	
	return false;
}

// Creates a default profile for %id.
// @bool
function ProfileManagerSO::createProfile(%so, %id)
{
	if(%so.isBL_ID(%id))
		if(!%so.isProfile(%id))
		{
			for(%a = 0; %a < %so.properties; %a++)
				eval(%so.wildcard @ %so.property[%a] @ " = \"" @ %so.defaults[%a] @ "\";");
			
			export(%so.wildcard @ "*", %so.filePath @ %id @ ".cs");
			deleteVariables(%so.wildcard @ "*");
			
			if(%so.isProfile(%id))
				return true;
			else
				error("ERROR: Failed to create profile for BL_ID (" @ %id @ ").");
		}
		else
			return false;
	else
		error("ERROR: Supplied invalid BL_ID (" @ %id @ ").");
	
	return false;
}

// Deletes %id's profile save and scriptobject (if applicable).
// @bool
function ProfileManagerSO::deleteProfile(%so, %id)
{
	if(%so.isBL_ID(%id))
		if(%so.isProfile(%id))
		{
			if(%so.isProfileActive(%id))
				%so.getProfile(%id).delete();
			
			fileDelete(%so.filePath @ %id @ ".cs");
			
			if(!%so.isProfile(%id) && !%so.isProfileActive(%id))
				return true;
			else
				error("ERROR: Failed to delete profile for BL_ID (" @ %id @ ") (method 1).");
		}
		else
		{
			if(%so.isProfileActive(%id))
				%so.getProfile(%id).delete();
			
			if(!%so.isProfile(%id) && !%so.isProfileActive(%id))
				return true;
			else
				error("ERROR: Failed to delete profile for BL_ID (" @ %id @ ") (method 2).");
		}
	else
		error("ERROR: Supplied invalid BL_ID (" @ %id @ ").");
	
	return false;
}

// Dumps a list of profiles for diagnostics.
// @null
function ProfileManagerSO::dumpProfiles(%so)
{
	echo("\n\c5== Dumping Profiles ==");
	
	for(%a = 0; %a < ClientGroup.getCount(); %a++)
	{
		%client = ClientGroup.getObject(%a);
		
		if(%so.isProfileActive(%client.bl_id))
			echo(" +- "@%client.name SPC "|" SPC %client.bl_id SPC "|" SPC %so.getProfile(%client.bl_id).getName());
		else
			echo(" +- "@%client.name SPC "|" SPC %client.bl_id SPC "| No Profile");
	}
	
	echo("\n");
}

// Returns column names and data, properly formatted for MySQL.
// @string
function ProfileManagerSO::fetchMySQL(%so)
{
	%data = "bl_id int";
	
	for(%a = 0; %a < %so.properties; %a++)
		%data = %data @ ", " @ %so.property[%a] @ " varchar(255)";
	
	$CRPG_Temp::MySQL = "CREATE TABLE Persons(" @ %data @ ");";
	export("$CRPG_Temp::MySQL", "config/server/CityRPG/MySQL.txt");
}

// Attempts to load or create a profile for the BL_ID.
// @profile OR int (error)
function ProfileManagerSO::fetchProfile(%so, %id)
{
	if(!%so.isProfileActive(%id))
		if(%so.isProfile(%id))
			if(%so.loadProfile(%id))
				return %so.getProfile(%id);
			else
				return -2;
		else
			if(%so.createProfile(%id))
				if(%so.loadProfile(%id))
					return %so.getProfile(%id);
				else
					return -2;
			else
				return -1;
	else
		return %so.getProfile(%id);
}

// Fetches %id's profile if it exists.
// @profile OR false
function ProfileManagerSO::getProfile(%so, %id)
{
	if(%so.isBL_ID(%id))
		return "Profile_" @ %id;
	else
		error("ERROR: Supplied invalid BL_ID (" @ %id @ ").");
	
	return false;
}

// Fetches the default value of property name.
// @any value
function ProfileManagerSO::getDefaultValue(%so, %name)
{
	for(%a = 0; %a < %so.properties; %a++)
		if(%so.property[%a] $= %name)
			return %so.defaults[%a];
}

// Tests to see if %id is a valid BL_ID.
// @boolean
function ProfileManagerSO::isBL_ID(%so, %id)
{
	if(mFloatLength(%id, 0) !$= %id)
		return false;
	
	if(%id < 0)
		return false;
	
	return true;
}

// Determiens if %id has a save.
// @boolean
function ProfileManagerSO::isProfile(%so, %id)
{
	if(%so.isBL_ID(%id))
		if(isFile(%so.filePath @ %id @ ".cs"))
			return true;
		else
			return false;
	else
		error("ERROR: Supplied invalid BL_ID (" @ %id @ ").");
	
	return false;
}

// Determines if %id has a scriptobject.
// @bool
function ProfileManagerSO::isProfileActive(%so, %id)
{
	if(%so.isBL_ID(%id))
		return isObject(%so.getProfile(%id));
	
	return false;
}

// Loads %id's profile from their savefile and inputs properties to scriptobject.
// @profile OR false
function ProfileManagerSO::loadProfile(%so, %id)
{
	if(%so.isBL_ID(%id))
		if(%so.isProfile(%id))
			if(!%so.isProfileActive(%id))
			{
				exec(%so.filePath @ %id @ ".cs");
				
				%profile = new ScriptObject("Profile_" @ %id)
				{
					superclass = "ProfileSO";
					class = %so.profileClass;
					
					parent = %so.getID();
					isProfile = true;
					
					bl_id = %id;
				};
				
				%so.group.add(%profile);
				
				for(%a = 0; %a < %so.properties; %a++)
				{
					eval("%property = " @ %so.wildcard @ %so.property[%a] @ ";");
					
					if(%property $= "")
						%property = %so.defaults[%a];
					
					eval(%profile.getID() @ "." @ %so.property[%a] @ " = \"" @ %property @ "\";");
				}
				
				deleteVariables(%so.wildcard @ "*");
				
				if(isObject(%client = findClientByBL_ID(%id)))
				{
					%profile.name	= %client.name;
					%profile.ip		= addFieldInto(%profile.ip, %client.getRawIP());
				}
				
				return %profile;
			}
			else
				return %so.getProfile(%id);
		else
			return false;
	else
		error("ERROR: Supplied invalid BL_ID (" @ %id @ ").");
	
	return false;
}

// Exports %id's scriptobject's properties to their savefile.
// @bool (success)
function ProfileManagerSO::saveProfile(%so, %id)
{
	if(%so.isBL_ID(%id))
		if(%so.isProfileActive(%id))
		{
			%profile = %so.getProfile(%id);
			
			for(%a = 0; %a < %so.properties; %a++)
				eval(%so.wildcard @ %so.property[%a] @ " = " @ %profile.getID() @ "." @ %so.property[%a] @ ";");
			
			export(%so.wildcard @ "*", %so.filePath @ %id @ ".cs");
			deleteVariables(%so.wildcard @ "*");
			
			return true;
		}
		else
			return false;
	else
		error("ERROR: Supplied invalid BL_ID (" @ %id @ ").");
	
	return false;
}


// ============================================================
// Section 2 : ProfileSO
// ============================================================
// This is the pseudo-class that all profile datums belong to.

// @null
function ProfileSO::onAdd(%so)
{
	
}

// Fetches MySQL String for Profile.
// @string
function ProfileSO::fetchMySQL(%so)
{
	%file = %so.parent.filePath @ %so.bl_id @ ".cs";
	
	if(!isFile(%file))
		return;
	
	%fo = new FileObject();
	
	if(!isObject(%fo) || !%fo.openForRead(%file))
		return;
	
	%fields = "";
	%data = "";
	
	while(!%fo.isEOF())
	{
		%line = %fo.readLine();
		%line = strReplace(%line, %so.parent.wildcard, "");
		%line = strReplace(%line, "\";", "\"");
		%line = strReplace(%line, " = \"", "\t\"");
		
		%field = getField(%line, 0);
		%datum = getField(%line, 1);
		
		if(%field !$= "")
		{
			%fields = addField(%fields, %field);
			%data = addField(%data, %datum);
		}
	}
	
	%fieldCount = getFieldCount(%fields);
	%datumCount = getFieldCount(%data);
	
	if(!%fieldCount || %fieldCount != %datumCount)
		return;
	
	%colText = "";
	%valText = "";
	
	for(%a = 0; %a < %fieldCount; %a++)
	{
		%colText = %colText @ getField(%fields, %a) @ ", ";
		%valText = %valText @ getField(%data, %a) @ ", ";
		
		if(%a + 1 == %fieldCount)
		{
			%colText = %colText @ "bl_id";
			%valText = %valText @ %so.bl_id;
		}
	}
	
	return "&blid=" @ %so.bl_id @ "&cols=" @ %colText @ "&fields=" @ %valText;
}

// Gets the first client with the same BL_ID.
// @GameConnection
function ProfileSO::getClient(%so)
{
	%client = %so.client;
	
	if(!isObject(%client))
	{
		%client = findClientByBL_ID(%so.bl_id);
		
		if(!isObject(%client))
		{
			%client = -1;
		}
	}
	
	return %client;
}

// Messages all clients with the Profile's BL_ID.
// @null
function ProfileSO::messageClients(%so, %type, %message)
{
	for(%a = 0; %a < ClientGroup.getCount(); %a++)
	{
		%client = ClientGroup.getObject(%a);
		
		if(%client.bl_id == %so.bl_id)
		{
			messageClient(%client, %type, %message);
		}
	}
}

// Quick-hand function to save the profile.
// @boolean
function ProfileSO::saveProfile(%so)
{
	return %so.parent.saveProfile(%so.bl_id);
}
