// ============================================================
// File Name		:	ScriptObjects
// Description		:	ScriptObjects, SimSets, SimGroups, oh my!
// ============================================================
// Table of Contents
// 
// 1. Profiler
// 2. Profiles
// 3. Calendar
// 4. Crime
// 5. Outfit
// 6. Jobs
// 7. Missions
// 8. Restrictions
// 9. Sales
// 10. Spawns
// 11. System
// 12. TCPObject
// 13. Trigger
// ============================================================

// Mission Cleanup
if(!isObject(CityCleanup))
{
	new SimGroup(CityCleanup);
	CityCleanup.add(new SimGroup(CityGroups));
	CityCleanup.add(new SimGroup(CityScripts));
	
	if(isObject(LotSS))
		CityScripts.add(LotSS);
}

// ============================================================
// Section : Profiler
// ============================================================

// Constructor - Adds specific properties.
// @null
function Profiler::onAdd(%so)
{
	parent::onAdd(%so);
	
	// Looks
	%so.addProperty("faceName", "", false, true);
	%so.addProperty("headColor", "", false, true);
	%so.addProperty("outfit", "", false, true);
	
	// Basic
	%so.addProperty("gender", 0, false, false);
	%so.addProperty("problem", "", false, true);
	%so.addProperty("hunger", 100, false, false);
	%so.addProperty("reincarnate", false, false, false);
	
	// Elections
	%so.addProperty("votedOn", "0/0/0", false, true);
	%so.addProperty("votesFor", 0, false, true);
	%so.addProperty("votedFor", -1, false, true);
	%so.addProperty("campaign", 0, false, true);
	
	// Money
	%so.addProperty("money", 20, false, false);
	%so.addProperty("bank", 100, false, false);
	%so.addProperty("debt", 0, false, false);
	%so.addProperty("plastic", 0, false, false);
	%so.addProperty("refplastic", 0, false, false);
	
	// Education
	%so.addProperty("education", 0, false, false);
	%so.addProperty("student", 0, false, false);
	%so.addProperty("job", 10101, false, false);
	%so.addProperty("workhours", 0, false, false);
	
	// Property
	%so.addProperty("tools", "hammerItem" TAB "wrenchItem" TAB "printGun", true, true);
	%so.addProperty("safe", "", true, true);
	%so.addProperty("garage", "", true, false);
	%so.addProperty("lostv", "", true, true);
	
	// Property +- Ammunition
	%so.addProperty("ammoSniper", 0, false, true);
	
	// Criminal Record
	%so.addProperty("felon", false, false, false);
	%so.addProperty("sentence", 0, false, false);
	%so.addProperty("jailvalue", 0, false, true);
	%so.addProperty("jailtime", 0, false, false);
	%so.addProperty("demerits", 0, false, false);
	%so.addProperty("stolenMoney", "", false, false);
	
	// Guilt Tripping
	%so.addProperty("timeTotal", 0, false, true);
	%so.addProperty("timeToday", 0, false, true);
	%so.addProperty("timeDay", getWord(getDateTime(), 0), false, true);
	
	// Job Experience
	%so.addProperty("expBum", 0, false, false);
	%so.addProperty("expDrone", 0, false, false);
	%so.addProperty("expLabor", 0, false, false);
	%so.addProperty("expStore", 0, false, false);
	%so.addProperty("expLaw", 0, false, false);
	%so.addProperty("expPC", 0, false, false);
	
	// Title Properties
	%so.addProperty("deaths", 0, false, true);
	%so.addProperty("donated", 0, false, true);
	%so.addProperty("kills", 0, false, true);
	%so.addProperty("stolen", 0, false, true);
	%so.addProperty("timeserved", 0, false, true);
	%so.addProperty("votes", 0, false, true);
	
	// Beliefs
	%so.addProperty("beliefChangeTime", 0, false, true);
	%so.addProperty("beliefOrder", 6, false, false);
	%so.addProperty("beliefPeace", 6, false, false);
	%so.addProperty("beliefCommercial", 6, false, false);
	%so.addProperty("beliefNight", 6, false, false);
	%so.addProperty("beliefTradition", 6, false, false);
	
	// Award Properties
	%so.addProperty("awardBeta", -1, false, true);
	%so.addProperty("awardContributor", false, false, true);
	%so.addProperty("awardFemale", false, false, true);
}

// Constructs Profiler ScriptObject with Properties.
if(!isObject(Profiler))
{
	new ScriptObject(Profiler)
	{
		class = "ProfileManagerSO";
		
		// Script namespace for generated profiles.
		profileClass = "CityProfileSO";
		
		// Directory of which Profiler accesses profile files.
		filePath = $CRPG::Path::Saving @ "Civilians/";
		
		// Does an automatic backup of files every X minutes. (0 for off)
		backupEvery = 1;
		
		// Wildcard used to save and delete profile properties.
		wildcard = "$CRPG_Temp::Profile::";
	};
	
	CityScripts.add(Profiler);
}

// ============================================================
// Section : CityProfileSO
// ============================================================
// This is the pseudo-class that all profile datums belong to.

// Adds beta tester award.
// @null
function CityProfileSO::onAdd(%so)
{
	parent::onAdd(%so);
	
	if(%so.awardBeta == -1)
		if(RestrictionSO.isBeta(%so.bl_id))
			%so.awardBeta = true;
		else
			%so.awardBeta = false;
}

// Reduces incoming damage.
// @bool
function CityProfileSO::getArmor(%so)
{
	%armor = 0;
	%armor += (0.02 * %so.getBeliefPeace());
	%armor += %so.getJob().getArmor();
	
	return %armor;
}

// Calculates the ticks a player will serve (in hours).
// @integer
function CityProfileSO::getArrestTime(%so)
{
	%time = %so.getWantedLevel() * 24;
	%time -= mFloor( (%time / 2) * (%so.getBeliefAnarchy() / 6));
	
	return %time;
}

// Fetches the player's data string.
// @string
function CityProfileSO::getData(%so, %client)
{
	for(%a = 0; %a < %so.parent.properties; %a++)
	{
		%name = %so.parent.property[%a];
		
		if(!%so.parent.hideData[%a])
		{
			eval("%value = " @ %so.getID() @ "." @ %so.parent.property[%a] @ ";");
			
			if(%so.parent.objectID[%a])
			{
				for(%b = 0; %b < getFieldCount(%value); %b++)
				{
					%field = getField(%value, %b);
					
					if(isObject(%field))
						%value = setField(%value, %b, %field.getID());
					else
						%value = setField(%value, %b, 0);
				}
			}
			
			%value = strReplace(%value, "\t", " ");
			
			if(%a)
			{
				%names = %names TAB %name;
				%values = %values TAB %value;
			}
			else
			{
				%names = %name;
				%values = %value;
			}
		}
	}
	
	// Special Values
	%names = %names TAB "wantedlevel";
	%values = %values TAB %so.getWantedLevel();
	
	%names = %names TAB "jobexp";
	%values = %values TAB %so.exp[%so.getJob().expTree];
	
	%names = %names TAB "jobname";
	%values = %values TAB %so.getJob().name;
	
	if(isObject(%client.player.crpgLot.trigger))
	{
		%triggerDB = %client.player.crpgLot.trigger.getDatablock();
		
		if(%triggerDB.jailTrigger)
		{
			%lotOwner = "<color:ff7f00>Prison";
		}
		else
		{
			%lotGroup = %client.player.crpgLot.getGroup();
			%lotClient = %lotGroup.client;
			%lotBLID = %lotGroup.bl_id;
			
			if(isObject(%lotClient))
			{
				%lotOwner = "<color:ffff00>" @ %lotClient.name;
			}
			else
			{
				if(%so.parent.isProfileActive(%lotBLID))
				{
					%lotOwner = "<color:ffff00>" @ %so.parent.getProfile(%lotBLID).name;
				}
				else
				{
					%lotOwner = "<color:0000ff>BL_ID" SPC %lotBLID;
				}
			}
		}
		
		%names = %names TAB "lotowner";
		%values = %values TAB %lotOwner;
	}
	else
	{
		%names = %names TAB "lotowner";
		%values = %values TAB "";
	}
	
	// Job Add-Ins
	%addendum = %so.getJob().getDataAddendum(%so, %client);
	
	if(%addendum !$= "")
	{
		%names = %names TAB getLine(%addendum, 0);
		%values = %values TAB getLine(%addendum, 1);
	}
	
	%data = %names NL %values;
	
	return %data;
}

// Fetches the player's job.
// @JobSO
function CityProfileSO::getJob(%so)
{
	JobsSO.getJobSO(%so.job);
}

// Fetches the player's outfit.
// @OutfitSO
function CityProfileSO::getOutfit(%so)
{
	if(!%so.isInmate())
	{
		%job = %so.getJob();
		
		if(isObject(%job))
		{
			%outfit = %job.outfit;
			
			if(isObject(%outfit))
				return %outfit;
			else
				return OutfitDefaultSO;
		}
		else
			return OutfitDefaultSO;
	}
	else
		return OutfitInmateSO;
}

// Calculates the amount of money needed to upgrade the campaign level. TODO: Buff by change.
// @int
function CityProfileSO::getCampaignMoneyRequired(%so) {
	return mFloor(100 * mPow(1.5, %so.campaign));
}

// Determines what currency denominations %so has, then picks one at random.
// @int
function CityProfileSO::getRandomBill(%so)
{
	if(%so.money >= 100) %max = 6;
	else if(%so.money >= 50) %max = 5;
	else if(%so.money >= 20) %max = 4;
	else if(%so.money >= 10) %max = 3;
	else if(%so.money >= 5) %max = 2;
	else if(%so.money >= 1) %max = 1;
	else %max = 0;
	
	%fetch = getRandom(0, %max);
	
	switch(%fetch)
	{
		case 1: return 1;
		case 2: return 5;
		case 3: return 10;
		case 4: return 20;
		case 5: return 50;
		case 6: return 100;
		default: return 0;
	}
}

// Determines title.
// @string
function CityProfileSO::getTitle(%so)
{
	return;
	
	switch(%so.bl_id % 4)
	{
		case 0: %title = "Generic";
		case 1: %title = "Dull";
		case 2: %title = "Boring";
		case 3: %title = "Common";
	}
	
	if(%so.education > 6)
		%title = "Genius";
	else if(%so.education > 2)
		%title = "Bright";
	else if(%so.education > 0)
		%title = "Educated";
	
	return "\c3 the " @ %title;
}

// Calcultes wanted level on a scale of 0 to 6.
// @integer
function CityProfileSO::getWantedLevel(%so)
{
	%dems = %so.demerits;
	
	if(%dems == 0)
		return 0;
	else if(%dems < 100)
		return 1;
	else if(%dems < 250)
		return 2;
	else if(%dems < 500)
		return 3;
	else if(%dems < 750)
		return 4;
	else if(%dems < 1000)
		return 5;
	else if(%dems < 10000)
		return 6;
	else if(%dems < 15000)
		return 7;
	else if(%dems < 25000)
		return 8;
	else if(%dems < 50000)
		return 9;
	else if(%dems < 75000)
		return 10;
	else if(%dems < 100000)
		return 11;
	else
		return 12;
}

// Determines if the profile takes headshots.
// @bool
function CityProfileSO::isHeadProtected(%so)
{
	if(%so.getJob().isHeadProtected())
		return true;
	
	return false;
}

// Checks BL_ID against host's.
function CityProfileSO::isHost(%so)
{
	return (%so.bl_id == getNumKeyID());
}

// Returns if %so is an inmate.
// @bool
function CityProfileSO::isInmate(%so)
{
	return (%so.sentence != 0 ? true : false);
}

// Returns if %so is an inmate for life.
// @bool
function CityProfileSO::isInmateLife(%so)
{
	return (%so.sentence == -1 ? true : false);
}

// Checks BL_ID against mayor's.
// @bool
function CityProfileSO::isMayor(%so)
{
	return (%so.bl_id == SystemSO.getMayor().bl_id);
}

// Deducts money from the user in this manner:
// - Removes from Wallet.
// - Remainder is removed from bank.
// - Remainder is added to debt.
// @null
function CityProfileSO::removeFunds(%so, %amount, %debt)
{
	if(mFloatLength(%amount, 0) $= %amount)
	{
		%so.money -= %amount;
		
		if(%so.money < 0)
		{
			%so.bank -= mAbs(%so.money);
			%so.money = 0;
			
			if(%so.bank < 0)
			{
				if(%debt)
				{
					%so.debt += mAbs(%so.bank);
				}
				
				%so.bank = 0;
			}
		}
	}
	else
		error("%amount is not valid integer. (" @ %amount @ ")");
}

// Properly alters Profile's demerits.
// @null
function CityProfileSO::setDemerits(%so, %amt)
{
	%so.demerits = mClamp(%amt, 0, 10000);
	
	if(isObject(%client = %so.getClient()))
	{
		%obj = %client.player;
		
		if(isObject(%obj))
		{
			%obj.setShapeNameColor("1 1 1 1");
			%obj.setShapeNameDistance(-1);
		}
		
		%client.crpgEvil = (%so.demerits > 0);
		%so.setVillainy(%client.crpgEvil);
		%client.sendStatus();
		
		if(%client.crpgLastWantedLevel != %so.getWantedLevel())
		{
			%client.crpgLastWantedLevel = %so.getWantedLevel();
			
			if(%so.getWantedLevel() >= 6)
			{
				messageAll('', '\c0Attention\c6: Elite Criminal \c3%1\c6 has achieved \c0Wanted Level %2\c6. Lethal Force from all civilians is authorized.',
					%client.name, %so.getWantedLevel());
			}
		}
	}
}

// Properly alters Profile's demerits for a life sentence.
// @null
function CityProfileSO::setDemeritsMax(%so)
{
	%so.demerits = 1000000;
	JobRogueSO.setJobFor(%so, false);
	
	if(isObject(%client = %so.getClient()))
	{
		%obj = %client.player;
		
		if(isObject(%obj))
		{
			%obj.setShapeNameColor("1 1 1 1");
			%obj.setShapeNameDistance(-1);
			%so.setVillainy(%client.crpgEvil);
		}
		
		%client.sendStatus();
		
		if(%client.crpgLastWantedLevel != %so.getWantedLevel())
		{
			%client.crpgLastWantedLevel = %so.getWantedLevel();
			messageAll('', '\c0WARNING\c6: Extremely dangerous criminal \c3%1\c6 has achieved \c0Wanted Level %2\c6. Report all sightings of this lunatic!', %client.name, %so.getWantedLevel());
		}
	}
}

// Properly sets the Profile as an inmate.
// @null
function CityProfileSO::setInmate(%so, %multiplier)
{
	if(%wanted = %so.getWantedLevel())
	{
		%multiplier = mClamp(%multiplier, 1, %multiplier);
		
		for(%a = 0; %a < CluesSS.getCount(); %a++)
		{
			%clue = CluesSS.getObject(%a);
			
			if(%clue.clueBLID == %so.bl_id)
			{
				%so.demerits += %clue.clueDems;
				SystemSO.unsolved--;
				SystemSO.solved++;
				%clue.delete();
				%a--;
			}
		}
		
		%so.sentence	+= (%wanted < 12 ? (%so.getArrestTime() * %multiplier) : -1);
		%so.demerits	= 0;
		%so.felon		= (%so.sentence > 24) ? true : %so.felon;
		%so.tools 		= "";
		SystemSO.addInmate(%so.bl_id);
		
		%job = %so.getJob();
		
		if(%job.cleanRecord)
		{
			if(%so.felon)
			{
				%defJob = JobsSO.getJobSO(%so.parent.defaultsByName["Job"]);
				
				if(isObject(%defJob))
				{
					for(%a = 0; %a < ClientGroup.getCount(); %a++)
					{
						%client = ClientGroup.getObject(%a);
						
						if(%client.bl_id == %so.bl_id)
						{
							%defJob.setJobFor(%client, true);
						}
					}
				}
			}
		}
	}
}

// Sets user's villainy.
// @bool
function CityProfileSO::setVillainy(%so, %villainy)
{
	%client	= findClientByBL_ID(%so.bl_id);
	%evil		= %client.crpgEvil;
	
	if (%so.getWantedLevel())
		%evil = true;
	
	else if(%so.isInmate() || %so.getJob().doNoEvil)
		%evil = false;
	
	else
		%evil = %villainy;
	
	%client.crpgEvil = %evil;
	commandToClient(%client, 'CRPGVillainy', %client.crpgEvil);
}

// @null
function CityProfileSO::setVoreFor(%so)
{
	echo("Om nom nom.");
}

// Sets a user's vote.
// @bool (success)
function CityProfileSO::setVoteFor(%so, %blid)
{
	%date = getWord(getDateTime(), 0);
	%newVote = 0;
	
	if(%so.votedOn !$= %date)
	{
		%newVote = 1;
		%so.votesFor = 0;
		%so.votedFor = -1;
	}
	
	%candidate = %so.parent.loadProfile(%blid);
	
	if(isObject(%candidate))
	{
		if(%so.votedFor != -1)
		{
			%prevVote = %so.parent.loadProfile(%so.votedFor);
			
			if(isObject(%prevVote))
			{
				%prevVote.votesFor--;
			}
			
			%so.votedFor = -1;
		}
		
		if(%candidate.votedOn !$= %date)
		{
			%candidate.votesFor = 0;
			%candidate.votedFor = -1;
		}
		
		%candidate.votedOn = %date;
		%candidate.votesFor++;
		
		%so.votedOn = %date;
		%so.votedFor = %candidate.bl_id;
		%so.votes += %newVote;
		
		addFieldInto(SystemSO.elecList, %candidate.bl_id);
		
		return true;
	}
	
	return false;
}


// Belief System
// objects.cs CRPG_HandcuffsU_Image::onHitObject
// objects.cs CRPG_Baton_Image::onRaycastDamage
// scriptobjects.cs CityProfileSO::getArrestTime
function CityProfileSO::getBeliefAnarchy(%so)
{
	return (%so.belief["Order"] - 6) / -5;
}

// functions.cs serverCmdStats_ChangeBelief
function CityProfileSO::getBeliefChange(%so)
{
	return (%so.belief["Tradition"] - 6) / -5;
}

function CityProfileSO::getBeliefCommercial(%so)
{
	return (%so.belief["Commercial"] - 6) / 5;
}

// functions.cs Player::onAutoHeal
// package.cs Player::setShapeNameDistance
function CityProfileSO::getBeliefDay(%so)
{
	return (%so.belief["Night"] - 6) / -5;
}

function CityProfileSO::getBeliefIndustry(%so)
{
	return (%so.belief["Commercial"] - 6) / -5;
}

// functions.cs Player::onAutoHeal
// package.cs Player::setShapeNameDistance
function CityProfileSO::getBeliefNight(%so)
{
	return (%so.belief["Night"] - 6) / 5;
}

// objects.cs CRPG_HandcuffsU_Image::onHitObject
// objects.cs CRPG_Baton_Image::onRaycastDamage
function CityProfileSO::getBeliefOrder(%so)
{
	return (%so.belief["Order"] - 6) / 5;
}

// scriptobjects.cs CityProfileSO::getArmor
function CityProfileSO::getBeliefPeace(%so)
{
	return (%so.belief["Peace"] - 6) / 5;
}

function CityProfileSO::getBeliefTradition(%so)
{
	return (%so.belief["Tradition"] - 6) / 5;
}

// functions.cs WeaponImage::onRaycastDamage
function CityProfileSO::getBeliefWar(%so)
{
	return (%so.belief["Peace"] - 6) / -5;
}

// ============================================================
// Section : Calendar
// ============================================================
function CalendarSO::onAdd(%so)
{
	%so.tickSched = %so.schedule(%so.cycle * 60 * 1000, "tick");
}

function CalendarSO::announceHoliday(%so, %day, %month, %year)
{
	%annual = %so.holiday[%day, %month];
	
	if(%annual !$= "")
	{
		messageAll('', "\c6 - " @ %annual);
		%eval = %so.eval[%day, %month];
		
		if(%eval !$= "")
			eval(%eval);
	}
}

function CalendarSO::getDaysInSeconds(%so, %days)
{
	%days = mClamp(%days, 1, %days);
	return (60 * %so.cycle * (24 / %so.length)) * %days;
}

function CalendarSO::getDOM(%so)
{
	%dom = %so.getDOY();
	
	for(%month = 0; %month < %so.months; %month++)
	{
		if(%dom - %so.days[%month] < 1)
			break;
		else
			%dom -= %so.days[%month];
	}
	
	return %dom;
}

function CalendarSO::getDOW(%so)
{
	%time = %so.time + 24;
	%day = mFloor(%time / 24);
	%dow = %day % 7;
	
	return %dow;
}

function CalendarSO::getDOY(%so)
{
	%time = %so.time + 24;
	%day = mFloor(%time / 24);
	%doy = %day % 365;
	
	return %doy;
}

function CalendarSO::getMonth(%so)
{
	%dom = %so.getDOY();
	
	for(%month = 0; %month < %so.months; %month++)
	{
		if(%dom - %so.days[%month] < 1)
			break;
		else
			%dom -= %so.days[%month];
	}
	
	return %month;
}

function CalendarSO::getYear(%so)
{
	%time = %so.time + 24;
	%day = mFloor(%time / 24);
	%years = mFloor(%day / 365) + 1990;
	
	return %years;
}

function CalendarSO::getTimeString(%so, %hours)
{
	%time = %so.time + 24;
	%hour = %time % 24;
	
	if(%hour >= 12)
	{
		%noon = "P.M.";
		%hour -= 12;
	}
	else
	{
		%noon = "A.M.";
	}
	
	if(%hour == 0)
		%hour = 12;
	
	%day = mFloor(%time / 24);
	%years = mFloor(%day / 365) + 1;
	
	%dow = %day % 7;
	%doy = %day % 365;
	%dom	=	%doy;
	
	for(%month = 0; %month < %so.months; %month++)
	{
		if(%dom - %so.days[%month] < 1)
			break;
		else
			%dom -= %so.days[%month];
	}
	
	if(%dom > 9 && %dom < 20)
		%suffix = "th";
	else
		switch(getsubstr(%dom, strlen(%dom) - 1, 1))
		{
			case 1: %suffix = "st";
			case 2: %suffix = "nd";
			case 3: %suffix = "rd";
			default: %suffix = "th";
		}
	
	switch(%dow)
	{
		case 0: %dow = "Monday";
		case 1: %dow = "Tuesday";
		case 2: %dow = "Wednesday";
		case 3: %dow = "Thursday";
		case 4: %dow = "Friday";
		case 5: %dow = "Saturday";
		case 6: %dow = "Sunday";
	}
	
	%timestr = "\c3%1\c6, \c3%2 %3%4\c6, \c3Year %5\c6" @ (%hours ? " at \c3%6 %7" : ".");
	%timestr = strReplace(%timestr, "%" @ %place++, %dow);
	%timestr = strReplace(%timestr, "%" @ %place++, %so.month[%month]);
	%timestr = strReplace(%timestr, "%" @ %place++, %dom);
	%timestr = strReplace(%timestr, "%" @ %place++, %suffix);
	%timestr = strReplace(%timestr, "%" @ %place++, %years);
	%timestr = strReplace(%timestr, "%" @ %place++, %hour);
	%timestr = strReplace(%timestr, "%" @ %place++, %noon);
	
	return %timestr;
}

function CalendarSO::tick(%so)
{
	%so.time	+= %so.length;
	%newDay	= mFloor(%so.time / 24) > mFloor((%so.time - %so.length) / 24) ? true : false;
	%dow		= %so.getDOW();
	
	if(MissionInfo.crpg)
	{
		%timeFactor = mFloor((%so.time % 24) / 2) + 1;
		EnvironmentSO.setTimeFactor(%timeFactor);
	}
	
	if(%newDay)
	{
		messageAll('', "\c6Today is" SPC %so.getTimeString(false));
		%so.announceHoliday(%so.getDOM(), %so.getMonth() + 1, %so.getYear());
	}
	
	for(%a = 0; %a < Profiler.group.getCount(); %a++)
	{
		%profile = Profiler.group.getObject(%a);
		%client = findClientByBL_ID(%profile.bl_id);
		
		if(isObject(%client))
		{
			// Beliefs
			if(%profile.beliefChangeTime > 0)
				%profile.beliefChangeTime -= %so.length;
			
			// Playtime Tracking
			%profile.timeTotal += mFloor(60 * %so.cycle);
			
			if(%profile.timeDay !$= getWord(getDateTime(), 0))
			{
				%profile.timeDay = getWord(getDateTime(), 0);
				%profile.timeToday = 0;
			}
			
			%profile.timeToday += mFloor(60 * %so.cycle);
			
			// If this tick sets us over a new hour benchmark, notify client.
			if(mFloor(%profile.timeToday / 60 / 60) > mFloor((%profile.timeToday - mFloor(60 * %so.cycle)) / 60 / 60))
			{
				//commandToClient(%client, 'MessageBoxOK', "", "<font:arial:16>Today, you have played CityRPG for\n" NL
				//"<font:impact:32>" @ mFloor(%profile.timeToday / 60 / 60) @ " Hour" @ (mFloor(%profile.timeToday / 60 / 60) > 1 ? "s" : ""));
				commandToClient(%client, 'Nag_SendNag', mFloor(%profile.timeToday / 60 / 60), mFloor(%profile.timeTotal / 60 / 60));
				echo(" +- " @ %client.name @ " Playtime = " @ mFloor(%profile.timeToday / 60 / 60));
			}
			
			// Cutomized Job Updates
			if(isObject(%job = %profile.getJob()))
				%job.onTick(%client);
			
			if(isObject(%player = %client.player))
			{
				// Hunger
				%profile.hunger -= mClamp(mCeil(%so.length / 2), 1, 100);
				%profile.hunger = mClamp(%profile.hunger, 0, %profile.hunger);
				
				if(%profile.isInmate())
					%profile.hunger = mClamp(%profile.hunger, 50, %profile.hunger);
				
				// Once a Day
				if(%newDay)
				{
					// Demerits
					if(%profile.demerits > 0 && %profile.getWantedLevel() < 6)
					{
						%profile.setDemerits(%profile.demerits - 50);
						
						if(%profile.demerits <= 0)
							messageClient(%client, '', "\c6 - Your demerits have been removed entirely.");
						else
							messageClient(%client, '', '\c6 - Your demerits have faded to \c3%1\c6.', %profile.demerits);
					}
					
					// Schooling
					if(%profile.student)
					{
						%profile.student--;
						
						if(%profile.student > 1 && %profile.student < 7)
						{
							messageClient(%client, '', '\c6 - There %3 \c3%1 day%2\c6 left in your semester.', %profile.student,
								(%profile.student > 1 ? "s" : ""),
								(%profile.student > 1 ? "are" : "is"));
						}
						else if(%profile.student == 1)
						{
							%profile.student = 0;
							%profile.education++;
							
							messageClient(%client, '', '\c6 - \c2You Graduated!\c6 Your Diploma level is now \c3%1\c6.', %profile.education);
						}
						else if(%profile.student % %dow == 0)
						{
							messageClient(%client, '', '\c6 - There %3 \c3%1 week%2\c6 left in your semester.', mFloor(%profile.student / 7),
								(%profile.student / %dow > 1 ? "s" : ""),
								(%profile.student / %dow > 1 ? "are" : "is"));
						}
					}
					
					// Weekly
					switch(%dow)
					{
						// Mon
						case 0:
							
						// Tue
						case 1:
							
						// Wed
						case 2:
							
						// Thur
						case 3:
							
						// Fri
						case 4:
							
						// Sat
						case 5:
							
						// Sun
						case 6:
							
					}
				}
				
				// Sun Burn
				if(%profile.getBeliefNight() == 1)
					%player.onSunBurn();
				
				// Name visibility
				%player.setShapeNameDistance(0);
			}
			
			// Jailing
			if(%profile.isInmate())
			{
				%profile.timeserved += %so.length;
				%profile.jailtime = %so.time;
				
				if(!%profile.isInmateLife())
					%profile.sentence -= %so.length;
				
				if(%profile.sentence == 0)
				{
					messageClient(%client, '', "\c6 - Your sentence has ended.");
					
					if(isObject(%player))
						%client.spawnPlayer();
				}
				else if(%newDay)
				{
					if(!%profile.isInmateLife())
					{
						%left = mFloor(%profile.sentence / 24);
						messageClient(%client, '', '\c6 - You have \c3%1 day%2\c6 left in your sentence.', %left + 1, %left > 1 ? "s" : "");
					}
					else
					{
						messageClient(%client, '', '\c6 - You are serving a \c3life sentence\c3.');
					}
				}
			}
			
			// Status Update
			%profile.name = %client.name;
			%client.sendStatus();
		}
		
		if(!isObject(%client))
		{
			// Jailing
			if(%profile.isInmate())
			{
				%profile.timeserved += %so.length;
				%profile.jailtime = %so.time;
				
				if(!%profile.isInmateLife())
					%profile.sentence -= %so.length;
				
				if(%profile.sentence == 0)
				{
					%profile.problem = addField(%profile.problem, "\c6You have gotten out of jail.");
				}
			}
			
			// Posts to Database
			//%to = new TCPObject(DatabaseTO)
			//{
			//	address = "bl.ichverbot.com:80";
			//	page = "/scripts/post.php";
			//	password = $CRPG::MySQL::Pass;
			//	port = 80;
			//};
			
			if(isObject(%to))
				%to.post(%profile.fetchMySQL());
		}
	}
	
	SystemSO.onTick(%newDay);
	
	if(!isEventPending(%so.tickSched))
		%so.tickSched = %so.schedule(%so.cycle * 60 * 1000, "tick");
}

if(!isObject(CalendarSO))
{
	new ScriptObject(CalendarSO)
	{
		cycle = 0.75;
		length = 2;
		time = 0;
		
		months = 12;
		month[0] = "January";
		month[1] = "February";
		month[2] = "March";
		month[3] = "April";
		month[4] = "May";
		month[5] = "June";
		month[6] = "July";
		month[7] = "August";
		month[8] = "September";
		month[9] = "October";
		month[10] = "November";
		month[11] = "December";
		
		days[0] = 31;
		days[1] = 28;
		days[2] = 31;
		days[3] = 30;
		days[4] = 31;
		days[5] = 30;
		days[6] = 31;
		days[7] = 31;
		days[8] = 30;
		days[9] = 31;
		days[10] = 30;
		days[11] = 31;
		
		// Heartbreak Day [Annual]
		holiday	[27, 1]	= "\c6Happy \c3Heartbreak Day\c6! \c3Iban\c6 has kicked everyone in the dick.";
		
		// Valentine's Day [Annual]
		holiday	[14, 2]	= "\c6Happy <color:ffaaaa>Valentine's Day\c6! \c3Iban\c6 has given everyone a <color:ffaaaa>Box of Chocolates\c6, adding \c2125%\c6 to their hunger!";
		eval		[14, 2]	= "CRPG_MassProfileUpdate(\"hunger\", \"+=\", 125);";
		
		// Halloween [Annual]
		holiday 	[30, 10]	= "\c3Happy Halloween!\c6 Everyone is getting \c2$100\c6!";
		eval		[30, 10]	= "CRPG_MassProfileUpdate(\"money\", \"+=\", 100);";
		
		// Patriot's Day [Annual]
		holiday	[9, 11]	= "\c3Happy Patriot's Day!\c6 Everyone blame the Muslims!";
		
		// The Creator's Birthday [Annual]
		holiday	[19, 12]	= "\c4Happy Birthday, Iban!\c6 Everyone has been granted \c2$1000\c6 in allowance money.";
		eval		[19, 12]	= "CRPG_MassProfileUpdate(\"money\", \"+=\", 1000);";
		
		// Christmas Birthday [Annual]
		holiday	[25, 12]	= "\c2M\c0e\c2r\c0r\c2y \c0C\c2h\c0r\c2i\c0s\c2t\c0m\c2a\c0s\c2!\c6 Bellies are filled and bank accounts bloated!";
		eval		[25, 12]	= "CRPG_MassProfileUpdate(\"hunger\", \"=\", \"getRandom(110, 500)\"); CRPG_MassProfileUpdate(\"bank\", \"+=\", \"getRandom(1000, 10000)\");";
	};
	
	CalendarSO.time = SystemSO.time;
	CityScripts.add(CalendarSO);
}

if(!isObject(TimeBrickSS))
{
	new SimSet(TimeBrickSS);
	CityScripts.add(TimeBrickSS);
}

// ============================================================
// Section : Crime
// ============================================================
function CrimeSO::createClue(%so, %profile, %message, %demerits, %visible, %transform, %group)
{
	// Information Sanitizing
	if(!isObject(%profile))
		return -1;
	
	if(%message $= "")
		return -1;
	
	if( (%demerits = mClamp(%demerits, 0, %demerits)) <= 0)
		return -1;
	
	%visible = (%visible ? true : false);
	
	if(!isObject(%group))
		%group = new SimGroup(Clue_Group);
	
	// Object Creation
	%clue = new Item()
	{
		datablock	= (%visible ? CRPG_Clue_Item : CRPG_Clue_Hidden_Item);
		
		isClue		= true;
		cluePerp		= %profile;
		clueBLID		= %profile.bl_id;
		clueMessage	= %message;
		clueDems		= %demerits;
		clueHidden	= !%visible;
		cluePos		= %transform;
		clueGroup	= %group;
	};
	
	// Cleanup
	if(isObject(%clue))
	{
		%clue.canPickup = false;
		%clue.setTransform(%transform);
		
		%group.add(%clue);
		CluesSS.add(%clue);
		MissionCleanup.add(%clue);
		
		return %clue;
	}
	else
	{
		echo("\c2ERROR: Could not create a clue item. Item count overload?");
		return -1;
	}
}

if(!isObject(CrimeSO))
{
	new ScriptObject(CrimeSO)
	{
		
	};
}

if(!isObject(CrimeGroups))
{
	new SimGroup(CrimeGroups);
	CityScripts.add(CrimeGroups);
}


// ============================================================
// Section : Environment
// ============================================================
function EnvironmentSO::clearSky(%so)
{
	if(MissionInfo.crpg)
	{
		if(!isObject(Precipitation))
			return;
		
		Precipitation.delete();
		
		if(!isObject(Sky))
		{
			for(%a = 0; %a < MissionGroup.getCount(); %a++)
			{
				%obj = MissionGroup.getObject(%a);
				
				if(%obj.getClassName() $= "Sky")
				{
					%obj.setName("Sky");
					break;
				}
			}
			
			if(!isObject(Sky))
				return;
		}
		
		Sky.setName("Sky_Old");
		
		new Sky(Sky : Sky_Old)
		{
			materialList = %so.getSkyMedia();
		};
		
		Sky_Old.delete();
	}
}

function EnvironmentSO::getSkyMedia(%so)
{
	if(isObject(Precipitation))
		return $CRPG::ClientPath @ "skies/Sky_Dynamic/stormy.dml";
	else
		return $CRPG::ClientPath @ "skies/Sky_Dynamic/clear.dml";
}

function EnvironmentSO::playMayorTheme(%so)
{
	for(%a = 0; %a < ClientGroup.getCount(); %a++)
		ClientGroup.getObject(%a).play2D(CRPG_Hail_Chief_Sound);
}

function EnvironmentSO::resetSky(%so)
{
	%so.setTimeFactor();
}

function EnvironmentSO::setNorthStar(%so, %visible)
{
	if(!isObject(Celest_North_Star))
		return false;
	
	Celest_North_Star.setEnable(%visible);
	
	return true;
}

function EnvironmentSO::setRain(%so)
{
	if(isObject(Precipitation))
		Precipitation.delete();
	
	new Precipitation(Precipitation)
	{
		position = "0 0 0";
		rotation = "1 0 0 0";
		scale = "1 1 1";
		
		dataBlock = "CRPG_Rain";
		
		minSpeed = "1.5";
		maxSpeed = "2";
		minMass = "0.75";
		maxMass = "0.85";
		
		maxTurbulence = "0.1";
		turbulenceSpeed = "0.2";
		rotateWithCamVel = "1";
		useTurbulence = "0";
		
		numDrops = "5000";
		
		boxWidth = "200";
		boxHeight = "100";
		doCollision = "1";
	};
	
	MissionGroup.add(Precipitation);
}

function EnvironmentSO::setSky(%so, %color)
{
	if(MissionInfo.crpg)
	{
		if(isObject(Sky))
			Sky.delete();
		
		new Sky(Sky)
		{
			// Transform
			position = "336 136 0";
			rotation = "1 0 0 0";
			scale = "1 1 1";
			
			// Media
			materialList = %so.getSkyMedia();
			
			// Clouds
			cloudHeightPer[0] = "0.4";
			cloudHeightPer[1] = "0.3";
			cloudHeightPer[2] = "0.2";
			cloudSpeed[1] = "0.0001";
			cloudSpeed[2] = "0.0002";
			cloudSpeed[3] = "0.0004";
			
			// Visibility
			visibleDistance = "500";
			
			// Fog
			fogDistance = "800";
			fogColor = %color;
			fogStorm[1] = false;
			fogStorm[2] = false;
			fogStorm[3] = false;
			fogVolume[1] = "200 100 200";
			fogVolume[2] = "400 0 100";
			fogVolume[3] = "600 200 300";
			fogVolumeColor[1] = "1 0 0";
			fogVolumeColor[2] = "0 1 0";
			fogVolumeColor[3] = "0 0 1";
			
			// Wind
			windVelocity = "1 1 0";
			windEffectPrecipitation = "1";
			
			// Misc
			SkySolidColor = %color;
			useSkyTextures = false;
			renderBottomTexture = false;
			noRenderBans = true;
		};
	}
}

function EnvironmentSO::setSnow(%so)
{
	if(isObject(Precipitation))
		Precipitation.delete();
	
	new Precipitation(Precipitation)
	{
		position = "0 0 0";
		rotation = "1 0 0 0";
		scale = "1 1 1";
		nameTag = "snow";
		dataBlock = "SnowA";
		minSpeed = "0.5";
		maxSpeed = "1";
		minMass = "0.75";
		maxMass = "0.85";
		maxTurbulence = "0.1";
		turbulenceSpeed = "0.2";
		rotateWithCamVel = "1";
		useTurbulence = "1";
		numDrops = "500";
		boxWidth = "100";
		boxHeight = "100";
		doCollision = "1";
	};
	
	MissionGroup.add(Precipitation);
}

function EnvironmentSO::setSun(%so, %color, %ambient, %elevation, %azimuth)
{
	if(MissionInfo.crpg)
	{
		if(!isObject(Sun))
		{
			for(%a = 0; %a < MissionGroup.getCount(); %a++)
			{
				%obj = MissionGroup.getObject(%a);
				
				if(%obj.getClassName() $= "Sun")
				{
					%obj.setName("Sun");
					break;
				}
			}
			
			if(!isObject(Sun))
				return;
		}
		
		%elevation = mClamp(%elevation, 0, 360);
		%azimuth = mClamp(%azimuth, 0, 360);
		
		Sun.setName("Sun_Old");
		
		%sun = new Sun(Sun)
		{
			azimuth = %azimuth;
			elevation = %elevation;
			color = %color;
			ambient = %ambient;
			rotation = "1 0 0 0";
			locked = true;
			scale = "1 1 1";
			direction = "0 0 0";
			position = "0 0 0";
		};
		
		MissionGroup.add(%sun);
		Sun_Old.delete();
	}
}

function EnvironmentSO::setSunFX(%so, %visible, %azimuth, %brightMin, %brightMax, %brightKeys, %eleMin, %eleMax, %eleKeys)
{
	if(MissionInfo.crpg)
	{
		if(!isObject(SunLight))
		{
			for(%a = 0; %a < MissionGroup.getCount(); %a++)
			{
				%obj = MissionGroup.getObject(%a);
				
				if(%obj.getClassName() $= "fxSunLight")
				{
					%obj.setName("SunLight");
					break;
				}
			}
			
			if(!isObject(SunLight))
				return;
		}
		
		SunLight.setEnable(%visible ? true : false);
		
		if(%visible)
		{
			%azimuth = mClamp(%azimuth, 0, 360);
			%eleMin = mClamp(%eleMin, 0, 360);
			%eleMax = mClamp(%eleMax, 0, 360);
			
			SunLight.setMinAzimuth(%azimuth);
			SunLight.setMaxAzimuth(%azimuth);
			
			SunLight.setMaxBrightness(%brightMin);
			SunLight.setMinBrightness(%brightMax);
			SunLight.setBrightnessKeys(brightKeys);
			SunLight.setBrightnessTime(60 * CalendarSO.cycle * CalendarSO.length);
			
			SunLight.setMinElevation(%eleMin);
			SunLight.setMaxElevation(%eleMax);
			SunLight.setElevationKeys(%eleKeys);
			SunLight.setElevationTime(60 * CalendarSO.cycle * CalendarSO.length);
		}
	}
}

function EnvironmentSO::setTimeFactor(%so, %timeFactor)
{
	if(%so.timeFactor $= "" || %so.timeFactor != %timeFactor || %timeFactor $= "")
	{
		if(%timeFactor $= "")
			%timeFactor = %so.timeFactor;
		
		%timeFactor = mClamp(%timeFactor, 1, 12);
		
		%skyColor	= "1.00 0.00 1.00";
		
		%sunColor	= "0.00 1.00 0.00";
		%sunAmbient	= "0.00 0.00 1.00";
		%sunEle		= 0;
		%sunAzimuth	= 270;
		
		%fxVisible	= false;
		%fxAzimuth	= 0;
		%fxMin		= 0;
		%fxMax		= 0;
		%fxKeys		= "";
		%eleMin		= 90;
		%eleMax		= 90;
		%eleKeys		= "AZZ";
		
		%isDay		= false;
		%isNight		= false;
		%isDawn		= false;
		%isDusk		= false;
		
		// Time-Specific Alterations
		switch(%timeFactor)
		{
			// Midnight
			case 1:
				%skyColor	= "0.050 0.100 0.200";
				
				%sunColor	= "0.020 0.050 0.050";
				%sunAmbient	= "0.000 0.075 0.100";
				
				%isNight		= true;
			
			// Wee hours
			case 2:
				%skyColor	= "0.100 0.200 0.200";
				
				%sunColor	= "0.150 0.100 0.100";
				%sunAmbient	= "0.050 0.100 0.100";
				
				%isNight		= true;
			
			// Pre-Dawn
			case 3:
				%skyColor	= "0.200 0.300 0.350";
				
				%sunColor	= "0.275 0.250 0.225";
				%sunAmbient	= "0.150 0.175 0.175";
				%sunEle		= 180;
				
				%fxVisible	= true;
				%fxAzimuth	= 270;
				%fxMin		= 0.00;
				%fxMax		= 0.50;
				%fxKeys		= "AZ";
				%eleMin		= -25;
				%eleMax		= -15;
				%eleKeys		= "AZZ";
				
				%isNight		= true;
			
			// Dawn
			case 4:
				%skyColor	= "0.300 0.400 0.500";
				
				%sunColor	= "0.400 0.400 0.350";
				%sunAmbient	= "0.250 0.250 0.250";
				%sunEle		= 160;
				
				%fxVisible	= true;
				%fxAzimuth	= 270;
				%fxMin		= 0.50;
				%fxMax		= 1.00;
				%fxKeys		= "AZ";
				%eleMin		= -15;
				%eleMax		= 20;
				%eleKeys		= "AZZ";
				
				%isDawn		= true;
			
			// Morning
			case 5:
				%skyColor	= "0.700 0.800 0.900";
				
				%sunColor	= "0.800 0.750 0.500";
				%sunAmbient	= "0.600 0.500 0.400";
				%sunEle		= 140;
				
				%fxVisible	= true;
				%fxAzimuth	= 270;
				%fxMin		= 1.00;
				%fxMax		= 1.00;
				%fxKeys		= "";
				%eleMin		= 20;
				%eleMax		= 55;
				%eleKeys		= "AZZ";
				
				%isDay		= true;
			
			// Noon
			case 6:
				%skyColor	= "0.500 0.700 0.900";
				
				%sunColor	= "0.700 0.600 0.400";
				%sunAmbient	= "0.500 0.400 0.300";
				%sunEle		= 120;
				
				%fxVisible	= true;
				%fxAzimuth	= 270;
				%fxMin		= 1.00;
				%fxMax		= 1.00;
				%fxKeys		= "";
				%eleMin		= 55;
				%eleMax		= 90;
				%eleKeys		= "AZZ";
				
				%isDay		= true;
			
			// Afternoon-ish
			case 7:
				%skyColor	= "0.550 0.700 0.850";
				
				%sunColor	= "0.700 0.600 0.400";
				%sunAmbient	= "0.500 0.400 0.300";
				%sunEle		= 100;
				
				%fxVisible	= true;
				%fxAzimuth	= 270;
				%fxMin		= 1.00;
				%fxMax		= 1.00;
				%fxKeys		= "";
				%eleMin		= 90;
				%eleMax		= 125;
				%eleKeys		= "AZZ";
				
				%isDay		= true;
			
			// Afternoon
			case 8:
				%skyColor	= "0.600 0.700 0.800";
				
				%sunColor	= "0.700 0.600 0.400";
				%sunAmbient	= "0.500 0.400 0.300";
				%sunEle		= 80;
				
				%fxVisible	= true;
				%fxAzimuth	= 90;
				%fxMin		= 1.00;
				%fxMax		= 1.00;
				%fxKeys		= "";
				%eleMin		= 20;
				%eleMax		= 55;
				%eleKeys		= "ZAA";
				
				%isDay		= true;
			
			// Late in the Day
			case 9:
				%skyColor	= "0.600 0.700 0.800";
				
				%sunColor	= "0.700 0.600 0.400";
				%sunAmbient	= "0.500 0.400 0.300";
				%sunEle		= 80;
				
				%fxVisible	= true;
				%fxAzimuth	= 90;
				%fxMin		= 1.00;
				%fxMax		= 1.00;
				%fxKeys		= "";
				%eleMin		= 20;
				%eleMax		= 55;
				%eleKeys		= "ZAA";
				
				%isDay		= true;
			
			// Twilight
			case 10:
				%skyColor	= "0.600 0.700 0.800";
				
				%sunColor	= "0.700 0.600 0.400";
				%sunAmbient	= "0.500 0.400 0.300";
				%sunEle		= 80;
				
				%fxVisible	= true;
				%fxAzimuth	= 90;
				%fxMin		= 0.00;
				%fxMax		= 0.50;
				%fxKeys		= "ZA";
				%eleMin		= -25;
				%eleMax		= -15;
				%eleKeys		= "ZAA";
				
				%isDusk		= true;
			
			// Dusk
			case 11:
				%skyColor	= "0.450 0.225 0.100";
				
				%sunColor	= "0.490 0.125 0.025";
				%sunAmbient	= "0.250 0.210 0.050";
				
				%isNight		= true;
			
			// Night
			case 12:
				%skyColor	= "0.050 0.200 0.200";
				
				%sunColor	= "0.020 0.050 0.050";
				%sunAmbient	= "0.000 0.075 0.100";
				
				%isNight		= true;
			
			default: error("EnvironmentSO::setTimeFactor(\"" @ %timeFactor @ "\") : %timeFactor is not valid.");
		}
		
		// Daylight Variables
		%so.dawn		= %isDawn;
		%so.day		= %isDay;
		%so.dusk		= %isDusk;
		%so.night	= %isNight;
		
		if(%so.dawn)
		{
			%so.setNorthStar(true);
			
			for(%a = 0; %a < TimeBrickSS.getCount(); %a++)
			{
				%brick = TimeBrickSS.getObject(%a);
				%brick.schedule(getRandom(32, 2500), "onTimeDay", %a);
			}
		}
		else if(%so.dusk)
		{
			%so.setNorthStar(false);
			
			for(%a = 0; %a < TimeBrickSS.getCount(); %a++)
			{
				%brick = TimeBrickSS.getObject(%a);
				%brick.schedule(getRandom(32, 2500), "onTimeNight", %a);
			}
		}
		
		// Transmutation
		if(SystemSO.anarchy)
		{
			
		}
		
		// Environment Application
		%so.setSky(%skyColor);
		%so.setSun(%sunColor, %sunAmbient, %sunEle, %sunAzimuth);
		%so.setSunFX(%fxVisible, %fxAzimuth, %fxMin, %fxMax, %fxKeys, %eleMin, %eleMax, %eleKeys);
		
		%so.timeFactor = %timeFactor;
	}
}

if(!isObject(EnvironmentSO))
{
	new ScriptObject(EnvironmentSO)
	{
		timeFactor = 6;
		
		day = true;
		night = false;
		dawn = false;
		dusk = false;
	};
	
	CityScripts.add(EnvironmentSO);
}

// ============================================================
// Section : Items
// ============================================================
// Clues
if(!isObject(CluesSS))
{
	new SimSet(CluesSS);
	CityScripts.add(CluesSS);
}

// Crates
if(!isObject(CrateSS))
{
	new SimSet(CrateSS);
	CityScripts.add(CrateSS);
}

// ============================================================
// Section : Outfit
// ============================================================
function OutfitSO::applyOutfit(%so, %client)
{
	%profile = %client.profile;
	
	if(isObject(%profile))
	{
		%obj = %client.player;
		
		if(isObject(%obj))
		{
			hideAllNodes(%obj);
			
			%obj.unHideNode("headskin");
			%obj.setNodeColor("headskin", %client.headColor);
			
			for(%a = 0; %a < %so.outfit; %a++)
			{
				%node = %so.node[%a];
				%color = %so.color[%a];
				
				switch$(%node)
				{
					case "gender": %node = (%profile.gender == 2 ? "femchest" : "chest");
					case "genderlarm": %node = (%profile.gender == 2 ? "LArmSlim" : "LArm");
					case "genderrarm": %node = (%profile.gender == 2 ? "RArmSlim" : "RArm");
				}
				
				if(%color $= "skin")
					%color = %client.headColor;
				
				%obj.unHideNode(%node);
				%obj.setNodeColor(%node, %color);
			}
			
			if(%profile.gender == 2)
			{
				for(%a = 0; %a < %so.outfitFem; %a++)
				{
					%node = %so.nodeFem[%a];
					%color = %so.colorFem[%a];
					
					switch$(%node)
					{
						case "gender": %node = (%profile.gender == 2 ? "femchest" : "chest");
						case "genderlarm": %node = (%profile.gender == 2 ? "LArmSlim" : "LArm");
						case "genderrarm": %node = (%profile.gender == 2 ? "RArmSlim" : "RArm");
					}
					
					if(%color $= "skin")
						%color = %client.headColor;
					
					%obj.unHideNode(%node);
					%obj.setNodeColor(%node, %color);
				}
			}
			else
			{
				for(%a = 0; %a < %so.outfitMale; %a++)
				{
					%node = %so.nodeMale[%a];
					%color = %so.colorMale[%a];
					
					switch$(%node)
					{
						case "gender": %node = (%profile.gender == 2 ? "femchest" : "chest");
						case "genderlarm": %node = (%profile.gender == 2 ? "LArmSlim" : "LArm");
						case "genderrarm": %node = (%profile.gender == 2 ? "RArmSlim" : "RArm");
					}
					
					if(%color $= "skin")
						%color = %client.headColor;
					
					%obj.unHideNode(%node);
					%obj.setNodeColor(%node, %color);
				}
			}
			
			%obj.setDecalName(%so.decal);
		}
	}
}

function OutfitSO::applyOutfitColors(%so, %client)
{
	%profile = %client.profile;
	
	if(isObject(%profile))
	{
		%obj = %client.player;
		
		if(isObject(%obj))
		{
			%obj.setNodeColor("headskin", %client.headColor);
			
			for(%a = 0; %a < %so.outfit; %a++)
			{
				%node = %so.node[%a];
				%color = %so.color[%a];
				
				if(%profile.favoriteColor !$= "" && %so.isCustomColor(%node))
					%color = %profile.favoriteColor;
				
				switch$(%node)
				{
					case "gender": %node = (%profile.gender == 2 ? "femchest" : "chest");
					case "genderlarm": %node = (%profile.gender == 2 ? "LArmSlim" : "LArm");
					case "genderrarm": %node = (%profile.gender == 2 ? "RArmSlim" : "RArm");
				}
				
				if(%color $= "skin")
					%color = %client.headColor;
				
				%obj.setNodeColor(%node, %color);
			}
			
			if(%profile.gender == 2)
			{
				for(%a = 0; %a < %so.outfitFem; %a++)
				{
					%node = %so.nodeFem[%a];
					%color = %so.colorFem[%a];
					
					switch$(%node)
					{
						case "gender": %node = (%profile.gender == 2 ? "femchest" : "chest");
						case "genderlarm": %node = (%profile.gender == 2 ? "LArmSlim" : "LArm");
						case "genderrarm": %node = (%profile.gender == 2 ? "RArmSlim" : "RArm");
					}
					
					if(%color $= "skin")
						%color = %client.headColor;
					
					%obj.setNodeColor(%node, %color);
				}
			}
			else
			{
				for(%a = 0; %a < %so.outfitMale; %a++)
				{
					%node = %so.nodeMale[%a];
					%color = %so.colorMale[%a];
					
					switch$(%node)
					{
						case "gender": %node = (%profile.gender == 2 ? "femchest" : "chest");
						case "genderlarm": %node = (%profile.gender == 2 ? "LArmSlim" : "LArm");
						case "genderrarm": %node = (%profile.gender == 2 ? "RArmSlim" : "RArm");
					}
					
					if(%color $= "skin")
						%color = %client.headColor;
					
					%obj.setNodeColor(%node, %color);
				}
			}
			
			%obj.setDecalName(%so.decal);
		}
	}
}

function OutfitSO::applyOutfitParts(%so, %client)
{
	%profile = %client.profile;
	
	if(isObject(%profile))
	{
		%obj = %client.player;
		
		if(isObject(%obj))
		{
			hideAllNodes(%obj);
			
			%obj.unHideNode("headskin");
			%obj.setHeadUp(%so.headUp ? true : false);
			
			for(%a = 0; %a < %so.outfit; %a++)
			{
				%node = %so.node[%a];
				
				switch$(%node)
				{
					case "gender": %node = (%profile.gender == 2 ? "femchest" : "chest");
					case "genderlarm": %node = (%profile.gender == 2 ? "LArmSlim" : "LArm");
					case "genderrarm": %node = (%profile.gender == 2 ? "RArmSlim" : "RArm");
				}
				
				%obj.unHideNode(%node);
			}
			
			if(%profile.gender == 2)
			{
				for(%a = 0; %a < %so.outfitFem; %a++)
				{
					%node = %so.nodeFem[%a];
					
					switch$(%node)
					{
						case "gender": %node = (%profile.gender == 2 ? "femchest" : "chest");
						case "genderlarm": %node = (%profile.gender == 2 ? "LArmSlim" : "LArm");
						case "genderrarm": %node = (%profile.gender == 2 ? "RArmSlim" : "RArm");
					}
					
					%obj.unHideNode(%node);
				}
			}
			else
			{
				for(%a = 0; %a < %so.outfitMale; %a++)
				{
					%node = %so.nodeMale[%a];
					
					switch$(%node)
					{
						case "gender": %node = (%profile.gender == 2 ? "femchest" : "chest");
						case "genderlarm": %node = (%profile.gender == 2 ? "LArmSlim" : "LArm");
						case "genderrarm": %node = (%profile.gender == 2 ? "RArmSlim" : "RArm");
					}
					
					%obj.unHideNode(%node);
				}
			}
		}
	}
}

function OutfitSO::isCustomColor(%so, %part)
{
	for(%a = 0; %a < getFieldCount(%so.customColor); %a++)
	{
		%field = getField(%so.customColor);
		
		if(%field $= %part)
		{
			return true;
		}
	}
	
	return false;
}

function OutfitSO::getOutfitString(%so, %profile)
{
	if(isObject(%profile))
	{
		%nodes = "";
		%colors = "";
		
		for(%a = 0; %a < %so.outfit; %a++)
		{
			%node = %so.node[%a];
			%color = %so.color[%a];
			
			switch$(%node)
			{
				case "gender": %node = (%profile.gender == 2 ? "femchest" : "chest");
				case "genderlarm": %node = (%profile.gender == 2 ? "LArmSlim" : "LArm");
				case "genderrarm": %node = (%profile.gender == 2 ? "RArmSlim" : "RArm");
			}
			
			if(%color $= "skin")
				%color = %profile.headColor;
			
			if(%a)
			{
				%nodes = %nodes TAB %node;
				%colors = %colors TAB %color;
			}
			else
			{
				%nodes = %node;
				%colors = %color;
			}
			
			%parts++;
		}
		
		if(%profile.gender == 2)
		{
			for(%a = 0; %a < %so.outfitFem; %a++)
			{
				%node = %so.nodeFem[%a];
				%color = %so.colorFem[%a];
				
				switch$(%node)
				{
					case "gender": %node = (%profile.gender == 2 ? "femchest" : "chest");
					case "genderlarm": %node = (%profile.gender == 2 ? "LArmSlim" : "LArm");
					case "genderrarm": %node = (%profile.gender == 2 ? "RArmSlim" : "RArm");
				}
				
				if(%color $= "skin")
					%color = %profile.headColor;
				
				if(%parts)
				{
					%nodes = %nodes TAB %node;
					%colors = %colors TAB %color;
				}
				else
				{
					%nodes = %node;
					%colors = %color;
				}
				
				%parts++;
			}
		}
		else
		{
			for(%a = 0; %a < %so.outfitMale; %a++)
			{
				%node = %so.nodeMale[%a];
				%color = %so.colorMale[%a];
				
				switch$(%node)
				{
					case "gender": %node = (%profile.gender == 2 ? "femchest" : "chest");
					case "genderlarm": %node = (%profile.gender == 2 ? "LArmSlim" : "LArm");
					case "genderrarm": %node = (%profile.gender == 2 ? "RArmSlim" : "RArm");
				}
				
				if(%color $= "skin")
					%color = %profile.headColor;
				
				if(%parts)
				{
					%nodes = %nodes TAB %node;
					%colors = %colors TAB %color;
				}
				else
				{
					%nodes = %node;
					%colors = %color;
				}
				
				%parts++;
			}
		}
		
		%outfit = %parts NL %so.decal NL %nodes NL %colors;
		%profile.outfit = %outfit;
		return %outfit;
	}
}

function OutfitSO::onAdd(%so)
{
	OutfitSG.add(%so);
}

// OutfitSG
if(!isObject(OutfitSG))
{
	new SimGroup(OutfitSG);
	CityGroups.add(OutfitSG);
}

// Default Outfits
if(!isObject(OutfitDefaultSO))
{
	new ScriptObject(OutfitDefaultSO)
	{
		class = "OutfitSO";
		
		decal = "";
		outfit = 8;
		headUp = 0;
		
		node[0] = "gender";
		node[1] = "genderlarm";
		node[2] = "LHand";
		node[3] = "genderrarm";
		node[4] = "RHand";
		node[5] = "pants";
		node[6] = "LShoe";
		node[7] = "RShoe";
		
		color[0] = "1.00 1.00 1.00 1.00";
		color[1] = "1.00 1.00 1.00 1.00";
		color[2] = "skin";
		color[3] = "1.00 1.00 1.00 1.00";
		color[4] = "skin";
		color[5] = "0.00 0.14 0.33 1.00";
		color[6] = "0.20 0.20 0.20 1.00";
		color[7] = "0.20 0.20 0.20 1.00";
	};
}

if(!isObject(OutfitInmateSO))
{
	new ScriptObject(OutfitInmateSO)
	{
		class = "OutfitSO";
		
		decal = "Mod-Prisoner";
		outfit = 8;
		headUp = false;
		
		node[0] = "gender";
		node[1] = "genderlarm";
		node[2] = "LHand";
		node[3] = "genderrarm";
		node[4] = "RHand";
		node[5] = "pants";
		node[6] = "LShoe";
		node[7] = "RShoe";
		
		color[0] = "0.90 0.34 0.08 1.00";
		color[1] = "0.90 0.34 0.08 1.00";
		color[2] = "skin";
		color[3] = "0.90 0.34 0.08 1.00";
		color[4] = "skin";
		color[5] = "0.90 0.34 0.08 1.00";
		color[6] = "0.90 0.34 0.08 1.00";
		color[7] = "0.90 0.34 0.08 1.00";
	};
}

if(!isObject(OutfitTuxedoSO))
{
	new ScriptObject(OutfitTuxedoSO)
	{
		class = "OutfitSO";
		
		decal = "Mod-Suit";
		outfit = 8;
		headUp = false;
		
		node[0] = "gender";
		node[1] = "genderlarm";
		node[2] = "genderrarm";
		node[3] = "LHand";
		node[4] = "RHand";
		node[5] = "pants";
		node[6] = "LShoe";
		node[7] = "RShoe";
		
		color[0] = "0.20 0.20 0.20 1.00";
		color[1] = "0.20 0.20 0.20 1.00";
		color[2] = "0.20 0.20 0.20 1.00";
		color[3] = "skin";
		color[4] = "skin";
		color[5] = "0.20 0.20 0.20 1.00";
		color[6] = "0.20 0.20 0.20 1.00";
		color[7] = "0.20 0.20 0.20 1.00";
	};
}

// ============================================================
// Section : Jobs
// ============================================================
// JobSO Class
// This is a pseudoclass called by all jobs.

function JobSO::getDataAddendum(%so, %profile, %client)
{
	// Nothing!
}

function JobSO::getError(%so, %client)
{
	if(!isObject(%profile = %client.profile))
		return "No Profile" TAB "You have no profile!";
	
	if(%profile.job == %so.id)
		return "Same Job" TAB "You are applying for the same job you already are!";
	
	if(%profile.felon && %so.cleanRecord)
		return "You're a Felon!" TAB "As a convicted felon, you cannot apply for this job.";
	
	if(isObject(%job = %profile.getJob()) && %job.expTree $= %so.expTree && ((%job.expTree $= %so.expTree) && %so.expTier < %job.expTier))
		return "Top of your Game" TAB "You are already on a tier higher in this job's EXP Tree.\n\nYou don't want to move down, do you!?";
	
	if(%profile.exp[%so.expTree] < %so.expMin)
		return "Not Good Enough, Kid" TAB "You cannot apply for this job because you do not have enough experience in this EXP tree.\n\n" @
			"(" @ %profile.exp[%so.expTree] @ "/" @ %so.expMin @ " EXP)";
	
	if(%profile.education < mFloor(%so.education))
		return "Not Enough Education" TAB "Your level of education is below what is required for this job.";
	
	if((%profile.money + %profile.bank) < mFloor(%so.cost))
		return "Insufficient Funds" TAB "You cant afford the investment!";
	
	return -1;
}

function JobSO::getLaborBonus(%so, %client)
{
	return 0;
}

function JobSO::getSpawn(%so)
{
	if(isObject(%so.spawnBrick))
		return %so.spawnBrick.getID();
	else
		return -1;
}

// Determines if the profile takes headshots.
// @bool
function JobSO::getArmor(%so)
{
	return 0;
}

function JobSO::getSuccessMessage(%so, %profile)
{
	return "\c6You are now a \c3" @ %so.name @ "\c6!";
}

// Gets the job's tools.
// @string (ItemData List)
function JobSO::getTools(%so)
{
	return %so.tools;
}

function JobSO::giveLaborBonus(%so)
{
	return 0;
}

// Determines if you can, here and now, be this job.
// @bool
function JobSO::isApplicable(%so, %target)
{
	if(%target.isProfile)
		%profile = %target;
	else
		%profile = %target.profile;
	
	// Do we exist?
	if(isObject(%profile))
	// Demerits?
	if(!%profile.getWantedLevel() && !%profile.isInmate())
	// Are we applying for the same job?
	if(%profile.job != %so.id)
	// Do we need a clean record?
	if(!%profile.felon || !%so.cleanRecord)
	// Are we applying for an inferior job?
	if(!isObject(%profile.getJob()) || %job.expTree !$= %so.expTree || (%job.expTree $= %so.expTree && %so.expTier < %job.expTier))
	// Are we experienced enough?
	if(%profile.exp[%so.expTree] >= %so.expMin)
	// Are we educated enough?
	if(%profile.education >= mFloor(%so.education))
	// Can we afford it?
	if((%profile.money + %profile.bank) >= mFloor(%so.cost))
		return true;
	
	return false;
}

// Determines if you can theoretically be this job, foregoing education and investment.
// @bool
function JobSO::isAvailable(%so, %target)
{
	if(%target.isProfile)
		%profile = %target;
	else
		%profile = %target.profile;
	
	// Do we exist?
	if(isObject(%profile))
	// Are we applying for the same job?
	if(%profile.job != %so.id)
	// Do we need a clean record?
	if(!%profile.felon || !%so.cleanRecord)
	// Are we applying for an inferior job?
	if(!isObject(%profile.getJob()) || %job.expTree !$= %so.expTree || (%job.expTree $= %so.expTree && %so.expTier < %job.expTier))
	// Are we experienced enough?
	if(%profile.exp[%so.expTree] >= %so.expMin)
		return true;
	
	return false;
}

// Determines if the profile takes headshots.
// @bool
function JobSO::isHeadProtected(%so)
{
	return false;
}

// Determines if %item is a default %so spawn item.
// @bool
function JobSO::isJobItem(%so, %item)
{
	if(!isObject(%item))
		return false;
	
	for(%a = 0; %a < getFieldCount(%so.tools); %a++)
	{
		%tool = getField(%so.tools, %a);
		
		if(!isObject(%tool))
			continue;
		
		if(%tool.getID() == %item.getID())
			return true;
	}
	
	return false;
}

function JobSO::isListed(%so)
{
	return !%so.unlisted;
}

function JobSO::onActivate(%so, %client, %obj)
{
	// Nothing
}

function JobSO::onAdd(%so)
{
	// Adds job to the job list.
	JobsSG.add(%so);
	
	%spawnName = "CRPG_" @ strreplace(%so.name, " ", "_") @ "_Spawn_Brick";
	
	if(!isObject(%spawnName))
	{
		datablock fxDTSBrickData(CRPG_TEMP_SPAWN_NAME : brickSpawnPointData)
		{
			uiName = %so.name SPC "Spawn";
			category = "CityRPG";
			subCategory = "Spawn Points";
			iconName = $CRPG::Path::Images @ "brickIcons/Icon_SpawnPoint_Admin";
			
			adminOnly = true;
			crpgSpawn = true;
		};
		
		CRPG_TEMP_SPAWN_NAME.setName(%spawnName);
		%so.spawnBrick = %spawnName.getID();
	}
}

function JobSO::onActivate(%so, %client, %obj)
{
	// Nothing!
}

function JobSO::onEXP(%so, %profile, %exp)
{
	// Nothing!
}

function JobSO::onJobOp(%so, %client, %profile, %category, %opID, %parameters)
{
	// Nothing!
}

function JobSO::onJobOpEnd(%so, %client, %profile, %success)
{
	if(%success $= "")
		return;
	
	messageClient(%client, '', "\c6Your Job Operations have gone through.");
	
	for(%a = 0; %a < getLineCount(%success); %a++)
	{
		%line = getLine(%success, %a);
		messageClient(%client, '', "\c3 +- " @ %line);
	}
}

function JobSO::onKilled(%so, %client, %profile, %sourceClient, %sourceProfile, %sourceObject, %transform, %damageType)
{
	if(!%profile.getWantedLevel())
	{
		%sourceClient.commitCrime("Murder", 100, false, true, false, false, 240000, %transform);
	}
}

function JobSO::onTick(%so, %client)
{
	// Nothing!
}

function JobSO::sendMessage(%so, %message)
{
	for(%a = 0; %a < ClientGroup.getCount(); %a++)
	{
		%client = ClientGroup.getObject(%a);
		
		if(isObject(%client))
		{
			%profile = %client.profile;
			
			if(isObject(%profile))
			{
				%job = %profile.getJob();
				
				if(isObject(%job) && %job.getID() == %so.getID())
				{
					messageClient(%client, '', %message);
				}
			}
		}
	}
}

function JobSO::setJobFor(%so, %target, %automessage)
{
	if(%target.isProfile)
	{
		%profile = %target;
		%client = %profile.getClient();
	}
	else
	{
		%client = %target;
		%profile = %client.profile;
	}
	
	if(isObject(%profile))
	{
		%obj = %client.player;
		
		// Tool Removal && Saving
		if(isObject(%obj))
		{
			%toolMax	= %obj.dataBlock.maxTools;
			%toolDef	= $CRPG::DefaultTools;
			%toolJob	= %profile.getJob().getTools();
			%toolInm	= $CRPG::PrisonTools;
			%tools	= "";
			
			for(%a = 0; %a < %toolMax; %a++)
			{
				%tool = %obj.tool[%a];
				%valid = false;
				
				// If we have a tool...
				if(isObject(%tool))
				{
					%valid = true;
					
					// Check if it's a job tool.
					for(%b = 0; %b < %toolMax; %b++)
					{
						%toolSpc = getField(%toolJob, %b);
						
						// %tool is a job tool. Forget about it, move on.
						if(isObject(%toolSpc) && %toolSpc.getID() == %tool.getID())
						{
							%valid = false;
							break;
						}
					}
					
					// If it wasn't a job tool...
					if(%valid)
					{
						// Check if it's a default tool.
						for(%b = 0; %b < %toolMax; %b++)
						{
							%toolSpc = getField(%toolDef, %b);
							
							if(isObject(%toolSpc) && %toolSpc.getID() == %tool.getID())
							{
								%valid = false;
								break;
							}
						}
						
						// If it wasn't a default tool, and we're an inmate...
						if(!%valid && %profile.isInmate())
						{
							for(%b = 0; %b < %toolMax; %b++)
							{
								%toolSpc = getField(%toolInm, %b);
								
								if(isObject(%toolSpc) && %toolSpc.getID() == %tool.getID())
								{
									%valid = false;
									break;
								}
							}
						}
					}
				}
				
				if(%valid)
					%tools = %tools @ %tool.getName() @ "\t";
				
				%obj.tool[%a] = 0;
			}
			
			%profile.tools = %tools;
		}
		
		// Change Job
		%profile.workhours = 0;
		%profile.job = %so.id;
		%profile.removeFunds(mFloor(%so.cost), false);
		
		// Tool Addition
		if(isObject(%obj))
			%obj.giveDefaultEquipment();
		
		if(isObject(%client))
		{
			// Outfit
			%profile.getOutfit().applyOutfit(%client);
			
			// GUI
			serverCmdStats_GetJobs(%client);
			serverCmdStats_GetOutfit(%client);
			
			// Message
			if(%automessage)
				messageClient(%client, '', %so.getSuccessMessage(%profile));
			
			%client.sendJobOps();
		}
	}
	
	return false;
}

// JobsSO
// Overlying scriptobject.
function JobsSO::execJobs(%so)
{
	%file = findFirstFile(%so.filepath);
	
	while(isFile(%file))
	{
		exec(%file);
		%file = findNextFile(%so.filepath);
	}
}

function JobsSO::getJobSO(%so, %id)
{
	for(%a = 0; %a < JobsSG.getCount(); %a++)
		if(JobsSG.getObject(%a).id $= %id)
			return JobsSG.getObject(%a);
	
	return -1;
}

function JobsSO::messageBranch(%so, %tree, %message)
{
	for(%a = 0; %a < JobsSG.getCount(); %a++)
	{
		%job = JobsSG.getObject(%a);
		
		if(%job.expTree $= %tree)
		{
			%job.sendMessage(%message);
		}
	}
}

function JobsSO::onAdd(%so)
{
	%so.execJobs();
}

if(!isObject(JobsSG))
{
	new SimGroup(JobsSG);
	CityGroups.add(JobsSG);
}

if(!isObject(JobsSO))
{
	new ScriptObject(JobsSO)
	{
		// Where job script files are located.
		filepath = $CRPG::Path::Scripts @ "jobs/*.cs";
	};
	
	CityScripts.add(JobsSO);
}

// ============================================================
// Section : Missions
// ============================================================
// Test Mission
function Mission_Test::onStart(%so)
{
	%so.name = "Don't Be Arrested";
	parent::onStart(%so);
	
	%client = %so.client;
	%profile = %so.profile;
	messageClient(%client, '', "\c3Contact\c6: lol ur fucked.");
	%profile.setDemerits(10000);
}

// Individual Missions
function Mission_Convert_Worker::onStart(%so)
{
	%so.name = "Disloyal Immigrant";
	parent::onStart(%so);
	
	%client = %so.client;
	%profile = %so.profile;
	messageClient(%client, '', "\c3Contact\c6: What keeps you floating above the water is your Factory Workers.");
	messageClient(%client, '', "\c3Contact\c6: If you want to make it big, you need to keep your drones happy and loyal.");
	messageClient(%client, '', "\c3Contact\c6: Your competitors have their own drones. Taking theirs would be a good idea.");
	messageClient(%client, '', "\c4Mission Objective\c6: Convince a rival factory worker to work in your factory.");
}

function Mission_Convert_Worker::onPatronConvert(%so, %client)
{
	parent::onPatronConvert(%so, %client);
	%so.count_Patron_Convert = %client;
	%so.onSuccess();
}

function Mission_Convert_Worker::onSuccess(%so)
{
	messageClient(%so.client, '', "\c3Contact\c6: Excellent." SPC %so.count_Patron_Convert.name SPC "is working for you now.");
	messageClient(%so.client, '', "\c3Contact\c6: Lets hope they're the first in a tsunami of new employees!");
	
	parent::onSuccess(%so);
}

// Police Brutality
function Mission_Police_Brutality::onStart(%so)
{
	%so.name = "Police Brutality";
	parent::onStart(%so);
	
	messageClient(%client, '', "\c1Police HQ\c6: Officer" SPC %client.name @ ", I want a word with you.");
	messageClient(%client, '', "\c1Police HQ\c6: As you know, the streets are tough right now. The city is always at war with itself.");
	messageClient(%client, '', "\c1Police HQ\c6: I want you to go out and beat someone down, then arrest them. Demerits are not required.");
	messageClient(%client, '', "\c1Police HQ\c6: And don't get rubber legged on me, you know they've done something to deserve it!");
	messageClient(%client, '', "\c4Mission Objective\c6: Arrest anybody you want.");
	
	%so.obj_Can_Arrest_Anyone = true;
}

function Mission_Police_Brutality::onArrest(%so, %client)
{
	parent::onArrest(%so, %client);
	%so.count_Arrest_Client = %client;
	%so.onSuccess();
}

function Mission_Police_Brutality::onSuccess(%so)
{
	%vc = %so.count_Arrest_Client;
	%vp = %vc.getProfile();
	
	messageClient(%so.client, '', "\c1Police HQ\c6: Nice job bringing that dope up off the streets.");
	%exp = 10;
	
	if(isObject(%vp))
	{
		if(%vp.felon)
		{
			messageClient(%so.client, '', "\c1Police HQ\c6: \c3" @ %vc.name @ "\c6 has a felony record.");
			%exp += 5;
			
			if(%vp.getWantedLevel())
			{
				messageClient(%so.client, '', "\c1Police HQ\c6: They even had a warrant out for their arrest at the time!");
				%exp += mClamp(%vp.demerits / 10, 10, 50);
			}	
		}
		
		if(%vp.getWantedLevel())
		{
			messageClient(%so.client, '', "\c1Police HQ\c6: You'll be pleased to know \c3" @ %vc.name @ "\c6 had a warrant on them at the time.");
			messageClient(%so.client, '', "\c1Police HQ\c6: That'll look real good on your resume.");
			%exp += mClamp(%vp.demerits / 10, 10, 50);
		}
		
	}
	
	messageClient(%so.client, '', "\c1Police HQ\c6: We'll contact you if we're ever in need of your help again," SPC %so.client.name @ ".");
	
	%so.profile.addEXP(%exp, "Law");
	%so.client.sendStatus();
	parent::onSuccess(%so);
}

// Sell Goods
function Mission_Sell_Goods::isValid(%so)
{
	if(!parent::isValid(%so))
		return false;
	
	if(!isObject(%so.client.player))
		return false;
	
	return true;
}

function Mission_Sell_Goods::onStart(%so)
{
	%so.name = "Hot Topic";
	parent::onStart(%so);
	
	%client = %so.client;
	%obj = %client.player;
	
	messageClient(%client, '', "\c3Contact\c6: Hey, \c3" @ %client.name @ "\c6! I've got some heat on me.");
	messageClient(%client, '', "\c3Contact\c6: Take \c5this crate\c6. I don't care what you do with it, just get it out of my sight!");
	messageClient(%client, '', "\c4Mission Objective\c6: Take the crate of Stolen Goods to the \c3Shady Part of Town\c6 and dispose of them.");
	
	%obj.crpgCrateOwner = -1;
	%obj.crpgCrateRefPlastic = 0;
	%obj.crpgCratePlastic = 0;
	%obj.crpgCrateWorth = 500;
	%obj.crpgCrateMission = %so.getID();
	%obj.mountImage(CRPG_Crate_Image, 0);
}

function Mission_Sell_Goods::onSellCrate(%so)
{
	parent::onSellCrate(%so);
	%so.onSuccess();
}

// Smash Stuff
function Mission_Smash_Stuff::onStart(%so)
{
	%so.name = "Civil Disobedience";
	parent::onStart(%so);
	
	messageClient(%client, '', "\c3Contact\c6: Brother Brother!");
	messageClient(%client, '', "\c3Contact\c6: As you know, the labor unions have been failing us. Plastic prices are at an all-time low.");
	messageClient(%client, '', "\c3Contact\c6: Show big-business we MEAN business. We mine the plastic to make their factories, now we can mine it back.");
	messageClient(%client, '', "\c4Mission Objective\c6: Destroy \c0$1,000\c6 worth of bricks with your \c3Pickaxe\c6.");
	
	%so.obj_Destroy_Bricks = 1000;
	%so.count_Damage_Done = 0;
}

function Mission_Smash_Stuff::onSmashBrick(%so, %brick)
{
	parent::onSmashBrick(%so);
	%so.obj_Destroy_Bricks -= %brick.getDatablock().getPrice();
	%so.count_Damage_Done += %brick.getDatablock().getPrice();
	
	if(%so.obj_Destroy_Bricks <= 0)
	{
		%color = "<color:00ff00>";
		%so.onSuccess();
	}
	else
	{
		%color = "<color:ff0000>";
	}
	
	commandToClient(%so.client, 'CenterPrint', "<just:right>" @ %color @ "$" @ %so.count_Damage_Done, 2);
}

function Mission_Smash_Stuff::onSuccess(%so)
{
	messageClient(%so.client, '', "\c3Contact\c6: Hey hey! Nice work. That'll teach them to treat us like their slaves.");
	messageClient(%so.client, '', "\c3Contact\c6: I'll let you keep the \c51,000 Chunks of Plastic\c6 as a reward of sorts.");
	%so.profile.plastic += 1000;
	
	if(%so.profile.getWantedLevel())
	{
		messageClient(%so.client, '', "\c3Contact\c6: Also, I know a guy in a high place to take that heat off your back.");
		%so.profile.demerits = 0;
	}
	
	%so.client.sendStatus();
	
	parent::onSuccess(%so);
}


// Mission Class
// Determines if the mission can be created.
// @bool
function Mission::isValid(%so)
{
	if(!isObject(%so.client))
		return false;
	
	if(!isObject(%so.profile))
		return false;
	
	return true;
}

// Called when a mission so is added. Checks to make sure it is valid.
// @null
function Mission::onAdd(%so)
{
	if(%so.isValid())
		%so.onStart();
	else
		%so.schedule(0, "delete");
}

// Called when the mission runner arrests someone.
// @null
function Mission::onArrest(%so, %legitimate)
{
	echo("Mission Arrest Made.");
}

// Called whenever a mission ends, after the fail and success scripts.
// @null
function Mission::onEnd(%so)
{
	echo("Mission Ended.");
	%so.delete();
}

// Called when a mission is failed.
// @null
function Mission::onFail(%so)
{
	echo("Mission Failed.");
	
	if(%so.name !$= "")
		messageAll('', "\c3" @ %so.client.name @ "\c6 has \c0failed\c6 the mission \"\c3" @ %so.name @ "\c6.\"");
	else
		messageAll('', "\c3" @ %so.client.name @ "\c6 has \c0failed\c6 a mission.");
	
	%so.onEnd();
}

// Called when a factory worker completes a work cycle on your brick,
// while his last completed factory brick was in a different factory.
// @null
function Mission::onPatronConvert(%so, %client)
{
	echo("Mission Patron Converted.");
}

// Called when a player is arrested.
// @null
function Mission::onPlayerBusted(%so)
{
	messageClient(%so.client, 'CenterPrint', "<font:impact:64>\c1Busted!");
	%so.onFail();
}

// Called when a player is killed.
// @null
function Mission::onPlayerKilled(%so)
{
	messageClient(%so.client, 'CenterPrint', "<font:impact:64>\c0You're dead!");
	%so.onFail();
}

// Called when a mission crate is sold.
// @null
function Mission::onSellCrate(%so)
{
	echo("Mission Crate Sold.");
}

// Called when a brick is destroyed during a mission.
// @null
function Mission::onSmashBrick(%so, %brick)
{
	echo("Mission Brick Smashed.");
}

// Called when a mission is good to go and has started.
// @null
function Mission::onStart(%so)
{
	echo("Mission Started.");
	%client = %so.client;
	%profile = %so.profile;
	
	if(%so.name !$= "")
		messageClient(%client, '', "\c3Mission Started\c6: \"" @ %so.name @ "\"");
}

// Called when a mission is successful.
// @null
function Mission::onSuccess(%so)
{
	echo("Mission Completed.");
	
	if(%so.name !$= "")
		messageAll('', "\c3" @ %so.client.name @ "\c6 has \c2completed\c6 the mission \"\c3" @ %so.name @ "\c6.\"");
	else
		messageAll('', "\c3" @ %so.client.name @ "\c6 has \c2completed\c6 a mission.");
	
	%so.client.sendMissionComplete();
	%so.onEnd();
}

// MissionsSO
function MissionSO::giveMission(%so, %client, %id)
{
	%profile = %client.profile;
	
	if(isObject(%profile))
	{
		if(isObject(%profile.mission))
			%profile.mission.onEnd();
		
		%mission = new ScriptObject()
		{
			class = %id;
			superClass = "Mission";
			
			client = %client;
			profile = %profile;
			
			isMission = true;
		};
		
		if(isObject(%mission))
		{
			%profile.mission = %mission;
			%client.mission = %mission;
			CityScripts.add(%mission);
		}
		else
		{
			echo("Mission failed to instantiate.");
		}
	}
}

if(!isObject(MissionSO))
{
	new ScriptObject(MissionSO);
	CityScripts.add(MissionSO);
}

// ============================================================
// Section : Restrictions
// ============================================================
function RestrictionSO::isAllowed(%so, %bl_id)
{
	if(!$CRPG::BETAList)
		return true;
	
	%beta = %so.isBeta(%bl_id);
	
	if(%beta == -1)
		return true;
	else
		return %beta;
}

function RestrictionSO::isBeta(%so, %bl_id)
{
	%file = $CRPG::Path @ "accesslist.txt";
	
	if(isFile(%file))
	{
		%fo = new FileObject();
		%fo.openForRead(%file);
		
		%hasFound = false;
		%lineNum = 0;
		
		while(!%fo.isEOF())
		{
			%line = %fo.readLine();
			%name = getField(%line, 0);
			%foid = getField(%line, 1);
			
			if(%foid == %bl_id)
			{
				%hasFound = true;
				break;
			}
			else
			{
				%lineNum++;
			}
		}
		
		%fo.close();
		%fo.delete();
		
		return %hasFound;
	}
	
	return -1;
}

if(!isObject(RestrictionSO))
{
	new ScriptObject(RestrictionSO);
	CityScripts.add(RestrictionSO);
}

// ============================================================
// Section : Sales
// ============================================================
function SalesSO::onAdd(%so)
{
	// Price Items
	%so.addItem("CRPG_Key_Item", 50, 1);
	%so.addItem("CRPG_Cellphone_Item", 250, 5);
	%so.addItem("CRPG_GasMask_Item", 250, 5);
	%so.addItem("CRPG_Handgun_Item", 1000, 20);
	%so.addItem("CRPG_Shotgun_Item", 2500, 50);
	%so.addItem("CRPG_SMG_Item", 5000, 100);
	%so.addItem("CRPG_Assault_Item", 7500, 150);
	%so.addItem("CRPG_Sniper_Item", 10000, 200);
	
	// Price Vehicles
	%so.addVehicle("ArmoredVanVehicle",	"Phydeoux Co.",	20000,	3, "Although slowed by its additional armor, this vehicle is trusted by banks to move money.");
	%so.addVehicle("BlockoCarVehicle",	"Kaje Inc.",		20000,	3, "A standard four-door sedan by a trusted manufacturer.");
	%so.addVehicle("GolfCartVehicle",	"Kaje Inc.",		2500,		2, "Beep beep!");
	%so.addVehicle("JeepVehicle",			"Step 1: Games",	5000,		2, "A trusted vehicle that has been in use since the dawn of Blockland. Reliable and sturdy.");
	%so.addVehicle("privatejetVehicle", "Phydeoux Co.",	50000,	5, "A massive airplane with all the comforts one could want. Own the skies.");
	%so.addVehicle("RowBoatArmor",		"Rot Co.",			500,		1, "Human powered and easily destroyed, but the only line of water-craft available.");
	%so.addVehicle("ScooterVehicle",		"Kaje Inc.",		500,		1, "The scooter is the perfect first vehicle. It's cheap, albeit slow and hard to control.");
	%so.addVehicle("SemiTractorVehicle","Phydeoux Co.",	10000,	3, "Semi Tractors are men's vehicles. They're durable, but hard to control.");
	%so.addVehicle("StuntPlaneVehicle",	"Kaje Inc.",		15000,	3, "Take to the skies! Kaje Inc. is the most trusted name in planes.");
	%so.addVehicle("TurismoVehicle",		"Phydeoux Co.",	30000,	5, "A fast Eurpoean-style sports car. Unrivaled in terms of speed and control.");
}

// Adds am item property.
// @null
function SalesSO::addItem(%so, %name, %price, %plastic)
{
	if(%name !$= "")
	{
		%so.item[%so.items]	= %name;
		%so.price[%so.items]	= %price;
		%so.plastic[%so.items] = %plastic;
		%so.items++;
	}
	else
		error("Item" SPC %so.items SPC "has no name.");
}

// Adds a vehicle property.
// @null
function SalesSO::addVehicle(%so, %name, %corp, %price, %rating, %desc)
{
	if(%name !$= "")
	{
		%so.vehicle[%so.vehicles]	= %name;
		%so.vcorp[%so.vehicles]		= %corp;
		%so.vprice[%so.vehicles]	= %price;
		%so.vrating[%so.vehicles]	= %rating;
		%so.vdesc[%so.vehicles]		= %desc;
		%so.vehicles++;
	}
	else
		error("Item" SPC %so.vehicles SPC "has no name.");
}

function SalesSO::isItemValuable(%so, %item)
{
	if(isObject(%item) && %item.canDrop)
		for(%a = 0; %a < %so.items; %a++)
			if(isObject(%so.item[%a]) && %so.item[%a].getID() == %item.getID())
				return true;
	
	return false;
}

function SalesSO::isVehicleProduct(%so, %db)
{
	if(isObject(%db))
		for(%a = 0; %a < %so.vehicles; %a++)
			if(isObject(%so.vehicle[%a]) && %so.vehicle[%a].getID() == %db.getID())
				return true;
	
	return false;
}

function SalesSO::getItemID(%so, %item)
{
	if(isObject(%item))
		for(%a = 0; %a < %so.items; %a++)
			if(isObject(%so.item[%a]) && %so.item[%a].getID() == %item.getID())
				return %a+1;
	
	return -1;
}

function SalesSO::getIDItem(%so, %id)
{
	%item = %so.item[%id--];
	
	if(isObject(%item))
		return %item.getID();
	
	return -1;
}

function SalesSO::getItemPrice(%so, %id)
{
	%item = %so.item[%id--];
	
	if(isObject(%item))
		return %so.price[%id];
	
	return 0;
}

function SalesSO::getItemString(%so)
{
	%items = 0;
	%sales = "";
	
	for(%a = 1; %a <= %so.items; %a++)
	{
		%item = %so.item[%a-1];
		
		if(isObject(%item))
		{
			%price = %so.price[%a-1];
			
			if(mFloatLength(%price, 0) $= %price && %price > 0)
			{
				%name = %item.uiName;
				%name = stripTrailingSpaces(%name);
				%name = strReplace(%name, " ", "_");
				
				if(%items)
					%sales = %sales SPC %name SPC %a;
				else
					%sales = %name SPC %a;
				
				%items++;
			}
		}
	}
	
	return %sales;
}

function SalesSO::getVehiclePrice(%so, %db)
{
	if(isObject(%db))
		for(%a = 0; %a < %so.vehicles; %a++)
			if(isObject(%so.vehicle[%a]) && %so.vehicle[%a].getID() == %db.getID())
				return %so.vprice[%a];
	
	return -1;
}

function SalesSO::getVehicleString(%so)
{
	%vehicles = 0;
	%sales = "";
	
	for(%a = 1; %a <= %so.vehicles; %a++)
	{
		%vehicle = %so.vehicle[%a-1];
		
		if(isObject(%vehicle))
		{
			%price = %so.price[%a-1];
			
			if(mFloatLength(%price, 0) $= %price && %price > 0)
			{
				%name = %vehicle.uiName;
				%name = stripTrailingSpaces(%name);
				%name = strReplace(%name, " ", "_");
				
				if(%vehicles)
					%sales = %sales SPC %name SPC %a;
				else
					%sales = %name SPC %a;
				
				%vehicles++;
			}
		}
	}
	
	return %sales;
}

if(!isObject(SalesSO))
{
	new ScriptObject(SalesSO)
	{
		items = 0;
		vehicles = 0;
	};
	
	CityScripts.add(SalesSO);
}

// ============================================================
// Section : Spawns
// ============================================================
function SpawnsSO::getSpawnPointForBrick(%so, %brick, %client)
{
	if(isObject(%brick) && %brick.getClassName() $= "fxDTSBrickData" && (!%brick.personalSpawn || isObject(%client)))
	{
		%spawns = "";
		
		for(%a = 0; %a < %so.ss.getCount(); %a++)
		{
			%spawn = %so.ss.getObject(%a);
			
			if(%spawn.getDatablock().getID() == %brick.getID() && (!%brick.personalSpawn || %spawn.getGroup().bl_id == %client.bl_id))
			{
				if(%spawns $= "")
					%spawns = %spawn.getID();
				else
					%spawns = %spawns TAB %spawn.getID();
			}
		}
		
		if(%spawns !$= "")
		{
			%spawnBrick = getField(%spawns, getRandom(0, getFieldCount(%spawns) - 1));
			
			%box	= %spawnBrick.getWorldBox();
			%point = vectorAdd(getWords(%box, 0, 1) SPC 0, getWords(%box, 3, 4) SPC 0);
			%point = vectorScale(%point, 0.5);
			%point = vectorAdd(getWords(%point, 0, 1) SPC getWord(%box, 2), "0 0 0.2");
			%trans = getWords(%spawnBrick.getTransform(), 3, 6);
			%vect	=	%point SPC %trans;
			
			return %vect;
		}
	}
	else if(!isObject(%brick))
		error("SpawnSO::getSpawnPointForBrick :: %brick does not exist.");
	else if(%brick.getClassName() !$= "fxDTSBrickData")
		error("SpawnSO::getSpawnPointForBrick :: %brick is not fxDTSBrickData." SPC %brick.getname() SPC %brick.getClassName());
	else
		error("SpawnSO::getSpawnPointForBrick :: %brick is a personal spawn and no %client was provided." SPC !%brick.personalSpawn SPC isObject(%client));
	
	return -1;
}

if(!isObject(SpawnsSS))
{
	new SimSet(SpawnsSS);
	CityScripts.add(SpawnsSS);
}

if(!isObject(SpawnsSO))
{
	new ScriptObject(SpawnsSO)
	{
		ss = "SpawnsSS";
	};
	
	CityScripts.add(SpawnsSO);
}

// ============================================================
// Section : System
// ============================================================

// Adds %bl_id to the inmate list after checking for duplicates.
// @string (list)
function SystemSO::addCandidate(%so, %bl_id)
{
	%so.elecList = addFieldInto(%so.elecList, %bl_id);
	return %so.elecList;
}

// Adds %bl_id to the inmate list after checking for duplicates.
// @bool (success)
function SystemSO::addInmate(%so, %bl_id)
{
	for(%a = 0; %a < getFieldCount(%so.inmates); %a++)
		if(getField(%so.inmate, %a) $= %bl_id)
			return false;
	
	%so.inmates = addField(%so.inmates, %bl_id);
	return true;
}

// Adds funds to treasury.
// @bool
function SystemSO::addToTreasury(%so, %input)
{
	%input = mAbs(%input);
	
	if(%input)
	{
		%so.treasury += %input;
		return true;
	}
	
	return false;
}

// Deduct funds from treasury.
// @bool kind of
function SystemSO::deductFromTreasury(%so, %input)
{
	%input = mAbs(%input);
	
	if(%input)
	{
		%output = mClamp(%so.treasury - %input, 0, %so.treasury); 
		%so.treasury -= %output;
		return %output;
	}
	
	return false;
}

// Saves the SystemSO.
// @null
function SystemSO::backup(%so)
{
	%so.save($CRPG::Path::Saving @ "system.cs");
}

// Goes through the server's candidates and active users to see if the list is correct.
// @null
function SystemSO::checkCandidates(%so)
{

	// Fetch campaigners.
	%list = "";
	
	for(%a = 0; %a < Profiler.group.getCount(); %a++)
	{
		%profile = Profiler.group.getObject(%a);
		
		if(%profile.campaign > 0)
		{
			%list = addFieldInto(%list, %profile.getID());
		}
	}
	
	// Bubblesort them.
	%size = getFieldCount(%list);
	
	for(%a = 0; %a < %size; %a++)
	{
		%lastSize = 0;
		
		for(%b = 0; %b < (%size - %a); %b++)
		{
			if(%b > 0)
			{
				%last = getField(%list, %b - 1);
				%curr = getField(%list, %b);
				
				if(%curr.campaign > %last.campaign)
				{
					%list = setField(%list, %b - 1, %curr);
					%list = setField(%list, %b, %last);
				}
			}
		}
	}
	
	// Set & Send
	%so.elecList = getFields(%list, 0, 3);
	
	return %so.elecList;
}

// Goes through the server's inmates and removes them if they have served their sentence.
// @null
function SystemSO::checkInmates(%so)
{
	for(%a = 0; %a < getFieldCount(%so.inmates); %a++)
	{
		if(Profiler.isProfileActive(%bl_id = getField(%so.inmates, %a)))
		{
			%profile = Profiler.getProfile(%bl_id);
			
			if(!%profile.isInmate())
			{
				%profile.sentence = 0;
				%so.inmates = removeField(%so.inmates, %a);
				%a--;
			}
		}
	}
}

// Gets a list of candidates.
function SystemSO::getCandidates(%so)
{
	%candidates = "";
	
	// Fetch incumbent.
	%mayor = %so.getMayor();
	
	if(isObject(%mayor))
		%candidates = %mayor;
	else
		%candidates = Profiler.fetchProfile(getNumKeyID());
	
	return addFieldInto(%candidates, %so.elecList);
}

// Fetches the mayor's profile.
// @CityProfileSO
function SystemSO::getMayor(%so)
{
	return Profiler.loadProfile(%so.mayorBLID);
}

// Gets a server name.
// @string
function SystemSO::getName(%so)
{
	%name = "Server";
	%tag = "";
	
	if(%so.election)
		%tag = "Election Time";
	
	else if(%so.anarchy)
		%tag = "Anarchy!";
	
	
	if(%tag !$= "")
		return %name SPC "[" @ %tag @ "]";
	else
		return %name;
}

// The amount to be added each tick.
// @int
function SystemSO::getTreasuryTrickle(%so)
{
	%addDefault		= 0; //100
	%addInmate		= 0; //-50
	%addPlayer		= 0; //25
	%addProperty	= 0; //10
	
	%total = %addDefault;
	
	for(%a = 0; %a < ClientGroup.getCount(); %a++)
	{
		%client = ClientGroup.getObject(%a);
		%profile = %client.profile;
		
		if(isObject(%profile))
		{
			if(!%profile.isInmate())
			{
				%total += %addPlayer;
			}
			else
			{
				%total += %addInmate;
			}
		}
	}
	
	return %total;
}

// Ends the election. Picks a mayor.
// @CityProfileSO
function SystemSO::endElection(%so)
{
	%mostVoted = -1;
	%mostVotes = 0;
	
	for(%a = 0; %a < Profiler.group.getCount(); %a++)
	{
		%profile = Profiler.group.getObject(%a);
		%profile.campaign = mClamp(%profile.campaign, 0, 5);
		%client = %profile.getClient();
		
		if(isObject(%client))
			%client.sendCampaign();
		
		if(%profile.votesFor > %mostVotes)
		{
			%mostVoted = %profile.getID();
			%mostVotes = %profile.votesFor;
		}
	}
	
	if(!isObject(%mostVoted))
	{
		%mostVoted = %so.getMayor();
		%mostVotes = 0;
		
		if(!isObject(%mostVoted))
		{
			%mostVoted = Profiler.fetchProfile(getNumKeyID());
			%mostVotes = 0;
		}
	}
	
	%so.election = false;
	
	messageAll('', "\c6 - \c5Congratulations! \c3" @ %profile.name @ "\c6 is the new Mayor!");
	
	if(%mostVotes > 0)
		messageAll('', "\c6 - The new mayor won with \c3" @ %mostVotes @ "\c6 vote" @ (%mostVotes != 1 ? "s" : "") @ ".");
	else
		messageAll('', "\c6 - The new mayor won by a technical default.");
	
	%so.setMayor(%mostVoted.bl_id);
}

// Loads Candidate Profiles.
// @null
function SystemSO::loadCandidates(%so)
{
	for(%a = 0; %a < getFieldCount(%so.elecList); %a++)
		if(!Profiler.isProfileActive(%bl_id = getField(%so.elecList, %a)))
			Profiler.loadProfile(%bl_id);
}

// Loads Inmate Profiles.
// @null
function SystemSO::loadInmates(%so)
{
	for(%a = 0; %a < getFieldCount(%so.inmates); %a++)
		if(!Profiler.isProfileActive(%bl_id = getField(%so.inmates, %a)))
			Profiler.loadProfile(%bl_id);
}

// Adjusts the Calendar's time to the saved time + others
// @null
function SystemSO::onAdd(%so)
{
	CalendarSO.time = %so.time;
	
	%so.loadCandidates();
	%so.loadInmates();
	
	// Mayor Detections
	if(%so.mayorBLID == -1)
	{
		echo("No mayor! Setting server owner as mayor.");
		%so.setMayor(getNumKeyID());
	}
}

// Backs up the SystemSO on remove.
// @null
function SystemSO::onRemove(%so)
{
	%so.backup();
}

// Handles tick actions.
// @null
function SystemSO::onTick(%so, %newDay)
{
	// Election Notices
	if(%newDay)
	{
		if(!%so.election)
		{
			if(getWord(getDateTime(), 0) !$= %so.elecLast)
			{
				%time = getSubStr(getWord(getDateTime(), 1), 0, strPos(getWord(getDateTime(), 1), ":"));
				
				if(%time >= %so.elecTime)
				{
					%so.startElection();
				}
			}
		}
		else
		{
			%so.elecRem--;
			
			if(%so.elecRem == 0)
			{
				%so.endElection(%so);
			}
			else
			{
				messageAll('', "\c6 - There are \c5" @ (%so.elecRem + 1) @ "\c6 day" @ (%so.elecRem  != 0 ? "s" : "") @ " left until the new mayor is chosen.");
			}
		}
		
		if(%so.election)
		{
			%so.checkCandidates();
		}
		
		//%income = %so.getTreasuryTrickle();
		//%netinc = mClamp(%income - %so.mayorPay, 0, %income);
		//%so.addToTreasury(%netinc);
		//messageAll('', '\c6 - The treasury reported \c3$%1\c6 in income today.', %netinc);

		%income = %so.treasury - %so.lastTreasury; //Gets the difference

		if (%income >= 0)
		{
			messageAll('', '\c6 - The treasury reported \c2+$%1\c6 in income today.', %income);
		}
		else
		{
			%income = MAbs(%income);
			messageAll('', '\c6 - The treasury reported \c0-$%1\c6 in losses today.', %income);
		}
		
		%so.lastTreasury = %so.treasury; //Sets the last treasury to the current one.
	}
	$Pref::Server::Name = %so.getName();
	%so.time = CalendarSO.time;
	%so.checkInmates();
	%so.backup();
	
	if(%newDay)
		WebCom_PostServer();
	
	//if(isFunction("saveBricks"))
	//	saveBricks("CRPG_Backup.bls", true, true);
}

// Starts an election.
// @CityProfileSO
function SystemSO::startElection(%so)
{
	// Starting an Election
	if(!%so.election)
	{
		%so.election	= true;
		%so.elecLast	= getWord(getDateTime(), 0);
		%so.elecList	= "";
		%so.elecRem		= %so.elecLen;
		messageAll('', "\c6 - \c5The Mayoral Election has started!");
	}
}

// Sets a new mayor.
// @CityProfileSO
function SystemSO::setMayor(%so, %bl_id)
{
	%mayor = -1;
	
	if(%bl_id != %so.mayorBLID)
	{
		if(%so.mayorBLID != -1)
		{
			%oldMayor = Profiler.fetchProfile(%so.mayorBLID);
			JobCivilianSO.setJobFor(%oldMayor, false);
		}
		
		%newMayor = Profiler.fetchProfile(%bl_id);
		
		if(isObject(%newMayor))
		{
			%mayor = %newMayor;
			echo("Setting BL_ID " @ %bl_id @ " to mayor!");
			
			%so.mayorBLID = %bl_id;
			JobMayorSO.setJobFor(%newMayor, false);
		}
		else
		{
			echo("Could not set supplied ID (" @ %bl_id @ ") as mayor...");
		}
	}
	
	return %mayor;
}

if(!isObject(SystemSO))
{
	if(isFile($CRPG::Path::Saving @ "system.cs"))
		exec($CRPG::Path::Saving @ "system.cs");
	
	if(!isObject(SystemSO))
	{
		new ScriptObject(SystemSO)
		{
			// World Properties
			time		= 0;
			election	= false;
			elecTime	= 18; // Hour to start election.
			elecLast = "0/0/0";
			elecLen	= 7;
			elecList	= "";
			elecRem	= 0;
			
			// City Properties
			inmates	= "";
			anarchy	= false;
			
			// Statistics
			crimes	= 0;
			unsolved	= 0;
			solved	= 0;
			
			// === MAYORAL ===
			//  *- Elections
			mayorBLID		= -1;
			mayorElected	= 0;
			mayorPay			= 100;
			
			//  *- Intangible
			pricePlastic	= 0.75;
			priceRPlastic	= 1.25;
			treasury			= 0;
			lastTreasury			= 0;
			
			//  +- Taxes
			taxCommercial	= 10;
			taxLots			= 10;
			taxMonopoly		= 10;
			taxPlastic		= 10;
			taxRefPlastic	= 10;
			taxTarriff		= 10;
			taxWeapon		= 10;
			
			//  +- Moral
			moralAbortion	= true;
			moralFelonGun	= false;
			
			//  +- Superficial
			supCityName		= "The City";
			supMayorTitle	= "Mayor";
		};
	}
	
	CityScripts.add(SystemSO);
}

// ============================================================
// Section : TCPObject
// ============================================================
function DatabaseTO::init(%this, %url)
{
	%url = strReplace(%url, "http://", "");
	%host = "";
	%page = "";
	
	%host = getSubStr(%url, 0, strpos(%url, "/", 8) - 7);
	%page = getSubStr(%url, strpos(%url, "/", 8), 255);
	
	if(strpos(%host, ":") < 0)
		%host = %host @ ":" @ "80";
	
	%this.address = %host;
	%this.page = %page;
}

function DatabaseTO::get(%this)
{
	%this.buffer = "";
	%this.dobuffer = false;
	%this.method = "GET";
	%this.connect(%this.address);
}

function DatabaseTO::post(%this, %data)	
{
	%this.data = "valid=true" @ %data @ "&password=" @ %this.password;
	%this.method = "POST";
	%this.connect(%this.address);
}

function DatabaseTO::onConnected(%this)	
{
	%query = %this.method @ " " @ %this.page @ " HTTP/1.1\r\n";
	%query = %query @ "Host: " @ %this.address @ "\r\n";
	
	if(%this.method $= "POST")
	{
		%query = %query @ "User-Agent: Torque/1.0\r\n";
		%query = %query @ "Content-Type: application/x-www-form-urlencoded\r\n";
		%query = %query @ "Content-Length: " @ strlen(%this.data) @ "\r\n\r\n";
		%query = %query @ %this.data @ "\r\n";
	}
	
	%this.send(%query);
}

function DatabaseTO::onLine(%this, %line)	
{
	%funct = getField(%line, 0);
	%input = getField(%line, 1);
	
	if(isFunction(%funct))
		eval(%funct @ "(expandEscape(%input));");
}

function DatabaseTO::getResult(%this)	
{
	return %this.buffer;	
}

function DatabaseTO::onConnectFailed(%this)
{
	echo("DatabaseTO has failed to connect.");
	%this.delete();
}

function DatabaseTO::onDisconnect(%this)
{
	%this.delete();
}

// ============================================================
// Section : Trigger
// ============================================================
// Car Trigger
function CRPG_Car_Trigger::onEnterTrigger(%this, %trigger, %obj)
{
	%obj.crpgCarTrigger = %trigger.getID();
	
	if(isObject(%client = %obj.client))
	{
		%client.sendStatus();
		
		if(mFloatLength(%obj.crpgCrateOwner, 0) $= %obj.crpgCrateOwner && %obj.crpgCrateOwner != %client.bl_id)
		{
			commandToClient(%client, 'MessageBoxYesNo', "The Shady Part of Town",
			"You've made it to a shady part of town with a crate of stolen goods.\n" NL
			"If you want, you can cut a deal with someone here to open it for you.\n" NL
			"Naturally, they'll want to take half, and this is illegal.\n\n" NL
			"Do you want to bust open this crate?",
			'Event_DumpCrate');
		}
	}
}

function CRPG_Car_Trigger::onLeaveTrigger(%this, %trigger, %obj)
{
	if(isObject(%obj.crpgCarTrigger) && %obj.crpgCarTrigger.getID() == %trigger.getID())
	{
		%obj.crpgCarTrigger = "";
		%obj.crpgCarSales = "";
		
		if(%obj.getType() & $TypeMasks::PlayerObjectType)
		{
			%client = %obj.getControllingClient();
			
			if(isObject(%client) && isObject(%profile = %client.profile))
			{
				%client.crpgCarSpamTime = "";
				
				commandToClient(%client, 'BottomPrint', "", 0.0001);
			}
		}
	}
}

function CRPG_Car_Trigger::onTickTrigger(%this, %trigger, %obj)
{
	if(isObject(%obj.crpgCarTrigger) && %obj.crpgCarTrigger.getID() == %trigger.getID())
	{
		%client = %obj.getControllingClient();
		
		if(isObject(%client) && isObject(%profile = %client.profile))
		{
			if(%obj.getType() & $TypeMasks::VehicleObjectType)
			{
				// Has to be a spawned vehicle, client has to have crime on.
				if(!isObject(%obj.spawnBrick) || !%client.crpgEvil)
					return parent::onTickTrigger(%this, %trigger, %obj);
				
				// Load a brick group's profile.
				%brickgroup = %obj.spawnBrick.getGroup();
				%blid = %brickgroup.bl_id;
				%db = %obj.getDatablock();
				
				if(!Profiler.isProfileActive(%blid))
				{
					if(Profiler.isProfile(%blid))
					{
						if(!Profiler.loadProfile(%blid))
						{
							error("serverCmdEvent_Buy() - [Could not load profile for absentee client BL_ID " @ %blid @ "]");
						}
					}
					else
					{
						if(Profiler.createProfile(%blid))
							if(!Profiler.loadProfile(%blid))
								error("serverCmdEvent_Buy() - [Could not load newly-created profile for absentee client BL_ID " @ %blid @ "]");
							else
								error("serverCmdEvent_Buy() - [Could not load profile for absentee client BL_ID " @ %blid @ "]");
						else
							error("serverCmdEvent_Buy() - [Could not create profile for absentee client BL_ID " @ %blid @ "]");
					}
				}
				
				%brickProfile = Profiler.getProfile(%blid);
				
				if(isObject(%brickProfile) && %brickProfile.bl_id != %profile.bl_id)
				{
					%vehicles = getFieldCount(%brickProfile.garage);
					
					for(%a = 0; %a < %vehicles; %a++)
					{
						%vehicle = getField(%brickProfile.garage, %a);
						
						if(isObject(%vehicle) && %vehicle.getID() == %db.getID())
						{
							%canSpawn = true;
							break;
						}
					}
					
					if(%canSpawn)
					{
						%obj.crpgCarSales++;
						
						if(%obj.crpgCarSales < 100)
						{
							if(%obj.crpgLastMessage[%client.getID()] != %health || $sim::time - %obj.crpgLastMessage[%client.getID()] > 1)
							{
								%obj.crpgLastHealth[%client.getID()]	= %health;
								%obj.crpgLastMessage[%client.getID()]	= $sim::time;
								%worked = mFloor(100 * (%obj.crpgCarSales / 100));
								%client.sendProgress(%worked, "SELLING STOLEN VEHICLE");
							}
						}
						else
						{
							%obj.spawnBrick.setVehicle(0);
							%brickProfile.garage = removeField(%brickProfile.garage, %a);
							%brickProfile.lostv = addField(%brickProfile.lostv, %db.getName());
							
							%price = SalesSO.getVehiclePrice(%db) / 10;
							%price = mClamp(%price, 0, %price);
							%profile.money += %price;
							%client.commitCrime("Grand Theft Auto", 500, false, true, false, false, 180000);
							commandToClient(%client, 'CRPGCashChange', %price, %profile.money, %profile.bank, %profile.debt);
							commandToClient(%client, 'BottomPrint', "<just:center>\c2VEHICLE SOLD!", 3);
						}
					}
				}
			}
			else if(%obj.getType() & $TypeMasks::PlayerObjectType)
			{
				%image = %obj.getMountedImage(0);
				
				if($sim::time - %client.crpgCarSpamTime >= 1)
				{
					%client.crpgCarSpamTime = $sim::time;
					
					commandToClient(%client, 'BottomPrint', "<just:center>\c3SHADY PART OF TOWN\n\c6Throw a weapon to open the illegal sales dialog.", 1.25);
				}
			}
		}
	}
	
	parent::onTickTrigger(%this, %trigger, %obj);
}

if(!isObject(CRPG_Car_Trigger))
{
	datablock TriggerData(CRPG_Car_Trigger)
	{
		tickPeriodMS = 100;
		carTrigger = true;
	};
}

// Jail Trigger
function CRPG_Jail_Trigger::onEnterTrigger(%this, %trigger, %obj)
{
	if(isObject(%trigger.parent) && (%obj.getType() & $TypeMasks::PlayerObjectType))
	{
		%obj.crpgLot = %trigger.parent;
		
		if(isObject(%obj.client))
		{
			%obj.client.sendStatus();
			cancel(%obj.client.crpgJailBreak);
			%obj.client.crpgJailBreak = "";
		}
	}
	
	parent::onEnterTrigger(%this, %trigger, %obj);
}

function CRPG_Jail_Trigger::onLeaveTrigger(%this, %trigger, %obj)
{
	if(isObject(%obj.crpgLot) && isObject(%trigger.parent) && %obj.crpgLot.getID() == %trigger.parent.getID())
	{
		%obj.crpgLot = "";
		
		if(isObject(%obj.client))
		{
			%obj.client.sendStatus();
			%obj.client.crpgJailBreak = %obj.client.schedule(5000, "escapePrison");
		}
	}
	
	parent::onLeaveTrigger(%this, %trigger, %obj);
}

function CRPG_Jail_Trigger::onTickTrigger(%this, %trigger, %obj)
{
	if(!isObject(%obj.crpgLot) && isObject(%trigger.parent) && (%obj.getType() & $TypeMasks::PlayerObjectType))
	{
		%obj.crpgLot = %trigger.parent;
		
		if(isObject(%obj.client))
		{
			%obj.client.sendStatus();
			cancel(%obj.client.crpgJailBreak);
			%obj.client.crpgJailBreak = "";
		}
	}
}

if(!isObject(CRPG_Jail_Trigger))
{
	datablock TriggerData(CRPG_Jail_Trigger)
	{
		tickPeriodMS = 100;
		jailTrigger = true;
		lotTrigger = true;
	};
}

// Lot Trigger
function CRPG_Lot_Trigger::onEnterTrigger(%this, %trigger, %obj)
{
	if(%obj.getType() & $TypeMasks::PlayerObjectType && isObject(%trigger.parent))
	{
		%obj.crpgLot = %trigger.parent;
		
		if(isObject(%client = %obj.client))
		{
			%client.sendStatus();
			
			if(%obj.crpgCrateOwner == %trigger.parent.getGroup().bl_id)
			{
				commandToClient(%client, 'MessageBoxYesNo', "You Made It",
				"You've made it to your lot with a crate of goods." NL
				"Would you like to collect the material inside?",
				'Event_DumpCrate');
			}
		}
		
		// Brick Search
		%pos		= %trigger.position;
		%boxSize	= vectorSub(%trigger.scale, "0.1 0.1 0.0");
		%mask 	= $TypeMasks::FxBrickAlwaysObjectType;
		
		initContainerBoxSearch(%pos, %boxSize, %mask);
		
		while(isObject(%brick = containerSearchNext()))
		{
			if(%brick.numEvents)
			{
				%brick.onLotEnter(%obj);
			}
		}
	}
	
	parent::onEnterTrigger(%this, %trigger, %obj);
}

function CRPG_Lot_Trigger::onLeaveTrigger(%this, %trigger, %obj)
{
	if(%obj.getType() & $TypeMasks::PlayerObjectType && %obj.crpgLot $= %trigger.parent && isObject(%trigger.parent))
	{
		// Brick Search
		%pos		= %trigger.position;
		%boxSize	= vectorSub(%trigger.scale, "0.1 0.1 0.0");
		%mask 	= $TypeMasks::FxBrickAlwaysObjectType;
		
		initContainerBoxSearch(%pos, %boxSize, %mask);
		
		while(isObject(%brick = containerSearchNext()))
		{
			if(%brick.numEvents)
			{
				%trigger.parent.onLotLeave(%obj);
			}
		}
		
		%obj.crpgLot = "";
		
		if(isObject(%obj.client))
			%obj.client.sendStatus();
	}
	
	parent::onLeaveTrigger(%this, %trigger, %obj);
}

function CRPG_Lot_Trigger::onTickTrigger(%this, %trigger, %obj)
{
	if(!isObject(%obj.crpgLot) && isObject(%trigger.parent) && (%obj.getType() & $TypeMasks::PlayerObjectType))
	{
		%obj.crpgLot = %trigger.parent;
	}
}

if(!isObject(CRPG_Lot_Trigger))
{
	datablock TriggerData(CRPG_Lot_Trigger)
	{
		tickPeriodMS = 100;
		lotTrigger = true;
	};
}

// Trigger SimSet
if(!isObject(TriggerSS))
{
	new SimSet(TriggerSS);
	CityScripts.add(TriggerSS);
}
