// Tire
datablock WheeledVehicleTire(CRPG_Police_Truck_Tire)
{
   // Tires act as springs and generate lateral and longitudinal
   // forces to move the vehicle. These distortion/spring forces
   // are what convert wheel angular velocity into forces that
   // act on the rigid body.
   shapeFile = $CRPG::Path::Shapes @ "Police_Truck_Tire.dts";
	
	mass = 10;
    radius = 1;
    staticFriction = 7;
   kineticFriction = 7;
   restitution = 0.5;	

   // Spring that generates lateral tire forces
   lateralForce = 13000;
   lateralDamping = 6000;
   lateralRelaxation = 0.01;

   // Spring that generates longitudinal tire forces
   longitudinalForce = 12000;
   longitudinalDamping = 2000;
   longitudinalRelaxation = 0.01;
};


// Spring
datablock WheeledVehicleSpring(CRPG_Police_Truck_Spring)
{
   // Wheel suspension properties
   length = 0.4;			 // Suspension travel
   force = 9000; //3000;		 // Spring force
   damping = 600; //600;		 // Spring damping
   antiSwayForce = 6; //6;		 // Lateral anti-sway force
};


// Explosion
datablock DebrisData(CRPG_Police_Truck_Tire_Debris)
{
   emitters = "JeepTireDebrisTrailEmitter";

	shapeFile = $CRPG::Path::Shapes @ "Police_Truck_Tire.dts";
	lifetime = 2.0;
	minSpinSpeed = -400.0;
	maxSpinSpeed = 200.0;
	elasticity = 0.5;
	friction = 0.2;
	numBounces = 3;
	staticOnMaxBounce = true;
	snapOnMaxBounce = false;
	fade = true;

	gravModifier = 2;
};

datablock ExplosionData(CRPG_Police_Truck_Explosion : vehicleExplosion)
{
   debris = CRPG_Police_Truck_Tire_Debris;
   debrisNum = 4;
   debrisNumVariance = 0;
   debrisPhiMin = 0;
   debrisPhiMax = 360;
   debrisThetaMin = 40;
   debrisThetaMax = 85;
   debrisVelocity = 14;
   debrisVelocityVariance = 3;
};

datablock ProjectileData(CRPG_Police_Truck_Explosion_Projectile : vehicleExplosionProjectile)
{
   directDamage        = 0;
   radiusDamage        = 0;
   damageRadius        = 0;
   explosion           = CRPG_Police_Truck_Explosion;

   directDamageType  = $DamageType::VehicleExplosion;
   radiusDamageType  = $DamageType::VehicleExplosion;

   explodeOnDeath		= 1;

   armingDelay         = 0;
   lifetime            = 0;

   uiName = "";
};


// Final Explosion
datablock DebrisData(CRPG_Police_Truck_Debris)
{
   emitters = "JeepDebrisTrailEmitter";

	shapeFile = $CRPG::Path::Shapes @ "Police_Truck_Wreck.dts";
	lifetime = 10;
	minSpinSpeed = -100.0;
	maxSpinSpeed = 100.0;
	elasticity = 0.5;
	friction = 0.2;
	numBounces = 0;
	staticOnMaxBounce = true;
	snapOnMaxBounce = true;
	fade = true;

	gravModifier = 4;
};

datablock ExplosionData(CRPG_Police_Truck_Final_Explosion : vehicleFinalExplosion)
{
   debris = CRPG_Police_Truck_Debris;
   debrisNum = 1;
   debrisNumVariance = 0;
   debrisPhiMin = 0;
   debrisPhiMax = 360;
   debrisThetaMin = 0;
   debrisThetaMax = 20;
   debrisVelocity = 18;
   debrisVelocityVariance = 3;
};

datablock ProjectileData(CRPG_Police_Truck_Final_Explosion_Projectile : vehicleFinalExplosionProjectile)
{
   directDamage        = 0;
   radiusDamage        = 0;
   damageRadius        = 0;
   explosion           = CRPG_Police_Truck_Final_Explosion;

   directDamageType  = $DamageType::VehicleExplosion;
   radiusDamageType  = $DamageType::VehicleExplosion;

   explodeOnDeath		= 1;

   armingDelay         = 0;
   lifetime            = 0;

   uiName = "";
};


// Cannon
datablock TSShapeConstructor(CRPG_Police_Truck_Cannon_DTS)
{
	baseShape  = $CRPG::Path::Shapes @ "Police_Truck_Cannon.dts";
	sequence0  = $CRPG::Path::Shapes @ "Police_Truck_root.dsq root";

	sequence1  = $CRPG::Path::Shapes @ "Police_Truck_root.dsq run";
	sequence2  = $CRPG::Path::Shapes @ "Police_Truck_root.dsq walk";
	sequence3  = $CRPG::Path::Shapes @ "Police_Truck_root.dsq back";
	sequence4  = $CRPG::Path::Shapes @ "Police_Truck_root.dsq side";

	sequence5  = $CRPG::Path::Shapes @ "Police_Truck_root.dsq crouch";
	sequence6  = $CRPG::Path::Shapes @ "Police_Truck_root.dsq crouchRun";
	sequence7  = $CRPG::Path::Shapes @ "Police_Truck_root.dsq crouchBack";
	sequence8  = $CRPG::Path::Shapes @ "Police_Truck_root.dsq crouchSide";

	sequence9  = $CRPG::Path::Shapes @ "Police_Truck_look.dsq look";
	sequence10 = $CRPG::Path::Shapes @ "Police_Truck_root.dsq headside";
	sequence11 = $CRPG::Path::Shapes @ "Police_Truck_root.dsq headUp";

	sequence12 = $CRPG::Path::Shapes @ "Police_Truck_root.dsq jump";
	sequence13 = $CRPG::Path::Shapes @ "Police_Truck_root.dsq standjump";
	sequence14 = $CRPG::Path::Shapes @ "Police_Truck_root.dsq fall";
	sequence15 = $CRPG::Path::Shapes @ "Police_Truck_root.dsq land";

	sequence16 = $CRPG::Path::Shapes @ "Police_Truck_root.dsq armAttack";
	sequence17 = $CRPG::Path::Shapes @ "Police_Truck_root.dsq armReadyLeft";
	sequence18 = $CRPG::Path::Shapes @ "Police_Truck_root.dsq armReadyRight";
	sequence19 = $CRPG::Path::Shapes @ "Police_Truck_root.dsq armReadyBoth";
	sequence20 = $CRPG::Path::Shapes @ "Police_Truck_root.dsq spearready";  
	sequence21 = $CRPG::Path::Shapes @ "Police_Truck_root.dsq spearThrow";

	sequence22 = $CRPG::Path::Shapes @ "Police_Truck_root.dsq talk";  

	sequence23 = $CRPG::Path::Shapes @ "Police_Truck_root.dsq death1"; 
	
	sequence24 = $CRPG::Path::Shapes @ "Police_Truck_root.dsq shiftUp";
	sequence25 = $CRPG::Path::Shapes @ "Police_Truck_root.dsq shiftDown";
	sequence26 = $CRPG::Path::Shapes @ "Police_Truck_root.dsq shiftAway";
	sequence27 = $CRPG::Path::Shapes @ "Police_Truck_root.dsq shiftTo";
	sequence28 = $CRPG::Path::Shapes @ "Police_Truck_root.dsq shiftLeft";
	sequence29 = $CRPG::Path::Shapes @ "Police_Truck_root.dsq shiftRight";
	sequence30 = $CRPG::Path::Shapes @ "Police_Truck_root.dsq rotCW";
	sequence31 = $CRPG::Path::Shapes @ "Police_Truck_root.dsq rotCCW";

	sequence32 = $CRPG::Path::Shapes @ "Police_Truck_root.dsq undo";
	sequence33 = $CRPG::Path::Shapes @ "Police_Truck_root.dsq plant";

	sequence34 = $CRPG::Path::Shapes @ "Police_Truck_root.dsq sit";

	sequence35 = $CRPG::Path::Shapes @ "Police_Truck_root.dsq wrench";
};    

datablock PlayerData(CRPG_Police_Truck_Cannon_Player)
{
   renderFirstPerson = true;
   emap = false;
   
   className = Armor;
   shapeFile = $CRPG::Path::Shapes @ "Police_Truck_Cannon.dts";
   cameraMaxDist = 8;
   cameraTilt = 0.261;
   cameraVerticalOffset = 2.3;
   computeCRC = false;
  
   canObserve = true;
   cmdCategory = "Clients";

   cameraDefaultFov = 90.0;
   cameraMinFov = 5.0;
   cameraMaxFov = 120.0;
   
   //debrisShapeName = "~/data/player/debris_player.dts";
   //debris = horseDebris;

   aiAvoidThis = true;

   minLookAngle = -1.5708;
   maxLookAngle = 1.5708;
   maxFreelookAngle = 3.0;

   mass = 99999;
   drag = 0.1;
   maxdrag = 0.52;
   density = 0.7;
   maxDamage = 250;
   maxEnergy =  10;
   repairRate = 0.33;
   energyPerDamagePoint = 75.0;

   rechargeRate = 0.4;

   runForce = 0 * 90;
   runEnergyDrain = 0;
   minRunEnergy = 0;
   maxForwardSpeed = 0;
   maxBackwardSpeed = 0;
   maxSideSpeed = 0;

   maxForwardCrouchSpeed = 0;
   maxBackwardCrouchSpeed = 0;
   maxSideCrouchSpeed = 0;

   maxForwardProneSpeed = 0;
   maxBackwardProneSpeed = 0;
   maxSideProneSpeed = 0;

   maxForwardWalkSpeed = 0;
   maxBackwardWalkSpeed = 0;
   maxSideWalkSpeed = 0;

   maxUnderWaterForwardSpeed = 0;
   maxUnderWaterBackwardSpeed = 0;
   maxUnderWaterSideSpeed = 0;

   jumpForce = 0; //8.3 * 90;
   jumpEnergyDrain = 0;
   minJumpEnergy = 0;
   jumpDelay = 0;

   minJetEnergy = 0;
	jetEnergyDrain = 0;
	canJet = 0;

   recoverDelay = 0;
   recoverRunForceScale = 1.2;

   minImpactSpeed = 250;
   speedDamageScale = 3.8;

   boundingBox			= vectorScale("2.5 2.5 1.7", 4); 
   crouchBoundingBox	= vectorScale("2.5 2.5 1.7", 4); 
   proneBoundingBox	= vectorScale("2.5 2.5 1.7", 4); 

   pickupRadius = 0.75;
   
   // Damage location details
   boxNormalHeadPercentage       = 0.83;
   boxNormalTorsoPercentage      = 0.49;
   boxHeadLeftPercentage         = 0;
   boxHeadRightPercentage        = 1;
   boxHeadBackPercentage         = 0;
   boxHeadFrontPercentage        = 1;

   // Foot Prints
   //decalData   = HorseFootprint;
   //decalOffset = 0.25;
	
   jetEmitter = playerJetEmitter;
   jetGroundEmitter = playerJetGroundEmitter;
   jetGroundDistance = 4;
  
   //footPuffEmitter = LightPuffEmitter;
   footPuffNumParts = 10;
   footPuffRadius = 0.25;

   //dustEmitter = LiftoffDustEmitter;

   splash = PlayerSplash;
   splashVelocity = 4.0;
   splashAngle = 67.0;
   splashFreqMod = 300.0;
   splashVelEpsilon = 0.60;
   bubbleEmitTime = 0.1;
   splashEmitter[0] = PlayerFoamDropletsEmitter;
   splashEmitter[1] = PlayerFoamEmitter;
   splashEmitter[2] = PlayerBubbleEmitter;
   mediumSplashSoundVelocity = 10.0;   
   hardSplashSoundVelocity = 20.0;   
   exitSplashSoundVelocity = 5.0;

   // Controls over slope of runnable/jumpable surfaces
   runSurfaceAngle  = 85;
   jumpSurfaceAngle = 86;

   minJumpSpeed = 0;
   maxJumpSpeed = 0;

   horizMaxSpeed = 0;
   horizResistSpeed = 0;
   horizResistFactor = 0;

   upMaxSpeed = 0;
   upResistSpeed = 0;
   upResistFactor = 0;
   
   footstepSplashHeight = 0.35;

   //NOTE:  some sounds commented out until wav's are available

   JumpSound			= HorseJumpSound;
	deathCry = -1;

   // Footstep Sounds
   //FootSoftSound        = HorseFootFallSound;
   //FootHardSound        = HorseFootFallSound;
   //FootMetalSound       = HorseFootFallSound;
   //FootSnowSound        = HorseFootFallSound;
   //FootShallowSound     = HorseFootFallSound;
   //FootWadingSound      = HorseFootFallSound;
   //FootUnderCRPG_Police_Truck_Sound  = HorseFootFallSound;
   //FootBubblesSound     = FootLightBubblesSound;
   //movingBubblesSound   = ArmorMoveBubblesSound;
   //CRPG_Police_Truck_BreathSound     = CRPG_Police_Truck_BreathMaleSound;

   //impactSoftSound      = ImpactLightSoftSound;
   //impactHardSound      = ImpactLightHardSound;
   //impactMetalSound     = ImpactLightMetalSound;
   //impactSnowSound      = ImpactLightSnowSound;
   
   //impactCRPG_Police_Truck_Easy      = ImpactLightCRPG_Police_Truck_EasySound;
   //impactCRPG_Police_Truck_Medium    = ImpactLightCRPG_Police_Truck_MediumSound;
   //impactCRPG_Police_Truck_Hard      = ImpactLightCRPG_Police_Truck_HardSound;
   
   groundImpactMinSpeed    = 10.0;
   groundImpactShakeFreq   = "4.0 4.0 4.0";
   groundImpactShakeAmp    = "1.0 1.0 1.0";
   groundImpactShakeDuration = 0.8;
   groundImpactShakeFalloff = 10.0;
   
   //exitingCRPG_Police_Truck_         = ExitingCRPG_Police_Truck_LightSound;
   
   observeParameters = "0.5 4.5 4.5";

   // Inventory Items
	maxItems   = 10;	//total number of bricks you can carry
	maxWeapons = 5;		//this will be controlled by mini-game code
	maxTools = 5;
	 
	uiName = "Police Water Cannon";
	rideable = true;
   lookUpLimit = 0.5;
   lookDownLimit = 0.5;

	canRide = false;
	showEnergyBar = false;
	paintable = true;

	brickImage = horseBrickImage;	//the imageData to use for brick deployment

   numMountPoints = 1;
   mountThread[0] = "Sit";

   //protection for passengers
   protectPassengersBurn   = true;  //protect passengers from the burning effect of explosions?
   protectPassengersRadius = true;  //protect passengers from radius damage (explosions) ?
   protectPassengersDirect = false; //protect passengers from direct damage (bullets) ?
};


// Vehicle
datablock WheeledVehicleData(CRPG_Police_Truck_Vehicle)
{
	category = "Vehicles";
	displayName = " ";
	shapeFile = $CRPG::Path::Shapes @ "Police_Truck.dts"; //"~/data/shapes/skivehicle.dts"; //
	emap = true;
	minMountDist = 3;
   
   numMountPoints = 3;
   mountThread[0] = "sit";
   mountThread[1] = "sit";
   mountThread[2] = "sit";

	maxDamage = 270.00;
	destroyedLevel = 160.00;
	energyPerDamagePoint = 160;
	speedDamageScale = 1.04;
	collDamageThresholdVel = 20.0;
	collDamageMultiplier   = 0.02;

	massCenter = "0 0 0";
   //massBox = "2 5 1";

	maxSteeringAngle = 0.97;  // Maximum steering angle, should match animation
	integration = 4;           // Force integration time: TickSec/Rate
	tireEmitter = VehicleTireEmitter; // All the tires use the same dust emitter

	// 3rd person camera settings
	cameraRoll = false;         // Roll the camera with the vehicle
	cameraMaxDist = 15;         // Far distance from vehicle
	cameraOffset = 9.5;        // Vertical offset from camera mount point
	cameraLag = 0.0;           // Velocity lag of camera
	cameraDecay = 0.75;        // Decay per sec. rate of velocity lag
	cameraTilt = 0.4;
   collisionTol = 0.1;        // Collision distance tolerance
   contactTol = 0.1;

	useEyePoint = false;	

	defaultTire	= CRPG_Police_Truck_Tire;
	defaultSpring	= CRPG_Police_Truck_Spring;
	//flatTire	= jeepFlatTire;
	//flatSpring	= jeepFlatSpring;

   numWheels = 6;

	// Rigid Body
	mass = 800;
	density = 3.0;
	drag = 1.6;
	bodyFriction = 0.7;
	bodyRestitution = 0.7;
	minImpactSpeed = 10;        // Impacts over this invoke the script callback
	softImpactSpeed = 10;       // Play SoftImpact Sound
	hardImpactSpeed = 15;      // Play HardImpact Sound
	groundImpactMinSpeed    = 10.0;

	// Engine
	engineTorque = 5500; //4000;       // Engine power
	engineBrake = 2000;         // Braking when throttle is 0
	brakeTorque = 3500;        // When brakes are applied
	maxWheelSpeed = 22;        // Engine scale by current speed / max speed

	rollForce		= 100;
	yawForce		= 300;
	pitchForce		= 1000;
	rotationalDrag		= 0.2;

	// Advanced Steering
   steeringAutoReturn = true;
   steeringAutoReturnRate = 0.9;
   steeringAutoReturnMaxSpeed = 10;
   steeringUseStrafeSteering = true;
   steeringStrafeSteeringRate = 0.1;
	
	// Energy
	maxEnergy = 100;
	jetForce = 3000;
	minJetEnergy = 30;
	jetEnergyDrain = 2;

	splash = jeepSplash;
	splashVelocity = 4.0;
	splashAngle = 67.0;
	splashFreqMod = 300.0;
	splashVelEpsilon = 0.60;
	bubbleEmitTime = 1.4;
	splashEmitter[0] = VehicleFoamDropletsEmitter;
	splashEmitter[1] = VehicleFoamEmitter;
	splashEmitter[2] = VehicleBubbleEmitter;
	mediumSplashSoundVelocity = 10.0;   
	hardSplashSoundVelocity = 20.0;   
	exitSplashSoundVelocity = 5.0;
		
	//mediumSplashSound = "";
	//hardSplashSound = "";
	//exitSplashSound = "";
	
	// Sounds
	//   jetSound = ScoutThrustSound;
	//engineSound = idleSound;
	//squealSound = skidSound;
	softImpactSound = slowImpactSound;
	hardImpactSound = fastImpactSound;
	//wheelImpactSound = slowImpactSound;
	
	//   explosion = VehicleExplosion;
	justcollided = 0;

   uiName = "Police Water Truck";
	rideable = true;
		lookUpLimit = 0.50;
		lookDownLimit = 0.50;

	paintable = true;
   
   damageEmitter[0] = VehicleBurnEmitter;
	damageEmitterOffset[0] = "0.0 0.0 0.0 ";
	damageLevelTolerance[0] = 0.99;

   damageEmitter[1] = VehicleBurnEmitter;
	damageEmitterOffset[1] = "0.0 0.0 0.0 ";
	damageLevelTolerance[1] = 1.0;

   numDmgEmitterAreas = 1;

   initialExplosionProjectile = CRPG_Police_Truck_Explosion_Projectile;
   initialExplosionOffset = 0;         //offset only uses a z value for now

   burnTime = 4000;

   finalExplosionProjectile = CRPG_Police_Truck_Final_Explosion_Projectile;
   finalExplosionOffset = 0.5;          //offset only uses a z value for now


   minRunOverSpeed    = 4;   //how fast you need to be going to run someone over (do damage)
   runOverDamageScale = 6;   //when you run over someone, speed * runoverdamagescale = damage amt
   runOverPushScale   = 1.7; //how hard a person you're running over gets pushed
};


// Scripts
function CRPG_Police_Truck_Cannon_Player::Damage(%data,%obj,%source,%pos,%amm,%type)
{
	//scale the damage ammount as if this was a vehicle     
	%amm *= $Damage::VehicleDamageScale[%type];
	
	if(%obj.getDamageLevel()+%amm > %data.maxDamage && !isObject(%obj.client))
		%obj.setDamageLevel(%data.maxDamage);
	else
		Parent::Damage(%data,%obj,%source,%pos,%amm,%type);
}

function CRPG_Police_Truck_Cannon_Player::onAdd(%this, %obj)
{
   %obj.lastShotTime = getSimTime() - 2000;
   Parent::onAdd(%this, %obj);
}

function CRPG_Police_Truck_Cannon_Player::onDisabled(%this,%obj,%state)
{
	%tank = %obj.getObjectMount();
	if(isObject(%tank))
		%pos = vectorAdd(%obj.getObjectMount().getPosition(),"0 0" SPC %obj.getObjectMount().dataBlock.initialExplosionOffset);
	else
		%pos = %obj.getHackPosition();
	
	%p = new Projectile()
	{
		dataBlock = TankTurretExplosionProjectile;
		initialPosition = %pos;
		initialVelocity = "0 0 1";
		client = %obj.lastDamageClient;
		sourceClient = %obj.lastDamageClient;
	};
	MissionCleanup.add(%p);
	%obj.schedule(10,"delete");

	//mount the guy who was in the turret to the tank
	if(isObject(%tank))
	{
		%player = %obj.getMountedObject(0);
		if(isObject(%player))
		{
			%obj.getObjectMount().schedule(10,"mountObject",%player,2);
		}
	}
	
	if(!isObject(%tank))
	{
		if(isObject(%obj.spawnBrick))
		{
			%mg = getMiniGameFromObject(%obj);
			if(isObject(%mg))
			{
				%obj.spawnBrick.spawnVehicle(%mg.vehicleReSpawnTime);
			}
			else
			{
				%obj.spawnBrick.spawnVehicle(0);
			}         
		}
  
		Parent::onDisabled(%this,%obj,%state);              
	}
}  

function CRPG_Police_Truck_Cannon_Player::onDriverLeave(%this,%obj)
{
   if(%obj.isMounted())
      %obj.setTransform("0 0 0 0 0 0 0");
}

function CRPG_Police_Truck_Cannon_Player::rigTurret(%this, %obj)
{
   if(!isObject(%obj))
	   return;

   %parent = %obj.getObjectMount();
   if(!isObject(%parent))
	   return;

   %obj.spawnBrick = %parent.spawnBrick;
   %obj.brickGroup = %parent.brickGroup;
}

function CRPG_Police_Truck_Vehicle::Damage(%this,%obj,%source,%pos,%amm,%type)
{
	if((%obj.getDamageLevel()+%amm) >= %this.maxDamage)
	{
		if(%obj.destroyed)
			return;
		
		%obj.setNodeColor("ALL","0 0 0 1");
		if(isObject(%obj.turret))
		{
			%obj.turret.delete();
			%p = new Projectile()
			{
				dataBlock = TankTurretExplosionProjectile;
				initialPosition = vectorAdd(%obj.getPosition(),"0 0" SPC %this.initialExplosionOffset);
				initialVelocity = "0 0 1";
				client = %obj.lastDamageClient;
				sourceClient = %obj.lastDamageClient;
			};
			MissionCleanup.add(%p);
		}
		else
		{
			%p = new Projectile()
			{
				dataBlock = TankExplosionProjectile;
				initialPosition = vectorAdd(%obj.getPosition(),"0 0" SPC %this.initialExplosionOffset);
				initialVelocity = "0 0 1";
				client = %obj.lastDamageClient;
				sourceClient = %obj.lastDamageClient;
			};
			MissionCleanup.add(%p);
		}
		if(%obj.destroyed)
			return;
		
		%obj.setDamageLevel(%this.maxDamage);
		%obj.destroyed = 1;
		%obj.schedule(%this.burnTime,"finalExplosion");
		if(isObject(%obj.spawnBrick.client.minigame))
			%respawn = %obj.spawnBrick.client.minigame.vehicleReSpawnTime;
		%obj.spawnBrick.schedule(%respawn,"spawnVehicle");
	}
	else
		Parent::Damage(%this,%obj,%source,%pos,%amm,%type);
}

function CRPG_Police_Truck_Vehicle::onAdd(%this, %obj)
{
   // Setup the car with some defaults tires & springs
   for(%i = 0; %i < %this.numWheels; %i++)
   {
      %obj.setWheelTire(%i, %this.defaultTire);
      %obj.setWheelSpring(%i, %this.defaultSpring);
   }

   // 4 wheel steering
   %obj.setWheelSteering(0,1);
   %obj.setWheelSteering(1,1);
   %obj.setWheelSteering(2,-0.8);
   %obj.setWheelSteering(3,-0.8);

   // 4 wheel drive
   %obj.setWheelPowered(0,true);
   %obj.setWheelPowered(1,true);
   %obj.setWheelPowered(2,true);
   %obj.setWheelPowered(3,true);

   // turret
   %t = new AIPlayer()
   {
      dataBlock = CRPG_Police_Truck_Cannon_Player;
   };
   MissionCleanup.add(%t);
   %obj.mountObject(%t, 2);
   %obj.turret = %t;

   %obj.creationTime = getSimTime();
   %t.getDatablock().schedule(10, "rigTurret", %obj);
}

function CRPG_Police_Truck_Vehicle::onRemove(%this,%obj)
{
   if(isObject(%obj.turret))
	   %obj.turret.delete();
}

package CRPG_Police_Truck_Package
{
   function Player::burn(%obj,%time)
   {
      if(%obj.dataBlock !$= CRPG_Police_Truck_Cannon_Player)
         Parent::burn(%obj,%time);
   }
	
   function ShapeBase::setNodeColor(%obj,%node,%color)
   {
      Parent::setNodeColor(%obj,%node,%color);
		
      if(isObject(%obj.turret))
         %obj.turret.setNodeColor("ALL",%color);
   }
	
   function Player::emote(%obj,%emote)
   {
      if(%obj.dataBlock $= CRPG_Police_Truck_Cannon_Player)
	      return;
      Parent::emote(%obj,%emote);
   }
	
   function armor::onMount(%this,%obj,%col,%slot)
   {
      Parent::onMount(%this,%obj,%col,%slot);
      if(%col.getDataBlock() == CRPG_Police_Truck_Vehicle.getId())
      {
         if(%slot $= 2)
         {
            %obj.setLookLimits(1,0);
         }
      }
      else if(%col.getDataBlock() == CRPG_Police_Truck_Cannon_Player.getId())
      {
         %client = %obj.client;
         if(isObject(%client))
            ServerCmdUnUseTool(%client);
      }
   }

   function armor::onTrigger(%this, %obj, %triggerNum, %val)
   {
      %mount = %obj.getObjectMount();

      //hack so we can shoot if we are a tank turret
      if(%obj.getDataBlock().getID() == CRPG_Police_Truck_Cannon_Player.getID())
         %mount = %obj;

      if(isObject(%mount))
      {
         if(%mount.getDataBlock() == CRPG_Police_Truck_Cannon_Player.getId() && %triggerNum == 0 && %val)
         {
            %client = %obj.client;
            if(isObject(%client))
               ServerCmdUnUseTool(%client);

            if(getSimTime() - %obj.lastShotTime < 2500)
               return;
            
            %scaleFactor = getWord(%mount.getScale(), 2);
            %p = new Projectile()
            {
               dataBlock = tankShellProjectile;
               initialPosition = %mount.getSlotTransform(1);
               initialVelocity = vectorScale(%mount.getMuzzleVector(1),140 * %scaleFactor);
               sourceObject = %obj;
               client = %obj.client;
               sourceSlot = 0;
               originPoint = %mount.getSlotTransform(1);
            };
            MissionCleanup.add(%p);
            %p.setScale(%scaleFactor SPC %scaleFactor SPC %scaleFactor);
				
            serverPlay3D(TankshotSound,%obj.getPosition());
         
            %tank = %mount.getObjectMount();
            if(isObject(%tank))
            {
               %theVector = %mount.getEyeVector();
               %theVector = vectorAdd(%theVector,"0 0 1");
               %theVector = vectorScale(%theVector,%tank.getDataBlock().mass*5);
               %theVector = vectorSub(%theVector,vectorScale(%thevector,2));
               
               %tank.applyImpulse(getWords(%mount.getEyeTransform(),0,2),%theVector);	
            }
            
            %obj.lastShotTime = getSimTime();

            return;
         }
      }
      
      Parent::onTrigger(%this,%obj,%triggerNum,%val);
   }
};

if(!$CRPG_Temp::Packages)
	activatepackage(CRPG_Police_Truck_Package);
